#include "common\\include\\commondef.h"
#include "systems\\common\\include\\systemdef.h"
#include "systems\\common\\include\\plugindef.h"
#include <userint.h>    // UIR  , -  -
#include <ansi_c.h>
#include "core.h"

static systemManager sysMng;
static pluginManager plugMng;		
static systemDefine  *sysInstance= NULL;  //      
static pluginDefine  *plugDefine= NULL;  // !!! TMP -         
static pluginList    pList= {NULL, NULL, NULL, NULL, NULL};
static int           nPlugin;

static char currentPluginPath[FNAMELEN+1]= "";

#define PLUGROOT_DIR "systems\\"

char *core_getSysDescr(void) {return sysMng.system()->info.descr;};

systemManager *systemCore(void) { return &sysMng;};
pluginManager *moduleCore(void) { return &plugMng; };

int   core_getPlugNum(void) { return nPlugin; };
char *core_getPlugName(int n) {return pList.name[n]; };
char *core_getPlugFileName(int n) {return pList.filename[n]; };
char *core_getPlugPath(int n) {return pList.path[n]; };
char *core_getPlugDescr(int n) {return pList.descr[n]; };

void closeUserSystem(void) {
  closeUTFSystemDLL(&sysMng); 
};

int openUserSystem(char *syspath, char *sysname) {
 int r;
 r= openUTFSystemDLL(&sysMng, syspath);
 if(!r) strcpy(sysname, sysMng.system()->info.name);
 return r;
};


int openPlugList4System(char *sysname) 
{
  char mask[FNAMELEN+1];
  char rootpath[FNAMELEN+1];
  
  strcpy(mask, sysname);
  strcat(mask, "*.");
  
  strcpy(rootpath, PLUGROOT_DIR);
  strcat(rootpath, sysname);
  
  return createPluginList(&pList, &nPlugin, rootpath, mask);
};

void closePlugList4System(void) 
{
  discardPluginList(&pList);
};

int  insertModuleList2Bar(int pnl, int bar, int menuItem, 
  void CVICALLBACK ModuleFunction ( int menuBarHandle,
         int menuItemID, void *callbackData, int panelHandle )) {
         
 int  did= -1;
 char tmp[2*FNAMELEN + 16];
 int  firstId; 
 moduleDefine *m; 
 
 GetMenuBarAttribute (bar, menuItem, ATTR_FIRST_ITEM_ID, &firstId);
 
 did= InsertSeparator(bar, menuItem, firstId); 
 if(did<= 0) return 1;
 
 for(m= sysInstance->module; m!= NULL; m= m->next) {
     sprintf(tmp, "%s", m->descr);
	 NewMenuItem (bar, menuItem, tmp, did, 0, ModuleFunction, NULL); 
 }; 
 return 0; 
};

int disconnectSystem(void) {
 if(sysInstance == NULL) return 0;
 sysMng.system()->close(); // sysInstance);
 sysInstance= NULL;
 return 0;
};

int connect2System(void) {
   if(sysInstance != NULL) {
      disconnectSystem();
   };
   
   if(sysMng.system()->open()) return 1;
   sysInstance= sysMng.system();
   if(sysInstance->module == NULL ) {
        sysMng.system()->close(); // sysInstance);
        sysInstance= NULL;
        return 1;
   };
   return 0;
};

char *core_getModuleName(char *moduleDescr) {
 moduleDefine *m; 
 
 for(m= sysInstance->module; m!= NULL; m= m->next) {
     if(!stricmp(m->descr, moduleDescr)) return m->name;
 };
 return NULL;
};

int core_getPluginNum4Module(char *moduleName) {
 int n= 0, j;
 
 for(j= 0; j< nPlugin; j++) {
     if(!stricmp(moduleName, pList.name[j])) n++;
 };
 return n;
};

void freeIntList(IntList *list) {
	IntList *next;
	while (list) {
		next = list->next;
		free(list);
		list = next;
	}
}

/**
*      , 
*   .
*
*/
IntListAndLength core_getPluginList4Module(char *moduleName) {
	IntListAndLength result;
	IntList *item, **tail;
	int n= 0, j;

	result.list = NULL;/*  .*/
	tail = &result.list;/*    .*/
	for(j= 0; j< nPlugin; j++) {/*  */
		if(!stricmp(moduleName, pList.name[j])) {/* .*/
			if(!(item = (IntList*)malloc(sizeof(IntList)))) {/*    */
				/* */
				freeIntList(result.list);
				result.list = NULL;
				n = 0;
				break;
			}
			item->data = j;/* .*/
			item->next = NULL;
			*tail = item;/* .*/
			tail = &item->next;/*    .*/
			n++;
		}
	}
	
	result.number = n;
	return result;
};

char *core_getPluginPath4Module(int num, char *moduleName) {
 int n= 0, j;
 for(j= 0; j< nPlugin; j++) {
     if(!stricmp(moduleName, pList.name[j])) {
        if(n == num) return pList.path[j];
        n++;
     };
 };   
 return NULL;
};

char *core_getPluginDescr4Module(int num, char *moduleName) {
 int n= 0, j;
 for(j= 0; j< nPlugin; j++) {
     if(!stricmp(moduleName, pList.name[j])) {
        if(n == num) return pList.descr[j];
        n++;
     };
 };   
 return NULL;
};

int  deselectPlugin(void) {
 if(plugDefine == NULL) return 0;
 closePlugin(&plugMng);
 plugDefine= NULL;
 return 0;
};

/*int  selectPlugin(char *path) {
  int ind= 0;  // tmp
  int j, n;

  if(plugDefine!= NULL) {
     deselectPlugin();
  };
  
  if(openPlugin(&plugMng, path)) return 1;

  plugDefine= plugMng.plugin();
  strcpy(currentPluginPath, path);
  return 0;
};*/



int  selectPlugin(int index) {
	if(plugDefine!= NULL) {
		deselectPlugin();
	};

	if(openPlugin(&plugMng, index)) return 1;

	plugDefine= plugMng.plugin();
	strcpy(currentPluginPath, core_getPlugPath(index));
	return 0;
}


char *pluginCurrentPath(void) { return currentPluginPath; };

char *pluginCurrentDescr(void) { return plugDefine->descr; };


// typedef void         * __stdcall (*getUTFSystemTContext_Ptr)(moduleDefine *mi);

void * core_getSysTContext(char *moduleDescr) {
 moduleDefine *m; 
 
 for(m= sysInstance->module; m!= NULL; m= m->next) {
     if(!stricmp(m->descr, moduleDescr)) 
         return sysMng.tcontext(m);
 };
  return NULL;
};


static int testCBFunction_Default (unsigned int id, char *msg) 
{
 return 0;
};

int fillTContext(TCONTEXT *tc, tCbfPtr f, char *currentModule_Descr, void *te01) 
{
  int j, n= strlen(currentModule_Descr);
  
  memset(tc, 0x0, sizeof(TCONTEXT));
  strcpy(tc->dllpath, pluginCurrentPath());
  
  if(f == NULL) tc->cbf= testCBFunction_Default;
  else tc->cbf= f;
  
  tc->tcontext= core_getSysTContext(currentModule_Descr);
  
  strcpy(tc->mdescr, currentModule_Descr);
  for(j= 0; j< n; j++) {
      if(tc->mdescr[j] == ' ' || tc->mdescr[j] == '-' || tc->mdescr[j] == ':')
        tc->mdescr[j]= '_';  // !!! tmp
  };
  
  tc->te01= te01;
  return 0;
};


static int LocalTFUseFlags[256];
void core_fillLocalTFUseFlags(void) 
{
  int n, j;
  
  n= moduleCore()->fnum();
  for(j= 0; j<256; j++) LocalTFUseFlags[j]= 0;
  for(j= 0; j<n; j++) LocalTFUseFlags[j]= moduleCore()->f(j)->set.use;
};

void core_updateLocalTFUseFlags(int n, int use[256]) 
{
 int j;
 
 if(n>256) n= 256;
 if(n<0) n= 0;
 for(j= 0; j<n; j++) LocalTFUseFlags[j]= use[j];
};

int core_isUseTF(int fnum) 
{
 if(fnum > 255) return 0; // ??? !!! tmp
 if(fnum < 0) return 0; // ??? !!! tmp
 return LocalTFUseFlags[fnum];
};

#define COREINIFNAME "utsini.ini"

				//   0123456789012345
#define COREINISIGN "UTS Settings V1"
void coreReadIni(COREINISET *set) 
{
  FILE *f= fopen(COREINIFNAME, "rb"); 
  COREINISET x;
  
  memset(set, 0x0, sizeof(COREINISET));
  strcpy(set->sign, COREINISIGN);
  set->loop= 1;
  set->breakOnError= 0;
  set->launchAll= 0;
  set->te01.port= 0;	  // com1
  set->te01.use= 1;
  set->keithley.port= 1;	// com2
  set->keithley.use= 0;  
  set->burn.port= 0;  // lpt1
  set->burn.use= 1;   

  set->crc= 0xFFD5AA55;
  if(f == NULL) return;
  
  if(fread (&x, sizeof(COREINISET), 1, f) == 1) 
     memcpy(set, &x, sizeof(COREINISET));
  if(set->te01.port == set->keithley.port) {
     set->te01.port= 0;
     set->keithley.port= 1;
  };
  fclose(f);
};

void coreWriteIni(COREINISET *set) {
  FILE *f= fopen(COREINIFNAME, "wb"); 
  if(f == NULL) return;
  fwrite (set, sizeof(COREINISET), 1, f);
  fclose(f);
};

