#include <userint.h>
#include "systems\\common\\include\\plugindef.h"
#include "toolbox.h"
#include <utility.h>
#include <ansi_c.h>
#include "executoruir.h"
#include "core.h"
#include "reporter.h"

static int Pnl= -1;
#define P(x) TPNL_##x

#define EXEC_SINGLE_TEST__NEXT        0
#define EXEC_SINGLE_TEST__ABORT       1
#define EXEC_SINGLE_TEST__STOPONERROR 2

extern void *getTE01Handle(void);
       int  exec_SingleTest(int tnum, TCONTEXT *tc, int force);
       static void updateStatistics(void);
       
static int  SaveInfo4Refs_Flag;
       void run_SaveInfo4Refs(void);
static int  testCBFunction (unsigned int id, char *msg);
static char currentModule_Descr[FNAMELEN+1];
static void (*functionPtr_WhenQuit)(void)= NULL;

extern int  createExecEnv(pluginManager *pm);
extern int  destroyExecEnv(void);
extern int  execSetMode_Run(int Pnl);
extern int  execSetMode_Stop(int Pnl);
extern void execUpdateResult(int Pnl, int testNum, int result);

#pragma pack(1)
typedef struct {						  //
  int passed;							  //
  int fail;								  //
  int total;							  //
} execCntr;								  //

typedef struct {						  //
  int      button;		                  // button controlID 
  int      led;							  // led() controlID
  int      ledtotal;					  // ledtotal() controlID
  int      failnum;						  // 
} execUIR;								  //

typedef struct {
  execUIR  uir;
  execCntr cntr;
  int      tnum;						  //   
} execResult;
#pragma pack()

static execResult *execTResult= NULL;
static int buttonDefColor;
// ----------------------------------------------------------------------------
//    
// ----------------------------------------------------------------------------
static int freeUirExecID(execResult  **p) {
 int j= 0, flag= 0;
 
 if(*p == NULL) return 0;
 free(*p);
 *p= NULL;  
 return 0;
};
// ----------------------------------------------------------------------------
//      
// ----------------------------------------------------------------------------
static int allocateUirExecID(int n, execResult **p) {
  if((*p= (execResult *)malloc(sizeof(execResult )*(n+1))) == NULL) goto __bad_ret;
  memset(*p, 0x0, sizeof(execResult)*(n+1));
  (*p)[n].tnum= -1;
  return 0;
__bad_ret:
  freeUirExecID(p);
  return 1;
};
// ----------------------------------------------------------------------------
//         
//      
// ----------------------------------------------------------------------------
int createExecEnv(pluginManager *pm) 
{
 int top, left, w;
 int j, n;
 int xtop, xleft;
 int pBw, pBh, pLw, pLh;
 int pNw, pNh, pL2w, pL2h;
 int idB, idL, idL2, idN, q;
 #define XYGAP 5

 //       
 //  ()   
 //   
 GetCtrlBoundingRect (Pnl, P(decor), &top, &left, 0, &w);
 GetCtrlBoundingRect (Pnl, P(patternB), 0, 0, &pBh, &pBw);
 GetCtrlBoundingRect (Pnl, P(patternL), 0, 0, &pLh, &pLw);
 GetCtrlBoundingRect (Pnl, P(patternL2), 0, 0, &pL2h, &pL2w);
 GetCtrlBoundingRect (Pnl, P(patternN), 0, 0, &pNh, &pNw);
 GetCtrlAttribute(Pnl, P(patternB), ATTR_CMD_BUTTON_COLOR, &buttonDefColor);
 //        
 //    
 n= pm->fnum() & 0xFF;
 if(allocateUirExecID(n, &execTResult)) return 1;
 q= pLh + pL2h + XYGAP/2;
 //      :
 // , ,      
 for(j= 0; j<n; j++) {
     //   .     pm->f(j)->name
     xtop= top + (j/2)*(q + 2*XYGAP);
     if(!(j%2)) xleft= left;
     else xleft= left + w - pNw - XYGAP - pLw - XYGAP - pBw;
	 idB= DuplicateCtrl (Pnl, P(patternB), Pnl, pm->f(j)->name, xtop, xleft);
	 SetCtrlAttribute(Pnl, idB, ATTR_VISIBLE, 1);
	 //    
	 idL= DuplicateCtrl (Pnl, P(patternL), Pnl, "", xtop + pBh/2 - q/2, xleft + pBw + XYGAP);
	 SetCtrlAttribute(Pnl, idL, ATTR_VISIBLE, 1); 
	 //     
	 idL2= DuplicateCtrl (Pnl, P(patternL2), Pnl, "", xtop + pBh/2 - q/2 + XYGAP/2 + pLh, xleft + pBw + XYGAP);
	 SetCtrlAttribute(Pnl, idL2, ATTR_VISIBLE, 1); 	 
	 //   
	 idN= DuplicateCtrl (Pnl, P(patternN), Pnl, "", xtop, xleft + pBw + XYGAP + pLw + XYGAP);
	 SetCtrlAttribute(Pnl, idN, ATTR_VISIBLE, 1); 	 	 
	 // 
	 if(pm->f(j)->set.manual) SetCtrlAttribute (Pnl, idB, ATTR_LABEL_COLOR, VAL_BLUE);
	 if(pm->f(j)->set.passport) SetCtrlAttribute (Pnl, idB, ATTR_LABEL_BOLD, 1);
	 
	 if(!core_isUseTF(j) /*!(pm->f(j)->set.use)*/) 
		SetCtrlAttribute (Pnl, idB, ATTR_LABEL_STRIKEOUT, 1);
	 //	 controlID  
	 execTResult[j].uir.button= idB;
	 execTResult[j].uir.led= idL;
	 execTResult[j].uir.ledtotal= idL2;
	 execTResult[j].uir.failnum= idN;
     execTResult[j].tnum= j;
 };
 return 0;
};
			
int destroyExecEnv(void) {
 freeUirExecID(&execTResult);
 return 0;
};			

// ----------------------------------------------------------------------------
//        
// ----------------------------------------------------------------------------
int execSetMode_Run(int Pnl) 
{
 execResult *p= execTResult;
 
 SetCtrlAttribute(Pnl, P(run), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(stop), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(close), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(clear), ATTR_DIMMED, 1);
 
 SetPanelAttribute (Pnl, ATTR_CLOSE_ITEM_VISIBLE, 0);
 DisReporterClose();
			
 while(p->tnum!= -1) {
   SetCtrlAttribute (Pnl, p->uir.button, ATTR_CTRL_MODE, VAL_INDICATOR);
   SetCtrlAttribute (Pnl, p->uir.button, ATTR_CMD_BUTTON_COLOR, VAL_TRANSPARENT);
   p++;
 };
 
 return 0;
}
// ----------------------------------------------------------------------------
//        
// ----------------------------------------------------------------------------
int execSetMode_Stop(int Pnl) 
{
 execResult *p= execTResult;
 
 while(p->tnum!= -1) {
   SetCtrlAttribute (Pnl, p->uir.button, ATTR_CTRL_MODE, VAL_HOT);
   SetCtrlAttribute (Pnl, p->uir.button, ATTR_CMD_BUTTON_COLOR, buttonDefColor);
   p++;
 };
 
 EnaReporterClose();
 SetPanelAttribute (Pnl, ATTR_CLOSE_ITEM_VISIBLE, 1);
 SetCtrlAttribute(Pnl, P(run), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(stop), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(close), ATTR_DIMMED, 0);	 
 SetCtrlAttribute(Pnl, P(clear), ATTR_DIMMED, 0);
 return 0;
}

void execUpdateResult(int Pnl, int testNum, int result) {  // here result or bad or fail for test itself
 execResult *p= execTResult;
 
 while(p->tnum!= -1) {
  if(p->tnum == testNum) {
     (p->cntr.total)++;
     if(result) (p->cntr.fail)++;
     else (p->cntr.passed)++;

	 SetCtrlAttribute (Pnl, p->uir.led, ATTR_ON_COLOR, result ? VAL_RED : VAL_GREEN);
	 SetCtrlVal (Pnl, p->uir.led, 1);
	 
	 SetCtrlAttribute (Pnl, p->uir.ledtotal, ATTR_ON_COLOR, p->cntr.fail ? VAL_RED : VAL_GREEN);
	 SetCtrlVal (Pnl, p->uir.ledtotal, 1);	 
	 
	 SetCtrlVal(Pnl, p->uir.failnum, 100.0*p->cntr.fail/p->cntr.total);
     break;
  };
  p++;
 };
 
};

void closeExecutor(void) 
{
  tfuncpnl_close (Pnl, -1, EVENT_COMMIT, NULL, 0, 0);
};
// ----------------------------------------------------------------------------
//         
// ----------------------------------------------------------------------------
void openExecutor(char *moduleDescr, void cbf(void)) 
{
   int n, f;
   execResult *p;   
   
   strcpy(currentModule_Descr, moduleDescr);
   functionPtr_WhenQuit= cbf;
   Pnl= LoadPanel(0, "executoruir.uir", TPNL);
   createExecEnv(moduleCore());   
   
   
   SetCtrlAttribute(Pnl, P(ledkeithley), ATTR_VISIBLE, moduleCore()->plugin()->flag.keithley);
   SetCtrlAttribute(Pnl, P(ledte01), ATTR_VISIBLE, moduleCore()->plugin()->flag.te01);
   SetCtrlAttribute(Pnl, P(ledte01ex), ATTR_VISIBLE, moduleCore()->plugin()->flag.te01ext == NULL ? 0 : 1);
   
   p= execTResult;
   f= 0;
   while(p->tnum!=-1) {
     if(core_isUseTF(p->tnum)) { f= 1; break; };
     p++;
   };
   
   SetCtrlAttribute(Pnl, P(run), ATTR_DIMMED, !f);   
   
   
   DisplayPanel(Pnl);  // ??? !!! tmp
   
   
   SaveInfo4Refs_Flag= 0;
   run_SaveInfo4Refs();   
}
// ----------------------------------------------------------------------------
//  
// ----------------------------------------------------------------------------
int CVICALLBACK tfuncpnl_close (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
		    if(Pnl == -1) break;
		    
		    destroyExecEnv();
			DiscardPanel(Pnl);
			Pnl= -1;
			
			if(functionPtr_WhenQuit!=NULL) functionPtr_WhenQuit();
			break;
	}
	return 0;
}

static int runtest_StopFlag;
// ----------------------------------------------------------------------------
//     
// ----------------------------------------------------------------------------
int CVICALLBACK tfuncpnl_run (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) 
{
int testNum, NTests;
int loop, isloop, r;
TCONTEXT tc;
char msg[FNAMELEN+1];
int id= 0;  // ??? tmp

	switch (event) {
		case EVENT_COMMIT:
		    run_SaveInfo4Refs();
		    
		    runtest_StopFlag= 0;
			execSetMode_Run(Pnl);
			
			NTests= moduleCore()->fnum() & 0xFF;
			
            fillTContext(&tc, testCBFunction, currentModule_Descr, getTE01Handle());
			// **************************************************************************************

			while(1) {
			  GetCtrlVal(Pnl, P(loop), &loop);
			  SetCtrlVal(Pnl, P(loop), loop+1);			
			  GetCtrlVal(Pnl, P(loop), &loop);
			  
              sprintf(msg, "# Run-all session started: date= %s on %s, loop= %d", DateStr(), TimeStr(), loop);
		      testCBFunction(MIDCSRC_CORE | MIDCT_EALL | MIDCER_BEGIN, msg);			  

              SetCtrlVal(Pnl, P(tpercent), "0 %");
   			  SetCtrlVal(Pnl, P(tbar), 0.0);
              SetCtrlVal(Pnl, P(percent), "0 %");
   			  SetCtrlVal(Pnl, P(bar), 0.0);
   			  
			  for(testNum= 0; testNum< NTests; testNum++) {
			      SetCtrlVal(Pnl, P(tname), moduleCore()->f(testNum)->descr);
			      ProcessSystemEvents();
			      if(runtest_StopFlag) {
                     sprintf(msg, "# Run-all session user aborted: date= %s on %s, loop= %d", DateStr(), TimeStr(), loop);
                     id= MIDCSRC_CORE | MIDCT_EALL | MIDCER_ABORT;
			         goto __stop_tests; 
			      };

                  // #define EXEC_SINGLE_TEST__NEXT  0
                  // #define EXEC_SINGLE_TEST__ABORT 1
                  //         EXEC_SINGLE_TEST__STOPONERROR 2
                  if((r= exec_SingleTest(testNum, &tc, 0)) == EXEC_SINGLE_TEST__ABORT) {
                     sprintf(msg, "# Run-all session stopped on a test aborted: date= %s on %s, loop= %d", DateStr(), TimeStr(), loop);
			         id= MIDCSRC_CORE | MIDCT_EALL | MIDCER_STOP_ABORT;
                     goto __stop_tests;
                  };
                  if(r == EXEC_SINGLE_TEST__STOPONERROR) {
                     sprintf(msg, "# Run-all session stopped on error: date= %s on %s, loop= %d", DateStr(), TimeStr(), loop);
			         id= MIDCSRC_CORE | MIDCT_EALL | MIDCER_STOP_ERROR;
			         goto __stop_tests;
                  };

	              sprintf(msg, "%.0lf %%", (testNum + 1) *100.0/ NTests);
	              SetCtrlVal(Pnl, P(tpercent), msg);
	   			  SetCtrlVal(Pnl, P(tbar), (testNum + 1) *100.0/ NTests);
			  }; // for testNum= ... 

              sprintf(msg, "# Run-all session finished: date= %s on %s, loop= %d", DateStr(), TimeStr(), loop);
			  id= MIDCSRC_CORE | MIDCT_EALL | MIDCER_END;
			  
			  GetCtrlVal(Pnl, P(isloop), &isloop);
			  
              //testCBFunction(id, msg);
              //testCBFunction(MIDCSRC_CORE | MIDCT_LOG, "");
			  if(!isloop) goto __stop_tests; // __quit_tests;
			}; // while(1)
			
   __stop_tests:
            testCBFunction(id, msg);
            testCBFunction(MIDCSRC_CORE | MIDCT_LOG, "");
            updateStatistics();
			execSetMode_Stop(Pnl);
			break;
			
   //__quit_tests:
     //      execSetMode_Stop(Pnl);
       //    break;
	}
	return 0;
}

int CVICALLBACK tfuncpnl_stop (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
			runtest_StopFlag= 1;
			break;
	}
	return 0;
}



// *************************************************************************************************************
// 
// *************************************************************************************************************
int  isExecutorActive(void) {
   return Pnl==-1 ? 0 : 1;
};
void functionCall_when_repoter_close (void) {
  tfuncpnl_close (-1, -1, EVENT_COMMIT, NULL, 0, 0);
};
//-----------------------------------------------------------------------------
// callback        
//     
//-----------------------------------------------------------------------------
static int testCBFunction (unsigned int id, char *msg) {
  double xp;
  char tmp[512];
  ProcessSystemEvents();
  //
  
  if (Pnl<0) return runtest_StopFlag;
  if( id == (MIDFD_CORE | MIDFC_ISTART) ) {
      SetCtrlVal(Pnl, P(ledinit), 1);
  }
  if (Pnl<0) return runtest_StopFlag;
  if( id == (MIDFD_CORE | MIDFC_IEND) ) {
      SetCtrlVal(Pnl, P(ledinit), 0);
  }
  if( (id & 0xFFFFFF00UL) == (MIDFD_CORE | MIDFC_PROGRESS) ) {
     xp= id & 0xFF;
     if(xp > 100.0) xp= 100.0;
     sprintf(tmp, "%.0lf%%", xp);
     if (Pnl<0) return runtest_StopFlag;
     SetCtrlVal(Pnl, P(percent), tmp);
     SetCtrlVal(Pnl, P(bar), xp);
  }
  if(msg!=NULL) {
	openReporter(functionCall_when_repoter_close);  //       
  	updateReporter(id, msg);
  }	
  //
  return runtest_StopFlag;
};
//-----------------------------------------------------------------------------
//     
//-----------------------------------------------------------------------------
int CVICALLBACK singletest_func (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) 
{
execResult *p= execTResult;
TCONTEXT tc;
int isloop= 1, r;

	switch (event) {
		case EVENT_COMMIT:
            while(p->tnum!= -1) {
               if(p->uir.button == control) break;  // !!! tmp
               p++;
            };

            run_SaveInfo4Refs();
            
            SetCtrlVal(Pnl, P(percent), "0 %");
            SetCtrlVal(Pnl, P(bar), 0.0);
   			  
		    runtest_StopFlag= 0;
			execSetMode_Run(Pnl);
			
            fillTContext(&tc, testCBFunction, currentModule_Descr, getTE01Handle());
 			//  
            testCBFunction(MIDCSRC_CORE | MIDCT_LOG, "");
            SetCtrlVal(Pnl, P(tname), moduleCore()->f(p->tnum)->descr);
			//
			runtest_StopFlag= 0;      
            while(isloop) {
               SetCtrlVal(Pnl, P(percent), "0 %");
   			   SetCtrlVal(Pnl, P(bar), 0.0);            
   			   
			   ProcessSystemEvents();
			   if(runtest_StopFlag) break;
			   
               r= exec_SingleTest(p->tnum, &tc, 1);
               
               GetCtrlVal(Pnl, P(isloop), &isloop);
               
               if(r) break;
            };
            testCBFunction(MIDCSRC_CORE | MIDCT_LOG, "");
            updateStatistics();
			//
			execSetMode_Stop(Pnl);
			break;
	}
	return 0;
}

int CVICALLBACK tfuncpnl_clear (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
int j;
execResult *p= execTResult;
	switch (event) {
		case EVENT_COMMIT:
		    if(!ConfirmPopup(MSG_WARNING, "Are you sure to clear results?")) break;
		    closeReporterFromExecEnv();
            
            SaveInfo4Refs_Flag= 0;
            
            SetCtrlVal(Pnl, P(tpercent), "0 %");
   			SetCtrlVal(Pnl, P(tbar), 0.0);
            SetCtrlVal(Pnl, P(percent), "0 %");
   			SetCtrlVal(Pnl, P(bar), 0.0);
   			  
			SetCtrlVal(Pnl, P(ledoverall), 0);
			SetCtrlVal(Pnl, P(led), 0);
			SetCtrlVal(Pnl, P(loop), 0);
			
			SetCtrlVal(Pnl, P(ledhw), 0);      
            SetCtrlVal(Pnl, P(ledte01), 0);    
            SetCtrlVal(Pnl, P(ledte01ex), 0);  
            SetCtrlVal(Pnl, P(ledkeithley), 0);
            SetCtrlVal(Pnl, P(ledinitres), 0);
            SetCtrlVal(Pnl, P(ledinit), 0); 

            while(p->tnum!= -1) {
              SetCtrlVal(Pnl, p->uir.led, 0);
              SetCtrlVal(Pnl, p->uir.ledtotal, 0);
              SetCtrlVal(Pnl, p->uir.failnum, 0.0);
              
              p->cntr.passed= 0;
              p->cntr.fail= 0;
              p->cntr.total= 0;
              
              p++;
            };			
			break;
	}
	return 0;
}

int  CVICALLBACK testpanel_cb(int panel, int event, void *callbackData, int eventData1, int eventData2) {
 execResult *p= execTResult;
 int top, left, w, h, right;
 char tmp[128];
 int  tnum= 0, X, Y;
 
 if(panel!= Pnl) return 0;
 if(event!= EVENT_RIGHT_CLICK) return 0;
 
 // eventData1 = mouse vertical position. eventData2 = mouse horizontal position.

 GetRelativeMouseState (Pnl, 0, &X, &Y, NULL, NULL, NULL);
 
 while(p->tnum!= -1) {
    GetCtrlBoundingRect (Pnl, p->uir.button, &top, &left, &h, 0); 
    GetCtrlBoundingRect (Pnl, p->uir.failnum, 0, &right, 0, &w);
    right+= w;
    if(Y >= top && Y <= (top + h) && X >=left && X <= right) {
       
       sprintf(tmp, "FUNCTION <%s> STATISTICS\n\n   - failed:   %d\n   - passed:   %d\n\n   - total:   %d", 
          moduleCore()->f(tnum)->name,
          p->cntr.fail,
          p->cntr.passed, 
          p->cntr.total);
          
       MessagePopup(MSG_PROMPT, tmp);
       
    };
    p++;
    tnum++;
 };
 
 return 0;
};
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
void run_SaveInfo4Refs(void) 
{
 int j, n;
 int id;
 char x[512], qq[512]= "Operator: ";
 int r;
 TCONTEXT tc;
 
 if(SaveInfo4Refs_Flag) return;
 SaveInfo4Refs_Flag= 1;
 
 n= moduleCore()->fnum() & 0xFF;  // !!! tmp   must be less than 255
 for(j= 0; j< n; j++) {
     id= MIDCSRC_CORE | MIDCT_REFS | (j << 8) | (n - 1);
     sprintf(x, "Test%02d: ", j+1);
     strcat(x, moduleCore()->f(j)->name);
     strcat(x, " (");
     strcat(x, moduleCore()->f(j)->descr);
     strcat(x, ") ");
     if(moduleCore()->f(j)->set.manual)   { id|= MIDCF_BIT_MANUAL; strcat(x, " : manual"); };
     if(moduleCore()->f(j)->set.passport) { id|= MIDCF_BIT_PASSPORT; strcat(x, " : passport"); };
     if(/*moduleCore()->f(j)->set.use*/ core_isUseTF(j))      { id|= MIDCF_BIT_USE; strcat(x, " : use"); };
     testCBFunction (id, x);
 };

 if(GetCurrentUser (x, 128, &r)) strcat(qq, "???");
 else strcat(qq, x);
 testCBFunction (MIDCSRC_CORE | MIDCT_OPER, qq);
 
 strcpy(qq, "Module description: "); strcat(qq, currentModule_Descr); testCBFunction (MIDCSRC_CORE | MIDCT_MDESCR, qq);
 strcpy(qq, "Plugin: "); strcat(qq, pluginCurrentDescr()); testCBFunction (MIDCSRC_CORE | MIDCT_PDESCR, qq);
 strcpy(qq, "Module name: "); strcat(qq, core_getModuleName(currentModule_Descr)); testCBFunction (MIDCSRC_CORE | MIDCT_MNAME, qq);
 strcpy(qq, "System: "); strcat(qq, core_getSysDescr()); testCBFunction (MIDCSRC_CORE | MIDCT_SNAME, qq);
 
 testCBFunction (MIDCSRC_CORE | MIDCT_LOG, ""); 
 
  if(moduleCore()->plugin()->module_info != NULL) {
     fillTContext(&tc, testCBFunction, currentModule_Descr, getTE01Handle());
     if(!moduleCore()->plugin()->module_info(&tc, NULL)) {
         
     };
  }; 
};

// ******************************************************
// #define EXEC_SINGLE_TEST__NEXT  0
// #define EXEC_SINGLE_TEST__ABORT 1
// EXEC_SINGLE_TEST__STOPONERROR 2

/*
#define TRES_T_OK             0
#define TRES_T_FAIL           1
#define TRES_T_RSV			  2
#define TRES_HW_FAIL		  3
#define TRES_TE01_FAIL		  4
#define TRES_TE01EX_FAIL	  5
#define TRES_KEITHLEY_FAIL	  6
#define TRES_ABORT   		  7
#define TRES_INIT_FAIL		  8
#define TRES_RSV			  9
  */

static void updateGlobalResult(void);
//-----------------------------------------------------------------------------
//    
//-----------------------------------------------------------------------------
int  exec_SingleTest(int testNum, TCONTEXT *tc, int force) 
{
 int isbreak, r= EXEC_SINGLE_TEST__NEXT;
 int id= 0; // ??? tmp 
 char msg[1024];
 int  tres, isloop;
 int  etest_mark= MIDCT_ERTEST; // run-all test by def.
 int  tfproperties_mark= 0, totalTNum= 0;
 static char *tres_descr[]= {"","",""," (H/W) ", " (TE-01) ", " (TE-01 Ext.) ", " (Keithley) ", ""," (Init) ",""};
 
   
   if(force) {
      etest_mark= MIDCT_ESTEST;  // this is a single test
      GetCtrlVal(Pnl, P(isloop), &isloop);
      if(isloop && moduleCore()->f(testNum)->set.manual) {
        SetCtrlVal(Pnl, P(isloop), 0);
        MessagePopup(MSG_WARNING, "Loop mode has been disabled for manual test.\nYou should return it if need for another test");
      };
   };
   
   if(moduleCore()->f(testNum)->set.manual)       tfproperties_mark|= MIDCF_BIT_MANUAL; 
   if(moduleCore()->f(testNum)->set.passport)     tfproperties_mark|= MIDCF_BIT_PASSPORT; 
   if(/*moduleCore()->f(testNum)->set.use*/core_isUseTF(testNum) || force) tfproperties_mark|= MIDCF_BIT_USE;   // !!! ??? TMP --> ' || force '
   
   totalTNum= moduleCore()->fnum() & 0xFF;  // ??? tmp
   
   GetCtrlVal(Pnl, P(isbreak), &isbreak);
   
   //  -   +    ( )
   sprintf(msg, "*Test%02d: %s - started, date= %s on %s", testNum + 1, moduleCore()->f(testNum)->name, DateStr(), TimeStr());			
   testCBFunction(MIDCSRC_CORE | etest_mark | MIDCET_BEGIN | tfproperties_mark | (testNum << 8) | totalTNum, msg); 

   id= MIDCSRC_CORE | etest_mark | tfproperties_mark | (testNum << 8) | totalTNum;
   
   if(/*!moduleCore()->f(testNum)->set.use*/ !core_isUseTF(testNum) && !force) {
       sprintf(msg, "->Test%02d: %s - skipped (not need), date= %s on %s", testNum + 1, moduleCore()->f(testNum)->name, DateStr(), TimeStr());			          
       id |= MIDCET_NOTNEED;
       goto __next_test;
   };
   
   if((moduleCore()->f(testNum)->set.manual && /*moduleCore()->f(testNum)->set.use*/ core_isUseTF(testNum)) && !force) {
      SetCtrlAttribute (Pnl, execTResult[testNum].uir.led, ATTR_ON_COLOR, VAL_YELLOW);
      SetCtrlVal (Pnl, execTResult[testNum].uir.led, 1);			            
      sprintf(msg, "->Test%02d: %s - skipped (manual, must be run later), date= %s on %s", testNum + 1, moduleCore()->f(testNum)->name, DateStr(), TimeStr());			          
      id|= MIDCET_LATER;
      goto __next_test;
   };
   
   tres= moduleCore()->f(testNum)->run(tc);
   
   if(tres == TRES_ABORT) {
      sprintf(msg, "->Test%02d: %s - aborted, date= %s on %s", testNum + 1, moduleCore()->f(testNum)->name, DateStr(), TimeStr());			          
      id|= (MIDCET_END | MIDCR_ABORT);
      r= EXEC_SINGLE_TEST__ABORT;
      goto __next_test;
   };
   
   execUpdateResult(Pnl, testNum, tres == TRES_T_OK ? TRES_T_OK : TRES_T_FAIL);
   
   // !!! tmp  ???
   if(tres < 0) tres= TRES_RSV;
   if(tres > TRES_RSV) tres= TRES_RSV;
   
   sprintf(msg, "->Test%02d: %s - %s%s, date= %s on %s", testNum + 1, moduleCore()->f(testNum)->name, 
     tres == TRES_T_OK ? "passed" : "FAIL", 
     tres_descr[tres], DateStr(), TimeStr());			           
     
   id|= (MIDCET_END  | (tres << 16L));
   
   testCBFunction(id, msg);
   
   if( (tres == TRES_T_OK) || (tres == TRES_T_FAIL) ) {
   }
   else {
        if(tres == TRES_HW_FAIL)            SetCtrlVal(Pnl, P(ledhw), 1);
        else if(tres == TRES_TE01_FAIL)     SetCtrlVal(Pnl, P(ledte01), 1);
        else if(tres == TRES_TE01EX_FAIL)   SetCtrlVal(Pnl, P(ledte01ex), 1);
        else if(tres == TRES_KEITHLEY_FAIL) SetCtrlVal(Pnl, P(ledkeithley), 1);
        else if(tres == TRES_INIT_FAIL)     SetCtrlVal(Pnl, P(ledinitres), 1);
        else {
        };
   };
   
   /*
   sprintf(msg, " -> (%d/%d) %s", 
                 result ? execTResult[testNum].cntr.fail : (loop - execTResult[testNum].cntr.fail), loop,
                 result ? " -> FAILED" : " -> passed");
   testCBFunction(MIDCSRC_CORE, msg);
     */
     
   updateGlobalResult();     
   if(tres && isbreak) return EXEC_SINGLE_TEST__STOPONERROR;
   else return EXEC_SINGLE_TEST__NEXT;
   
__next_test:
  testCBFunction(id, msg);
  updateGlobalResult();
  return r;
};


// *****************************************************************************************************
static void updateLocalResult(void) {
 execResult *p= execTResult; 
 int f_red= 0, f_yellow= 0;
 int val, state;
 
 while(p->tnum!= -1) {
    GetCtrlVal(Pnl, execTResult[p->tnum].uir.led, &state);
    if(!state) {
       if(/*moduleCore()->f(p->tnum)->set.use*/ core_isUseTF(p->tnum)) f_yellow= 1;
    }
    else {
      GetCtrlAttribute(Pnl, execTResult[p->tnum].uir.led, ATTR_ON_COLOR, &val);      
      if(val == VAL_YELLOW) f_yellow= 1;
      if(val == VAL_RED) f_red= 1;
    };
    
    SetCtrlAttribute(Pnl, P(led), ATTR_ON_COLOR, VAL_GREEN);
    if(f_yellow) SetCtrlAttribute(Pnl, P(led), ATTR_ON_COLOR, VAL_YELLOW);
    if(f_red) SetCtrlAttribute(Pnl, P(led), ATTR_ON_COLOR, VAL_RED);
    SetCtrlVal(Pnl, P(led), 1);
    
    p++;
 };

};

// *****************************************************************************************************
static void updateOverallResult(void) {
 execResult *p= execTResult; 
 int f_red= 0, f_yellow= 0;
 int val, state;
 
 while(p->tnum!= -1) {
    GetCtrlVal(Pnl, execTResult[p->tnum].uir.ledtotal, &state);
    if(!state) {
       if(/*moduleCore()->f(p->tnum)->set.use*/core_isUseTF(p->tnum)) f_yellow= 1;
    }
    else {
      GetCtrlAttribute(Pnl, execTResult[p->tnum].uir.ledtotal, ATTR_ON_COLOR, &val);      
      if(val == VAL_YELLOW) f_yellow= 1;
      if(val == VAL_RED) f_red= 1;
    };
    
    SetCtrlAttribute(Pnl, P(ledoverall), ATTR_ON_COLOR, VAL_GREEN);
    if(f_yellow) SetCtrlAttribute(Pnl, P(ledoverall), ATTR_ON_COLOR, VAL_YELLOW);
    if(f_red) SetCtrlAttribute(Pnl, P(ledoverall), ATTR_ON_COLOR, VAL_RED);
    SetCtrlVal(Pnl, P(ledoverall), 1);
    
    p++;
 };

};

// *****************************************************************************************************
static void updateGlobalResult(void) {
 updateLocalResult();
 updateOverallResult();
};

// *****************************************************************************************************

static void updateStatistics(void) {
 execResult *p= execTResult; 
 char msg[1024];
 int id;
 int xp_hi, xp_lo;
 char mark[2]= "";
 char tfprop[3+1]= "";
 
 testCBFunction(MIDCSRC_CORE | MIDCT_LOG, "$ Statistics: START");
 while(p->tnum!= -1) {
 
   if(moduleCore()->f(p->tnum)->set.manual) tfprop[0]= 'M'; else tfprop[0]= '-'; 
   if(moduleCore()->f(p->tnum)->set.passport) tfprop[1]= 'P'; else tfprop[1]= '-'; 
   if(/*moduleCore()->f(p->tnum)->set.use*/core_isUseTF(p->tnum)) tfprop[2]= 'U'; else tfprop[2]= '-'; 
   tfprop[3]= 0x0;
   
    xp_hi= 0;
    xp_lo= 0;
    if(execTResult[p->tnum].cntr.total) {
       xp_hi= 1000 * execTResult[p->tnum].cntr.fail/execTResult[p->tnum].cntr.total;
       xp_lo= xp_hi % 10;
       xp_hi/= 10;
    }; 
    
    strcpy(mark, " ");
    if(!execTResult[p->tnum].cntr.fail && !execTResult[p->tnum].cntr.total) {
       if(moduleCore()->f(p->tnum)->set.passport || 
          moduleCore()->f(p->tnum)->set.use /*core_isUseTF(p->tnum)*/) {
            strcpy(mark, "*");
       }
    };

    sprintf(msg, "  Test%02d: T= %04d, P= %04d, F= %04d :: %sfail= %03d.%01d%% : %s : (%s)", 
                      p->tnum + 1,
                      execTResult[p->tnum].cntr.total,
                      execTResult[p->tnum].cntr.passed,
                      execTResult[p->tnum].cntr.fail,
                      mark,
                      xp_hi, xp_lo,
                      tfprop,
                      moduleCore()->f(p->tnum)->name);

    id= MIDCSRC_CORE | MIDCT_STATISTIC | (p->tnum << 8) | (moduleCore()->fnum() & 0xFF);
    testCBFunction(id, msg);                   
    p++;
 };
 
 testCBFunction(MIDCSRC_CORE | MIDCT_LOG, "$ Statistics: END");
 testCBFunction(MIDCSRC_CORE | MIDCT_LOG, ""); 
};




// *******************************************************************************
//// *******************************************************************************
//// ******************** BURNING MCU FLASH ******************************************
//// *******************************************************************************
// *******************************************************************************
static int McuFlashPnl;
static int McuFlash_cbf(unsigned int id, char *msg) 
{
  char   tmp[512];
  double xp;
  ProcessSystemEvents();
  //
  if(id == (MIDFD_CORE|MIDFC_ISTART)) {
      SetCtrlVal(McuFlashPnl, MCUFLASH_ledinit, 1);
  }
  if( id == (MIDFD_CORE | MIDFC_IEND) ) {
      SetCtrlVal(McuFlashPnl, MCUFLASH_ledinit, 0);
  }
  if( (id & 0xFFFFFF00UL) == (MIDFD_CORE | MIDFC_PROGRESS) ) {
     xp= id & 0xFF;
     if(xp > 100.0) xp= 100.0;
     sprintf(tmp, "%.0lf%%", xp);
     SetCtrlVal(McuFlashPnl, MCUFLASH_percent, tmp);
     SetCtrlVal(McuFlashPnl, MCUFLASH_bar, xp);
  }
  if(msg!= NULL) {
	if(id==TFMSG_SERIAL) {
     	SetCtrlVal(McuFlashPnl, MCUFLASH_serial, msg);
     	SetCtrlVal(McuFlashPnl, MCUFLASH_text, " : ");
  	}
    if(id==TFMSG_VER) SetCtrlVal(McuFlashPnl, MCUFLASH_text, ": ");
    SetCtrlVal(McuFlashPnl, MCUFLASH_text, msg);
    SetCtrlVal(McuFlashPnl, MCUFLASH_text, "\n");
  }
  return runtest_StopFlag;
};
//--------------------------------------------------------------
//
//--------------------------------------------------------------
int BurnMcuFlash(char *moduleDescr, char *fname) 
{
  TCONTEXT tc;
  strcpy(currentModule_Descr, moduleDescr);
  fillTContext(&tc, McuFlash_cbf, currentModule_Descr, getTE01Handle());		
  McuFlashPnl= LoadPanel(0, "executoruir.uir", MCUFLASH);
  SetCtrlVal(McuFlashPnl, MCUFLASH_fname, fname);
  runtest_StopFlag= 0;
  InstallPopup(McuFlashPnl);
  if(moduleCore()->plugin()->module_info != NULL) {
     if(!moduleCore()->plugin()->module_info(&tc, NULL)) {
         
     };
  };
  return 0;
}
//--------------------------------------------------------------
//
//--------------------------------------------------------------
int CVICALLBACK mcuflash(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
 switch (event) {
	case EVENT_CLOSE:
		DiscardPanel(McuFlashPnl);
	break;
 }
 return 0;
}

/*int CVICALLBACK burnmcuf_close (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) 
{
 switch (event) {
	case EVENT_COMMIT:
		
	break;
 }
 return 0;
}*/
//--------------------------------------------------------------
//
//--------------------------------------------------------------
static void EnaDis_McuFlashPnl(int ena) 
{
 SetCtrlAttribute(McuFlashPnl, MCUFLASH_write_start, ATTR_DIMMED, !ena);
// SetCtrlAttribute(McuFlashPnl, MCUFLASH_close, ATTR_DIMMED, !ena);
 SetCtrlAttribute(McuFlashPnl, MCUFLASH_write_stop , ATTR_DIMMED, ena);
 ProcessSystemEvents();
};
//--------------------------------------------------------------
//   mcu 
//--------------------------------------------------------------
int CVICALLBACK mcuflash_write_stop (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) 
{
 switch (event) {
	case EVENT_COMMIT:
		runtest_StopFlag=1;
	break;
 }
 return 0;
}
//--------------------------------------------------------------
//   mcu 
//--------------------------------------------------------------
int CVICALLBACK mcuflash_write_start(int panel, int control, int event, void *callbackData, int eventData1, int eventData2) 
{
 burnMcuContext bc;
 TCONTEXT tc;  
 char userSerial[COMMENTLEN + 1];
 int r;
	switch (event) 
	{
		case EVENT_COMMIT:
  
            fillTContext(&tc, McuFlash_cbf, currentModule_Descr, getTE01Handle());				
            ResetTextBox(McuFlashPnl, MCUFLASH_text, "");
            SetCtrlVal(McuFlashPnl, MCUFLASH_percent, "0 %");
            SetCtrlVal(McuFlashPnl, MCUFLASH_bar, 0.0);
		    GetCtrlVal(McuFlashPnl, MCUFLASH_fname, bc.fname);
		    GetCtrlVal(McuFlashPnl, MCUFLASH_serial, bc.serial);
		    
		    strcpy(userSerial, bc.serial);
		    
		    runtest_StopFlag= 0;
		    EnaDis_McuFlashPnl(0);
			r= moduleCore()->plugin()->flash_write(&tc, (void *)&bc);
			EnaDis_McuFlashPnl(1);
			
			GetCtrlVal(McuFlashPnl, MCUFLASH_serial, bc.serial);
			if(!r) {
			    if(!stricmp(bc.serial, userSerial)) 
			      MessagePopup(MSG_PROMPT, "Process finished succesfully!");
			    else 
			      MessagePopup(MSG_WARNING, "Check out the serial number!");
			}
			else 
			   MessagePopup(MSG_ERROR, "Process failed/aborted, try again!");
			
			break;
	}
	return 0;
}



int CVICALLBACK tfuncpnl_clearspec (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
			SetCtrlVal(Pnl, P(ledhw), 0);      
            SetCtrlVal(Pnl, P(ledte01), 0);    
            SetCtrlVal(Pnl, P(ledte01ex), 0);  
            SetCtrlVal(Pnl, P(ledkeithley), 0);
            SetCtrlVal(Pnl, P(ledinitres), 0);
            SetCtrlVal(Pnl, P(ledinit), 0); 
			break;
	}
	return 0;
}










// *******************************************************************************
//// *******************************************************************************
//// ******************** BURNING MCU FUSES ******************************************
//// *******************************************************************************
// *******************************************************************************
static int McuFusesPnl;
static int fuses[20]=
{
 MCUFUSES_FUSE00, MCUFUSES_FUSE01, MCUFUSES_FUSE02, MCUFUSES_FUSE03,
 MCUFUSES_FUSE04, MCUFUSES_FUSE05, MCUFUSES_FUSE06, MCUFUSES_FUSE07,
 MCUFUSES_FUSE10, MCUFUSES_FUSE11, MCUFUSES_FUSE12, MCUFUSES_FUSE13,
 MCUFUSES_FUSE14, MCUFUSES_FUSE15, MCUFUSES_FUSE16, MCUFUSES_FUSE17,
 MCUFUSES_FIRST_TIME, 
 MCUFUSES_read, MCUFUSES_write, MCUFUSES_default
};
//--------------------------------------------------------------
//
//--------------------------------------------------------------
static void EnaDis_McuFusesPnl(int ena) 
{
 int i;
 for(i=0; i<20; i++) {
	SetCtrlAttribute(McuFusesPnl, fuses[i], ATTR_DIMMED, !ena);
 }
 ProcessSystemEvents();
}
//--------------------------------------------------------------
//     
//--------------------------------------------------------------
int CVICALLBACK mcufuses_read(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 int i;
 TCONTEXT tc;  
 burnMcuContext bc;
 //
 switch (event) {
	case EVENT_COMMIT:
		fillTContext(&tc, NULL/*McuFlash_cbf*/, currentModule_Descr, getTE01Handle());				
		EnaDis_McuFusesPnl(0);
		if(!moduleCore()->plugin()->fuses_read(&tc, (void *)&bc)) {
			for(i=0; i<16; i++) {	
				SetCtrlVal(McuFusesPnl, fuses[i], !((bc.fuses>>i)&1));
			}
			//
			MessagePopup(MSG_PROMPT, "Process finished succesfully!");			
		} else MessagePopup(MSG_ERROR, "Process failed/aborted, try again!");
		EnaDis_McuFusesPnl(1);
	break;
 }
 return 0;
}
//--------------------------------------------------------------
//     
//--------------------------------------------------------------
int CVICALLBACK mcufuses_write(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 int i;
 TCONTEXT tc;  
 burnMcuContext bc;
 //
 switch(event) {
	case EVENT_COMMIT:
		fillTContext(&tc, NULL/*McuFlash_cbf*/, currentModule_Descr, getTE01Handle());				
		//
		bc.fuses=0xFFFFFFFFl;
		for(i=0; i<16; i++) {	
			int k;
			GetCtrlVal(McuFusesPnl, fuses[i], &k);
			if(k) bc.fuses&=~(1<<i);
		}
		//
		GetCtrlVal(McuFusesPnl, MCUFUSES_FIRST_TIME, &i);
		bc.flags=i ? (1<<0) : (0<<0);
		//
		EnaDis_McuFusesPnl(0);
		if(!moduleCore()->plugin()->fuses_write(&tc, (void *)&bc)) {
			MessagePopup(MSG_PROMPT, "Process finished succesfully!");			
		} else MessagePopup(MSG_ERROR, "Process failed/aborted, try again!");
		EnaDis_McuFusesPnl(1);
	break;
 }
 return 0;
}
//--------------------------------------------------------------
//     
//--------------------------------------------------------------
int CVICALLBACK mcufuses_default (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
 int i;
 WORD fuses_default=0xFFFFFFF0l;
 switch (event) {
	case EVENT_COMMIT:
		for(i=0; i<16; i++) {	
			SetCtrlVal(McuFusesPnl, fuses[i], !((fuses_default>>i)&1));
		}
	break;
 }
 return 0;
}
//--------------------------------------------------------------
//
//--------------------------------------------------------------
int CVICALLBACK mcufuses(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
 switch (event) {
	case EVENT_GOT_FOCUS:
	break;
	case EVENT_LOST_FOCUS:
	break;
	case EVENT_CLOSE:
		DiscardPanel(McuFusesPnl);
	break;
 }
 return 0;
}
//--------------------------------------------------------------
//     -
//--------------------------------------------------------------
int BurnMcuFuses(char *moduleDescr) 
{
 TCONTEXT tc;
 strcpy(currentModule_Descr, moduleDescr);
 fillTContext(&tc, NULL/*McuF_cbf*/, currentModule_Descr, getTE01Handle());		
 McuFusesPnl=LoadPanel(0, "executoruir.uir", MCUFUSES);
 runtest_StopFlag= 0;
 InstallPopup(McuFusesPnl);
 if(moduleCore()->plugin()->module_info != NULL) {
    if(!moduleCore()->plugin()->module_info(&tc, NULL)) {

	}
 }
 return 0;
}
