#include <math.h>
#include <ansi_c.h>
#include "te01api.h"

typedef unsigned short WORD;
typedef signed short SHORT;

void SetSDAC_Value(TE01 *te01, double t_val, char range) {
	double value;

	te01->sdac_range=range;
	value=te01->Ks1[range]*t_val+te01->Bs1[range];
	TE01SetSDACConfiguration(te01, range, value, 1);
	TE01SetConfiguration(te01, FORCE_UPDATE_SDAC|FORCE_UPDATE_UMODULE); // ToDo: FORCE_UPDATE_UMODULE      API    ADC
	if(range==0) Sleep(1000); //      0 
	else Sleep(1000); // ToDo: ???
}


/**
 * \brief     TE-01.
 *
 *
 * \param range  
 *  - 0 10 
 *  - 1 2.5  ( !)
 *  - 2 500 
 *  - 3 100 
 */
void SetADC_Range(TE01 *te01, int range) {
	te01->uadc_range=range;
	TE01SetConfiguration(te01, FORCE_UPDATE_UMODULE);
	Sleep(3000); // ToDo: ????
}


double GetUADC_Val(TE01 *te01) {
	double value;

	TE01UMeasurement(te01, &value, 8, 1000.0, NULL);
	value=TE01UModuleValue(te01, value, 1);
	//      .
	value=value*te01->Ku[te01->uadc_range]+te01->Bu[te01->uadc_range];
	return value;//(value*UADCdividerCC);
}


/***************** FDAC ******************************************/
#define MAX_FDAC_CODE 0xFFFF
#define MIN_FDAC_CODE 0x0000
#define FDAC_RAM_SIZE 20000
#define M_PI		3.14159265358979323846
#define M_TWOPI         (M_PI * 2.0)

/*    . */
double FDAC_BASE_FREQ_VALUES[] = {20E6, 2E6, 200E3, 20E3, 2E3};
#define FDAC_BASE_FREQ_NUMBER (sizeof(FDAC_BASE_FREQ_VALUES)/sizeof(FDAC_BASE_FREQ_VALUES[0]))

#if 0
int calc_best_base_freq(double frequency, int size, int *base_freq_index) {
	/*        fbase*size.
	    ,    1/frequency.
	 ,     : fbase<=FDAC_RAM_SIZE*frequency*/
	int criteria = size*frequency, index;
	double fbase;
	
	for (index=0;index<FDAC_BASE_FREQ_NUMBER;++index) {
		fbase = FDAC_BASE_FREQ_VALUES[index];
		if (fbase <= criteria) {
			*base_freq_index = index;
			if (2*fbase<frequency) /*    . */
				return 1;
			else {
				*base_freq_index = index;
				return 0;
			}
		}
	}
	return 1;
}

int calc_real_frequency(double *frequency, int *base_freq_index, int *step, int ram_size) {
	double Fmin;

	if (calc_best_base_freq(*frequency, ram_size, base_freq_index))
		return 1;
	Fmin = FDAC_BASE_FREQ_VALUES[*base_freq_index]/ram_size;
	if (*frequency>FDAC_BASE_FREQ_VALUES[*base_freq_index])
		*frequency = FDAC_BASE_FREQ_VALUES[*base_freq_index];
	else
		*frequency = (int)(*frequency/Fmin + 0.5)*Fmin;

	*step = (int)(*frequency/Fmin);

	return 0;
}

/**
 *           TE-01.
 *
 * \param image    
 * \param size  
 */
void FDAC_sin_image_make(WORD *image, int size) {
	int i;
	for (i=0;i<size;++i) {
		image[i] = (WORD)((MAX_FDAC_CODE-MIN_FDAC_CODE)*sin(M_TWOPI*i/size)/2+(MAX_FDAC_CODE+MIN_FDAC_CODE)/2);
	}
}


/**
 *         TE-01.
 *
 * \param te01     TE-01
 * \param size   
 */
int FDAC_set_image(TE01 *te01, int size) {
	int ram_start=0, rom_start=0;
	int ret_code;
	WORD *image;

	image = (WORD*)malloc(size*sizeof(WORD));
	if (image==NULL) return 1;
	FDAC_sin_image_make(image, size);
	ret_code = TE01SystemAreaWriteEnable(te01);
	if (ret_code==0)
		ret_code = TE01WriteMemoryRandom(te01, ram_start, size, image, NULL);
	free(image);
	if (ret_code) return ret_code;
	return ret_code;
}

int FDAC_ramsize_from_frequency(double freq, int *base_freq_index, int *ram_size) {
	int index;
	for (index=0;index<FDAC_BASE_FREQ_NUMBER;++index) {
		*ram_size = FDAC_BASE_FREQ_VALUES[index]/freq;
		if ((*ram_size > 0) && (*ram_size <= FDAC_RAM_SIZE)) {
			*base_freq_index = index;
			return 0;
		}
	}

	return 1;
}

void FDAC_frequency_from_ramsize(int base_freq_index, int ram_size, double *freq) {
	*freq = FDAC_BASE_FREQ_VALUES[base_freq_index]/ram_size;
}
#endif


int calc_sin_params(double *frequency, int *base_freq_index, int *image_size, int *periods) {
	int criteria = FDAC_RAM_SIZE * *frequency, index;
	double fbase, size;
	
	for (index=0;index<FDAC_BASE_FREQ_NUMBER;++index) {
		fbase = FDAC_BASE_FREQ_VALUES[index];
		if (fbase <= criteria) {
			*base_freq_index = index;
			if (2*fbase < *frequency) /*    . */
				return 1;
			else {
				*base_freq_index = index;     /*   . */
				size = fbase / *frequency;       /*     . */
				*periods = FDAC_RAM_SIZE/size;/*    . */
				*image_size = size * *periods;/*  . */
				*frequency = fbase/(*image_size / *periods); /*   . */
				return 0;
			}
		}
	}
	return 1;
}

/**
 *           TE-01.
 *
 * \param image    
 * \param size  
 */
void FDAC_sin_image_make(WORD *image, int size, int periods) {
	int i;
	double period = size/periods;
	for (i=0;i<size;++i) {
		image[i] = (WORD)((MAX_FDAC_CODE-MIN_FDAC_CODE)*sin(M_TWOPI*i/period)/2+(MAX_FDAC_CODE+MIN_FDAC_CODE)/2);
	}
}


/**
 *         TE-01.
 *
 * \param te01     TE-01
 * \param size   
 */
int FDAC_set_image(TE01 *te01, int size, int periods) 
{
	int ram_start=0, rom_start=0;
	int ret_code;
	WORD *image;

	image = (WORD*)malloc(size*sizeof(WORD));
	if (image==NULL) return 1;
	FDAC_sin_image_make(image, size, periods);
	ret_code = TE01SystemAreaWriteEnable(te01);
	if (ret_code==0)
		ret_code = TE01WriteMemoryRandom(te01, ram_start, size, image, NULL);
	free(image);
	if (ret_code) return ret_code;
	return ret_code;
}

int FDAC_set_sin_signal(TE01 *te01, int image_size, int base_freq_index, int step, double voltage, int band) {
	int filter_code=0;
	double K = te01->Kf1[band], B = te01->Bf1[band];

	return TE01SetFDACConfiguration(te01, image_size, step, base_freq_index, band, filter_code,
																	-voltage*K+B, voltage*K+B, 1);
}

int FDAC_SetSin(TE01 *te01, double *freq, double voltage, int band) {
	int image_size, base_freq_index, periods;
	
	if (calc_sin_params(freq, &base_freq_index, &image_size, &periods)) {
		printf("calc_sin_params error\n");
		return 1;
	}
	
	if (FDAC_set_image(te01, image_size, periods)) {
		printf("FDAC_set_image error\n");
		return 2;
	}
#if 0	
	if (FDAC_ramsize_from_frequency(*freq, &base_freq_index, &image_size)) {
		printf("FDAC_ramsize_from_frequency error\n");
		return 1;
	}
	if (FDAC_set_image(te01, image_size)) {
		printf("FDAC_set_image error\n");
		return 2;
	}
	if (calc_real_frequency(freq, &base_freq_index, &step, image_size)) {
		printf("calc_real_frequency error\n");
		return 3;
	}
#endif
	if (FDAC_set_sin_signal(te01, image_size, base_freq_index, 1, -voltage, band)) {
		printf("FDAC_set_sin_signal error\n");
		return 4;
	}

	TE01SetConfiguration(te01, FORCE_UPDATE_RAM|FORCE_UPDATE_FDAC);

	return 0;
}
