#include <utility.h>
#include <userint.h>
#include <ansi_c.h>
#include "common\\include\\commondef.h"
#include "reporter.h"
#include "reporteruir.h"

static int Pnl= -1;
#define P(x) PANEL_##x

static utsReporter *TheReporter= NULL;
static int isTFHeadInfo(unsigned int id);

int EnaReporterClose(void) {
 SetPanelAttribute (Pnl, ATTR_CLOSE_ITEM_VISIBLE, 1);
 
 SetCtrlAttribute(Pnl, P(log), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(summary), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(save), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(report), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(passport), ATTR_DIMMED, 0);
 SetCtrlAttribute(Pnl, P(close), ATTR_DIMMED, 0);
 return 0;
};

int DisReporterClose(void) {
 SetPanelAttribute (Pnl, ATTR_CLOSE_ITEM_VISIBLE, 0);
 
 SetCtrlAttribute(Pnl, P(log), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(summary), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(save), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(report), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(passport), ATTR_DIMMED, 1);
 SetCtrlAttribute(Pnl, P(close), ATTR_DIMMED, 1); 
 return 0;
};


static utsReporter *newReporterEntry(int id, char *msg) {
 utsReporter *newp;
 
 if((newp= (utsReporter *)malloc(sizeof(utsReporter))) == NULL) return NULL;
 if( (newp->msg= (char *)malloc(strlen(msg)+1)) == NULL) {
      free(newp);
      return NULL;
 };
 
 strcpy(newp->msg, msg);
 newp->id= id;
 newp->next= NULL;
 return newp;
};

// -------------------------------------------------------------------------
utsReporter* addReporter(utsReporter *list, int id, char *msg) {
 utsReporter *p, *newp;
 
 if( (newp= newReporterEntry(id, msg)) == NULL) return NULL;
 if(list == NULL) {
    newp->prev= NULL;
    return newp;
 };
 
 for(p= list; p->next!= NULL; p= p->next);
 p->next= newp;
 p->next->prev= p;
 return list;
};

// -------------------------------------------------------------------------
void freeReporter(utsReporter **list) {
  utsReporter *next;
  
  for(; (*list)!= NULL; (*list)= next) {
    next= (*list)->next;
    free((*list)->msg);
    free(*list);
  };
  *list= NULL;
};

// -------------------------------------------------------------------------

int  CVICALLBACK msgpnl_cbf(int panel, int event, void *callbackData, int eventData1, int eventData2);

typedef void (*fcb_CloseReporter)(void);

static fcb_CloseReporter fcb_ReporterClose;

int isReporterActive(void) {
  return Pnl == -1 ? 0 : 1;
};

int openReporter(void cbf(void)) {
 if(Pnl!=-1) return 0;
 fcb_ReporterClose= cbf;
 freeReporter(&TheReporter);
 Pnl= LoadPanel(0, "reporteruir.uir", PANEL);
 DisplayPanel(Pnl); 
 return 0;
};

extern int isExecutorActive(void);

int closeReporter(void) {
 if(isExecutorActive() && Pnl!= -1) {
    if(!ConfirmPopup(MSG_WARNING, "Closing the reporter now will destroy the Executor environment also.\nContinue?")) return 0;
 };
 
 if(Pnl!=-1) {
    DiscardPanel(Pnl);
    if(fcb_ReporterClose!= NULL) fcb_ReporterClose();
 };
 freeReporter(&TheReporter); 
 Pnl= -1; 
 return 0;
};

int closeReporterFromExecEnv(void) {
 if(Pnl!=-1) DiscardPanel(Pnl);
 freeReporter(&TheReporter); 
 Pnl= -1; 
 return 0;
};

int updateReporter(unsigned int id, char *msg) {
 int res= 0;
 // InsertTextBoxLine (Pnl, P(text), -1, msg);
 //  SetCtrlVal (Pnl, P(text), msg);
  int j, n= strlen(msg);
  int lnum;
  #define MAX_TEXT_LINES 1024 // ??? !!! TMP
  
  for(j= 0; j< n; j++) {
      if(msg[j] == 0xA || msg[j] == 0xD) msg[j]= 0xFF;  // !!! ??? tmp
      // if(msg[j] < 0x20) msg[j]= 0x20;
  };
 
  if((TheReporter= addReporter(TheReporter, id, msg)) == NULL) res= 1;   // !!! tmp ---> NULL , 
    //   free --->   
    //     ---    !!!!!!!!!!!!!
    

 if( !(id & MIDCSRC_CORE) ) {  //    
 
 /*
    if( (id & 0xF0000L) == MIDFD_SPEC) {
	   GetNumTextBoxLines (Pnl, P(spec), &lnum);
	   if(lnum > MAX_TEXT_LINES) ResetTextBox(Pnl, P(spec), "");
    
       
       SetCtrlVal (Pnl, P(spec), msg);    
       SetCtrlVal (Pnl, P(spec), "\n");  
	   return res;
    };
 */
 
    if(isTFHeadInfo(id)) {
	   GetNumTextBoxLines (Pnl, P(spec), &lnum);
	   if(lnum > MAX_TEXT_LINES) ResetTextBox(Pnl, P(spec), "");
	   if(id == TFMSG_SERIAL) SetCtrlVal (Pnl, P(spec), " : ");
	   if(id == TFMSG_VER) SetCtrlVal (Pnl, P(spec), ": ");
       SetCtrlVal (Pnl, P(spec), msg);    
       SetCtrlVal (Pnl, P(spec), "\n");  
	   return res;       
    };
    
    GetNumTextBoxLines (Pnl, P(text), &lnum);
    if(lnum > MAX_TEXT_LINES) ResetTextBox(Pnl, P(text), "");    
    
    SetCtrlVal (Pnl, P(text), "   ");    
    SetCtrlVal (Pnl, P(text), msg);    
    SetCtrlVal (Pnl, P(text), "\n");    
    return res;
 };
   
   
  // if(id & MIDCT_REFS) return res;
  
  if(!isTFHeadInfo(id)) {
     SetCtrlVal (Pnl, P(text), msg);        
     SetCtrlVal (Pnl, P(text), "\n");
  }
  else {
     SetCtrlVal (Pnl, P(spec), msg);    
     SetCtrlVal (Pnl, P(spec), "\n");    
  };
  return res;
};

utsReporter *GetReporter(void) {
  return TheReporter;
};

int  CVICALLBACK msgpnl_cbf(int panel, int event, void *callbackData, int eventData1, int eventData2) {
  int wp, hp, wc, hc;
  int ws, hd;
  
  if(panel!= Pnl) return 0;
  if(event == EVENT_PANEL_SIZE) {
  

	 GetPanelAttribute (Pnl, ATTR_HEIGHT, &hp);
	 GetPanelAttribute (Pnl, ATTR_WIDTH, &wp);
     GetCtrlAttribute (Pnl, P(decor), ATTR_HEIGHT, &hd);
     
	 SetCtrlAttribute (Pnl, P(text), ATTR_HEIGHT, hp - hd);
	 SetCtrlAttribute (Pnl, P(text), ATTR_WIDTH, wp);     
	 
	 GetCtrlAttribute (Pnl, P(text), ATTR_HEIGHT, &hc);
	 GetCtrlAttribute (Pnl, P(text), ATTR_WIDTH, &wc);
	 
	 SetCtrlAttribute (Pnl, P(spec), ATTR_WIDTH, wc);     	 
	 
	 SetPanelAttribute (Pnl, ATTR_HEIGHT, hc + hd);
	 SetPanelAttribute (Pnl, ATTR_WIDTH, wc);	 
	 
	 
     return 0;
  };
  
  if(event == EVENT_CLOSE) {
    closeReporter();
    return 0;
  };
  
  return 0;
};

// **************************************************************************************
//
// **************************************************************************************
int CVICALLBACK rf_close (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
			closeReporter();
			break;
	}
	return 0;
}

#define LOGFILENAME "test.log"

static int isTFHeadInfo(unsigned int id) {
  int type;
  
  if((id & 0xF0000L) == MIDFD_SPEC) return 1;
  
  if( !(id & MIDCSRC_CORE) ) return 0;
  
  type= (id >> 25L) & 0xF;
  type= type << 25L;
  
  if(type == MIDCT_OPER ||
     type == MIDCT_MDESCR ||
     type == MIDCT_PDESCR ||
     type == MIDCT_REFS ||
     type == MIDCT_MNAME ||
     type == MIDCT_SNAME) 
       return 1;
  
  return 0;
}

int CVICALLBACK rf_log (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
FILE *f;
char buf[4096];
utsReporter *p;
	switch (event) {
		case EVENT_COMMIT:
			if((f= fopen(LOGFILENAME, "w")) == NULL) {
			    sprintf(buf, "Failed create file \"%s\" to update with log information", LOGFILENAME);
			    MessagePopup(MSG_WARNING, buf);
			    break;
			};
			
			fprintf(f, "@ Head: START\n\n");
			for(p= TheReporter; p!= NULL; p= p->next) {
			      if(!isTFHeadInfo(p->id)) continue;
                  if( !(p->id & MIDCSRC_CORE) ) {  //    
                     fprintf(f, "TF-> %s\n", p->msg);
                     continue;
                   };
                  fprintf(f, "%s\n", p->msg);			    
			};
			fprintf(f, "@ Head: END\n\n");
			
			fprintf(f, "@ Reporter: START\n\n");
			// ------------------------------------------------------------
			for(p= TheReporter; p!= NULL; p= p->next) {
			      if(isTFHeadInfo(p->id)) continue;
                  if( !(p->id & MIDCSRC_CORE) ) {  //    
                     fprintf(f, "   %s\n", p->msg);
                     continue;
                   };
                  fprintf(f, "%s\n", p->msg);
			};										\
			fprintf(f, "\n@ Reporter: END\n\n");
			fclose(f);
			sprintf(buf, "start %s", LOGFILENAME);
			LaunchExecutableEx (buf, LE_HIDE, NULL);
			break;
	}
	return 0;
}

int CVICALLBACK rf_save (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
FILE *f;
char tmp[FNAMELEN+1], x[2*FNAMELEN+1];
utsReporter *p= TheReporter;
int n= 0;
	switch (event) {
		case EVENT_COMMIT:
			 if(!FileSelectPopup ("", "*.rpt", "*.txt;*.log",
							  "Save reporter information", VAL_SAVE_BUTTON, 0, 0,
							  1, 1, tmp)) break;
             if((f= fopen(tmp, "w")) == NULL) {
                 MessagePopup(MSG_WARNING, "Failed open the specified file");
                 break;
             };
             fprintf(f, "L-CARD UTS: Reporter Information: START\n");
			 for(; p!= NULL; p= p->next) {   // --
                 fprintf(f, "%08X : %d %02d %s <%s %s %s> %02d %03d %03d : %s\n", 
                             p->id, 
                             (p->id>>29) & 0x7,
                             (p->id >> 25) & 0xF,
                             ((p->id >> 24) & 1) ? "C" : "F",
                             ((p->id >> 20) & 1) ? "m" : " ",
                             ((p->id >> 21) & 1) ? "p" : " ",
                             ((p->id >> 22) & 1) ? "u" : " ",
                             (p->id >> 16) & 0xF,
                             (p->id >> 8) & 0xFF,
                             p->id & 0xFF,
                             p->msg);
                 n++;
			 };							             
			 fprintf(f, "L-CARD UTS: Reporter Information: END (lines: %d)", n);
             fclose(f);
             
             // MessagePopup(MSG_PROMPT, "Reporter output has been saved");
             
             sprintf(x, "start notepad.exe %s", tmp);
             LaunchExecutableEx (x, LE_HIDE, NULL);
			break;
	}
	return 0;
}

int CVICALLBACK rf_summary (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:

			break;
	}
	return 0;
}

int CVICALLBACK rf_report (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:

			break;
	}
	return 0;
}

int CVICALLBACK rf_passport (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:

			break;
	}
	return 0;
}


