#include <analysis.h>
#include <ansi_c.h>


/*===============================================================================================*/
#define THESPAN (15)
/*===============================================================================================*/

/*===============================================================================================*/
int        utsMath_RemoveDC      (double *X, int n, double *Y);
static int utsMath_FindSINAD     (double *s, int n, double w, double g, double *snr, int ipeak);
int        utsMath_FindSyncIndex (double *X, int n, double tau, int dir, double *t, double u,
                                  int isDC);
/*===============================================================================================*/
  
/*===============================================================================================*/
/*-----------------------------------------------------------------------------------------------*/

#define max(a,b) (a>b?a:b)
#define min(a,b) (a<b?a:b)
int _CalcSNR(double *Values, int ValuesDimension1, int ValuesChannel,
							 double * SNR, double * THD, double * SINAD, double * SFDR, double *ENOB,
							 double PeakFreq, double SampleFreq, WindowConst wc)
{
	int res=0;
	int i;
	
	int ipeak = (int)(PeakFreq*ValuesDimension1/(SampleFreq/2));
	
	double PeakPower=0;
	double NoisePower=0;
	double HarmPower=0;
	double AllPower =0;	
	double MaxPower=-100000000;
	double MaxSygnal=-100000000;
	
	Values = (double *)(Values+(ValuesDimension1*ValuesChannel));    
	
	if (SNR!=NULL) *SNR=0;
	if (THD!=NULL) *THD=0;
	if (SINAD!=NULL) *SINAD=0;
	if (SFDR!=NULL) *SFDR =0;
	if (ENOB!=NULL) *ENOB=0;
    
	if (ipeak<(THESPAN / 2)) return -1;
	
	for(i=(THESPAN / 2);i<ValuesDimension1;i++)
	{
		int NearestPeak = (i+(THESPAN / 2))/ipeak;
		int NearestMaxPeak = (NearestPeak*ipeak)+(THESPAN / 2);
		int NearestMinPeak = (NearestPeak*ipeak)-(THESPAN / 2);
		
		double RMS = Values[i];
		
		if(i>=NearestMinPeak&&i<=NearestMaxPeak&&NearestPeak>0&&NearestPeak<7) //    6 
		{   
			if(NearestPeak>1)
			{   
				MaxSygnal = max(MaxSygnal, RMS);
				HarmPower+=(RMS*RMS);
			}
			else
			{
				MaxPower = max(MaxPower, RMS);
				PeakPower+=RMS;	  
			}
		}
		else
		{
			MaxSygnal = max(MaxSygnal, RMS);
			NoisePower+=(RMS*RMS);
		}
	}							
	
	NoisePower = NoisePower/wc.enbw;
	AllPower = NoisePower+HarmPower;
	
	NoisePower = sqrt(NoisePower);
	HarmPower =sqrt(HarmPower);
	AllPower = sqrt(AllPower);

	if(PeakPower!=0)
	{   
		double DynamicRange = MaxPower/MaxSygnal;
		if (DynamicRange>0&&SFDR!=NULL) *SFDR = 20 * log10(DynamicRange);
			
    	if (NoisePower)
		{
			double SNRPower =  PeakPower / NoisePower;
			if(SNRPower>0&&SNR!=NULL)
			{
				*SNR = 20 * log10(SNRPower);
				if (*SNR < 0)
			    {
			        *SNR = 0;
					res=-1;
			    }
			}
		}
		
		if(HarmPower)
		{   double THDPower = HarmPower/PeakPower;     
			if(THDPower>0&&THD!=NULL)
			{
				*THD = 20 * log10(THDPower);
				if (*THD >0)
			    {
			        *THD = 0;
			        res=-1;
			    }
			}
		}
		
		if (AllPower)
		{
			double SINADPower = PeakPower/AllPower;
			if (SINADPower>0&&SINAD!=NULL) 
			{
				*SINAD = 20*log10(SINADPower);
				if (*SINAD<0)
				{
					*SINAD=0;
					res=-1;
				}
				else if (ENOB!=NULL) *ENOB = (*SINAD - 1.76) / 6.02;         
			}
		}
	}

    return res;
}		
		


/*-----------------------------------------------------------------------------------------------*/
int utsMath_DoFFT(double *In, int ValuesDimension1, int isDC,
								 int win, double fs, int *nfft, double *y,  
								 double *fsignal, double *pwr,
								 double * SNR, double * THD, double * SINAD, double * SFDR, double *ENOB) 
{   
	double      df;
	int         r = 0;
	double      *spectr;
	char        unit[20] = "V\0";
	char        unit1[20] = "V\0";
	int n=ValuesDimension1;
	double *Values;
	WindowConst wc;  
	
	*pwr = 0.0;

	if (fs == 0.0)
		return 1;

	n = log(n + 0.5) / log(2.0);
	n = pow(2, n);
	if (n < 2)
		return 1;
	if (*nfft<(n/2)) return 1;
	
	*nfft=n/2;
	
	Values = malloc(ValuesDimension1*sizeof(double));
	memcpy(Values, In, ValuesDimension1*sizeof(double));
	
	if (!isDC) utsMath_RemoveDC(Values, ValuesDimension1, y);     

	ScaledWindow(Values, n, win, &wc);  
	AutoPowerSpectrum(Values, n, 1.0 / fs, y, &df);

	PowerFrequencyEstimate(y, n/2, -1, wc, df, THESPAN, fsignal, pwr);
	*pwr = sqrt((*pwr) * 2.0);

	SpectrumUnitConversion(y, n / 2, 0, 0, 0, df, wc, Values, unit); //   V2 RMS
	
	_CalcSNR(Values, n/2, 0, SNR, THD, SINAD, SFDR, ENOB, *fsignal, fs, wc);
	
	SpectrumUnitConversion(y, n / 2, 0, 1, 0, df, wc, Values, unit1); //   Db
	
	memcpy(y, Values, (n/2)*sizeof(double));
	free(Values);
	
	return r;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utsMath_RemoveDC(double *X, int n, double *Y)
{
    int    imax;
    int    imin;
    int    j;
    double max;
    double min;
    double z;

    MaxMin1D(X, n, &max, &imax, &min, &imin);
    z = (max + min) / 2.0; 
    for (j = 0; j < n; j++)
        Y[j] = X[j] - z;  
        
    return 0;
};
/*-----------------------------------------------------------------------------------------------*/

// isDC= 1  -   DC-
// isDC= 0  -    AC - 
double utsMath_GetDCLine(double *X, int n, int isDC) {
 double   min, max, z;
 int      imin, imax;
 
 if(isDC) return 0.0;
 
 MaxMin1D (X, n, &max, &imax, &min, &imin);
 z= (max + min) / 2.0; 
 return z;
};

int utsMath_FindSyncIndex(double *X, int n, double tau, int dir, double *t, double u, int isDC) {
 int j, index= 0;
 int tindex;
 double zero;
 
 tindex= (int)(*t / tau + 0.5);
 zero= utsMath_GetDCLine(X, n, isDC);
 u= u + zero;  // !!!    !!!  ;-)
 
 for(j= 0; j< n - 1; j++) {
		if(dir) {
			if(u >= X[j] && u <= X[j+1]) { 
			   if(j >= tindex ) {
			      index= j; 
			      break; 
			   }
			};
		}   
		else {
			if(u <= X[j] && u >= X[j+1]) {   // !!! tmp
			   if(j >= tindex ) {
			      index= j; 
			      break; 
			   }
			}
		};        
 };
 
 // if(!flag) return 1;

 if(!index) *t= 0;
 return index;
};

// ************************************************************************
void utsMath_ACDC(double *X, int N, double *URms, double *UDc) {
int    ind1, ind2;
int    NZeros= 0, flag1, flag2, VeryFirst= 0, VeryLast= N - 1;
int    flag_veryfirst= 0, j;
double rms= 0.0, dc= 0.0, thresh_hi= 10.0, thresh_lo= -10.0, UZero;
double xMax, xMin;
int    iMin, iMax;
 
	flag1= 0;
	flag2= 0;
 
    MaxMin1D (X, N, &xMax, &iMax, &xMin, &iMin);
	UZero= (xMax+xMin)/2.; 
	for(j=0; j<N; j++) X[j]-= (UZero); 
 
	xMax-=UZero;
	xMin-=UZero;
 
	if(xMax < thresh_hi) thresh_hi= 0.0; 
	if(xMin > thresh_lo) thresh_lo= 0.0;
 
	for(j= 0; j< N; j++) {
    if(X[j]<=thresh_hi) {  // upper threshold
			flag1= 1;
			ind1= j;
    };
    if(X[j]>=thresh_lo) { // lower threshold
			flag2= 1;
			ind2= j;
    };
    if(flag1&flag2) {
      flag1= 0;
      flag2= 0;    
      NZeros++;
      if(!flag_veryfirst) {
				flag_veryfirst= 1;
				VeryFirst= (ind1+ind2)/2;
      };
      if( (NZeros&1) ) {
				VeryLast= (ind1+ind2)/2;       
      }
      
    };
	}
 
	for(j= VeryFirst; j<= VeryLast; j++) rms+= X[j]*X[j];
	rms= rms/(VeryLast-VeryFirst+1);
	rms= sqrt(rms);
	// rms=rms*sqrt(2);
 
	*URms= rms;
	
	// --------------
	for(j= VeryFirst; j<= VeryLast; j++) X[j]+= (UZero); 	
    MaxMin1D (X + VeryFirst, VeryLast - VeryFirst + 1, &xMax, &iMax, &xMin, &iMin);
	dc= (xMax+xMin)/2.; 
	
	*UDc=  dc;
}
/*===============================================================================================*/
