#include <windows.h>
#include <analysis.h>
#include <utility.h>
#include <ansi_c.h>
#include <userint.h>
#include "srvuir.h"
#include "uts\\services\\common\\include\\utsmath.h"
#include "uts\\services\\common\\include\\servicedef.h"

#define NPOINTMAX  1024
#define P(x)       PANEL_##x
#define SNAME      "Multimeter"

#define FFTSIZEMAX 16384
// ------------------------------------------------------------------------------------------------
static int         Pnl= -1;
// ------------------------------------------------------------------------------------------------
#define FUNCID_NO        0
#define FUNCID_MEAN      1   //  
#define FUNCID_STDDEV    2   // 
#define FUNCID_RMS       3   // 
#define FUNCID_WIDTH     4   //  - 
#define FUNCID_MIN       5   // min  
#define FUNCID_MAX       6   // max  
#define FUNCID_INSTVAL   7   //     
#define FUNCID_DC        8
#define FUNCID_DCUTS     9
#define FUNCID_RMSUTS    10
#define FUNCID_MIDDLE    11
#define FUNCID_AMPL      12
#define FUNCID_RMSAMPL   13
#define FUNCID_RMSFORMFACT 14
#define FUNCID_PERIOD      15
#define FUNCID_FREQ        16
// ------------------------------------------------------------------------------------------------
typedef struct { 
  char *name;
  int   fid;
} listFunc;
// ------------------------------------------------------------------------------------------------
static listFunc functionList[]= {
 {"Mean",        FUNCID_MEAN    },
 {"StdDev",      FUNCID_STDDEV  },
 {"Inst.Val.",   FUNCID_INSTVAL }, 
 
 {"\033m-",      FUNCID_NO},
 
 {"AC (Rms)",    FUNCID_RMS     },
 {"Rms Ampl",    FUNCID_RMSAMPL     }, 
 {"DC",          FUNCID_DC      }, 

 {"\033m-",      FUNCID_NO},
 
 {"Min",         FUNCID_MIN     },
 {"Max",         FUNCID_MAX     }, 
 {"Pik-Pik",     FUNCID_WIDTH   },
 {"Middle",      FUNCID_MIDDLE   },
 {"Amplitude",   FUNCID_AMPL   },
 
 {"\033m-",      FUNCID_NO},

 {"Frequency, Hz",        FUNCID_FREQ   },
 {"Period, ms",           FUNCID_PERIOD   },    
 {"Form Factor, a.u.",    FUNCID_RMSFORMFACT   },   

// {"Rms:UTS",       FUNCID_RMSUTS     },
// {"DC:UTS",        FUNCID_DCUTS      },  
 {NULL, 0}
};
// ------------------------------------------------------------------------------------------------
static void fillList(void) {
  listFunc *p= &functionList[0];
  while(p->name!=NULL) {
    InsertListItem(Pnl, P(list), -1, p->name, p->fid);
    p++;
  };
  SetCtrlIndex(Pnl, P(list), 0);
};
// ------------------------------------------------------------------------------------------------
typedef struct {
  int n;   //   ,  
  
  int  value;
  int  unit;
  int  descr;
  int  code;
  int  point;
} uirControl;
// ------------------------------------------------------------------------------------------------
static uirControl *theControl=NULL;
// ------------------------------------------------------------------------------------------------
static void discardUIR(void);

static int createUIR(int n, pluginADC *p) {
 int j, top, left, h;
 char tmp[512];
 
 if((theControl= (uirControl *)malloc(sizeof(uirControl)*n)) == NULL) return 1;
 memset(theControl, 0x0, sizeof(uirControl)*n);
 for(j= 0; j< n; j++) theControl[j].n= n;
 
 GetCtrlBoundingRect (Pnl, P(value), &top, 0, &h, 0);
 
 for(j= 0; j< n; j++) {
     GetCtrlBoundingRect (Pnl, P(value), 0, &left, 0, 0);
	 theControl[j].value= DuplicateCtrl (Pnl, P(value), Pnl, 0, top, left);
	 
	 GetCtrlBoundingRect (Pnl, P(unit), 0, &left, 0, 0);
	 theControl[j].unit= DuplicateCtrl (Pnl, P(unit), Pnl, "", top, left);
	 
	 GetCtrlBoundingRect (Pnl, P(descr), 0, &left, 0, 0);
	 theControl[j].descr= DuplicateCtrl (Pnl, P(descr), Pnl, "", top, left);
	 
	 GetCtrlBoundingRect (Pnl, P(code), 0, &left, 0, 0);
	 theControl[j].code= DuplicateCtrl (Pnl, P(code), Pnl, "", top, left);
	 
	 GetCtrlBoundingRect (Pnl, P(point), 0, &left, 0, 0);
	 theControl[j].point= DuplicateCtrl (Pnl, P(point), Pnl, "", top, left);
	 
	 top+= h;
 };
 
 for(j= 0; j< n; j++) {
    if(theControl[j].value <= 0 || 
       theControl[j].unit <= 0 || 
       theControl[j].descr <= 0 || 
       theControl[j].code <= 0 || 
       theControl[j].point <= 0) {
         discardUIR();
         return 1;
    };
    
    SetCtrlAttribute(Pnl, theControl[j].value , ATTR_VISIBLE, 1);
    SetCtrlAttribute(Pnl, theControl[j].unit , ATTR_VISIBLE, 1);
    SetCtrlAttribute(Pnl, theControl[j].descr , ATTR_VISIBLE, 1);
    SetCtrlAttribute(Pnl, theControl[j].code , ATTR_VISIBLE, 1);
    SetCtrlAttribute(Pnl, theControl[j].point , ATTR_VISIBLE, 1);
    
    SetCtrlVal(Pnl, theControl[j].unit, p[j].unit);
    SetCtrlVal(Pnl, theControl[j].descr, p[j].comment);
    sprintf(tmp, "%02d", j+1);
	SetCtrlAttribute (Pnl, theControl[j].value, ATTR_LABEL_TEXT, tmp);

 };
 return 0;
};
// ------------------------------------------------------------------------------------------------
static void discardUIR(void) {
  int j, n;
  
  n= theControl[0].n;
  for(j= 0; j< n; j++) {
      if(theControl[j].value > 0) DiscardCtrl(Pnl, theControl[j].value);
      if(theControl[j].unit > 0) DiscardCtrl(Pnl, theControl[j].unit);
      if(theControl[j].descr > 0) DiscardCtrl(Pnl, theControl[j].descr);
      if(theControl[j].code > 0) DiscardCtrl(Pnl, theControl[j].code);
      if(theControl[j].point > 0) DiscardCtrl(Pnl, theControl[j].point);
  };
  free(theControl);
  theControl=NULL;
};

// *************************************************************************************************
int numUTService(void) { return 1; };

// *************************************************************************************************
int nameUTService(int num, char *name) { strcpy(name, SNAME); return 0; };
// *************************************************************************************************
int bring2frontUTService(char *name) { SetActivePanel(Pnl); return 0; };
// *************************************************************************************************
static int (*closeCBF)(char *name)= NULL;
// -------------------------------------------------------------------------------------------------
int  openUTService(serviceStrucOpen *X) {
 int j, ind, q;
 char tmp[512];
 int t, l;
 
 if(Pnl > 0) {
    MessagePopup("Multimeter service: WARNING", "Supposed only one instance of the service");
    return 0; // 
 };

 closeCBF= X->closeCBF;
 Pnl= LoadPanelEx (0, "services\\multimeter\\srvuir.uir", PANEL, __CVIUserHInst);  // !!! path - tmp !!!
 fillList();
 if(createUIR(X->nAdc, X->adc)) {
    DiscardPanel(Pnl);
    Pnl= -1;
    
    MessagePopup("Multimeter service: ERROR", "Failed allocate memory for channels");
    return 1;
 };
 
 SetCtrlAttribute (Pnl, P(n), ATTR_MAX_VALUE, FFTSIZEMAX);
 DisplayPanel(Pnl);
 return 0;
};
// *************************************************************************************************
int  reopenUTService(serviceStrucOpen *X) { return 0; };
// *************************************************************************************************
void closeUTService(void) {   // !!! ??? tmp  P(list) 
  if(Pnl > 0) {
     discardUIR();
     DiscardPanel(Pnl); 
  };
  Pnl= -1; 
};
// *************************************************************************************************
// 
// !!! tmp --> ,   ,      
//      !!!!!!!!!!!!!!! tmp !!!!!!!!!!
// 
static double YOut[FFTSIZEMAX/2], darrtmp[FFTSIZEMAX];
// -------------------------------------------------------------------
int  updateUTService(int nchan, serviceStrucData X[], char *info) {
 int ch, led, flag= 0, q, jj;
 serviceStrucData *y;
 uirControl *c;
 int   fid;
 double value, d, max, min, ac, dc;
 int imax, imin, np;
 WindowConst wc;
 double df;
 
 // if(info!=NULL) InsertTextBoxLine(Pnl, P(info), -1, info);  // !!! tmp
 
 GetCtrlVal(Pnl, P(list), &fid);
 if (theControl==NULL) return -1;
 
 q= 0;
 if(fid == FUNCID_FREQ || fid == FUNCID_PERIOD || fid == FUNCID_RMSFORMFACT) q= 1;
 for(jj= 0; jj< nchan; jj++) SetCtrlAttribute(Pnl, theControl[jj].unit, ATTR_DIMMED, q);

 for(ch= 0; ch< nchan; ch++) {
     y= &X[ch];
     c= &theControl[ch];
     
     SetCtrlVal(Pnl, c->point, y->point);
     
     GetCtrlVal(Pnl, P(n), &np);
     np= np > y->point ? y->point : np;
     SetCtrlVal(Pnl, P(nreal), np);  // !!! tmp --   ..   
     
     if(!np) continue;
     
     if(y->iarray != NULL && np) {
        
        if(np > 1) Mean(y->iarray, np, &value);   // ??? tmp
        else value= y->iarray[0];
        SetCtrlVal(Pnl, c->code, value);  
        
        flag= 1;
        
        SetCtrlAttribute(Pnl, c->code, ATTR_DIMMED, 0);
     }
     else {
        SetCtrlAttribute(Pnl, c->code, ATTR_DIMMED, 1);
     };
     
     if(y->darray == NULL) { 
        SetCtrlAttribute(Pnl, c->value, ATTR_DIMMED, 1);
        continue;
     };
     
     SetCtrlAttribute(Pnl, c->value, ATTR_DIMMED, 0);
     
     value= 0.0;
     flag= 1; // ??? !!! tmp
     
     switch(fid) {
        case FUNCID_MEAN:
           if(np > 1) Mean(y->darray, np, &value);   
           else value= y->darray[0];        
        break;
        case FUNCID_STDDEV:
           if(np > 1) StdDev (y->darray, np, &d, &value);        
           else value= 0.0; 
        break; 
        case FUNCID_RMS:
        case FUNCID_RMSAMPL:
        case FUNCID_RMSFORMFACT:
        case FUNCID_DC:
		   ACDCEstimator(y->darray, np, &ac, &dc);  // ??? !!! tmp -->   
		   if(fid == FUNCID_RMS) value= ac;
		   if(fid == FUNCID_DC)  value= dc;
		   if(fid == FUNCID_RMSAMPL)  value= ac * sqrt(2);
		   if(fid == FUNCID_RMSFORMFACT)  {
              if(np > 1) MaxMin1D (y->darray, np, &max, &imax, &min, &imin);        
              else { max= y->darray[0]; min= y->darray[0]; };		   
              dc= (max - min) / 2;
		      if(dc > (double) 0.0) value= ac * sqrt(2) / dc;
		      else value= 1.0; // ??? tmp
		   };
        break;    
        case FUNCID_RMSUTS:
        case FUNCID_DCUTS:
		   utsMath_ACDC(y->darray, np, &ac, &dc);  // ??? !!! tmp -->   
		   if(fid == FUNCID_RMSUTS) value= ac;
		   else value= dc;
        break;            
        case FUNCID_WIDTH:
        case FUNCID_MIDDLE:
        case FUNCID_AMPL:
        case FUNCID_MIN:
        case FUNCID_MAX:
            if(np > 1) MaxMin1D (y->darray, np, &max, &imax, &min, &imin);        
            else {
               max= y->darray[0];
               min= y->darray[0];
            };
            if(fid == FUNCID_MIN) value= min;
            if(fid == FUNCID_MAX) value= max;
            if(fid == FUNCID_WIDTH) value= max - min;
            if(fid == FUNCID_AMPL) value= (max - min) / 2;
            if(fid == FUNCID_MIDDLE) value= (max + min) / 2;
        break;    
        case FUNCID_INSTVAL:
           value= y->darray[np / 2];
        break;
        case FUNCID_PERIOD:
        case FUNCID_FREQ:
          value= 0.0;
          if(np < 2) break;
          if(y->fs <= (double) 0.0) break;
          
          np= (int) (log(np) / log(2.0));
          np= pow(2, np);
          if(np < 2) break;
          if(np > FFTSIZEMAX) np= FFTSIZEMAX;
          
          MaxMin1D (y->darray, np, &max, &imax, &min, &imin);     
	      d= (max + min) / 2.0; 
          for(jj= 0; jj< np; jj++) darrtmp[jj]= y->darray[jj] - d;
          
          ScaledWindow (darrtmp, np, 7, &wc);   // 7 : 4-term Black/Harr
		  AutoPowerSpectrum (darrtmp, np, 1.0/y->fs, YOut, &df);
		  PowerFrequencyEstimate (YOut, np/2, -1.0 /* 2.0/y->fs */, wc, df, 11, &value, &d);   // ??? !!! tmp
		  
		  if(fid == FUNCID_PERIOD) {
		     if(value <= (double)0.0) break;
		     value= 1000.0/value;
		  };
          
        break;
     };
     
     SetCtrlVal(Pnl, c->value, value);
 };

 if(flag) {
    GetCtrlVal(Pnl, P(led), &led);		// ??? !!! tmp --> here ?
    SetCtrlVal(Pnl, P(led), !led);
 }
 else {
    
 };
 
 return 0;
};

// ************************************************************************************************
// 
// ************************************************************************************************
int CVICALLBACK mf_close (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
		    discardUIR();
			DiscardPanel(Pnl);
			Pnl= -1;
			if(closeCBF == NULL) break;
		    closeCBF(SNAME);
			break;
	}
	return 0;
}

#define HLPMSG \
"Displays code/physical value of all channels\n\n\
"

int CVICALLBACK mf_help (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
			MessagePopup("Multimeter HELP", HLPMSG);
			break;
	}
	return 0;
}

int CVICALLBACK mf_list (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:

			break;
	}
	return 0;
}

int CVICALLBACK mf_precision (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
int v, j;
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal(Pnl, P(precision), &v);
			for(j= 0; j< theControl[0].n; j++) 
			  SetCtrlAttribute(Pnl, theControl[j].value, ATTR_PRECISION, v);
			break;
	}
	return 0;
}

int CVICALLBACK PANEL_CALLBACK (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
			 discardUIR();
			DiscardPanel(Pnl);
			Pnl= -1;
			if(closeCBF == NULL) break;
		    closeCBF(SNAME);
			break;
		}
	return 0;
}
