#include "uts\\systems\\common\\include\\plugindef.h"
#include "uts\\services\\common\\include\\servicedef.h"
#include <analysis.h>
#include <utility.h>
#include <ansi_c.h>
#include <userint.h>
#include "srvuir.h"
#include "uts\\systems\\common\\include\\plugindef.h"
#include "toolbox.h"
#include "uts\\services\\common\\include\\utsmath.h"

#define      MAXIDNUM 2
#define P(x) PANEL_##x

#define      NFFTMAX 32768

static double FFTArray[MAXIDNUM][NFFTMAX];

#define      OSCHNUM 16
static int nADC;
static pluginADC *theADC= NULL;


static int     ipnl[MAXIDNUM], Pnl= -1;
static double  theSHIFT[MAXIDNUM][OSCHNUM];

static int 	UseMiddleMeanFourier=0;
static int  MiddleMeanFourierCount=0;

// -------------------------------------------------------------------------
#pragma pack(1)
typedef struct {
  double scale[OSCHNUM];
  double x, y;  // 
  double time;  //  
  int    dc, sync, front, syncchannel;
  double shift[OSCHNUM];
} oscIniOsc;

typedef struct {
  double x, y;  // 
  double f;
  double win, size;
  double base;
} oscIniFft;

typedef struct {  
  int mode;              // fft/osc
  int onoff[OSCHNUM];
  int extchan[OSCHNUM];  //       
  int color[OSCHNUM];
  int shiftchannel;
  
  oscIniOsc osc;
  oscIniFft fft;
} oscIni;
#pragma pack()

// *********************************************************************
#define OSCINIPATH "services\\osc\\_oscfft.ini"  // !!! tmp -- .    
static oscIni IniSet[MAXIDNUM];


// -------------------------------------------------------------------------
static int findFreeId(void) {
 int j;
 for(j= 0; j< MAXIDNUM; j++) if(ipnl[j] <= 0) return j;
 return -1;
};

// -------------------------------------------------------------------------
int  numUTService(void) {
 int j;
 int q= 0;
 for(j= 0; j<MAXIDNUM; j++) if(ipnl[j] > 0) q++;
 return q;
};

// -------------------------------------------------------------------------
int nameUTService(int num, char *name) {
 int j, q= 0;
 
 for(j= 0; j<MAXIDNUM; j++) {
     if(ipnl[j] <= 0) continue;
     if(q == num) {
	    GetPanelAttribute (ipnl[j], ATTR_TITLE, name);
	    return 0;
     };
     q++;
 };
 return 1;
};

// -------------------------------------------------------------------------
int bring2frontUTService(char *name) {
 int j;
 char tmp[512];
 for(j= 0; j<MAXIDNUM; j++) {
     if(ipnl[j] <= 0) continue;
	 GetPanelAttribute (ipnl[j], ATTR_TITLE, tmp);
	 if(!stricmp(name, tmp)) {
	    SetActivePanel(ipnl[j]);
	    return 0;
	 };
 };
 return 1;
};

static winOCH[OSCHNUM]= {
P(ch_1),P(ch_2),P(ch_3),P(ch_4),P(ch_5),P(ch_6),P(ch_7),P(ch_8),
P(ch_9),P(ch_10),P(ch_11),P(ch_12),P(ch_13),P(ch_14),P(ch_15),P(ch_16)};
static winPCH[OSCHNUM]= {
P(p_1),P(p_2),P(p_3),P(p_4),P(p_5),P(p_6),P(p_7),P(p_8),
P(p_9),P(p_10),P(p_11),P(p_12),P(p_13),P(p_14),P(p_15),P(p_16)};
static winScale[OSCHNUM]= {
P(s_1),P(s_2),P(s_3),P(s_4),P(s_5),P(s_6),P(s_7),P(s_8),
P(s_9),P(s_10),P(s_11),P(s_12),P(s_13),P(s_14),P(s_15),P(s_16)};
static winColor[OSCHNUM]= {
P(c_1),P(c_2),P(c_3),P(c_4),P(c_5),P(c_6),P(c_7),P(c_8),
P(c_9),P(c_10),P(c_11),P(c_12),P(c_13),P(c_14),P(c_15),P(c_16)};
static winInfo[OSCHNUM]= {
P(t_1),P(t_2),P(t_3),P(t_4),P(t_5),P(t_6),P(t_7),P(t_8),
P(t_9),P(t_10),P(t_11),P(t_12),P(t_13),P(t_14),P(t_15),P(t_16)};
// -------------------------------------------------------------------------
typedef struct {
  double  value;
  char   *name;
} listControl;

// -------------------------------------------
static listControl listOfNFFT[]= {
 {128, "128"},
 {256, "256"}, 
 {512, "512"},
 {1024, "1024"}, 
 {2048, "2048"},
 {4096, "4096"}, 
 {8192, "8192"},
 {16384, "16384"},
 {NFFTMAX, "32768"},  
 {0.0, NULL} 
};
// -------------------------------------------
static listControl listOfWin[]= {
 {0, "Uniform"},
 {1, "Hanning"},
 {2, "Hamming"},
 {3, "Blackman-Harris"},
 {4, "Exact Blackman"},
 {5, "Blackman"},
 {6, "Flattop"},
 {7, "Four Term Blackman-Harris"},
 {8, "Seven Term Blackman-Harris"},
 {0.0, NULL} 
};
// --------------------------------------------
static listControl listOfFreq[]= {
 {1.0, "1 Hz"},
 {2.0, "2 Hz"},
 {5.0, "5 Hz"},
 {10.0, "10 Hz"},
 {20.0, "20 Hz"},
 {50.0, "50 Hz"},
 {100.0, "100 Hz"},
 {200.0, "200 Hz"},
 {500.0, "500 Hz"},
 {1e3, "1 kHz"},
 {2e3, "2 kHz"},
 {5e3, "5 kHz"},  
 {10e3, "10 kHz"},  
 {20e3, "20 kHz"},  
 {50e3,  "50 kHz"},
 {100e3, "100 kHz"},   
 {200e3, "200 kHz"},   
 {0.0, NULL}
};

// ----------------------------------
static listControl listOfScales[]= {
 { 10e-6, "10 mkV"},
 { 20e-6, "20 mkV"},
 { 50e-6, "50 mkV"},
 { 100e-6, "100 mkV"},
 { 200e-6, "200 mkV"},
 { 500e-6, "500 mkV"},  
 { 1e-3, "1 mV"},
 { 2e-3, "2 mV"},
 { 5e-3, "5 mV"},
 { 10e-3, "10 mV"}, 
 { 20e-3, "20 mV"},
 { 50e-3, "50 mV"},
 { 0.1, "0.1 V"},
 { 0.2, "0.2 V"},
 { 0.5 , "0.5 V"},
 { 1.0, "1 V"},
 { 2.0, "2 V"}, 
 { 5.0, "5 V"},
 { 0.0, NULL}
};

// -------------------------------------------------
static listControl listOfTimes[]= {
 { 1e-6, "1 mks"},
 { 2e-6, "2 mks"},
 { 5e-6, "5 mks"}, 
 { 10e-6, "10 mks"},
 { 20e-6, "20 mks"},
 { 50e-6, "50 mks"}, 
 { 100e-6, "100 mks"}, 
 { 200e-6, "200 mks"},
 { 500e-6, "500 mks"}, 
 { 1e-3, "1 ms"},
 { 2e-3, "2 ms"},
 { 5e-3, "5 ms"}, 
 { 10e-3, "10 ms"},
 { 20e-3, "20 ms"},
 { 50e-3, "50 ms"}, 
 { 100e-3, "100 ms"},
 { 200e-3, "200 ms"},
 { 0.5, "0.5 s"}, 
 { 1.0, "1 s"}, 
 { 0.0, NULL}
};

#define mkColor(x, y, z)  (((x) << 16) | ((y) << 8) | (z))

static int colorTable[OSCHNUM]={
 //VAL_RED, 
 mkColor(255, 193, 100),
 VAL_GREEN, 
 VAL_BLUE, 
 VAL_YELLOW, 
 mkColor(255, 128, 128),
 mkColor(128, 255, 255),  
 mkColor(255, 128, 64),
 mkColor(179, 102, 255), 
 mkColor(163, 92, 135),
 mkColor(233, 233, 233),  
 mkColor(235, 220, 86),
 mkColor(221, 221, 255),
 mkColor(254, 227, 222),
 mkColor(223, 255, 239),  
 mkColor(176, 170, 134),
 mkColor(201, 104, 206)
};
// -------------------------------------------------------------------------
//
// -------------------------------------------------------------------------
void GetCtrlVals(int p, int ind) 
{
	int j;
     GetCtrlVal(p, P(fft), &IniSet[ind].mode);
     for(j= 0; j< OSCHNUM; j++) 
     {
         GetCtrlVal(p, winOCH[j], &IniSet[ind].onoff[j]);
         GetCtrlVal(p, winPCH[j], &IniSet[ind].extchan[j]);
         GetCtrlVal(p, winColor[j], &IniSet[ind].color[j]);         
         GetCtrlVal(p, winScale[j], &IniSet[ind].osc.scale[j]);   
         IniSet[ind].osc.shift[j]= theSHIFT[ind][j];
     };
     GetCtrlVal(p, P(shiftchan), &IniSet[ind].shiftchannel);
     GetCtrlVal(p, P(synctime), &IniSet[ind].osc.x);
     GetCtrlVal(p, P(synclevel), &IniSet[ind].osc.y);
     GetCtrlVal(p, P(time), &IniSet[ind].osc.time);
     GetCtrlVal(p, P(dc), &IniSet[ind].osc.dc);
     GetCtrlVal(p, P(sync), &IniSet[ind].osc.sync);
     GetCtrlVal(p, P(rise), &IniSet[ind].osc.front);
     GetCtrlVal(p, P(syncchan), &IniSet[ind].osc.syncchannel);

     // x,y,win,size,base
     GetCtrlVal(p, P(fftf), &IniSet[ind].fft.x);
     GetCtrlVal(p, P(fftdb), &IniSet[ind].fft.y);
     GetCtrlVal(p, P(f), &IniSet[ind].fft.f);     
     GetCtrlVal(p, P(win), &IniSet[ind].fft.win);     
     GetCtrlVal(p, P(nfft), &IniSet[ind].fft.size);     
     GetCtrlVal(p, P(base), &IniSet[ind].fft.base);
};

// -------------------------------------------------------------------------
//
/*-----------------------------------------------------------------------------------------------*/
static void refreshUIR(int p, int useini, int ind)
{
    double      ftmp;
    int         i;
    int         j;
    listControl *lc;
    char        tmp[512];
    double      ttmp;


    lc = &listOfTimes[0];
    SetCtrlAttribute(p, P(time), ATTR_DATA_TYPE, VAL_DOUBLE);
    while (lc->name != NULL)
    {
        InsertListItem(p, P(time), -1, lc->name, lc->value);
        lc++;
    }
    SetCtrlIndex(p, P(time), 8);            /* 50 mks ??? !!! tmp ??? */
    GetCtrlVal(p, P(time), &ttmp);
    /* 10 :      */
    SetAxisScalingMode(p, P(graf), VAL_XAXIS, VAL_MANUAL, 0, ttmp * 10);


    lc = &listOfFreq[0];
    SetCtrlAttribute(p, P(f), ATTR_DATA_TYPE, VAL_DOUBLE);
    while (lc->name != NULL)
    {
        InsertListItem(p, P(f), -1, lc->name, lc->value);
        lc++;
    }
    SetCtrlIndex(p, P(f), 9);               /* 1 kHz ??? !!! tmp ??? */
    GetCtrlVal(p, P(f), &ftmp);
    /* 10 :     F */
    SetAxisScalingMode(p, P(fftgraf), VAL_XAXIS, VAL_MANUAL, 0, ftmp * 10);


    lc = &listOfWin[0];
    SetCtrlAttribute(p, P(win), ATTR_DATA_TYPE, VAL_DOUBLE);
    while (lc->name != NULL)
    {
        InsertListItem(p, P(win), -1, lc->name, lc->value);
        lc++;
    }
    SetCtrlIndex(p, P(win), 7);             /* 4-term blcakman-harris */


    lc = &listOfNFFT[0];
    SetCtrlAttribute(p, P(nfft), ATTR_DATA_TYPE, VAL_DOUBLE);
    while (lc->name != NULL)
    {
        InsertListItem(p, P(nfft), -1, lc->name, lc->value);
        lc++;
    }
    SetCtrlIndex(p, P(nfft), 6);            /* 8192 */


    SetAxisScalingMode(p, P(graf), VAL_LEFT_YAXIS, VAL_MANUAL, -1.0, 1.0);
    SetGraphCursor(p, P(graf), 1, 0.0, 0.0);

    SetAxisScalingMode(p, P(fftgraf), VAL_LEFT_YAXIS, VAL_MANUAL, -120.0, 10.0);  
    SetGraphCursor(p, P(fftgraf), 1, 0.0, 0.0);

    for (j = 0; j < OSCHNUM; j++)
    {
        sprintf(tmp, "%d", j + 1);
        SetCtrlAttribute(p, winOCH[j], ATTR_LABEL_TEXT, tmp);

        SetCtrlVal(p, winColor[j], colorTable[j]);
        for(i = 0; i < nADC; i++) 
            InsertListItem(p, winPCH[j], -1, theADC[i].comment, i);

        SetCtrlIndex(p, winPCH[j], j % nADC);
        SetCtrlVal(p, winInfo[j], theADC[j%nADC].comment);

        lc = &listOfScales[0];      
        SetCtrlAttribute(p, winScale[j], ATTR_DATA_TYPE, VAL_DOUBLE);
        while (lc->name != NULL)
        {
            InsertListItem(p, winScale[j], -1, lc->name, lc->value);
            lc++;
        }
        SetCtrlIndex(p, winScale[j], 17);   /* 5 V ??? !!! tmp ??? */
/*        GetCtrlIndex(panel, control, &index);
	    GetLabelFromIndex (panel, control, index, tmp);
		SetCtrlAttribute(panel, control, ATTR_LABEL_TEXT, tmp);*/
    }


    for(j = nADC; j < OSCHNUM; j++)
    {
        SetCtrlAttribute(p, winOCH[j], ATTR_VISIBLE, 0);
        SetCtrlAttribute(p, winPCH[j], ATTR_VISIBLE, 0);
        SetCtrlAttribute(p, winScale[j], ATTR_VISIBLE, 0);
        SetCtrlAttribute(p, winColor[j], ATTR_VISIBLE, 0);
        SetCtrlAttribute(p, winInfo[j], ATTR_VISIBLE, 0);
    }

    SetCtrlVal(p, winOCH[0], 1);
    SetCtrlVal(p, winOCH[1], 1);
    for(j = 2; j < OSCHNUM; j++)
    {
        SetCtrlVal(p, winOCH[j], 0);
        SetCtrlAttribute(p, winPCH[j], ATTR_DIMMED, 1);
        SetCtrlAttribute(p, winScale[j], ATTR_DIMMED, 1);
        SetCtrlAttribute(p, winColor[j], ATTR_DIMMED, 1);
        SetCtrlAttribute(p, winInfo[j], ATTR_DIMMED, 1);
    }

    if (useini)
    {
        SetCtrlVal(p, P(fft), IniSet[ind].mode);
        for(j = 0; j < OSCHNUM; j++)
        {
            SetCtrlVal(p, winOCH[j], IniSet[ind].onoff[j]);
            SetCtrlVal(p, winColor[j], IniSet[ind].color[j]);         
            SetCtrlVal(p, winScale[j], IniSet[ind].osc.scale[j]);   
            theSHIFT[ind][j] = IniSet[ind].osc.shift[j];
        }

        SetCtrlVal(p, P(time), IniSet[ind].osc.time);
        SetCtrlVal(p, P(dc), IniSet[ind].osc.dc);
        SetCtrlVal(p, P(sync), IniSet[ind].osc.sync);
        SetCtrlVal(p, P(rise), IniSet[ind].osc.front);

        SetCtrlVal(p, P(shiftchan), IniSet[ind].shiftchannel);     
        SetCtrlVal(p, P(syncchan), IniSet[ind].osc.syncchannel);     

        SetCtrlVal(p, P(f), IniSet[ind].fft.f);     
        SetCtrlVal(p, P(win), IniSet[ind].fft.win);     
        SetCtrlVal(p, P(nfft), IniSet[ind].fft.size);     
        SetCtrlVal(p, P(base), IniSet[ind].fft.base);

        for(j = 0; j< OSCHNUM; j++)
        {
            mf_channel(p, winOCH[j], EVENT_COMMIT, NULL, 0, 0);
            mf_physch(p, winPCH[j], EVENT_COMMIT, NULL, 0, 0);
        }
        /* !!! tmp ---> SHIFT    ??? */
        mf_shiftchannel(p, P(shiftchan), EVENT_COMMIT, NULL, 0, 0);
        mf_syncparams(p, -1, EVENT_COMMIT, NULL, 0, 0);
        mf_sync(p, P(sync), EVENT_COMMIT, NULL, 0, 0);
        mf_syncchan(p, P(syncchan), EVENT_COMMIT, NULL, 0, 0);

        mf_color(p, -1, EVENT_COMMIT, NULL, 0, 0);
        for(j = 0; j < OSCHNUM; j++)
            mf_scale(p, winScale[j], EVENT_COMMIT, NULL, 0, 0);
        mf_dc(p, P(dc), EVENT_COMMIT, NULL, 0, 0);
        mf_rise(p, P(rise), EVENT_COMMIT, NULL, 0, 0);
        mf_time(p, P(time), EVENT_COMMIT, NULL, 0, 0);

        mf_fftcursor(p, -1, EVENT_COMMIT, NULL, 0, 0);
        mf_f(p, P(f), EVENT_COMMIT, NULL, 0, 0);

        mf_fft(p, P(fft), EVENT_COMMIT, NULL, 0, 0);

        SetCtrlVal(p, P(shiftchan), IniSet[ind].shiftchannel);     
        SetCtrlVal(p, P(syncchan), IniSet[ind].osc.syncchannel);
        mf_shiftchannel(p, P(shiftchan), EVENT_COMMIT, NULL, 0, 0);     
        mf_syncchan(p, P(syncchan), EVENT_COMMIT, NULL, 0, 0);
    }

    mf_channel(p, P(ch_1), EVENT_COMMIT, NULL, 0, 0);

    SetCtrlVal(p, P(t), "");
    SetCtrlVal(p, P(u), "");
    SetCtrlVal(p, P(fabs), "");

    mf_syncparams(p, -1, EVENT_COMMIT, NULL, 0, 0);

    mf_time(p, P(time), EVENT_COMMIT, NULL, 0, 0);
    mf_f(p, P(f), EVENT_COMMIT, NULL, 0, 0);

    EnableExtendedMouseEvents(p, P(graf), 0.15);
    EnableExtendedMouseEvents(p, P(fftgraf), 0.15);

    SetCtrlVal(p, P(synclevel), 0.0);
    SetCtrlVal(p, P(synctime), 0.0);
    mf_syncparams(p, P(synctime), EVENT_COMMIT, NULL, 0, 0);

    SetCtrlVal(p, P(fftf), 0.0);
    SetCtrlVal(p, P(fftdb), 0.0);						
    mf_fftcursor(p, P(fftf), EVENT_COMMIT, NULL, 0, 0);
}
/*-----------------------------------------------------------------------------------------------*/

static int (*closeCBF)(char *name)= NULL;


static int readINI(void) 
{
  FILE *f;
  if((f= fopen(OSCINIPATH, "rb")) == NULL) return 1;
  fread (IniSet, sizeof(oscIni), MAXIDNUM, f);
  fclose(f);
  return 0;
};
// *********************************************************************
static int writeINI(void) 
{
  FILE *f;
  if((f= fopen(OSCINIPATH, "wb")) == NULL) return 1;
  fwrite (IniSet, sizeof(oscIni), MAXIDNUM, f);
  fclose(f);
  return 0;
};
// *********************************************************************
//   ENTRY POINT 
// *********************************************************************
int  openUTService(serviceStrucOpen *X) 
{
 int j, ind, q;
 char tmp[512];
 int t, l, iniresult;
 
 closeCBF= X->closeCBF;
 
 if(Pnl == -1) 
 {
    for(j= 0; j< MAXIDNUM; j++) 
    {
       ipnl[j]= -1;
    };
	Pnl = LoadPanelEx (0, "services\\osc\\srvuir.uir", PANEL, __CVIUserHInst);  // !!! path - tmp !!!
	
    if( (theADC= (pluginADC *)malloc(X->nAdc * sizeof(pluginADC))) == NULL) {
    	 DiscardPanel(Pnl);
    	 Pnl= -1;
         return 1;
    };
    nADC= X->nAdc;
    
    for(j= 0; j<nADC; j++) theADC[j]= X->adc[j];
 
 };
 
 for(q= 0, j= 0; j<MAXIDNUM; j++) if(ipnl[j] > 0) q++;
 if(q == X->nAdc) 
 {
    MessagePopup("Oscilloscope service: WARNING", "All channels are visualized already");
    return 0;  // !!! tmp
 };
 
 if((ind= findFreeId()) < 0) return -1;

 sprintf(tmp, "Oscilloscope (%d)", ind+1);
 
 if((ipnl[ind] = DuplicatePanel (0, Pnl, tmp, VAL_KEEP_SAME_POSITION, VAL_KEEP_SAME_POSITION)) <= 0) return -1;
 GetPanelAttribute (ipnl[ind], ATTR_TOP, &t);
 GetPanelAttribute (ipnl[ind], ATTR_LEFT, &l);
 SetPanelPos (ipnl[ind], t + ind * 20, l);

 for(j= 0; j< X->nAdc; j++) 
 {
     sprintf(tmp, "%d", j+1);
     //InsertListItem (ipnl[ind], P(list), -1, tmp, X->adc[j].comment); 
 };
 
 q= ind % X->nAdc;
 //SetCtrlIndex(ipnl[ind], P(list), q);
 //SetCtrlVal(ipnl[ind], P(text), X->adc[q].comment);
 
 for(j= 0; j<OSCHNUM; j++) theSHIFT[ind][j]= 0.0;

 iniresult= !readINI();
 refreshUIR(ipnl[ind], iniresult, ind);
 

 DisplayPanel(ipnl[ind]);
 return ind;
};

int  reopenUTService(serviceStrucOpen *X) 
{
 	return 0;
};

// NULL at the input : close all !
void closeUTService(void) 
{
 	int j, flag= 0;
 	SetMouseCursor (VAL_DEFAULT_CURSOR);
 
 	if(Pnl!=-1) DiscardPanel(Pnl);
	Pnl= -1;
	for(j= 0; j<MAXIDNUM; j++) 
	{
	  if(ipnl[j] > 0 ) 
	  {
	    DisableExtendedMouseEvents (ipnl[j], P(graf));
	    DisableExtendedMouseEvents (ipnl[j], P(fftgraf));
    
	    // ??? !!! 
	    ProcessSystemEvents();
    
	    GetCtrlVals(ipnl[j], j);
	    flag= 1;
    
	    DiscardPanel(ipnl[j]); 
	    ipnl[j]= -1; 
	  };
	};
	
	if(theADC!=NULL) 
	{
	   free(theADC);
	   theADC= NULL;    
	};
	if(flag) writeINI();
};

int CVICALLBACK mf_close (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int j;
	char name[512];

	switch (event) 
	{
		case EVENT_COMMIT:
		    SetMouseCursor (VAL_DEFAULT_CURSOR);
		    
		    for(j= 0; j<MAXIDNUM; j++) 
		    {
		       if(panel != ipnl[j]) continue;
		       break;
		    };
		    
		    GetCtrlVals(ipnl[j], j);
		    
		    ipnl[j]= -1;
		    GetPanelAttribute (panel, ATTR_TITLE, name);
		    DisableExtendedMouseEvents (panel, P(graf));
		    DisableExtendedMouseEvents (panel, P(fftgraf)); 
			DiscardPanel(panel);
			
			if(!numUTService()) 
			{
			    free(theADC);
			    theADC= NULL;
			    
			    writeINI();
			    DiscardPanel(Pnl);
			    Pnl= -1;
			};
			
			if(closeCBF == NULL) break;
		    closeCBF(name);
			break;
	}
	return 0;
}

// *****************************************************************************************

#define HLPMSG \
"Not yet realized"

int CVICALLBACK mf_help (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) 
{
	switch (event) 
	{
		case EVENT_COMMIT:
			MessagePopup("Oscilloscope HELP", HLPMSG);
			break;
	}
	return 0;
}

// *****************************************************************************************
/*
static int checkLock(int panel) {
 int state;

 GetCtrlVal(panel, P(run), &state);
 if(!state) return 0;
 
 SetCtrlAttribute(panel, P(warn), );
 
 return 1;
};*/


// -----------------------------------------------------------------------------------------
int CVICALLBACK mf_channel (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int j, v;
	int flag= 0, sindex, shiftindex, color;
	char tmp[512], lbl[512];

	switch (event) 
	{
		case EVENT_COMMIT:
		    // if(checkLock(panel)) break;
		    
		    for(j= 0; j< OSCHNUM; j++) 
		    {
		        GetCtrlAttribute(panel, winOCH[j], ATTR_VISIBLE, &v);
		        if(!v) continue;
		        GetCtrlVal(panel, winOCH[j], &v);
		        if(v) { flag= 1; break; };
		    };
		    if(!flag) 
		    {
		       SetCtrlVal(panel, control, 1);
		       break;  //      
		    };
		    
		    for(j= 0; j< OSCHNUM; j++) if(control == winOCH[j]) break;
		    GetCtrlVal(panel, winOCH[j], &v);
		    v= !v;
            SetCtrlAttribute(panel, winPCH[j], ATTR_DIMMED, v);
            SetCtrlAttribute(panel, winScale[j], ATTR_DIMMED, v);
            SetCtrlAttribute(panel, winColor[j], ATTR_DIMMED, v);
            SetCtrlAttribute(panel, winInfo[j], ATTR_DIMMED, v);
            
            GetCtrlIndex(panel, P(syncchan), &sindex);
            if(sindex!=-1) GetCtrlVal(panel, P(syncchan), &sindex);
            else sindex= 0;
            GetCtrlIndex(panel, P(shiftchan), &shiftindex);
            if(shiftindex!=-1) GetCtrlVal(panel, P(shiftchan), &shiftindex);
            else shiftindex= 0;
            
            DeleteListItem(panel, P(syncchan), 0, -1);
            DeleteListItem(panel, P(shiftchan), 0, -1);
		    
		    for(j= 0; j< OSCHNUM; j++) {
		        GetCtrlAttribute(panel, winOCH[j], ATTR_VISIBLE, &v);
		        if(!v) continue;
		        GetCtrlVal(panel, winOCH[j], &v);
		        if(!v) continue;
				GetCtrlAttribute (panel, winOCH[j], ATTR_LABEL_TEXT, tmp);
				
				GetCtrlVal (panel, winColor[j], &color);
				
				//sprintf(lbl, "%s\033p25lqqqq\033fg%06XQQQQ", tmp, color);
				
				         //sprintf(lbl, "%s\033p200l[NET %d]", tmp, color);
				         
				InsertListItem (panel, P(syncchan), -1, tmp, j);
				InsertListItem (panel, P(shiftchan), -1, tmp, j);
		    };
		    
		    SetCtrlVal(panel, P(syncchan), sindex);
		    SetCtrlVal(panel, P(shiftchan), shiftindex);
		    
            mf_syncchan (panel, P(syncchan), EVENT_COMMIT, NULL, 0, 0);
            mf_shiftchannel (panel, P(shiftchan), EVENT_COMMIT, NULL, 0, 0);
		
			break;
	}
	return 0;
}

int CVICALLBACK mf_physch (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int j, ind;

	switch (event) {
		case EVENT_COMMIT:
		    for(j= 0; j< OSCHNUM; j++) if(control == winPCH[j]) break;
		    GetCtrlVal(panel, winPCH[j], &ind);
            SetCtrlVal(panel, winInfo[j], theADC[ind].comment);
			break;
	}
	return 0;
}

/*-----------------------------------------------------------------------------------------------*/
int CVICALLBACK mf_scale(int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int  index;
    char tmp[512];
    

    switch (event)
    {
    case EVENT_COMMIT:
        GetCtrlIndex(panel, control, &index);
	    GetLabelFromIndex (panel, control, index, tmp);
		SetCtrlAttribute(panel, control, ATTR_LABEL_TEXT, tmp);
        mf_syncchan(panel, P(syncchan), EVENT_COMMIT, NULL, 0, 0);
        mf_shiftchannel(panel, P(shiftchan), EVENT_COMMIT, NULL, 0, 0);
        break;
    }
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

int CVICALLBACK mf_color (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	switch (event) 
	{
		case EVENT_COMMIT:
			mf_syncchan (panel, P(syncchan), EVENT_COMMIT, NULL, 0, 0);
			mf_shiftchannel (panel, P(shiftchan), EVENT_COMMIT, NULL, 0, 0);
			break;
	}
	return 0;
}

int CVICALLBACK mf_time (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	double t;
	char tmp[512];
	int index;
	double s;
	switch (event) 
	{
		case EVENT_COMMIT:
			GetCtrlVal(panel, P(time), &t);
			SetAxisScalingMode (panel, P(graf), VAL_XAXIS, VAL_MANUAL, 0, t * 10); // 10 :    
			GetCtrlIndex(panel, control, &index);
			GetLabelFromIndex (panel, control, index, tmp);
			SetCtrlAttribute(panel, control, ATTR_LABEL_TEXT, tmp);
		
			GetCtrlVal(panel, P(time), &s);
			SetCtrlAttribute (panel, P(synctime), ATTR_INCR_VALUE, s/10.0);
			SetCtrlAttribute (panel, P(synctime), ATTR_MAX_VALUE, s*10.0);
			
			mf_graf (panel, -1, EVENT_COMMIT, NULL, 0, 0);
			break;
	}
	return 0;
}

/*-----------------------------------------------------------------------------------------------*/
int CVICALLBACK mf_sync(int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int    i;
    double t;
    double u;
    
    
    switch (event)
    {
    case EVENT_COMMIT:
        GetCtrlVal(panel, P(sync), &i);
        SetCtrlAttribute(panel, P(sync), ATTR_LABEL_TEXT, (!i) ? "Sync" : "N/S");
        SetCtrlAttribute(panel, P(rise), ATTR_VISIBLE, i);
        SetCtrlAttribute(panel, P(syncchan), ATTR_VISIBLE, i);
        SetCtrlAttribute(panel, P(synclevel), ATTR_VISIBLE, i);
        SetCtrlAttribute(panel, P(synctime), ATTR_VISIBLE, i);
        SetCtrlAttribute(panel, P(scale_sync), ATTR_VISIBLE, i);

        break;
    }
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

// *********************************************************************************************
// 
// *********************************************************************************************
int updateUTService(int nchan, serviceStrucData *X, char *info) 
{
 int    panel, p, ind;
 int    och, q, color, led, flag= 0;
 double scale;
 int    isSync, chSync, dirSync;
 double tSync, uSync, dcline, fsignal;
 int    indSync, dc, j, mode, nfft, win, fftresult;
 double nfft_dbl, win_dbl;
 double snr, pwr, base;
 char tmp[512];
 
 for(p= 0; p<MAXIDNUM; p++) 
 {
     panel= ipnl[p];
     if( panel<= 0) continue;
     
     GetCtrlVal(panel, P(run), &q);         
     if(!q) continue;

     GetCtrlVal(panel, P(sync), &isSync);
     GetCtrlVal(panel, P(dc),   &dc);
     GetCtrlVal(panel, P(fft),   &mode);
         
     indSync= 0;
     tSync= 0.0;
     // !!! ??? tmp --->     == ??? !!! TMP
     GetCtrlVal(panel, P(syncchan),  &chSync);
     if(isSync && X[chSync].darray!= NULL && mode) 
     {  // mode = 1 : OSC
        GetCtrlVal(panel, P(rise),      &dirSync);
        GetCtrlVal(panel, P(synctime),  &tSync);
        GetCtrlVal(panel, P(synclevel), &uSync);            
        indSync= utsMath_FindSyncIndex(X[chSync].darray, X[chSync].point,1.0/theADC[chSync].fs,dirSync,&tSync,uSync, dc);
     };
 
	 if(!mode) 
	 {  // FFT
	     DeleteGraphPlot (panel, P(fftgraf), -1, VAL_DELAYED_DRAW);
	     SetCtrlAttribute(panel, P(fftgraf), ATTR_REFRESH_GRAPH, 0);   
	     
	     GetCtrlVal (panel, P(shiftchan),  &och);    //   
         GetCtrlIndex (panel, winPCH[och], &ind);    //   _ ind _	 
         GetCtrlVal(panel, winColor[och],  &color);

         if(X[ind].darray == NULL) continue;
         if(X[ind].point < 2) continue; 
	     flag= 1;
	     
	     fsignal= -1.0; // !!! tmp  --> auto search
         GetCtrlVal(panel, P(win),  &win_dbl);     win= (int)win_dbl;
         GetCtrlVal(panel, P(nfft), &nfft_dbl);   nfft= (int)nfft_dbl;
         GetCtrlVal(panel, P(base), &base);
		 
		 if(UseMiddleMeanFourier)
		 {
			 double * TempFFTArray = malloc(nfft*sizeof(double));
			 fftresult= utsMath_DoFFT(X[ind].darray, X[ind].point, dc, win, theADC[ind].fs, &nfft,
            	         TempFFTArray, &fsignal, &pwr, &snr, NULL,NULL,NULL,NULL);

			 if (MiddleMeanFourierCount==0)
			 {
				 memset(FFTArray[p],0,sizeof(double)*nfft);
			 }
			 
			 if (TempFFTArray!=NULL)
			 {
				 int i=0;
				 for (i=0;i<nfft;i++)
				 {
					(FFTArray[p])[i] = (((FFTArray[p])[i]*MiddleMeanFourierCount)+TempFFTArray[i])/(MiddleMeanFourierCount+1);
				 }
				 
				 free(TempFFTArray);
			 }
			 MiddleMeanFourierCount++;
		 }
		 else
		 {

	     	fftresult= utsMath_DoFFT(X[ind].darray, X[ind].point, dc, win, theADC[ind].fs, &nfft,
            	         FFTArray[p], &fsignal, &pwr, &snr, NULL,NULL,NULL,NULL);
		 }
	     
	       // !!! tmp 10.0, 1, ---> 1 : win
	     

            SetCtrlAttribute(panel, P(peak), ATTR_VISIBLE, !fftresult);
            SetCtrlAttribute(panel, P(nfftreal), ATTR_VISIBLE, 1);//!fftresult);
            SetCtrlAttribute(panel, P(snr), ATTR_VISIBLE, !fftresult);
            
            sprintf(tmp, "Peak: %.3lf kHz, %.3lf Vrms", fsignal/1e3, pwr);
            SetCtrlVal(panel, P(peak), tmp);
	        SetCtrlVal(panel, P(snr), snr);	        
            sprintf(tmp, "%d", nfft*2);
            SetCtrlVal(panel, P(nfftreal), tmp);	        
	        
			PlotWaveform (panel, P(fftgraf), FFTArray[p], nfft, VAL_DOUBLE, 1.0,
						  -10.0*log10(base*base/2), 0.0, theADC[ind].fs/2.0/nfft,
						  VAL_THIN_LINE, VAL_SIMPLE_DOT, VAL_SOLID, 1, color);


	     RefreshGraph (panel, P(fftgraf));
         
	     goto __next_panel;
	 };
	 
	 
	 DeleteGraphPlot (panel, P(graf), -1, VAL_DELAYED_DRAW);
	 SetCtrlAttribute(panel, P(graf), ATTR_REFRESH_GRAPH, 0);     	 
	 
     for(och= 0; och < OSCHNUM; och++) 
     {
         GetCtrlAttribute(panel, winOCH[och], ATTR_VISIBLE, &q);
         if(!q) continue;
         GetCtrlVal(panel, winOCH[och], &q);         
         if(!q) continue;
         
	     GetCtrlIndex (panel, winPCH[och], &ind);    //   _ ind _
	     
         if(X[ind].darray == NULL) continue;
         
         if(X[ind].point < 2) continue; 
	     flag= 1;
	   
         GetCtrlVal(panel, winScale[och], &scale);
         GetCtrlVal(panel, winColor[och], &color);

         // 1.0 / 4 / scale   :  4  
         
         //     
         
         // printf("ind=%d, IS: %d, data= %.5lf\n", ind, indSync, dataPTR[indSync]);
         
         dcline= -utsMath_GetDCLine(X[ind].darray, X[ind].point, dc);
         dcline= dcline / (4 * scale);
         
		 PlotWaveform (panel, P(graf), X[ind].darray + indSync,
					   X[ind].point - indSync, VAL_DOUBLE, 1.0 / ( 4 * scale),
					   dcline + theSHIFT[p][och], 
					   tSync, 1.0/theADC[ind].fs,
					   VAL_THIN_LINE,  VAL_SIMPLE_DOT, VAL_SOLID, 1, color);
         // 
		 PlotWaveform (panel, P(graf), X[ind].darray,
					   indSync + 1, VAL_DOUBLE, 1.0 / ( 4 * scale),  // !!! indSync + 1
					   dcline + theSHIFT[p][och], 
					   tSync - 1.0/theADC[ind].fs * indSync, 1.0/theADC[ind].fs,
					   VAL_THIN_LINE,  VAL_SIMPLE_DOT, VAL_SOLID, 1, color);
					   
     };  // for( och... 
     
     RefreshGraph (panel, P(graf));
     
  __next_panel:
     
     if(flag) 
     {
        GetCtrlVal(panel, P(led), &led);
        SetCtrlVal(panel, P(led), !led);     
     };

 };  // for( panel...
 return 0;
};

/*-----------------------------------------------------------------------------------------------*/
int CVICALLBACK mf_rise(int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int i;  
    
    switch (event)
    {
    case EVENT_COMMIT:
        GetCtrlVal(panel, P(rise), &i);
        SetCtrlAttribute(panel, P(rise), ATTR_LABEL_TEXT, (!i) ? "Rise" : "Fall");
        break;
    }
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

int CVICALLBACK mf_syncchan (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) 
{
	char tmp[512], scale[512];
	int index, och, color;
	double s;

	switch (event) 
	{
		case EVENT_COMMIT:
			GetCtrlIndex(panel, control, &index);
			GetLabelFromIndex (panel, control, index, tmp);
			SetCtrlAttribute(panel, control, ATTR_LABEL_TEXT, tmp);
			
			GetCtrlVal(panel, P(syncchan), &och);
			GetCtrlVal(panel, winColor[och], &color);
			
			SetCtrlAttribute (panel, P(syncchan), ATTR_LABEL_COLOR, color);
			SetCtrlAttribute (panel, P(synclevel), ATTR_LABEL_COLOR, color);
			//SetCtrlAttribute (panel, P(synctime), ATTR_LABEL_COLOR, color);
			//SetCtrlAttribute (panel, P(synctime), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(synclevel), ATTR_TEXT_COLOR, color);
			
			SetCtrlAttribute (panel, control, ATTR_FRAME_COLOR, color);
			
			// -------------------------------------------------------
			GetCtrlIndex(panel, winScale[och], &index);
			GetLabelFromIndex (panel, winScale[och], index, scale);
			sprintf(tmp, "CH%d: %s", och + 1, scale);
			SetCtrlVal(panel, P(scale_sync), tmp);
			SetCtrlAttribute (panel, P(scale_sync), ATTR_TEXT_COLOR, color);
			
			// --------------------------------------------------------------
			GetCtrlVal(panel, winScale[och], &s);
			SetCtrlAttribute (panel, P(synclevel), ATTR_INCR_VALUE, s/10.0);
			SetCtrlAttribute (panel, P(synclevel), ATTR_MAX_VALUE, s*4.0);
			SetCtrlAttribute (panel, P(synclevel), ATTR_MIN_VALUE, -s*4.0);
			
			
			mf_graf (panel, -1, EVENT_COMMIT, NULL, 0, 0);
			break;
	}
	return 0;
}

/*-----------------------------------------------------------------------------------------------*/
int CVICALLBACK mf_dc (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int i;
    
    
    switch (event)
    {
    case EVENT_COMMIT:
        GetCtrlVal(panel, P(dc), &i);
        SetCtrlAttribute(panel, P(dc), ATTR_LABEL_TEXT, (!i) ? "DC" : "AC");
        break;
    }
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int CVICALLBACK mf_run(int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int i;

    
    switch (event)
    {
    case EVENT_COMMIT:
        GetCtrlVal(panel, P(run), &i);
        SetCtrlAttribute(panel, P(run), ATTR_LABEL_TEXT, (!i) ? "Run" : "Stop");

        SetCtrlAttribute(panel, P(warn), ATTR_VISIBLE, !i);
        SetCtrlAttribute(panel, P(fft), ATTR_DIMMED, !i);
        break;
    }
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

int CVICALLBACK mf_shift (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int panelIndex, chan;
	
	switch (event) 
	{
		case EVENT_COMMIT:
		case EVENT_VAL_CHANGED:
			for(panelIndex= 0; panelIndex< MAXIDNUM; panelIndex++) {
			   if(ipnl[panelIndex] == panel) break;
			};
			GetCtrlVal(panel, P(shiftchan), &chan);
			GetCtrlVal(panel, P(shift), &theSHIFT[panelIndex][chan]);
			break;
	}
	return 0;
}

int CVICALLBACK mf_rst (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int j;
	int panelIndex;
	
	switch (event) 
	{
		case EVENT_COMMIT:
			for(panelIndex= 0; panelIndex< MAXIDNUM; panelIndex++) {
			   if(ipnl[panelIndex] == panel) break;
			};		
			for(j= 0; j<OSCHNUM; j++) theSHIFT[panelIndex][j]= 0.0;
			SetCtrlVal(panel, P(shift), 0.0);
			
//			SetActiveCtrl(panel, P(shift));
			break;
	}
	return 0;
}

int CVICALLBACK mf_shiftchannel (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	char tmp[512], scale[512];
	int index, chan, ind;
	int panelIndex, och, color;
	
	switch (event) 
	{
		case EVENT_COMMIT:
			GetCtrlIndex(panel, control, &index);
			GetCtrlVal(panel, control, &chan);
			GetLabelFromIndex (panel, control, index, tmp);
			SetCtrlAttribute(panel, control, ATTR_LABEL_TEXT, tmp);
			
			for(panelIndex= 0; panelIndex< MAXIDNUM; panelIndex++) 
			{
			   if(ipnl[panelIndex] == panel) break;
			};
			
			SetCtrlVal(panel, P(shift), theSHIFT[panelIndex][chan]);

			GetCtrlVal(panel, P(shiftchan), &och);
			GetCtrlVal(panel, winColor[och], &color);
			
			SetCtrlAttribute (panel, P(shiftchan), ATTR_LABEL_COLOR, color);
			SetCtrlAttribute (panel, P(u), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(fabs), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(peak), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(nfftreal), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(t), ATTR_TEXT_COLOR, color);
			
			SetCtrlAttribute (panel, control, ATTR_FRAME_COLOR, color);
			SetCtrlAttribute (panel, P(shift), ATTR_SLIDER_COLOR, color);
			SetCtrlAttribute (panel, P(shift), ATTR_LABEL_COLOR, color);
			SetCtrlAttribute (panel, P(rst_2), ATTR_CMD_BUTTON_COLOR, color);
			
			SetCtrlAttribute (panel, P(snr), ATTR_LABEL_COLOR, color);
			SetCtrlAttribute (panel, P(snr), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(base), ATTR_TEXT_COLOR, color);
			SetCtrlAttribute (panel, P(base), ATTR_LABEL_COLOR, color);

			// -------------------------------------------------------
			GetCtrlIndex(panel, winScale[och], &index);
			GetLabelFromIndex (panel, winScale[och], index, scale);
			sprintf(tmp, "CH%d: %s", och + 1, scale);
			SetCtrlVal(panel, P(scale_shift), tmp);
			SetCtrlAttribute (panel, P(scale_shift), ATTR_TEXT_COLOR, color);

// -----------------------------
	        GetCtrlVal (panel, P(shiftchan),  &och);    //   
            GetCtrlIndex (panel, winPCH[och], &ind);    //   _ ind _	 						
            // !!!!!!!!!!!!!!! ??????? TMP
            
			SetCtrlVal(panel, P(base), (theADC[ind].maxU - theADC[ind].minU) / 2.0 );
			
			// -----------------------------------------------------------
			// SetActiveCtrl(panel, P(shift));
			
			
			break;
	}
	return 0;
}

static void showXY(int panel) 
{
 int    X, Y;
 int    w, h;
 char   tmp[512]= "???";
 double t, u, t0, u0;
 int    och;
 double scale, tscale;
 int    v;
 
 /*int    j, flag= 0;
 for(j= 0; j< MAXIDNUM; j++) if(panel == ipnl[j]) {
   flag= 1;
   break;
 };
 if(!flag) return;   */
 
 if(GetActivePanel()!= panel) return;
 GetCtrlAttribute(panel, P(graf), ATTR_VISIBLE, &v);
 if(!v) return;
 
 GetRelativeMouseState(panel, P(graf), &X, &Y, NULL, NULL, NULL);
 
 GetCtrlAttribute (panel, P(graf), ATTR_PLOT_AREA_HEIGHT, &h);
 GetCtrlAttribute (panel, P(graf), ATTR_PLOT_AREA_WIDTH, &w);
 
 if(X <= w && Y <= h && X>= 0 && Y >= 0) 
 {
   SetMouseCursor (VAL_CROSS_HAIR_CURSOR);

   GetCtrlVal(panel, P(shiftchan), &och);
   GetCtrlVal(panel, winScale[och], &scale);
   GetCtrlVal(panel, P(time), &tscale);
   
   GetCtrlVal(panel, P(synctime), &t0);
   GetCtrlVal(panel, P(synclevel), &u0);   
   
   u= (double)(h/2 - Y) / (h/2);
   u= u* 4.0 * scale;
   t= (double)(X) / w;
   t= t * 10.0 * tscale;
   
   t= t - t0;
   u= u - u0;
   
   sprintf(tmp, "mV: %+.2lf", u*1e3);   
   if(fabs(u) < 1e-3) sprintf(tmp, "mkV: %+.2lf", u*1e6);  
   if(fabs(u) >= (double)1.0) sprintf(tmp, "V: %+.2lf", u);
   SetCtrlVal(panel, P(u), tmp);

   sprintf(tmp, "ms: %.2lf", t*1e3);   
   if(fabs(t) < 1e-3) sprintf(tmp, "mks: %.2lf", t*1e6);
   if(fabs(t) >= (double)1.0) sprintf(tmp, "s: %.2lf", t);
   SetCtrlVal(panel, P(t), tmp);   
   
 }
 else {
   SetMouseCursor (VAL_DEFAULT_CURSOR);
   SetCtrlVal(panel, P(t), "");
   SetCtrlVal(panel, P(u), "");
   SetCtrlVal(panel, P(fabs), "");
 }
};

int CVICALLBACK mf_graf (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	double t, u;
	int och;
	double scale;
	
	switch (event) 
	{
		case EVENT_COMMIT:
		case EVENT_VAL_CHANGED:
		    GetGraphCursor (panel, P(graf), 1, &t, &u);
		    GetCtrlVal(panel, P(syncchan), &och);
		    GetCtrlVal(panel, winScale[och], &scale);
		    
		    u= u * 4 * scale;  // !!! ??? tmp  --->  4 :  
		   
			SetCtrlVal(panel, P(synclevel), u);
			SetCtrlVal(panel, P(synctime), t);
		
			showXY(panel); 
			break;
		   case EVENT_MOUSE_MOVE:
		       showXY(panel);
		   break;
		
			    /*
		   case EVENT_LOST_FOCUS:
		      SetMouseCursor (VAL_DEFAULT_CURSOR);     
		   break;
		   */
		   
		   case EVENT_LEFT_CLICK:
			SetCursorAttribute (panel, P(graf), 1, ATTR_CROSS_HAIR_STYLE,
								VAL_LONG_CROSS);
		break;
		   case EVENT_LEFT_MOUSE_UP:
			SetCursorAttribute (panel, P(graf), 1, ATTR_CROSS_HAIR_STYLE,
								VAL_SHORT_CROSS);
		   break;
	}
	
	return 0;
}

// ---------------------------------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------------------------------
int CVICALLBACK mf_syncparams (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	double u, t;
	int och;
	double scale;
	
	switch (event) 
	{
		case EVENT_COMMIT:
		case EVENT_VAL_CHANGED:
			GetCtrlVal(panel, P(synclevel), &u);
			GetCtrlVal(panel, P(synctime), &t);
		    GetCtrlVal(panel, P(syncchan), &och);
		    GetCtrlVal(panel, winScale[och], &scale);
		    
		    u= u / 4.0 / scale;
		    
			SetGraphCursor (panel, P(graf), 1, t, u);
			break;
	}
	return 0;
}

int CVICALLBACK mf_rstchannel (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int j;
	int panelIndex, ind, och;
	
	switch (event) 
	{
		case EVENT_COMMIT:
			for(panelIndex= 0; panelIndex< MAXIDNUM; panelIndex++) 
			{
			   if(ipnl[panelIndex] == panel) break;
			};		
			
			GetCtrlVal(panel, P(shiftchan), &j);
			
			theSHIFT[panelIndex][j]= 0.0;
			SetCtrlVal(panel, P(shift), 0.0);
			
			SetCtrlVal(panel, P(base), 0.0);
			
	        GetCtrlVal (panel, P(shiftchan),  &och);    //   
            GetCtrlIndex (panel, winPCH[och], &ind);    //   _ ind _	 						
            // !!!!!!!!!!!!!!! ??????? TMP
			SetCtrlVal(panel, P(base), (theADC[ind].maxU - theADC[ind].minU) / 2.0 );
			
			
//			SetActiveCtrl(panel, P(shift));
			break;
	}
	return 0;
}

/*
int  CVICALLBACK mf_panelF(int panel, int event, void *callbackData, int eventData1, int eventData2) {

 if(event == EVENT_LOST_FOCUS) SetMouseCursor (VAL_DEFAULT_CURSOR);     
 

 return 0;
}

  */
  
int CVICALLBACK mf_rstsync (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
int mode;
	switch (event) 
	{
		case EVENT_COMMIT:
		    GetCtrlVal(panel, P(fft), &mode);
		    
		    if(mode) 
		    {
			   SetCtrlVal(panel, P(synclevel), 0.0);
			   SetCtrlVal(panel, P(synctime), 0.0);
			   mf_syncparams (panel, P(synctime), EVENT_COMMIT, NULL, 0, 0);
			}
			else 
			{
			   SetCtrlVal(panel, P(fftf), 0.0);
			   SetCtrlVal(panel, P(fftdb), 0.0);						
			   mf_fftcursor (panel, P(fftf), EVENT_COMMIT, NULL, 0, 0);
			};
			break;
	}
	return 0;
}

// ********************************************************************************************************
// 
// ********************************************************************************************************
//    

int CVICALLBACK FnPanelPrint (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	switch (event) 
	{
		case EVENT_COMMIT:
			SetPrintAttribute (ATTR_PRINTER_NAME, 0);
			SetPrintAttribute (ATTR_ORIENTATION, VAL_PORTRAIT);
			SetPrintAttribute (ATTR_PRINT_AREA_WIDTH, VAL_INTEGRAL_SCALE);
			SetPrintAttribute (ATTR_PRINT_AREA_HEIGHT, VAL_INTEGRAL_SCALE);
			SetPrintAttribute (ATTR_XOFFSET, 0);
			SetPrintAttribute (ATTR_YOFFSET, 0);
			PrintCtrl (panel, P(graf), "test.pdf", VAL_VISIBLE_AREA, 1);
			
			
			PrintTextBuffer ("Test  ", "test.pdf");
			break;
	}
	return 0;
}


// ********************************************************************************************
// 
// ********************************************************************************************
int CVICALLBACK mf_fft (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	int mode;
	
	switch (event) 
	{
		case EVENT_COMMIT:
			GetCtrlVal(panel, P(fft), &mode);
			// mode= 1  // OSC
			SetCtrlAttribute(panel, P(scale_sync), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(synclevel), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(scale_shift), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(time), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(synctime), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(sync), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(rise), ATTR_VISIBLE, mode);
			//SetCtrlAttribute(panel, P(rstsync), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(syncchan), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(dc), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(shift), ATTR_VISIBLE, mode);
			// SetCtrlAttribute(panel, P(rst_2), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(rst), ATTR_VISIBLE, mode);
			SetCtrlAttribute(panel, P(graf), ATTR_VISIBLE, mode);
			//SetCtrlAttribute(panel, P(dec1), ATTR_VISIBLE, mode);
			
			SetCtrlAttribute(panel, P(fftgraf), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(fftdb), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(f), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(fftf), ATTR_VISIBLE, !mode);
		SetCtrlAttribute(panel, P(snr), ATTR_VISIBLE, 0);
			SetCtrlAttribute(panel, P(base), ATTR_VISIBLE, !mode);
			//SetCtrlAttribute(panel, P(slide), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(win), ATTR_VISIBLE, !mode);
			//SetCtrlAttribute(panel, P(rstf), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(fabs), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(fabst), ATTR_VISIBLE, !mode);
			
			SetCtrlAttribute(panel, P(frst), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(df), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(fleft), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(fright), ATTR_VISIBLE, !mode);
			SetCtrlAttribute(panel, P(nfft), ATTR_VISIBLE, !mode);
		SetCtrlAttribute(panel, P(peak), ATTR_VISIBLE, 0);
		SetCtrlAttribute(panel, P(nfftreal), ATTR_VISIBLE, 0);
		
			SetCtrlAttribute(panel, P(Mid_fourier), ATTR_VISIBLE, !mode);		

		    SetCursorAttribute (panel, P(graf), 1, ATTR_CURSOR_COLOR, mode ? VAL_WHITE : VAL_TRANSPARENT);
		    SetCursorAttribute (panel, P(graf), 1, ATTR_CURSOR_ENABLED, mode);
		    SetCursorAttribute (panel, P(fftgraf), 1, ATTR_CURSOR_COLOR, !mode ? VAL_WHITE : VAL_TRANSPARENT);
		    SetCursorAttribute (panel, P(fftgraf), 1, ATTR_CURSOR_ENABLED, !mode);		       
		    
			if(mode) {  // 1 : OSC
			   DeleteGraphPlot (panel, P(graf), -1, VAL_IMMEDIATE_DRAW);
			   mf_sync (panel, P(sync), EVENT_COMMIT, NULL, 0, 0);  // !!! tmp -- .. Sync    ..
			}
			else {
			   SetCtrlVal(panel, P(snr), 0.0);
			   SetCtrlVal(panel, P(peak), "");
			   DeleteGraphPlot (panel, P(fftgraf), -1, VAL_IMMEDIATE_DRAW);
			};
			

            //SetCursorAttribute (panel, P(fftgraf), 2, ATTR_CURSOR_COLOR, !mode ? VAL_WHITE : VAL_TRANSPARENT);
		    //SetCursorAttribute (panel, P(fftgraf), 2, ATTR_CURSOR_ENABLED, !mode);		       		    
			break;
	}
	return 0;
}

int CVICALLBACK mf_f (int panel, int control, int event,
void *callbackData, int eventData1, int eventData2) 
{
	double f;
	char tmp[512];
	int index, n;
	double s, min, max, fscaleprev;
	
	switch (event) 
	{
		case EVENT_COMMIT:
			/*
			GetCtrlVal(panel, P(f), &f);
			SetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, VAL_MANUAL, 0, f * 10); // 10 :    
			mf_fftgraf (panel, -1, EVENT_COMMIT, NULL, 0, 0);
			*/

			GetCtrlIndex(panel, control, &index);
			GetLabelFromIndex (panel, control, index, tmp);
			SetCtrlAttribute(panel, control, ATTR_LABEL_TEXT, tmp);
		
			GetCtrlVal(panel, P(f), &s);
			SetCtrlAttribute (panel, P(fftf), ATTR_INCR_VALUE, s/10.0);
			//SetCtrlAttribute (panel, P(fftf), ATTR_MAX_VALUE, s*10.0);
			
			//mf_slider (panel, P(slide), EVENT_COMMIT, NULL, 0, 0);
			
			GetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, 0, &min, &max);
			fscaleprev= (max - min) / 10;  //   fscale
			n= min / fscaleprev;           //   
			min= s*n;
			max= s*(10 + n);
			SetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, 0, min, max); 
			mf_fftgraf (panel, P(fftgraf), EVENT_VAL_CHANGED, NULL, 0, 0);			
			break;
	}
	return 0;
}

int CVICALLBACK mf_fftcursor (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
double db, f;
	switch (event) {
		case EVENT_COMMIT:
		case EVENT_VAL_CHANGED:
			GetCtrlVal(panel, P(fftdb), &db);
			GetCtrlVal(panel, P(fftf), &f);
			SetGraphCursor (panel, P(fftgraf), 1, f, db);
			break;
	}
	return 0;
}

// ---------------------------------------------------------------------------------------
static void showDB(int panel) 
{
 int    X, Y;
 char   tmp[512]= "???";
 double f, db, f0, db0, fscale, min, max;
 int    v;
 int    t1, l1, t2, l2, h, w; 
 
 if(GetActivePanel()!= panel) return;
 GetCtrlAttribute(panel, P(fftgraf), ATTR_VISIBLE, &v);
 if(!v) return;
 
 GetRelativeMouseState(panel, P(fftgraf), &X, &Y, NULL, NULL, NULL);
 
 GetCtrlAttribute (Pnl, P(fftgraf), ATTR_TOP, &t1);
 GetCtrlAttribute (Pnl, P(fftgraf), ATTR_LEFT, &l1); 
 GetCtrlAttribute (Pnl, P(fftgraf), ATTR_PLOT_AREA_TOP, &t2);
 GetCtrlAttribute (Pnl, P(fftgraf), ATTR_PLOT_AREA_LEFT, &l2); 
 GetCtrlAttribute (panel, P(fftgraf), ATTR_PLOT_AREA_HEIGHT, &h);
 GetCtrlAttribute (panel, P(fftgraf), ATTR_PLOT_AREA_WIDTH, &w);
 
 X= X - l2;
 Y= Y - t2;
  
 if(X <= w && Y <= h && X>= 0 && Y >= 0) {
   SetMouseCursor (VAL_CROSS_HAIR_CURSOR);

   GetCtrlVal(panel, P(f), &fscale);
   
   GetCtrlVal(panel, P(fftf), &f0);
   GetCtrlVal(panel, P(fftdb), &db0);   
   
   GetAxisScalingMode (panel, P(fftgraf), VAL_LEFT_YAXIS, 0, &min, &max);  
   db= (double)Y /h * (min - max) + max;
   
   /*
   f= (double)(X) / w;
   f= f * 10.0 * fscale;	 // 10 :  
   	*/
   GetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, 0, &min, &max);
   f= (double)X /w * (max - min) + min;
   
   sprintf(tmp, "kHz: %.3lf", f/1e3);   
   if(fabs(f) < 1e3) sprintf(tmp, "Hz: %.1lf", f);
   if(fabs(f) >= (double)1e6) sprintf(tmp, "MHz: %.3lf", f/1e6);
   SetCtrlVal(panel, P(fabs), tmp);      
   // -------------------
   
   f= f - f0;
   db= db - db0;
   
   sprintf(tmp, "dB: %+.1lf", db);   
   SetCtrlVal(panel, P(u), tmp);

   sprintf(tmp, "kHz: %+.3lf", f/1e3);   
   if(fabs(f) < 1e3) sprintf(tmp, "Hz: %+.1lf", f);
   if(fabs(f) >= (double)1e6) sprintf(tmp, "MHz: %+.3lf", f/1e6);
   SetCtrlVal(panel, P(t), tmp);   
   SetCtrlVal(panel, P(fabst), "Abs");
 }
 else {
   SetMouseCursor (VAL_DEFAULT_CURSOR);
   SetCtrlVal(panel, P(t), "");
   SetCtrlVal(panel, P(u), "");
   SetCtrlVal(panel, P(fabst), "");
   SetCtrlVal(panel, P(fabs), "");
 }
}; 

// ---------------------------------------------------------------------------------------
static void showDBPresize(int panel, double f, double db) {
 double f0, db0;
 char tmp[512];
 
   GetCtrlVal(panel, P(fftf), &f0);
   GetCtrlVal(panel, P(fftdb), &db0);   
   f= f - f0;
   db= db - db0;
   
   sprintf(tmp, "dB: %+.1lf", db);   
   SetCtrlVal(panel, P(u), tmp);

   sprintf(tmp, "kHz: %+.1lf", f/1e3);   
   if(fabs(f) < 1e3) sprintf(tmp, "Hz: %+.1lf", f);
   if(fabs(f) >= (double)1e6) sprintf(tmp, "MHz: %+.1lf", f/1e6);
   SetCtrlVal(panel, P(t), tmp);      
};

// ---------------------------------------------------------------------------------------
int CVICALLBACK mf_fftgraf (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
double f, db;
//int theCursor;
	switch (event) {
		case EVENT_COMMIT:
		case EVENT_VAL_CHANGED:
	      	//GetActiveGraphCursor (panel, P(fftgraf), &theCursor);
		
		    //if(theCursor == 1) {
		       GetGraphCursor (panel, P(fftgraf), 1, &f, &db);
			   SetCtrlVal(panel, P(fftdb), db);
			   SetCtrlVal(panel, P(fftf), f);
		//	};
		   // if(theCursor == 2) {
		   ///    GetGraphCursor (panel, P(fftgraf), 2, &f, &db);
		  //     showDBPresize(panel, f, db);
		//	};			
			showDB(panel); 
			break;
	    case EVENT_MOUSE_MOVE:
	     	showDB(panel);	  
	    break;
	    case EVENT_LEFT_CLICK:
	     //   GetActiveGraphCursor (panel, P(fftgraf), &theCursor);
	       // if(theCursor!=1) break;
			SetCursorAttribute (panel, P(fftgraf), 1, ATTR_CROSS_HAIR_STYLE,
								VAL_LONG_CROSS);
		break;
	    case EVENT_LEFT_MOUSE_UP:
	        //GetActiveGraphCursor (panel, P(fftgraf), &theCursor);
	        //if(theCursor!=1) break;	    
			SetCursorAttribute (panel, P(fftgraf), 1, ATTR_CROSS_HAIR_STYLE,
								VAL_SHORT_CROSS);
	    break;
	}
	return 0;
}

/*
int CVICALLBACK mf_slider (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
double x, fscale;
	switch (event) {
		case EVENT_COMMIT:
		case EVENT_VAL_CHANGED:
		    GetCtrlVal(panel, P(slide), &x);  // x= 0...1
		    x= (double)(((int)(x / 0.1 + 0.5)) * 0.1);
		    SetCtrlVal(panel, P(slide), x);
		    
            GetCtrlVal(panel, P(f), &fscale);  
            SetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, VAL_MANUAL, fscale * x * 10, fscale * 10 + fscale*x*10);  // 10 :     F
  
			mf_fftgraf (panel, P(fftgraf), EVENT_VAL_CHANGED, NULL, 0, 0);
			break;
	}
	return 0;
}

int CVICALLBACK mf_rstf (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
            SetCtrlVal(panel, P(slide), 0.0);
            mf_slider(panel, P(slide), EVENT_COMMIT, NULL, 0, 0);
			break;
	}
	return 0;
}
			 */
			 
			 

int CVICALLBACK mff_fzero (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
double fscale;
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlAttribute(panel, P(fleft), ATTR_DIMMED, 1);
		    SetCtrlAttribute(panel, P(frst), ATTR_DIMMED, 1);
            GetCtrlVal(panel, P(f), &fscale);  			
			SetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, VAL_MANUAL, 0, fscale * 10);
			mf_fftgraf (panel, P(fftgraf), EVENT_VAL_CHANGED, NULL, 0, 0);					    
			break;
	}
	return 0;
}

int CVICALLBACK mff_fleft (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
double min, max, fscale;
	switch (event) {
		case EVENT_COMMIT:
			GetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, 0, &min, &max);
            GetCtrlVal(panel, P(f), &fscale);  			
            min-= fscale;
            max-= fscale;
		    
		    if(min <= (double)0.0) {
		       SetCtrlAttribute(panel, P(fleft), ATTR_DIMMED, 1);
		       SetCtrlAttribute(panel, P(frst), ATTR_DIMMED, 1);
		    };
		    
			SetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, VAL_MANUAL, min, max);
			mf_fftgraf (panel, P(fftgraf), EVENT_VAL_CHANGED, NULL, 0, 0);			
			break;
	}
	return 0;
}

int CVICALLBACK mff_fright (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2) {
double min, max, fscale;
int j;

    GetCtrlVal(panel, control, &j);
    if(j) event= EVENT_COMMIT;
	switch (event) {
		case EVENT_COMMIT:
		    SetCtrlAttribute(panel, P(fleft), ATTR_DIMMED, 0);
		    SetCtrlAttribute(panel, P(frst), ATTR_DIMMED, 0);
			GetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, 0, &min, &max);
            GetCtrlVal(panel, P(f), &fscale);  			
            min+= fscale;
            max+= fscale;
			SetAxisScalingMode (panel, P(fftgraf), VAL_XAXIS, VAL_MANUAL, min, max);
			mf_fftgraf (panel, P(fftgraf), EVENT_VAL_CHANGED, NULL, 0, 0);			
			break;
	}
	return 0;
}

int CVICALLBACK mid_fourier (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			
			GetCtrlVal(panel,control,&UseMiddleMeanFourier);
			MiddleMeanFourierCount=0;
			break;
		}
	return 0;
}
