#include <ansi_c.h>
/*****************************************************************/
/*                                                               */
/*  PowerFrequencyEstimate Example Program          */
/*  National Instruments                                         */
/*  August, 1995                                                 */
/*                                                               */
/*****************************************************************/
#define HELP_MSG \
"This program serves as an example use of the Spectrum and \n\
PowerFrequencyEstimate functions in the Advanced Analysis library.\n\
The top graph displays the input signal in the time domain and the\n\
bottom graph displays the power spectrum of the filtered input\n\
signal.\n\n\
You can use the mouse to move the vertical red graph cursor\n\
to an area that you wish to know the peak frequency and its power.\n\
These values are displayed in the Output Signal Info section."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <userint.h>
#include <analysis.h>
#include <math.h>
#include "peakest.h"

static int panelHandle;
double  gNoise;

#define MAXPOINTS 65536

static void readWAV(int n, double *X) {
 FILE *f;
 f= fopen("wav.dat", "rb");
 if(f!= NULL) {
	fread (X, sizeof(double), n, f);
 };
 fclose(f);
};

static double SignalArray[MAXPOINTS];
static double NoiseArray[MAXPOINTS];
static double Converted_Spec[MAXPOINTS/2];    
static double Auto_Spec[MAXPOINTS/2];

static int Handle;   

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((Handle = LoadPanel (0, "peakest.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (Handle);
    RunUserInterface ();

    return 0;
}


int utsMath_FinsSINAD(double *s, int n, double,double,double *snr);

int CVICALLBACK TimerFunction(int panel, int control, int event, void *callbackData,
                            int eventData1, int eventData2){
    int     Win, Signal, Scaling, Display, i;    
    double  df, Freq_Peak, Power_Peak,  Freq, Amp; 
    char    Unit[20]="V";        
    WindowConst WinConst;    
    double phase=0.0, w= 0.0, qq;
    double base, ampl, fundf, samplerate, dc, snr; 
    int span, j;
    // #define NGARM 5
        static double *harmonic_amplitudes;
        static double *harmonic_frequencies;
        static double thd, thdnoise;
        int points, garm, tgarm;
        double sfact= 1.0;
        double q, adcrange, adcgap, a;
        int qint, bit;
        
    if (event==EVENT_TIMER_TICK)
    {
        /* Generate Input Signal according the SIGNAL ring */
        GetCtrlVal(panel, PANEL_SIGNAL, &Signal);
        GetCtrlVal(panel, PANEL_base, &base);
        GetCtrlVal(panel, PANEL_ampl, &ampl);
        GetCtrlVal(panel, PANEL_points, &points);
        GetCtrlVal(panel, PANEL_fundf, &fundf);
        
        fundf= 1.0e3;
        
        GetCtrlVal(panel, PANEL_samplerate, &samplerate);
        samplerate= 44.1e3;

SetCtrlVal(panel, PANEL_samplerate, samplerate);
//SetCtrlVal(panel, PANEL_fundf, fundf);

        GetCtrlVal(panel, PANEL_phase, &phase);
        GetCtrlVal(panel, PANEL_dc, &dc);
        GetCtrlVal(panel, PANEL_garm, &garm);
        GetCtrlVal(panel, PANEL_bit, &bit);
        
        adcrange= pow(2.0, bit) / 2.0;
        adcgap= base / adcrange;
        
        tgarm= (int)(samplerate / (fundf*2) + 1.5);
        SetCtrlVal(panel, PANEL_tgarm, tgarm);
        SetCtrlAttribute(panel, PANEL_garm, ATTR_MAX_VALUE, tgarm);
        
        harmonic_amplitudes= (double *)malloc(sizeof(double) * tgarm);
        harmonic_frequencies= (double *)malloc(sizeof(double) * tgarm);
        
        dc= -dc;
        
        GetCtrlVal (panel, PANEL_NOISE_LEVEL, &gNoise);
        GetCtrlVal (panel, PANEL_dc, &dc);
        gNoise/=100.0;
        WhiteNoise (points, ampl*gNoise, -1, NoiseArray);
        
        switch(Signal){
            case 0:
				#define PI 3.1415926535897932384626433832795028841971

				for(j= 0; j< points; j++) {
				    q= ampl * sin(2* PI  * fundf / samplerate * j + phase/180.0 * PI);
				    q+= dc;				    
				    qq= q;
				    
				    q+= NoiseArray[j];
						   
				    a= q / adcgap;
				    if(a >= 0.0) a+= 0.5;
				    else a-= -0.5;
				    a= (double)((int)a);
				    q= a * adcgap;
						
				    SignalArray[j]= q;
				    
				    w+= (q - qq) *(q - qq);
				};
				w= w / points;
			//	w= sqrt(w);
				w= 10.0*log10(ampl * ampl / w );
				SetCtrlVal(panel, PANEL_w, w);	  
				
				readWAV(points, SignalArray) ;
				
				for(j= 0; j< points; j++) SignalArray[j]= SignalArray[j] / 32768.0*10.0;
            break;
            /*
            case 1:
                SquareWave (NB_PTS, ampl, FREQ, &phase, 50.0, SignalArray);
            break;
            case 2:
                TriangleWave (NB_PTS, ampl, FREQ, &phase, SignalArray);
            break;
            */
        }   
        

        
        /* Display Input Signal */
        DeleteGraphPlot (panel, PANEL_GRAPH1, -1, VAL_DELAYED_DRAW);
        SetAxisScalingMode (panel, PANEL_GRAPH1, VAL_XAXIS, VAL_MANUAL, 0.0, 3/fundf);
        
        PlotWaveform (panel, PANEL_GRAPH1, SignalArray, points, VAL_DOUBLE, 1.0, 0.0, 0.0, 1/samplerate,
                                    VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_DK_GREEN);

        /* Apply a Window according the WINDOW Ring              */
        /* The Win is apply to the input signal in Time Space    */
        /* Keep Window parameter for later use                   */
        GetCtrlVal(panel, PANEL_WINDOW, &Win);
        ScaledWindow (SignalArray, points, Win, &WinConst);

        AutoPowerSpectrum (SignalArray, points, 1/samplerate, Auto_Spec, &df);
     //   {
     //   static double qq[1024];
	///	Spectrum (SignalArray, 1024, 1, 1/samplerate, Auto_Spec, qq, &df);
     //    };
         
        {
         double max, min;
         int    imax, imin;
		 MaxMin1D (Auto_Spec, points/2, &max, &imax, &min, &imin);
		 SetCtrlVal(panel, PANEL_qq, max / WinConst.coherentgain);
		 SetCtrlVal(panel, PANEL_qq1, max);
		 if(max > 0.0) max= 10.0 * log10(max / (base*base/2));
		 SetCtrlVal(panel, PANEL_max, max);
		 
		};
       
        
                                            

        /* Get cursor position                              */
        /* Calculate the nearest peak frequency and power   */
        //GetGraphCursor (panel, PANEL_GRAPH2, 1, &Freq, &Amp);
        Freq= -1.0;
        GetCtrlVal(panel, PANEL_span, &span);
        PowerFrequencyEstimate (Auto_Spec, points/2, Freq, WinConst, df, span, &Freq_Peak, &Power_Peak);
        SetCtrlVal(panel, PANEL_POWER_PEAK, sqrt(Power_Peak * 2.0) );
        SetCtrlVal(panel, PANEL_PEAK_FREQ, Freq_Peak);

        /* Get the scale information                                */
        /* Convert the spectrum and display the corresponding units */
        GetCtrlVal(panel, PANEL_SCALING, &Scaling);
        GetCtrlVal(panel, PANEL_DISPLAY, &Display);
        
        SpectrumUnitConversion (Auto_Spec, points/2, 0, Scaling, Display, df, WinConst, Converted_Spec, Unit);
       // for(j=0;j<points/2;j++) Converted_Spec[j]= 10*log10(Auto_Spec[j]);
        SetCtrlVal(panel, PANEL_UNIT,Unit);

        /* Display the spectrum */
        DeleteGraphPlot (panel, PANEL_GRAPH2, -1, VAL_DELAYED_DRAW);
        SetAxisScalingMode (panel, PANEL_GRAPH2, VAL_XAXIS, VAL_MANUAL, 0.0, fundf*garm*2);
		PlotWaveform (panel, PANEL_GRAPH2, Converted_Spec, points/2,
					  VAL_DOUBLE, 1.0, -10.0*log10(base*base/2), 0.0, df,
					  VAL_CONNECTED_POINTS, VAL_SMALL_CROSS, VAL_SOLID, 1,
					  VAL_YELLOW);
					  
		// **************************************************************************************			  
       
         utsMath_FinsSINAD(Auto_Spec, points/2, WinConst.enbw, WinConst.coherentgain, &snr);
         SetCtrlVal(panel, PANEL_snr, snr);
        // **************************************************************************************
				   /*
         if(harmonic_amplitudes!= NULL && harmonic_frequencies!= NULL) {
		  i = HarmonicAnalyzer (Auto_Spec, points/2, 0, garm, Win, samplerate,
								Freq_Peak, harmonic_amplitudes,
								harmonic_frequencies, &thd, &thdnoise);

          SetCtrlVal(panel, PANEL_a1, sqrt(2) * harmonic_amplitudes[0]);
          SetCtrlVal(panel, PANEL_a2, sqrt(2) * harmonic_amplitudes[1]);
          SetCtrlVal(panel, PANEL_a3, sqrt(2) * harmonic_amplitudes[2]);
          SetCtrlVal(panel, PANEL_a4, sqrt(2) * harmonic_amplitudes[3]);
          SetCtrlVal(panel, PANEL_a5, sqrt(2) * harmonic_amplitudes[4]);
          
          SetCtrlVal(panel, PANEL_f1, harmonic_frequencies[0]);
          SetCtrlVal(panel, PANEL_f2, harmonic_frequencies[1]);
          SetCtrlVal(panel, PANEL_f3, harmonic_frequencies[2]);
          SetCtrlVal(panel, PANEL_f4, harmonic_frequencies[3]);
          SetCtrlVal(panel, PANEL_f5, harmonic_frequencies[4]);
          
          SetCtrlVal(panel, PANEL_thd, thd);
          SetCtrlVal(panel, PANEL_thdnoise, thdnoise);	
         }; 		   */
          free(harmonic_amplitudes);
          free(harmonic_frequencies);

        // **************************************************************************************                           
        
    }
    return(0);
}


int CVICALLBACK Quit(int panel, int control, int event, void *callbackData,
                        int eventData1, int eventData2){
    if (event==EVENT_COMMIT)    
        QuitUserInterface(0);
    return(0);
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("PowerFrequencyEstimate Example Program", HELP_MSG);
        break;
    }
    return 0;
}


// --------------------------------------------------------------------------
int utsMath_FinsSINAD(double *s, int n, double w, double g, double *snr) {
 int     i1, i2, i3, span;
 double  signal= 0.0, snd= 0.0; // dc / signal / noise + distortion
 int     j, imax, imin;
 double  prev, max, min, fs;
 
 *snr= 0.0;
 
 
 SetCtrlVal(1, PANEL_ww, w);
 SetCtrlVal(1, PANEL_gg, g);
 
 j= 2;
 prev= s[1];
 while(j < n && prev > s[j]) {
    prev= s[j];
    j++;  
 };
 
 if(j == n) return 1;
 //  DC
 i1= j;
  i1= 0;  // !!! tmp
  
 MaxMin1D (s + i1, n - i1, &max, &imax, &min, &imin);
 
 if(max == min) return 1;

 imax+= i1;
 j= imax - 1;
 if(j<0) j= 0;
 // dc  ,  j>= 0
 if(imax == (n - 1) )  return 1;
 
 prev= max;
 
 while(j >= i1 && prev > s[j]) {
   prev= s[j];
   j--;
 };
 i2= j;
 
 // [i1...i2] - noise
 
 prev= max;
 j= imax + 1;
 while(j < n && prev > s[j]) {
   prev= s[j];
   j++;
 };
 i3= j;
 // [i3 ... n-1] - noise
 
 GetCtrlVal(1, PANEL_span, &span);
					/*
 i1= 4;
 i2= imax - 3 - 1;
 i3= imax + 3 + 1;
 				  */
 i1= span/2;
 i2= imax - span/2 - 1;
 i3= imax + span/2 + 1; 
 
if( (i2 <= i1) || (i3 - i2 - 2 < 1) || (i3 >= n - 1) ) return 1;

 SetCtrlVal(1, PANEL_i1, i1);
 
 for(j= i1; j<= i2; j++) snd+= s[j];
 for(j= i3; j<= n-1; j++) snd+= s[j];
 for(j= i2+1; j<= i3-1; j++) signal+= s[j];
 
 *snr= 10*log10(signal/snd);
 
 // !!! tmp
 SetCtrlVal(1, PANEL_pwr, sqrt(signal/w*2));
 GetCtrlVal(1, PANEL_samplerate, &fs);
 SetCtrlVal(1, PANEL_f, (double)imax/n*fs/2);
 return 0;
};

