#ifndef _UTS_PLUGINDEF__H
#define _UTS_PLUGINDEF__H

#include <windows.h>
#include "uts\\common\\include\\commondef.h"
#include "uts\\systems\\common\\include\\tfuncdef.h"

#pragma pack(1)

typedef struct {
  int   te01;       //    TE-01
  char *te01ext;    //      TE-01
  int   keithley;   //    Keithley
  int   type;
} pluginFlags;

typedef struct {
  char* manual;
  char* dhw;
  char* dscheme;
  char* cables;
  char* te01exthw;    //
  char* te01extscheme;
} pluginHelp;

typedef struct {
  int    type;      // 
  char   comment[COMMENTLEN+1]; // 
  int    nchan;      //       
  int    channel;
  double fs;         //   ,    
  double minU;      //   dB  
  double maxU;
  char   unit[COMMENTLEN+1];
} pluginADC;            //     

typedef struct {
  int    type;
  char   comment[COMMENTLEN+1];   // 
  int    nchan;          //  
  int    channel;
  double fs;             //   -- // --
  double minU;
  double maxU;
  char   unit[COMMENTLEN+1];
} pluginDAC;

typedef struct {
  int    type;           // 0, 1, 2  :  -  -  - ?  ? +  
  char   comment[COMMENTLEN+1];   // 
  int    tnline;         // total n:   + .
  int    line;           // = 0...31, <32...39 - . >
  int    bindmask[40];       //  00000000 11<line>11111 xxx...
} pluginDIO;

typedef int (*plugGetAdcDescr_FPtr)(TCONTEXT *tc, pluginADC *D, int channel);
typedef int (*plugGetDacDescr_FPtr)(TCONTEXT *tc, pluginDAC *D, int channel);
typedef int (*plugGetDIODescr_FPtr)(TCONTEXT *tc, pluginDIO *D, int channel);
typedef int (*plugUF_FPtr)(TCONTEXT *tc, void *);

typedef struct {
  void *coreCBF;
  int   dataMode;
} plugStartStruc;

typedef struct {
  char  *name;        									//  
  char  *version; 										// 
  char  *descr;  										// ,     
  //													//
  char  *mcutype;	   									//     mcu
  char  *mcuimage;	   									//     mcu 
  char  *adspimage;										//      dsp
  pluginFlags    flag;  
  pluginHelp     help;

  plugUF_FPtr             init_module;
  plugUF_FPtr             dread;
  plugUF_FPtr             dwrite;
  plugUF_FPtr             wfile;
  tfptr                   calibr;
  tfptr                   verify;
  plugUF_FPtr             tcfg_pnl;
  plugUF_FPtr             vdev_pnl;
  plugUF_FPtr             devcfg_pnl;
  plugUF_FPtr             te01ext_pnl; 
  plugGetAdcDescr_FPtr    adc_descr;
  plugGetDacDescr_FPtr    dac_descr;
  plugGetDIODescr_FPtr    dio_descr;
  plugUF_FPtr             get_dio;
  plugUF_FPtr             put_dio;
  plugUF_FPtr             flash_read;					  //   
  plugUF_FPtr             flash_write;					  //   
  plugUF_FPtr             fuses_read;					  //  
  plugUF_FPtr             fuses_write;					  //  
  plugUF_FPtr             load_adsp;					  //  adsp
  plugUF_FPtr             module_info;
  plugUF_FPtr             start;
  plugUF_FPtr             stop;
} pluginDefine;

typedef struct {
  pluginDefine *object;
  TCONTEXT     *core;
} vdevContext;

#define PCHNUM(x)         x
#define PFS_HZ(x)         x
#define PDBREF_VOLT(x)    x
#define PCOMMENT(x) x // ( strlen("x") > 255 ? "COMMENT BUFFER IS OVERFLOW" : "x")
#define PNAME(x)    x
#define PVERSION(x) x
#define PDESCR(x)   x

#define PIMAGE_MCU(x)    x
#define PIMAGE_ADSP(x)   x

#define PTE01EXT_NAME(x) x

#define PHELP_MANUAL(x)           x
#define PHELP_DEV_SCHEME(x)       x
#define PHELP_DEV_CABLES(x)       x
#define PHELP_DEV_HW_DESCR(x)     x
#define PHELP_TE01EXT_SCHEME(x)   x
#define PHELP_TE01EXT_HW_DESCR(x) x

#define PCHNUM_IN(x) x
#define PCHNUM_OUT(x) x

//#define NEED_PASSPORT 1
#define HAS_TESTCFG    1
#define HAS_MCU        1
#define HAS_DSP        1
#define HAS_CALIBR     1
#define ENA_FUSE_WRITE 1
#define NEED_TE01      1 
#define NEED_TE01EXT   1
#define NEED_KEITHLEY  1
#define ENA_OSC        1
#define ENA_SLOW       1
#define ENA_SPECTRUM   1
#define ENA_VOLTMETER  1
#define ENA_HISTOGRAM  1
#define ENA_AFR        1
#define ENA_DAC        1
#define ENA_DIO        1
#define HAS_VDEVPNL    1
#define HAS_DEVCFGPNL  1
#define HAS_TE01EXTPNL 1

#define ADC_EXIST 1
#define DAC_EXIST 1
#define DIO_EXIST 1


#define PTYPE_USER  0
#define PTYPE_METR  2
#define PTYPE_STD   1

#define PFPTR_WRITE_DATAFILE(x)  x
#define PFPTR_INIT_MODULE(x)     x
#define PFPTR_CALIBR(x)          x
#define PFPTR_VERIFY(x)          x
#define PFPTR_READ_DATA(x)       x
#define PFPTR_WRITE_DATA(x)      x

#define PFPTR_SHOWPNL_TESTCFG(x)      x
#define PFPTR_SHOWPNL_VIRTDEVICE(x)   x
#define PFPTR_SHOWPNL_DEVCFG(x)       x
#define PFPTR_SHOWPNL_TE01CFG(x)	  x
#define PFPTR_GETDESCR_ADC(x) x
#define PFPTR_GETDESCR_DAC(x) x
#define PFPTR_GETDESCR_DIO(x) x

#define PFPTR_GET_DIO(x) x
#define PFPTR_PUT_DIO(x) x

#define PFPTR_WRITE_MCU(x)       x
#define PFPTR_LOAD_ADSP(x)       x
#define PFPTR_MODULE_INFO(x)     x

#define PFPTR_START(x)     x
#define PFPTR_STOP(x)      x

#define TESTFUNC_PTR(x)  x
#define __TF__TERMINATOR__  NULL
 
#pragma pack()


typedef pluginDefine * __stdcall (*readPluginInfo_FPtr)(void);
typedef int            __stdcall (*readTFuncNum_FPtr)(void);
typedef TFuncDefine *  __stdcall (*readTFuncInfo_FPtr)(int num);

#pragma pack(1)
typedef struct {
  char **path;
  char **filename;
  char **name;
  int   *iduir;
  char **descr;
} pluginList;
//        openPlugin()
typedef struct {
  HINSTANCE            h;			  //
  int                  id;			  // integer ID of the loaded module
  int                  type;		  //
  readPluginInfo_FPtr  plugin;		  // readPluginDefine
  readTFuncNum_FPtr    fnum;		  // readTFuncNum
  readTFuncInfo_FPtr   f;			  //    readTFuncDefine
} pluginManager;					  // 
#pragma pack()

int  createPluginList(pluginList *pl, int *nl, char *root, char *mask);
void discardPluginList(pluginList *pl);

int  openPlugin(pluginManager *p, int index);
void closePlugin(pluginManager *p);

int  findPluginListIndex(int iduir, pluginList pl);

#endif
