#include <windows.h>
#include <utility.h>
#include <ansi_c.h>
#include "uts\\systems\\common\\include\\plugindef.h"
#include "uts\\systems\\common\\include\\tfuncdef.h"
#include "uts\\core.h"
// ----------------------------------------------------------------------------
typedef struct sPluginListItem{
	char *path;
	char *name;
	char *filename;
	/*int   iduir;*/
	char *descr;
	struct sPluginListItem *next;
} pluginListItem;

static pluginListItem* newPluginListItem()
{
	pluginListItem* result;

	if(!(result = (pluginListItem*)malloc(sizeof(pluginListItem)))) return NULL;
	if(!(result->path = (char *)malloc(FNAMELEN+1))) {
		free(result);
		return NULL;
	}
	if(!(result->filename = (char *)malloc(FNAMELEN+1))) {
		free(result->path);
		free(result);
		return NULL;
	}
	if(!(result->name = (char *)malloc(FNAMELEN+1))) {
		free(result->path);
		free(result->filename);
		free(result);
		return NULL;
	}
	if(!(result->descr = (char *)malloc(FNAMELEN+1))) {
		free(result->path);
		free(result->filename);
		free(result->name);
		free(result);
		return NULL;
	}
	result->next = NULL;
	return result;
}

/**
 *    .
 *    ,     .
 * \param root  .
 */
static void freePluginList(pluginListItem* root)
{
	pluginListItem* next;
	while (root) {
		next = root->next;
		free(root);
		root = next;
	}
}


int openPlugin(pluginManager *p, int index) 
{
 char cur_dir[FNAMELEN+1], path[FNAMELEN+1], tmp[FNAMELEN+1];
 int  s;
 char *sptr;
 
 GetCurrentDirectory(sizeof(cur_dir), cur_dir);
 SetCurrentDirectory(core_getPlugPath(index));

 memset(p, 0x0, sizeof(pluginManager));
 p->id= -2;

 if(GetFirstFile ("*tf.lib", 1, 0, 0, 0, 0, 0, tmp)) goto __bad_ret;    
 /*strcpy(path, dir); strcat(path, "\\");  strcat(path, tmp);*/
 sprintf(path, "%s\\%s", core_getPlugPath(index), core_getPlugFileName(index));
 if((p->id=LoadExternalModule(path)) < 0) goto __bad_ret;
 p->fnum= (void *)GetExternalModuleAddr (p->id, "readTFuncNum", &s);
 if(s) goto __bad_ret;
 p->f= (void *)GetExternalModuleAddr (p->id, "readTFuncDefine", &s);
 if(s) goto __bad_ret;
 p->plugin= (void *)GetExternalModuleAddr (p->id, "readPluginDefine", &s);
 if(s) goto __bad_ret; 
 
 // if((s= p->fnum()) == 0) goto __bad_ret; 
 sptr= p->plugin()->name;
 if(sptr == NULL) goto __bad_ret;
 if(p->plugin()->descr == NULL) goto __bad_ret;
 
 if(strlen(sptr) > FNAMELEN) sptr[FNAMELEN]= 0x0;
 if(!strlen(sptr)) goto __bad_ret;
 
 // strcpy(p->pname, sptr);
 //strcpy(p->dir, path);
 
 
 SetCurrentDirectory(cur_dir);
 return 0;
 
__bad_ret:
 SetCurrentDirectory(cur_dir);
 closePlugin(p);
 return 1;
};


void closePlugin(pluginManager *p) 
{
	if(p->id >= 0) UnloadExternalModule(p->id); 
	p->id= -2;
};

int addPlugin(pluginListItem ***tail, char *dir, char *filename)
{
	pluginManager p;
	char *sptr, path[FNAMELEN+1];
	int  s;
	pluginListItem *item;

	strcpy(path, dir); strcat(path, "\\");  strcat(path, filename);
	if((p.id=LoadExternalModule(path)) < 0) goto __bad_ret;
	p.fnum= (void *)GetExternalModuleAddr (p.id, "readTFuncNum", &s);
	if(s) goto __bad_ret;
	p.f= (void *)GetExternalModuleAddr (p.id, "readTFuncDefine", &s);
	if(s) goto __bad_ret;
	p.plugin= (void *)GetExternalModuleAddr (p.id, "readPluginDefine", &s);
	if(s) goto __bad_ret;

	sptr= p.plugin()->name;
	if(sptr == NULL) goto __bad_ret;
	if(p.plugin()->descr == NULL) return 0;

	if(strlen(sptr) > FNAMELEN) sptr[FNAMELEN]= 0x0;
	if(!strlen(sptr)) goto __bad_ret;

	/*  ,   .*/
	if(!(item = newPluginListItem())) return -1;
	strcpy(item->path, dir);
	strcpy(item->filename, filename);
	strcpy(item->name, p.plugin()->name);
	strcpy(item->descr, p.plugin()->descr);
	**tail = item;
	*tail = &(item->next);
	closePlugin(&p);
	return 1;

 __bad_ret:
	closePlugin(&p);
	return 0;
}

/* ----------------------------------------------------------------------------
 *    (dir)   "*tf.lib"
 *    .
 *          
 *  .
 *     (   ).
 ----------------------------------------------------------------------------*/
int addPluginDirectory(pluginListItem ***tail, char *dir)
{
	char cur_dir[FNAMELEN+1], filename[FNAMELEN+1];
	int result, ret_code;

	// SetDllDirectory(path); <- only WinXP & Win2003
	GetCurrentDirectory(sizeof(cur_dir), cur_dir);
	SetCurrentDirectory(dir);

	/*memset(p, 0x0, sizeof(pluginManager));
	p->id= -2;*/

	result = 0;
	if(!GetFirstFile ("*tf.lib", 1, 0, 0, 0, 0, 0, filename)) {
		ret_code = addPlugin(tail, dir, filename);
		if (ret_code>0)
			result += ret_code;
		if (ret_code>=0) {
			while(!GetNextFile (filename)) {
				ret_code = addPlugin(tail, dir, filename);
				if (ret_code<0) break;
				result += ret_code;
			}
		}
	}

	SetCurrentDirectory(cur_dir);
	if (result>=0)
		return result;
	else
		return -1;
};
// ----------------------------------------------------------------------------
void discardPluginList(pluginList *pl) {
	char **ppc= &(pl->path)[0];

	if(pl->path!= NULL) {
		while (*ppc) {
			free (*ppc);
			*ppc = NULL;
			ppc++;
		}; 
		free (pl->path);
		pl->path= NULL;			 
	};

	ppc= &(pl->filename)[0];
	if(pl->filename!= NULL) {
		while (*ppc) {
			free (*ppc);
			*ppc = NULL;
			ppc++;
		}; 
		free (pl->filename);
		pl->filename= NULL;			 
	}; 

	ppc= &(pl->name)[0];
	if(pl->name!= NULL) {
		while (*ppc) {
			free (*ppc);
			*ppc = NULL;
			ppc++;
		}; 
		free (pl->name);
		pl->name= NULL;			 
	}; 

	if(pl->iduir!= NULL) {
		free(pl->iduir);
		pl->iduir= NULL;
	};

	ppc= &(pl->descr)[0];
	if(pl->descr!= NULL) {
		while (*ppc) {
			free (*ppc);
			*ppc = NULL;
			ppc++;
		}; 
		free (pl->descr);
		pl->descr= NULL;			 
	}; 
};
// ----------------------------------------------------------------------------
//   (dirname)     .
//         ().
//  .
// ----------------------------------------------------------------------------
/*static int updatePluginList(char *flist, char *name, char *dirname) 
{  
	pluginManager pMng;
	if(openPlugin(&pMng, dirname)) return 1;
	strcpy(flist, dirname); 
	strcpy(name, pMng.plugin()->name);
	strcpy(dirname, pMng.plugin()->descr);    
	closePlugin(&pMng);
	return 0;
};*/


typedef struct sDirListItem {
	char dirname[FNAMELEN+1];
	struct sDirListItem *next;
} DirListItem;


void FreeDirList(DirListItem *list) {
	DirListItem *next;
	
	while(list) {
		next = list->next;
		free(list);
		list = next;
	}
}

DirListItem* CreateDirList(char *searchpath) {
	DirListItem *result = NULL, *item, **tail;
	char dirname[FNAMELEN+1];
	
	tail = &result;
	if(!GetFirstFile (searchpath, 1, 0, 0, 0, 0, 1, dirname)) {
		if(!(item=(DirListItem*)malloc(sizeof(DirListItem)))) {
			FreeDirList(result);
			return NULL;
		}
		strcpy(item->dirname, dirname);
		item->next = NULL;
		*tail = item;
		tail = &item->next;
		while(!GetNextFile(dirname)) {
			if(!(item=(DirListItem*)malloc(sizeof(DirListItem)))) {
				FreeDirList(result);
				return NULL;
			}
			strcpy(item->dirname, dirname);
			item->next = NULL;
			*tail = item;
			tail = &item->next;
		}
	}
	
	return result;
}

// ----------------------------------------------------------------------------
//   (pl)     
//  root ("systems\< >"). 
//      . 
//       mask ("LTR*.") 
// ----------------------------------------------------------------------------
int createPluginList(pluginList *pl, int *nl, char *root, char *mask) {
	int   n= 0, ret_code, j, k;
	char  dirname[FNAMELEN+1];
	char  searchpath[FNAMELEN+1];
	pluginListItem *pluginListRoot, *item, **tail;
	DirListItem *dir_list, *dir;
	*nl= 0;

	/*             .                */
	pluginListRoot = NULL;/*  -    . */
	tail = &pluginListRoot;/*  -    next  .*/
	n = 0;/*   (     ).*/
	sprintf(searchpath, "%s\\%s", root, mask);/*    .*/
	dir_list = dir = CreateDirList(searchpath);
	/*  . */
	while(dir) {
		sprintf(dirname, "%s\\%s", root, dir->dirname);
		ret_code = addPluginDirectory(&tail, dirname);
		if (ret_code<0) return 1;
		n += ret_code;
		dir = dir->next;
	};
	FreeDirList(dir_list);

	if(!n) return 1;

	/*                   .          */
	/*     . */
	if( (pl->path= (char **)malloc(sizeof(char *) * (n + 1))) == NULL) goto __bad_ret;
	if( (pl->filename= (char **)malloc(sizeof(char *) * (n + 1))) == NULL) goto __bad_ret;
	if( (pl->descr= (char **)malloc(sizeof(char *) * (n + 1))) == NULL) goto __bad_ret;
	if( (pl->name= (char **)malloc(sizeof(char *) * (n + 1))) == NULL) goto __bad_ret;
	if( (pl->iduir= (int *)malloc(sizeof(int) * (n+1))) == NULL) goto __bad_ret;
	/*  . */
	for(j= 0; j< (n+1); j++) {
		(pl->path)[j]= NULL;
		(pl->filename)[j]= NULL;
		(pl->name)[j]= NULL;
		(pl->descr)[j]= NULL;
	};
	memset(pl->iduir, 0x0, sizeof(int) * (n+1));

	/*       ( ). */
	for (item=pluginListRoot,j=0; item; item=item->next,++j) {
		(pl->path)[j] = item->path;
		(pl->filename)[j] = item->filename;
		(pl->name)[j] = item->name;
		(pl->descr)[j] = item->descr;
	}
	/*  . */
	freePluginList(pluginListRoot);
	*nl = n;

	return 0;

 __bad_ret:
	discardPluginList(pl);
	return 1;
};


int findPluginListIndex(int menuItemID, pluginList pl) {
	int ind= 0;

	while(pl.name[ind]!=NULL) {
		if(menuItemID == pl.iduir[ind]) return ind;
		ind++;
	};
	return -1;
};


