#define CRCCFGPATH "c:\\prj\\uts\\systems\\ltr11"

#include <ansi_c.h>
#include <analysis.h>
#include "uts\\systems\\common\\include\\tfuncdef.h"
#include "uts\\systems\\ltr\\common\\include\\ltrspecific.h"
#include "uts\\systems\\common\\include\\utspfunc.h"
#include "uts\\instruments\\te01\\te01api.h"
#include "ltr\\include\\ltr11api.h"
#include "ltr\\include\\ltravrapi.h"
#include "te01func.h"
#include <utility.h>
#include <userint.h>
#include "lib\\crc\\crc.h"
#include "uts\\services\\common\\include\\utsmath.h"
#include "localdefs.h"


/*===============================================================================================*/
#define CONFIG_DATA_SIZE   75
#define DIVIDER            36
//#define FRAME_SIZE         4096
#define LTR11BANDS         4
#define PRESCALER          1
#define RESOLUTION         8192
#define TE01_UADC_RANGEQNT 5              /*     TE-01 */
/*===============================================================================================*/

/*===============================================================================================*/
/*
typedef struct {
  int    type;     
  char   comment[COMMENTLEN+1]; 
  int    nchan;      
  int    channel;
  double fs;         
  double minU;     
  double maxU;
  char   unit[COMMENTLEN+1];
} pluginADC;     
*/
/*===============================================================================================*/

/*===============================================================================================*/
void        closeModule        (void);
static int  getframe           (TCONTEXT *tc, double *frame, BOOL correction, int *frame_size);
static void getframe_avg       (const double *frame_data, int data_size, double *frame_avg);
static int  ReadAVR_PM         (const char *filename, char **buf);
int         testPlolog         (TCONTEXT *tc);
static void VerifyDataRecorder (void *par);
static int  write_coeffs       (TCONTEXT *tc, const char *fname, int need_progress);
static int  writecoeffs        (void);
extern IniSet_LTR11     IniSet;
/*===============================================================================================*/

/*===============================================================================================*/
static const char bin_fname[] = "systems\\LTR\\ltr11\\ltr-11.tsk";
static DWORD buf[4096 * 16];
static const DWORD getframe_tmout = 5000;
TLTR11 hltr11;
/*     LTR-11,   ,    */
static const struct
{
    double gain;          /*    1     1 */
    double offset;        /*       */
} ltr11_max_cbrcoef[LTR11_ADC_RANGEQNT] = {{7.0, 0.2}, {7.0, 0.4}, {7.0, 0.6}, {7.0, 1.2}};
static const int max_adcdata_err = 3;
static int needOpen = 1;
/*    TE-01   */
static const int te01_uadc_ranges[TE01_UADC_RANGEQNT] = {12500, 2500, 620, 150, 39};
/*     */
const double ranges[LTR11_ADC_RANGEQNT] = {10.0, 2.5, 0.625, 0.156};
static int te01adc_band_ltr11[LTR11_ADC_RANGEQNT] = {0, 1, 2, 3};
static const int te01fdac_band_ltr11[LTR11_ADC_RANGEQNT] = {0, 1, 2, 4};
static const int te01sdac_band_ltr11[LTR11_ADC_RANGEQNT] = {0, 1, 1, 2};
/*===============================================================================================*/

/*===============================================================================================*/
/*-----------------------------------------------------------------------------------------------*/
void closeModule(void)
{
    if (!needOpen)
    {
        needOpen = 1;
        LTR11_Close(&hltr11); 
    }
}
/*-----------------------------------------------------------------------------------------------*/
#if 0
typedef struct {
	int size;
	double *buf;
	descr_ltr11 *hltr11;
	BOOL calibr;
} tDataRecStruct;

/*-----------------------------------------------------------------------------------------------*/
static DWORD rd_buf[MAXNPOINTS*LTR11_MAX_LCHANNEL];
static void MetrDataRecorder(void *par)
{
	int stored;
    int      cnt;
    int      r;
    /*int      dval;
    int      j;
    int      n =  numOfVChan();
    int      nline;
    int      p;
    int      q;
    int      stop_record = 0;
    double   t;*/
    tDataRecStruct *descr = par;
    /*char     tmp[512];
    int      tx;*/
    int     to = 1000/*iniset_Timeout()*/;
    int      xp;


	stored = 0;
	while (stored<descr->size) { 
        cnt = LTR_Recv(&(descr->hltr11->Channel), rd_buf, descr->size-stored, to);

        /*for (j = 0; j < n; j++)
        {
            serviceDATA[j].point  = 0;
            serviceDATA[j].iarray = NULL; 
            serviceDATA[j].darray = NULL; 
            serviceDATA[j].fs     = 0;
        }*/

        if (cnt < 0)
           	ExitThread(1);

        xp = descr->size-stored;
        r  = LTR11_ProcessData(descr->hltr11, rd_buf, descr->buf+stored, &xp, descr->calibr, TRUE);
        stored += xp;

        if (r != LTR_OK)
        {
           	ExitThread(1);
        }

        Sleep(0);
    }     /*while (runFlag)*/


    ExitThread(0);
}
/*-----------------------------------------------------------------------------------------------*/

/*void async_gf(int nchan, serviceStrucData X[]) {
}*/


/*-----------------------------------------------------------------------------------------------*/
int async_getframe(TCONTEXT *tc, double *frame, BOOL correction, int *frame_size) {
	HANDLE hThread;
	DWORD TmpDWord;
    tDataRecStruct descr;
    /*double buf[1024];*/
    int ret_code, j;
    char   msg[512];
    
    descr.size = *frame_size;
    descr.buf = frame;
    descr.hltr11 = &hltr11;
    descr.calibr = correction;

	hThread= CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)MetrDataRecorder,
												(void*)&descr, CREATE_SUSPENDED, (LPDWORD)&TmpDWord);

    for (j = 0; j < 10; j++)
    {
        ret_code = LTR11_SetADC(&hltr11);
        if (ret_code == LTR_OK)
            break;
        sprintf(msg, "LTR11_SetADC  %d:%5d", j, ret_code);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
    }
    if (ret_code != LTR_OK)
        return ret_code;

	if (ret_code=LTR11_Start(&hltr11)) {
		closeModule();
		return ret_code;
	};

	/*    . */
	ResumeThread(hThread);        // start thread here

	/*     . */
	WaitForMultipleObjects(1, &hThread, TRUE, INFINITE);
	/*    . */
	CloseHandle(hThread);
	/*  . */
	if (ret_code=LTR11_Stop(&hltr11)) {
		closeModule();
	}

	return ret_code;
}
#endif

/*-----------------------------------------------------------------------------------------------*/
static int getframe(TCONTEXT *tc, double *frame, BOOL correction, int *frame_size)
{
    int    err_code;
    int    j;
    char   msg[512];


    for (j = 0; j < 10; j++)
    {
        err_code = LTR11_SetADC(&hltr11);
        if (err_code == LTR_OK)
            break;
        sprintf(msg, "LTR11_SetADC  %d:%5d", j, err_code);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
    }
    if (err_code != LTR_OK)
        return err_code;

    for (j = 0; j < 10; j++)
    {
        err_code = LTR11_Start(&hltr11);
        if (err_code == LTR_OK)
            break;
        sprintf(msg, "LTR11_Start  %d:%5d", j, err_code);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
	}
    if (err_code != LTR_OK)
        return err_code;
    
        
    *frame_size = LTR_Recv(&hltr11.Channel, buf, NULL, *frame_size, getframe_tmout);

    err_code = LTR11_Stop(&hltr11);
    if (err_code != LTR_OK)
    {
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, "   ");
    }
    
    err_code = LTR11_ProcessData(&hltr11, buf, frame, frame_size, correction, TRUE);
	
    return err_code;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
static void getframe_avg(const double *frame_data, int data_size, double *frame_avg)
{
    /*
     *         .
     * 
     *        .
     *         LTR-11.
     *   
     *  
     *   frame_data -        LTR-11,    
     *                  ,   
     *                (: 1212   );
     *   data_size  -   frame_data (    LTR-11);
     *   frame_avg  -   ,      , 
     *                   .
     *  
     *   hltr11 -   LTR-11,       
     *             .
     */

    int    ch;
    double n = data_size / hltr11.LChQnt;
    double *pavg = frame_avg;
    
    
    for (ch = 0; ch < hltr11.LChQnt; ch++)
        frame_avg[ch] = 0.0;
        
    while (data_size-- > 0)
    {
        *pavg += *frame_data / n;
        if (++pavg >= frame_avg + hltr11.LChQnt)
            pavg = frame_avg;
        frame_data++;
    }
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
static int ReadAVR_PM(const char *filename, char **buf)
{
    FILE *f;
    int  size;
	
	
    if ((f = fopen(filename, "r")) == NULL)
    {
        return -1;
    }
	
    /*  :*/
    fseek(f, 0, SEEK_END);              /*   .*/
    size = ftell(f);                    /*  .*/
    fseek(f, 0, SEEK_SET);              /*   .*/
	
    /*   .*/
    *buf = (char *)malloc(size);
    /* .*/
	if (fread(*buf, 1, size, f) != size) /*    .*/
	{
        /*    .*/
        free(*buf);
        fclose(f);
        /* .*/
        return -2;
    }
	
    fclose(f);
    return size;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testExternalAcquisionStart(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (128)

    
    DWORD frame_adc[ADC_FRAME_SIZE];
    int   frame_size_adc;
    int   ltr11_err_code;
    char  msg[100] = "";
    TE01  *pte01;
    int   ret_val = 0;
    int   test_qnt;
    const struct
    {
        struct
        {
            unsigned prescaler;
            unsigned divider;
        } adc_rate;
        int start_mode;
    }     *ptest, test_tbl[] =
        {{{1, 36}, LTR11_STARTADCMODE_EXTRISE}, {{1, 74}, LTR11_STARTADCMODE_EXTFALL}};


    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(msg, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetExt_Inputs(pte01, 0);
    
    hltr11.InpMode   = LTR11_INPMODE_INT;
    hltr11.LChQnt    = 1;
    hltr11.ADCMode   = LTR11_ADCMODE_ACQ;
    hltr11.LChTbl[0] = 0;
    
    test_qnt = sizeof test_tbl / sizeof test_tbl[0];
    for (ptest = test_tbl; ptest < test_tbl + test_qnt; ptest++)
    {
        double adc_rate;
        int    ext_signal;
        
        
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_PROGRESS(100.0 * (ptest - test_tbl + 1) / test_qnt), NULL))
            {
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        
        
        if (   ptest->adc_rate.prescaler == 1 && ptest->adc_rate.divider < 37
            || ptest->adc_rate.prescaler == 2 && ptest->adc_rate.divider < 4
           )
        {
            adc_rate = 400.0;
        }
        else
        {
            adc_rate = LTR11_CLOCK /
                (ptest->adc_rate.prescaler * (ptest->adc_rate.divider + 1));
        }
        if (ptest->start_mode == LTR11_STARTADCMODE_EXTFALL)
        {
            sprintf(msg, "  ,  %.2f", adc_rate);
        }
        else
        {
            sprintf(msg, "  ,  %.2f", adc_rate);
        }
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
        
        hltr11.ADCRate.prescaler = ptest->adc_rate.prescaler;
        hltr11.ADCRate.divider   = ptest->adc_rate.divider;
        hltr11.StartADCMode      = ptest->start_mode;
    
        ext_signal = (ptest->start_mode == LTR11_STARTADCMODE_EXTFALL) ? EXT_START : 0;
        SetExt_Inputs(pte01, ext_signal);
    
        LTR11_Stop(&hltr11);
        ltr11_err_code = LTR11_SetADC(&hltr11);
        if (ltr11_err_code != LTR_OK)
        {
            sprintf(msg, "LTR11_SetADC  %5d", ltr11_err_code);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
        ltr11_err_code = LTR11_Start(&hltr11);
        if (ltr11_err_code != LTR_OK)
        {
            sprintf(msg, "LTR11_Start  %5d", ltr11_err_code);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
    	}

        ext_signal ^= EXT_START;
        SetExt_Inputs(pte01, ext_signal);
        frame_size_adc = LTR_Recv(&hltr11.Channel, frame_adc, NULL, ADC_FRAME_SIZE, getframe_tmout);
        LTR11_Stop(&hltr11);
    
        if (frame_size_adc != ADC_FRAME_SIZE)
        {
            sprintf(msg, "!       ");
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
    }
    
end_test:
    return ret_val;
    
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testExternalADCStart(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (16)

    
    DWORD frame_adc[ADC_FRAME_SIZE];
    int   frame_size_adc;
    int   ltr11_err_code;
    char  msg[100] = "";
    TE01  *pte01;
    int   ret_val = 0;
    int   test_qnt;
    const struct
    {
        struct
        {
            unsigned prescaler;
            unsigned divider;
        } adc_rate;
        int sync_mode;
    }     *ptest, test_tbl[] =
        {{{1, 36}, LTR11_INPMODE_EXTRISE}, {{1, 74}, LTR11_INPMODE_EXTFALL}};


    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(msg, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetExt_Inputs(pte01, 0);
    
    hltr11.StartADCMode = LTR11_STARTADCMODE_INT;
    hltr11.LChQnt       = 1;
    hltr11.ADCMode      = LTR11_ADCMODE_ACQ;
    hltr11.LChTbl[0]    = 0;
    
    test_qnt = sizeof test_tbl / sizeof test_tbl[0];
    for (ptest = test_tbl; ptest < test_tbl + test_qnt; ptest++)
    {
        double      adc_rate;
        int         ext_signal;
        const DWORD frame_timeout = 2000;
        const int   wait_frame_size = 1;
        
        
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_PROGRESS(100.0 * (ptest - test_tbl + 1) / test_qnt), NULL))
            {
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        
        
        if (   ptest->adc_rate.prescaler == 1 && ptest->adc_rate.divider < 37
            || ptest->adc_rate.prescaler == 2 && ptest->adc_rate.divider < 4
           )
        {
            adc_rate = 400.0;
        }
        else
        {
            adc_rate = LTR11_CLOCK /
                (ptest->adc_rate.prescaler * (ptest->adc_rate.divider + 1));
        }
        if (ptest->sync_mode == LTR11_INPMODE_EXTFALL)
        {
            sprintf(msg, "  ,  %.2f", adc_rate);
        }
        else
        {
            sprintf(msg, "  ,  %.2f", adc_rate);
        }
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
        
        hltr11.ADCRate.prescaler = ptest->adc_rate.prescaler;
        hltr11.ADCRate.divider   = ptest->adc_rate.divider;
        hltr11.InpMode           = ptest->sync_mode;
    
        ext_signal = (ptest->sync_mode == LTR11_INPMODE_EXTFALL) ? EXT_SYNC : 0;
        SetExt_Inputs(pte01, ext_signal);
    
        LTR11_Stop(&hltr11);
        ltr11_err_code = LTR11_SetADC(&hltr11);
        if (ltr11_err_code != LTR_OK)
        {
            sprintf(msg, "LTR11_SetADC  %5d", ltr11_err_code);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
        ltr11_err_code = LTR11_Start(&hltr11);
        if (ltr11_err_code != LTR_OK)
        {
            sprintf(msg, "LTR11_Start  %5d", ltr11_err_code);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
    	}

        
        ext_signal ^= EXT_SYNC;
        SetExt_Inputs(pte01, ext_signal);
        frame_size_adc = LTR_Recv(&hltr11.Channel, frame_adc, NULL, ADC_FRAME_SIZE, frame_timeout);
        LTR11_Stop(&hltr11);
    
        if (frame_size_adc != wait_frame_size)
        {
            sprintf(msg, "!    -");
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
        
        ext_signal ^= EXT_SYNC;
        SetExt_Inputs(pte01, ext_signal);
        ltr11_err_code = LTR11_Start(&hltr11);
        if (ltr11_err_code != LTR_OK)
        {
            sprintf(msg, "LTR11_Start  %5d", ltr11_err_code);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
    	}

        
        ext_signal ^= EXT_SYNC;
        SetExt_Inputs(pte01, ext_signal);
        frame_size_adc = LTR_Recv(&hltr11.Channel, frame_adc, NULL, ADC_FRAME_SIZE, frame_timeout);
        LTR11_Stop(&hltr11);
    
        if (frame_size_adc != wait_frame_size)
        {
            sprintf(msg, "!    -");
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
    }
    
end_test:
    return ret_val;
    
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testInphaseRejection(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (4096)
#define TEST_FREQ_QNT  (1)

    double         adcrate;
    int            band;
    const double   band_amp[LTR11_ADC_RANGEQNT] = {0.9, 0.9, 0.9, 0.8};
    int            ch_num;
    int            cnt_adcdata_err = 0;
    int            err_code;
    double         fft[ADC_FRAME_SIZE];
    double         frame[ADC_FRAME_SIZE];
    int            frame_size;
    int            fsin;
    int            ipeak;
    double         ltr11_max_snr[LTR11_ADC_RANGEQNT][TEST_FREQ_QNT] =
    {
        {-66.0}, {-66.0}, {-66.0}, {-66.0}
    };
    int            max_prcnt;
    int            prcnt;
    TE01           *pte01;
    int            ret_val = 0;
    char           s[100];
    const unsigned test_chnum[] = {0, 4, 8, 12};
    const struct
    {
        int freq;
        int filt;
        struct
        {
            int prescaler;
            int divider;
        }   adc_rate;
    }              test_sin[TEST_FREQ_QNT] = {{400, te01fdac_filt5kHz, {1, 749}}};
    
    
    
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetAModule_Config(pte01, UFPTOX_UFPTOY_CONNECT, U0TOX_U0TOY_CONNECT, UF0TOGND32_CONNECT,
        ZEROTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);

    
    /*    LTR-11 */
    hltr11.InpMode      = LTR11_INPMODE_INT;
    hltr11.LChQnt       = 1;
    hltr11.ADCMode      = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode = LTR11_STARTADCMODE_INT;
    
    max_prcnt = LTR11_ADC_RANGEQNT * TEST_FREQ_QNT * (sizeof test_chnum / sizeof test_chnum[0])
        - 1;
    prcnt     = 0;

    for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
    {
        double freq;
        double f_sin;
        double peak_pwr;
        double pwr;
        double snr;
        
        sprintf(s, " +-%.3f ", ranges[band]);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
            
        for (fsin = 0; fsin < TEST_FREQ_QNT; fsin++)
        {
            /*       LTR-11  */
            f_sin = test_sin[fsin].freq;
            SetFDAC_Sinus(pte01, &f_sin, ranges[band] * band_amp[band] * 1000.0,
                test_sin[fsin].filt, te01fdac_band_ltr11[band]);
            hltr11.ADCRate.prescaler = test_sin[fsin].adc_rate.prescaler;
            hltr11.ADCRate.divider   = test_sin[fsin].adc_rate.divider;
            if (   test_sin[fsin].adc_rate.prescaler == 1 && test_sin[fsin].adc_rate.divider < 37
                || test_sin[fsin].adc_rate.prescaler == 8 && test_sin[fsin].adc_rate.divider < 4
               )
            {
                adcrate = 400000.0;
            }
            else
            {
                adcrate = LTR11_CLOCK * 1000.0 /
                    (test_sin[fsin].adc_rate.prescaler * (test_sin[fsin].adc_rate.divider + 1));
            }
            
            /*     */
            pwr  = 0;
            freq = f_sin;
            SineWave(ADC_FRAME_SIZE, band_amp[band] * ranges[band], freq / adcrate, &pwr, frame);
            
            frame_size = ADC_FRAME_SIZE;
            ipeak      = freq * ADC_FRAME_SIZE / adcrate + 0.5;    /* ipeak = freq / df */
            utsMath_DoFFT(frame, ADC_FRAME_SIZE, 1, 7, adcrate, &frame_size, fft, &freq,
                &pwr, &snr, NULL,NULL,NULL,NULL);
            ipeak      = freq * frame_size * 2 / adcrate + 0.5;    /* ipeak = freq / df */
            peak_pwr = fft[ipeak];
                
            sprintf(s, ":  = %.0f ;  = %.0f ", freq, adcrate);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, s);
            
            for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)
            {
                ProcessSystemEvents();
                if (tc->cbf != NULL)
                {
                    if (tc->cbf(TFMSG_PROGRESS(100.0 * prcnt / max_prcnt), NULL))
                    {
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }                    
            
                /*     */
                hltr11.LChTbl[0] = (band << 6) | (0 << 4) | test_chnum[ch_num];
                frame_size       = ADC_FRAME_SIZE;
                err_code         = getframe(tc, frame, TRUE, &frame_size);
                
                if (err_code != LTR_OK)
                {
                    if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                        || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                        || err_code == LTR11_ERR_ADCDATA_CNT
                       )
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "!!!     ");
                        if (++cnt_adcdata_err >= max_adcdata_err)
                        {
                            if (tc->cbf != NULL)
                                tc->cbf(TFMSG_REPORT, "      ");
                            ret_val = TRES_ABORT;
                            goto end_test;
                        }
                    }
                    else
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, ".    LTR-11");
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }
                freq  = f_sin;
                ipeak = freq * frame_size / adcrate + 0.5;    /* ipeak = freq / df */
                
                utsMath_DoFFT(frame, frame_size, 1, 7, adcrate, &frame_size, fft, &freq,
                    &pwr, &snr, NULL,NULL,NULL,NULL);
                pwr = fft[ipeak];
                
                snr = pwr - peak_pwr;
                sprintf(s, " %u: c/ = %.1f ", test_chnum[ch_num] + 1, snr);
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, s);
                    
                if (snr > ltr11_max_snr[band][fsin])
                {
                    sprintf(s, "!  /   (%.0f)",
                        ltr11_max_snr[band][fsin]);
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, s);
                    ret_val = TRES_T_FAIL;
                    goto end_test;
                }

                prcnt++;
            } /*for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)*/
        }     /*for (fsin = 0; fsin < TEST_FREQ_QNT; fsin++)*/
    }         /*for (band = 0; band < LTR11_ADC_RANGEQNT; band++)*/


end_test:
    return ret_val;
    
#undef TEST_FREQ_QNT
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testInterchannel(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (4096)
#define TEST_FREQ_QNT  (1)

    double         adcrate;
    int            band;
    const double   band_amp[LTR11_ADC_RANGEQNT] = {0.9, 0.9, 0.9, 0.8};
    int            ch_num;
    int            cnt_adcdata_err = 0;
    int            err_code;
    double         fft[ADC_FRAME_SIZE];
    double         frame[ADC_FRAME_SIZE];
    int            frame_size;
    int            fsin;
    int            ipeak;
    double         ltr11_max_snr[LTR11_ADC_RANGEQNT][TEST_FREQ_QNT] =
    {
        {-70.0}, {-70.0}, {-70.0}, {-70.0}
    };
    int            max_prcnt;
    int            prcnt;
    TE01           *pte01;
    int            ret_val = 0;
    char           s[100];
    const unsigned test_chnum[] = {0, 4, 8, 12};
    const struct
    {
        int freq;
        int filt;
        struct
        {
            int prescaler;
            int divider;
        }   adc_rate;
    }              test_sin[TEST_FREQ_QNT] = {{5000, te01fdac_filt20kHz, {1, 149}}};
    
    
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetAModule_Config(pte01, UFPTOX_UFNTOY_CONNECT, U0TOX_U0TOY_CONNECT, UF0TOGND32_CONNECT,
        ZEROTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);

    
    /*    LTR-11 */
    hltr11.InpMode      = LTR11_INPMODE_INT;
    hltr11.LChQnt       = 1;
    hltr11.ADCMode      = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode = LTR11_STARTADCMODE_INT;
    
    max_prcnt = LTR11_ADC_RANGEQNT * TEST_FREQ_QNT * (sizeof test_chnum / sizeof test_chnum[0])
        - 1;
    prcnt     = 0;

    for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
    {
        double f_sin;
        double freq;
        double peak_pwr;
        double pwr;
        double snr;
        
        sprintf(s, " +-%.3f ", ranges[band]);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
            
        for (fsin = 0; fsin < TEST_FREQ_QNT; fsin++)
        {
            /*       LTR-11  */
            f_sin = test_sin[fsin].freq;
            SetFDAC_Sinus(pte01, &f_sin, ranges[band] * band_amp[band] * 1000.0,
                test_sin[fsin].filt, te01fdac_band_ltr11[band]);
            hltr11.ADCRate.prescaler = test_sin[fsin].adc_rate.prescaler;
            hltr11.ADCRate.divider   = test_sin[fsin].adc_rate.divider;
            adcrate                  = LTR11_CLOCK * 1000.0 /
                (test_sin[fsin].adc_rate.prescaler * (test_sin[fsin].adc_rate.divider + 1));
                
            sprintf(s, ":  = %.0f ;  = %.0f ", f_sin, adcrate);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, s);
            
            for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)
            {
                ProcessSystemEvents();
                if (tc->cbf != NULL)
                {
                    if (tc->cbf(TFMSG_PROGRESS(100.0 * prcnt / max_prcnt), NULL))
                    {
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }                    
            
                /*      */
                hltr11.LChTbl[0] = (band << 6) | (0 << 4) | test_chnum[ch_num];
                frame_size       = ADC_FRAME_SIZE;
                err_code         = getframe(tc, frame, TRUE, &frame_size);
                if (err_code != LTR_OK)
                {
                    if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                        || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                        || err_code == LTR11_ERR_ADCDATA_CNT
                       )
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "!!!     ");
                        if (++cnt_adcdata_err >= max_adcdata_err)
                        {
                            if (tc->cbf != NULL)
                                tc->cbf(TFMSG_REPORT, "      ");
                            ret_val = TRES_ABORT;
                            goto end_test;
                        }
                    }
                    else
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, ".    LTR-11");
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }
                freq  = f_sin;
                ipeak = freq * frame_size / adcrate + 0.5;    /* ipeak = freq / df */
                utsMath_DoFFT(frame, frame_size, 1, 7, adcrate, &frame_size, fft, &freq,
                    &pwr, &snr, NULL,NULL,NULL,NULL);
                peak_pwr = fft[ipeak];
                
                hltr11.LChTbl[0] = (band << 6) | (0 << 4) | (test_chnum[ch_num] + 1);
                frame_size       = ADC_FRAME_SIZE;
                err_code         = getframe(tc, frame, TRUE, &frame_size);
                if (err_code != LTR_OK)
                {
                    if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                        || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                        || err_code == LTR11_ERR_ADCDATA_CNT
                       )
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "!!!     ");
                        if (++cnt_adcdata_err >= max_adcdata_err)
                        {
                            if (tc->cbf != NULL)
                                tc->cbf(TFMSG_REPORT, "      ");
                            ret_val = TRES_ABORT;
                            goto end_test;
                        }
                    }
                    else
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, ".    LTR-11");
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }
                freq  = f_sin;
                ipeak = freq * frame_size / adcrate + 0.5;    /* ipeak = freq / df */
                utsMath_DoFFT(frame, frame_size, 1, 7, adcrate, &frame_size, fft, &freq,
                    &pwr, &snr, NULL,NULL,NULL,NULL);
                pwr   = fft[ipeak];
                
                snr = pwr - peak_pwr;
                sprintf(s, " %u: c/ = %.1f ", test_chnum[ch_num] + 1, snr);
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, s);
                    
                if (snr > ltr11_max_snr[band][fsin])
                {
                    sprintf(s, "!  /   (%.0f )",
                        ltr11_max_snr[band][fsin]);
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, s);
                    ret_val = TRES_T_FAIL;
                    goto end_test;
                }

                prcnt++;
            } /*for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)*/
        }     /*for (fsin = 0; fsin < TEST_FREQ_QNT; fsin++)*/
    }         /*for (band = 0; band < LTR11_ADC_RANGEQNT; band++)*/


end_test:
    return ret_val;
    
#undef TEST_FREQ_QNT
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testIntrinsicNoise(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (4096)

    int            band;
    int            ch_num;
    int            cnt_adcdata_err = 0;
    int            err_code;
    double         frame[ADC_FRAME_SIZE];
    int            frame_size;
    const double   ltr11_min_snr[LTR11_ADC_RANGEQNT] = {75.0, 75.0, 70.0, 70.0};
    TE01           *pte01;
    int            ret_val = 0;
    char           s[100];
    double         signal[ADC_FRAME_SIZE];
    double         signal_pattern[ADC_FRAME_SIZE];
    const unsigned test_chnum[] = {0, 4, 8, 12};
    
    
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetAModule_Config(pte01, U0TOX_U0TOY_CONNECT, U0TOX_U0TOY_CONNECT, UF0TOGND32_CONNECT,
        SIGNALTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);
    
    SetSDAC_Value(pte01, 0.0, 3);
    
    SetFDAC_DCValue(pte01, 0.0, 8);
    
    /*    LTR-11 */
    hltr11.InpMode           = LTR11_INPMODE_INT;
    hltr11.ADCRate.divider   = DIVIDER;
    hltr11.ADCRate.prescaler = PRESCALER;
    hltr11.LChQnt            = 1;
    hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;

    for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
    {
        double freq;
        double pwr;
        double snr;
        
        sprintf(s, " +-%.3f ", ranges[band]);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        
        /*    */
        SinePattern(ADC_FRAME_SIZE, 0.9 * ranges[band], 0.0, 100.0, signal_pattern);
        
        for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)
        {
            ProcessSystemEvents();
            
            if (tc->cbf != NULL)
            {
                double h;
                
                h  = 100.0 * (band * sizeof test_chnum / sizeof test_chnum[0] + ch_num);
                h /= (LTR11_ADC_RANGEQNT * sizeof test_chnum / sizeof test_chnum[0] - 1);
                if (tc->cbf(TFMSG_PROGRESS(h), NULL))
                {
                    ret_val = TRES_ABORT;
                    goto end_test;
                }
            }
            
            /*       LTR-11 */
            hltr11.LChTbl[0] = (band << 6) | (0 << 4) | test_chnum[ch_num];
            frame_size       = ADC_FRAME_SIZE;
            err_code         = getframe(tc, frame, TRUE, &frame_size);
            if (err_code != LTR_OK)
            {
                if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                    || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                    || err_code == LTR11_ERR_ADCDATA_CNT
                   )
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "!!!     ");
                    if (++cnt_adcdata_err >= max_adcdata_err)
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "      ");
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }
                else
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, ".    LTR-11");
                    ret_val = TRES_ABORT;
                    goto end_test;
                }
            }
                
            /*      */
            Add1D(frame, signal_pattern, frame_size, signal);
            
            freq = -1.0;
            utsMath_DoFFT(signal, frame_size, 1, 7, 400000, &frame_size, frame, &freq, &pwr, &snr, NULL,NULL,NULL,NULL);
            
            sprintf(s, " %u: c/ = %.1f ", test_chnum[ch_num] + 1, snr);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, s);
                
            if (snr < ltr11_min_snr[band])
            {
                sprintf(s, "!!!  /   (%.0f)",
                    ltr11_min_snr[band]);
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, s);
                    
                ret_val = TRES_T_FAIL;
                goto end_test;
            }
        } /*for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)*/
    }     /*for (band = 0; band < LTR11_ADC_RANGEQNT; band++)*/


end_test:
    return ret_val;
    
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testLTRR11TestMode(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (2048)
#define TEST_MODE_QNT  (4)

    int            cnt_adcdata_err = 0;
    double         e;
    int            err_code;
    double         frame[ADC_FRAME_SIZE];
    double         frame_avg[LTR11_MAX_CHANNEL];
    int            frame_size;
    /*     LTR-11,  % */
    const double   ltr11_max_err = 20;
    double         ltr11adc_val;
    const struct
    {
        int    mode;
        char   title[30];
        double u_rated;
    }              ltr11_test_mode[TEST_MODE_QNT] =
    {
        {LTR11_ADCMODE_TEST_U1P, "+U1", 95.6E-3}, {LTR11_ADCMODE_TEST_U1N, "-U1", -95.6E-3},
        {LTR11_ADCMODE_TEST_U2N, "-U2", -1.06}, {LTR11_ADCMODE_TEST_U2P, "+U2", 1.06}
    };
    unsigned       nch;
    int            nmode;
    TE01           *pte01;
    int            ret_val = 0;
    char           s[100];
    const double   te01_max_sdac_err = 0.2;       /*   */
    const int      te01sdac_band = 3;
    const int      te01uadc_band = 4;
    const unsigned test_band = 1;
    double         uadc_val;
    

    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
 
        
    pte01 = (TE01 *)tc->te01;

    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "Pluggable module not found\n");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetAModule_Config(pte01, USPTOX_US0TOY_CONNECT, USPTOX_US0TOY_CONNECT, US0TOGND32_CONNECT,
        SIGNALTOVOLTMETER_CONNECT);

    TE01ReadCoeffFromEEPROM(pte01);

    /*  ( )   */
    SetUADC_Config(pte01, te01uadc_band);
    SetSDAC_Value(pte01, 0, te01sdac_band);
    uadc_val = GetUADC_Val(pte01);
    if (fabs(uadc_val) > te01_max_sdac_err)
    {
        sprintf(s, "     TE-01 = %+.2f )", uadc_val);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    sprintf(s, " %.2f ", uadc_val);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_REPORT, s);
    uadc_val /= 1000.0;                /*    */

    
    for (nmode = 0; nmode < TEST_MODE_QNT; nmode++)
    {
        /*     */
        
        /*    LTR-11 */
        hltr11.InpMode           = LTR11_INPMODE_INT;
        hltr11.ADCRate.divider   = DIVIDER;
        hltr11.ADCRate.prescaler = PRESCALER;
        hltr11.LChQnt            = 1;
        hltr11.ADCMode           = ltr11_test_mode[nmode].mode;
        hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
        
        sprintf(s, ":    %s", ltr11_test_mode[nmode].title);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        
        for (nch = 0; nch < LTR11_MAX_CHANNEL / 2; nch++)
        {
            /*     */
            
            ProcessSystemEvents();
            if (tc->cbf != NULL)
            {
                double h = 0;
                
                h  = 100.0 * (nmode * LTR11_MAX_CHANNEL / 2 + nch);
                h /= (TEST_MODE_QNT * LTR11_MAX_CHANNEL / 2 - 1);
                if (tc->cbf(TFMSG_PROGRESS(h), NULL))
                {
                    ret_val = TRES_ABORT;
                    goto end_test;
                }
            }
            
            /*       LTR-11 */
            hltr11.LChTbl[0] = (test_band << 6) | (0 << 4) | nch;
            frame_size       = ADC_FRAME_SIZE;
            err_code         = getframe(tc, frame, TRUE, &frame_size);
            if (err_code != LTR_OK)
            {
                if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                    || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                    || err_code == LTR11_ERR_ADCDATA_CNT
                   )
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "!!!     ");
                    if (++cnt_adcdata_err >= max_adcdata_err)
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "      ");
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }
                else
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, ".    LTR-11");
                    ret_val = TRES_ABORT;
                    goto end_test;
                }
            }
            getframe_avg(frame, frame_size, frame_avg);
            ltr11adc_val = frame_avg[0];
            e            = 100.0 * (ltr11adc_val - ltr11_test_mode[nmode].u_rated)
                / ltr11_test_mode[nmode].u_rated;
            sprintf(s, " %u:  %.4f ,  %+.3f%%", nch, ltr11adc_val, e);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, s);
            if (fabs(e) > ltr11_max_err)
            {
                if (tc->cbf != NULL)
                {
                    tc->cbf(TFMSG_REPORT,
                        ".    LTR-11  ");
                }
                ret_val = TRES_T_FAIL;
                goto end_test;
            }            
        } /*for (nch = 0; nch < LTR11_MAX_CHANNEL / 2; nch++)*/
    }     /*for (nmode = 0; nmode < TEST_MODE_QNT; nmode++)*/

end_test:
    return ret_val;
    
#undef TEST_MODE_QNT
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testNonlinearDistortion(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (4096)
#define TEST_FREQ_QNT  (2)

    double         adcrate;
    int            band;
    const double   band_amp[LTR11_ADC_RANGEQNT] = {0.9, 0.9, 0.9, 0.8};
    int            ch_num;
    int            cnt_adcdata_err = 0;
    int            err_code;
    double         fft[ADC_FRAME_SIZE];
    double         frame[ADC_FRAME_SIZE];
    int            frame_size;
    int            fsin;
    double         ltr11_min_snr[LTR11_ADC_RANGEQNT][TEST_FREQ_QNT] =
    {
        {72.0, 64.0}, {69.0, 62.0}, {61.0, 58.0}, {53.0, 53.0}
    };
    int            max_prcnt;
    int            prcnt;
    TE01           *pte01;
    int            ret_val = 0;
    char           s[100];
    const unsigned test_chnum[] = {0, 4, 8, 12};
    const struct
    {
        int freq;
        int filt;
        struct
        {
            int prescaler;
            int divider;
        }   adc_rate;
    }              test_sin[TEST_FREQ_QNT] =
    {
        {400, te01fdac_filt5kHz, {1, 1499}}, {5000, te01fdac_filt5kHz, {1, 149}}
    };
    
    
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
	SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetAModule_Config(pte01, UFPTOX_UFNTOY_CONNECT, U0TOX_U0TOY_CONNECT, UF0TOGND32_CONNECT,
        ZEROTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);

    
    /*    LTR-11 */
    hltr11.InpMode      = LTR11_INPMODE_INT;
    hltr11.LChQnt       = 1;
    hltr11.ADCMode      = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode = LTR11_STARTADCMODE_INT;
    
    max_prcnt = LTR11_ADC_RANGEQNT * TEST_FREQ_QNT * (sizeof test_chnum / sizeof test_chnum[0])
        - 1;
    prcnt     = 0;

    for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
    {
        double f_sin;
        double freq;
        double pwr;
        double snr;
        
        sprintf(s, " +-%.3f ", ranges[band]);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
            
        for (fsin = 0; fsin < TEST_FREQ_QNT; fsin++)
        {
            /*       LTR-11  */
            f_sin = test_sin[fsin].freq;
            SetFDAC_Sinus(pte01, &f_sin, ranges[band] * band_amp[band] * 1000.0,
                test_sin[fsin].filt, te01fdac_band_ltr11[band]);
            hltr11.ADCRate.prescaler = test_sin[fsin].adc_rate.prescaler;
            hltr11.ADCRate.divider   = test_sin[fsin].adc_rate.divider;
            adcrate                  = LTR11_CLOCK * 1000.0 /
                (test_sin[fsin].adc_rate.prescaler * (test_sin[fsin].adc_rate.divider + 1));
                
            sprintf(s, ":  = %.0f ;  = %.0f ", f_sin, adcrate);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, s);
            
            for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)
            {
                ProcessSystemEvents();
                if (tc->cbf != NULL)
                {
                    if (tc->cbf(TFMSG_PROGRESS(100.0 * prcnt / max_prcnt), NULL))
                    {
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }                    
            
                /*     */
                hltr11.LChTbl[0] = (band << 6) | (0 << 4) | test_chnum[ch_num];
                frame_size       = ADC_FRAME_SIZE;
                err_code = getframe(tc, frame, TRUE, &frame_size);
                if (err_code != LTR_OK)
                {
                    if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                        || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                        || err_code == LTR11_ERR_ADCDATA_CNT
                       )
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "!!!     ");
                        if (++cnt_adcdata_err >= max_adcdata_err)
                        {
                            if (tc->cbf != NULL)
                                tc->cbf(TFMSG_REPORT, "      ");
                            ret_val = TRES_ABORT;
                            goto end_test;
                        }
                    }
                    else
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, ".    LTR-11");
                        ret_val = TRES_ABORT;
                        goto end_test;
                    }
                }
                
                freq = f_sin;
                utsMath_DoFFT(frame, frame_size, 1, 7, adcrate, &frame_size, fft, &freq,
                    &pwr, &snr, NULL,NULL,NULL,NULL);
                sprintf(s, " %u: c/ = %.1f ", test_chnum[ch_num] + 1, snr);
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, s);
                    
                if (snr < ltr11_min_snr[band][fsin])
                {
                    sprintf(s, "!      (%.0f)",
                        ltr11_min_snr[band][fsin]);
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, s);
                    ret_val = TRES_T_FAIL;
                    goto end_test;
                }

                prcnt++;
            } /*for (ch_num = 0; ch_num < sizeof test_chnum / sizeof test_chnum[0]; ch_num++)*/
        }     /*for (fsin = 0; fsin < TEST_FREQ_QNT; fsin++)*/
    }         /*for (band = 0; band < LTR11_ADC_RANGEQNT; band++)*/


end_test:
    return ret_val;
    
#undef TEST_FREQ_QNT
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testPlolog(TCONTEXT *tc)
{
    INT         err_code;
    ltrTContext *tclocal = tc->tcontext;
    
    
    if (!needOpen)
        return 0;
     
    if (tc->cbf)
        tc->cbf(TFMSG_INIT_START, NULL);   
    
    hltr11.size = sizeof hltr11;
    LTR11_Init(&hltr11);

    err_code = LTR11_Open(&hltr11, tclocal->serverAddr, tclocal->serverPort, tclocal->crateSerial,
        tclocal->slot);
    if (err_code != LTR_OK)
    {
        if (tc->cbf != NULL)
            tc->cbf(MIDCER_STOP_ERROR, (char *)LTR11_GetErrorString(err_code));
        LTR11_Close(&hltr11);
        if (tc->cbf)
            tc->cbf(TFMSG_INIT_END, NULL);   
        return TRES_INIT_FAIL;
    }

    err_code = LTR11_GetConfig(&hltr11); 
    if (err_code != LTR_OK)
    {
        if (tc->cbf != NULL)
            tc->cbf(MIDCER_STOP_ERROR, (char *)LTR11_GetErrorString(err_code));
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_INIT_END, NULL);
        return TRES_INIT_FAIL;
    }

    if (tc->cbf != NULL)
        tc->cbf(TFMSG_INIT_END, NULL);   

    needOpen = 0;
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testShortCircuitGND32AGND(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (512)
#define TEST_BAND 0U                    /*  ,     */
#if TEST_BAND >= LTR11_ADC_RANGEQNT
#error Invalid TEST_BAND value
#endif

    int          cnt_adcdata_err = 0;
    double       e;
    int          err_code;
    double       frame[ADC_FRAME_SIZE];
    double       frame_avg[LTR11_MAX_CHANNEL];
    int          frame_size;
    const double ltr11_max_err     = 20;
    double       ltr11adc_val;
    double       point;
    TE01         *pte01;
    int          ret_val           = 0;
    char         s[100];

 
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    SetAModule_Config(pte01, UFPTOX_UFNTOY_CONNECT, U0TOX_U0TOY_CONNECT, UFPTOGND32_CONNECT,
        SIGNALTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);
    
    /*    LTR-11 */
    hltr11.InpMode           = LTR11_INPMODE_INT;
    hltr11.ADCRate.divider   = DIVIDER;
    hltr11.ADCRate.prescaler = PRESCALER;
    hltr11.LChQnt            = 1;
    hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
    
    SetUADC_Config(pte01, te01adc_band_ltr11[TEST_BAND]);

    /*      TE-01 */
    point = ranges[TEST_BAND] / 2 * 1000.0;
    SetFDAC_DCValue(pte01, point, te01fdac_band_ltr11[TEST_BAND]);
    point    /= 1000.0;                /*    */
        
    /*       LTR-11 */
    hltr11.LChTbl[0] = (TEST_BAND << 6) | (3 << 4) | 0;
    frame_size       = ADC_FRAME_SIZE;
    err_code         = getframe(tc, frame, TRUE, &frame_size);
    if (err_code != LTR_OK)
    {
        if (   err_code == LTR11_ERR_ADCDATA_CHNUM
            || err_code == LTR11_ERR_ADCDATA_SLOTNUM
            || err_code == LTR11_ERR_ADCDATA_CNT
           )
        {
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, "!!!     ");
            if (++cnt_adcdata_err >= max_adcdata_err)
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, "      ");
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        else
        {
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, ".    LTR-11");
            ret_val = TRES_ABORT;
            goto end_test;
        }
    }
    getframe_avg(frame, frame_size, frame_avg);
    ltr11adc_val = -frame_avg[0];   /*  LTR-11      */
    e = 100 * (ltr11adc_val - point) / point;
    sprintf(s, " %.2f ,  %.2f ,  %+.1f%%", point, ltr11adc_val, e);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_REPORT, s);
        
    if (tc->cbf != NULL)
    {
        if (tc->cbf(TFMSG_PROGRESS(100.0), NULL))
        {
            ret_val = TRES_ABORT;
            goto end_test;
        }
    }
        
    if (fabs(e) > ltr11_max_err)
    {
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, "!    LTR-11  ");
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
end_test:
    return ret_val;
#undef TEST_BAND
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testShortCircuitXY(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (512)
#define TEST_BAND      (0U)             /*  ,     */
#if TEST_BAND >= LTR11_ADC_RANGEQNT
#error Invalid TEST_BAND value
#endif

    unsigned     ch_num;
    int          cnt_adcdata_err = 0;
    double       e;
    int          err_code;
    double       frame[ADC_FRAME_SIZE];
    double       frame_avg[LTR11_MAX_CHANNEL];
    int          frame_size;
    const double ltr11_max_err     = 20;
    double       ltr11adc_val;
    double       point;
    TE01         *pte01;
    int          ret_val           = 0;
    char         s[100];
    const double te01_max_sdac_err = 0.2;
    double       uadc_val;

 
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    SetAModule_Config(pte01, USPTOX_US0TOY_CONNECT, USPTOX_US0TOY_CONNECT, US0TOGND32_CONNECT,
        SIGNALTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);
    
    /*    LTR-11 */
    hltr11.InpMode           = LTR11_INPMODE_INT;
    hltr11.ADCRate.divider   = DIVIDER;
    hltr11.ADCRate.prescaler = PRESCALER;
    hltr11.LChQnt            = 1;
    hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
    
    SetADC_Range(pte01, te01adc_band_ltr11[TEST_BAND]);
    SetUADC_Config(pte01, te01adc_band_ltr11[TEST_BAND]);
    /*      TE-01 */
    point = ranges[TEST_BAND] / 2 * 1000.0;
    SetSDAC_Value(pte01, point, te01sdac_band_ltr11[TEST_BAND]);
    
    uadc_val = GetUADC_Val(pte01);
    e        = 100 * (uadc_val - point) / te01_uadc_ranges[te01adc_band_ltr11[TEST_BAND]];
    if (fabs(e) > te01_max_sdac_err)
    {
        sprintf(s, "     TE-01 ( %+.2f%%)", e);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    for (ch_num = 0; ch_num < LTR11_MAX_CHANNEL / 2; ch_num++)
    {
        /*       */
        
        ProcessSystemEvents();
        
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_PROGRESS(100.0 * ch_num / (LTR11_MAX_CHANNEL / 2 - 1)), NULL))
            {
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        
        sprintf(s, " %u", ch_num + 1);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
            
        
        /*       LTR-11 */
        hltr11.LChTbl[0] = (TEST_BAND << 6) | (0 << 4) | ch_num;
        frame_size       = ADC_FRAME_SIZE;
        err_code         = getframe(tc, frame, TRUE, &frame_size);
        if (err_code != LTR_OK)
        {
            if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                || err_code == LTR11_ERR_ADCDATA_CNT
               )
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, "!!!     ");
                if (++cnt_adcdata_err >= max_adcdata_err)
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "      ");
                    ret_val = TRES_ABORT;
                    goto end_test;
                }
            }
            else
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, ".    LTR-11");
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        getframe_avg(frame, frame_size, frame_avg);
        ltr11adc_val = frame_avg[0];
        e = 100 * (ltr11adc_val - uadc_val / 1000.0) / (uadc_val / 1000.0);
        sprintf(s, " %.6f ,  %.6f ,  %+.3f%%", uadc_val / 1000.0,
            ltr11adc_val, e);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        if (fabs(e) > ltr11_max_err)
        {
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, "!    LTR-11  ");
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
    } /*for (ch_num = 0; ch_num < LTR11_MAX_CHANNEL / 2; ch_num++)*/
    
    
    
end_test:
    return ret_val;
#undef TEST_BAND
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testShortCircuitXYGround(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (512)
#define TEST_BAND      (0U)              /*  ,     */
#if TEST_BAND >= LTR11_ADC_RANGEQNT
#error Invalid TEST_BAND value
#endif

    unsigned     ch_num;
    int          cnt_adcdata_err = 0;
    double       e;
    int          err_code;
    double       frame[ADC_FRAME_SIZE];
    double       frame_avg[LTR11_MAX_CHANNEL];
    int          frame_size;
    const double ltr11_max_err = 40;
    double       ltr11adc_val;
    double       point;
    TE01         *pte01;
    int          ret_val       = 0;
    char         s[100];

 
    if (testPlolog(tc))
    {
        ret_val = TRES_INIT_FAIL;
        goto end_test;
    }
    
    pte01 = (TE01 *)tc->te01;
    if (!Check_TE01A11(pte01))
    {
        sprintf(s, "!    ");
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
        ret_val = TRES_T_FAIL;
        goto end_test;
    }
    
    SetUADC_Config(pte01, te01adc_band_ltr11[0]);
    
    SetAModule_Config(pte01, UFPTOX_UFPTOY_CONNECT, UFPTOX_UFPTOY_CONNECT, UFNTOGND32_CONNECT,
        ZEROTOVOLTMETER_CONNECT);
    TE01ReadCoeffFromEEPROM(pte01);
    
    /*    LTR-11 */
    hltr11.InpMode           = LTR11_INPMODE_INT;
    hltr11.ADCRate.divider   = DIVIDER;
    hltr11.ADCRate.prescaler = PRESCALER;
    hltr11.LChQnt            = 1;
    hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
    hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
    
    
    /*      TE-01 */
    point = ranges[TEST_BAND] / 2 * 1000.0;
    SetFDAC_DCValue(pte01, point, te01fdac_band_ltr11[TEST_BAND]);
    point    /= 1000.0;                /*    */
    
    for (ch_num = 0; ch_num < LTR11_MAX_CHANNEL; ch_num++)
    {
        /*       */
        unsigned mm;            /*     */
    
        ProcessSystemEvents();
        
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_PROGRESS(100.0 * ch_num / (LTR11_MAX_CHANNEL - 1)), NULL))
            {
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        
        sprintf(s, " %u", ch_num + 1);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
            
        
        /*       LTR-11 */
        mm               = (ch_num < LTR11_MAX_CHANNEL / 2) ? 2 : 3;
        hltr11.LChTbl[0] = (TEST_BAND << 6) | (mm << 4) | (ch_num % (LTR11_MAX_CHANNEL / 2));
        frame_size       = ADC_FRAME_SIZE;
        err_code         = getframe(tc, frame, TRUE, &frame_size);
        if (err_code != LTR_OK)
        {
            if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                || err_code == LTR11_ERR_ADCDATA_CNT
               )
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, "!!!     ");
                if (++cnt_adcdata_err >= max_adcdata_err)
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "      ");
                    ret_val = TRES_ABORT;
                    goto end_test;
                }
            }
            else
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, ".    LTR-11");
                ret_val = TRES_ABORT;
                goto end_test;
            }
        }
        getframe_avg(frame, frame_size, frame_avg);
        ltr11adc_val = frame_avg[0];
        e = 100 * (ltr11adc_val - point) / point;
        sprintf(s, " %.2f ,  %.6f ,  %+.1f%%", point, ltr11adc_val, e);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
            
        if (fabs(e) > ltr11_max_err)
        {
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, "!    LTR-11  ");
            ret_val = TRES_T_FAIL;
            goto end_test;
        }
    } /*for (ch_num = 0; ch_num < LTR11_MAX_CHANNEL / 2; ch_num++)*/
    
end_test:
    return ret_val;
#undef TEST_BAND
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int testSNR(TCONTEXT *tc)
{
    int j;

 
    if (testPlolog(tc))
        return TRES_INIT_FAIL;
    for (j = 0; j < 17; j++)
    {
        Sleep(12);
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_PROGRESS(100.0 * j / 16), NULL))
                goto __uabort;
        }
    }
 
    if (tc->cbf)
        tc->cbf(TFMSG_RP, " 2.0 ,  1.998 ,  78.4 dB   1200 ");
    return 0;
 

__uabort:
    return TRES_ABORT;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspCalibrModule(TCONTEXT *tc)
{
#define ADC_FRAME_SIZE (1024)
    int          band;
    int          cnt_adcdata_err = 0;
    const int    divider = 749;
    double       e;
    int          err_code;
    double       frame[ADC_FRAME_SIZE];
    double       frame_avg[LTR11_MAX_CHANNEL];
    int          frame_size;
    double       ltr11_max;
    double       ltr11_min;
    double       ltr11_zero;
    char         msg[512];
    const int    prescaler = 1;
    double       scale;
    TE01         *te01;
    double       te01_adc_max;
    double       te01_adc_min;
    double       te01_adc_zero;
    double       te01_dac_value;
    const double te01_max_sdac_err = 0.1;
    double       zero;
	char         ini_file_name[256], ini_val_buf[64];
	int useTE01A11 = 1;

 
	/*  INI-. */
	sprintf(ini_file_name, "%s\\%s", tc->dllpath, "ltr11metr.ini");
	GetPrivateProfileString("TE01", "useTE01A11", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) useTE01A11 = atoi(ini_val_buf);
	else useTE01A11 = 1;

 
    if (testPlolog(tc))
        return TRES_INIT_FAIL;

    te01 = (TE01 *)tc->te01;


	if (useTE01A11) {
	    if (!Check_TE01A11(te01))
    	{
        	sprintf(msg, "!    .");
	        if (tc->cbf != NULL)
    	        tc->cbf(TFMSG_REPORT, msg);
        	goto bad_ret;
    	}
	    SetAModule_Config(te01, USPTOX_US0TOY_CONNECT, USPTOX_US0TOY_CONNECT, US0TOGND32_CONNECT,
    	    SIGNALTOVOLTMETER_CONNECT);
    }

    TE01ReadCoeffFromEEPROM(te01);
    for (band = 0; band < LTR11BANDS; band++)
    {
        ProcessSystemEvents();
        SetUADC_Config(te01, te01adc_band_ltr11[band]);
        sprintf(msg, " %.3f", ranges[band]);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_PROGRESS(100.0 * band / LTR11BANDS), NULL))
                goto uabort;
        }

		/* MAX */
        te01_dac_value = ranges[band] * 900.0;

        SetSDAC_Value(te01, te01_dac_value, te01sdac_band_ltr11[band]);
        te01_adc_max = GetUADC_Val(te01);
        
        e = 100 * (te01_adc_max - te01_dac_value) / te01_uadc_ranges[te01adc_band_ltr11[band]];
        if (fabs(e) > te01_max_sdac_err)
        {
            sprintf(msg, " SDAC TE-01   (%+.2f%%)", e);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            goto bad_ret;
        }
        
        te01_adc_max /= 1000;
	  
        sprintf(msg, ": %.3f, : %.3f", te01_dac_value / 1000.0, te01_adc_max);
        if (tc->cbf != NULL)
        {
            if (tc->cbf(TFMSG_REPORT, msg))
                goto uabort;
        }
	
        hltr11.InpMode           = LTR11_INPMODE_INT;
        hltr11.ADCRate.divider   = divider;
        hltr11.ADCRate.prescaler = prescaler;
        hltr11.LChQnt            = 1;
        hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
        hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
        hltr11.LChTbl[0]         = (band << 6) | (0 << 4);

        frame_size = ADC_FRAME_SIZE;
        err_code   = getframe(tc, frame, FALSE, &frame_size);
        if (err_code != LTR_OK)
        {
            if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                || err_code == LTR11_ERR_ADCDATA_CNT
               )
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, "!!!     ");
                if (++cnt_adcdata_err >= max_adcdata_err)
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "      ");
                    goto bad_ret;
                }
            }
            else
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, ".    LTR-11");
                goto bad_ret;
            }
        }
        getframe_avg(frame, frame_size, frame_avg);
		
        ltr11_max = frame_avg[0];
        sprintf(msg, ": te01=%10.5f, ltr11=%10.5f", te01_adc_max, ltr11_max);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);

		/* MIN */
        te01_dac_value = -ranges[band] * 900.0;

        SetSDAC_Value(te01, te01_dac_value, te01sdac_band_ltr11[band]);
        te01_adc_min = GetUADC_Val(te01);
        
        e = 100 * (te01_adc_min - te01_dac_value) / te01_uadc_ranges[te01adc_band_ltr11[band]];
        if (fabs(e) > te01_max_sdac_err)
        {
            sprintf(msg, " SDAC TE-01 SDAC   (%+.2f%%)", e);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            goto bad_ret;
        }
        te01_adc_min /= 1000.0;
	  
        sprintf(msg, ": %.5f, : %.5f", te01_dac_value / 1000.0, te01_adc_min);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, msg);
	
        hltr11.InpMode           = LTR11_INPMODE_INT;
        hltr11.ADCRate.divider   = divider;
        hltr11.ADCRate.prescaler = prescaler;
        hltr11.LChQnt            = 1;
        hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
        hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
        hltr11.LChTbl[0]         = (band << 6) | (0 << 4);

        frame_size = ADC_FRAME_SIZE;
        err_code   = getframe(tc, frame, FALSE, &frame_size);
        if (err_code != LTR_OK)
        {
            if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                || err_code == LTR11_ERR_ADCDATA_CNT
               )
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, "!!!     ");
                if (++cnt_adcdata_err >= max_adcdata_err)
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "      ");
                    goto bad_ret;
                }
            }
            else
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, ".    LTR-11");
                goto bad_ret;
            }
        }
        getframe_avg(frame, frame_size, frame_avg);
		
        ltr11_min = frame_avg[0];
        sprintf(msg, ": te01=%.5f, ltr11=%.5f", te01_adc_min, ltr11_min);
        if (tc->cbf)
            tc->cbf(TFMSG_REPORT, msg);
            
		/* ZERO */
        te01_dac_value = 0.0;

        SetSDAC_Value(te01, te01_dac_value, te01sdac_band_ltr11[band]);
        te01_adc_zero = GetUADC_Val(te01);
        
        e = 100 * (te01_adc_zero - te01_dac_value) / te01_uadc_ranges[te01adc_band_ltr11[band]];
        if (fabs(e) > te01_max_sdac_err)
        {
            sprintf(msg, " SDAC TE-01   (%+.2f%%)", e);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, msg);
            goto bad_ret;
        }
        te01_adc_zero /= 1000.0;
	  
        sprintf(msg, ": %.3f, : %.3f", te01_dac_value / 1000.0, te01_adc_zero);
        if (tc->cbf)
            tc->cbf(TFMSG_REPORT, msg);
	
        hltr11.InpMode           = LTR11_INPMODE_INT;
        hltr11.ADCRate.divider   = divider;
        hltr11.ADCRate.prescaler = prescaler;
        hltr11.LChQnt            = 1;
        hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
        hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
        hltr11.LChTbl[0]         = (band << 6) | (0 << 4);

        frame_size = ADC_FRAME_SIZE;
        err_code   = getframe(tc, frame, FALSE, &frame_size);
        if (err_code != LTR_OK)
        {
            if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                || err_code == LTR11_ERR_ADCDATA_CNT
               )
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, "!!!     ");
                if (++cnt_adcdata_err >= max_adcdata_err)
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "      ");
                    goto bad_ret;
                }
            }
            else
            {
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, ".    LTR-11");
                goto bad_ret;
            }
        }
        getframe_avg(frame, frame_size, frame_avg);

        ltr11_zero = frame_avg[0];
        sprintf(msg, ": te01=%.5f, ltr11=%.5f", te01_adc_zero, ltr11_zero);
        if (tc->cbf)
            tc->cbf(TFMSG_REPORT, msg);

	
        scale = (te01_adc_max - te01_adc_min) / (ltr11_max - ltr11_min);
        zero  = (te01_adc_zero / scale - ltr11_zero) * RESOLUTION / ranges[band];
        sprintf(msg, "zero %10.3f  scale %10.3f", zero, scale);
        if (tc->cbf)
            tc->cbf(TFMSG_REPORT, msg);

        e = 100 * fabs(zero / RESOLUTION);
        if (e > ltr11_max_cbrcoef[band].offset)
        {
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, "   ");
            goto bad_ret;
        }
        e = 100 * fabs(scale - 1);
        if (e > ltr11_max_cbrcoef[band].gain)
        {
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, "  ");
            goto bad_ret;
        }
        hltr11.ModuleInfo.CbrCoef[band].Offset = zero;
        hltr11.ModuleInfo.CbrCoef[band].Gain   = scale;
	} /*for (band = 0; band < LTR11BANDS; band++)*/
    if (tc->cbf != NULL)
    {
        if (tc->cbf(TFMSG_PROGRESS(100.0), NULL))
            goto uabort;
    }
	
	if (tc->cbf != NULL)
	{
	    tc->cbf(TFMSG_REPORT, " ...");
	    err_code = write_coeffs(tc, bin_fname, 0);
	    if (tc->cbf != NULL)
	    {
	        tc->cbf(TFMSG_REPORT, (!err_code) ? "  " :
	            "  ");
	    }
	    
	    if (err_code)
            goto bad_ret;
	        
	    if (tc->cbf != NULL)
	        tc->cbf(TFMSG_REPORT, "  (k - , b -  )");
	    for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
	    {
	        sprintf(msg, "Range %.3fV: k=%.4f  b=%.4f", ranges[band],
	            hltr11.ModuleInfo.CbrCoef[band].Gain, hltr11.ModuleInfo.CbrCoef[band].Offset);
	        if (tc->cbf != NULL)
	            tc->cbf(TFMSG_REPORT, msg);
	    }
	    
	    closeModule();
	    testPlolog(tc);
	    
	    utspGetModuleInfo(tc, NULL);
	}
 
    return TRES_T_OK;
 
bad_ret:
    return TRES_T_FAIL;

uabort:
    return TRES_ABORT;
    
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspGetDescrDAC(TCONTEXT *tc, pluginDAC *D, int channel)
{ 
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspGetDescrDIO(TCONTEXT *tc, pluginDIO *D, int channel)
{ 
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspGetDIO(TCONTEXT *tc, void *p)
{
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspGetModuleInfo(TCONTEXT *tc, void *p)
{
    char s[100];


    if (testPlolog(tc))
        return TRES_INIT_FAIL;
        
    
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_SERIAL, hltr11.ModuleInfo.Serial);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_IBLANK, "");
    sprintf(s, "%i.%i", (hltr11.ModuleInfo.Ver >> 8) & 0xFF, hltr11.ModuleInfo.Ver & 0xFF);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_VER, s);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_IBLANK, "");
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_IMODULE, ": ");
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_IBLANK, "");
    if (tc->cbf != NULL)
    {
        tc->cbf(TFMSG_BEGIN_ICALIBR,
            "  , : (A - , B -  :");
    }
    sprintf(s, "   10.0 : A= %2.5f, B= %2.5f", hltr11.ModuleInfo.CbrCoef[0].Gain,
        hltr11.ModuleInfo.CbrCoef[0].Offset);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_NEXT_ICALIBR, s);
        
    sprintf(s, "    2.5 : A= %2.5f, B= %2.5f", hltr11.ModuleInfo.CbrCoef[1].Gain,
        hltr11.ModuleInfo.CbrCoef[1].Offset);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_NEXT_ICALIBR, s);
        
    sprintf(s, "   0.625 : A= %2.5f, B= %2.5f", hltr11.ModuleInfo.CbrCoef[2].Gain,
        hltr11.ModuleInfo.CbrCoef[2].Offset);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_NEXT_ICALIBR, s);
        
    sprintf(s, "   0.156 : A= %2.5f, B= %2.5f", hltr11.ModuleInfo.CbrCoef[3].Gain,
        hltr11.ModuleInfo.CbrCoef[3].Offset);
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_END_ICALIBR, s);
        
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_IBLANK, "");
     
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspInitModule(TCONTEXT *tc, void *p)
{
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspPutDIO(TCONTEXT *tc, void *p)
{
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspReadData(TCONTEXT *tc, void *p)
{
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspReadMCUFuses(TCONTEXT *tc, void *p)
{ 
    TLTRAVR        hltr11avr;
    burnMcuContext *pbc = p;
    int            ret_val = TRES_T_OK;
    ltrTContext    *tclocal = tc->tcontext;
    
    
    if (!needOpen)
        LTR11_Close(&hltr11);
    needOpen = 1;

    LTRAVR_Init(&hltr11avr);
    if (LTRAVR_Open(&hltr11avr, tclocal->serverAddr, tclocal->serverPort, tclocal->crateSerial,
          CC_MODULE1 + tclocal->slot) == LTR_OK
       )
    {
        LTRAVR_SetSpeedFlag(&hltr11avr, FALSE);
        if (LTRAVR_ReadFuseBits(&hltr11avr, (BYTE *)&pbc->fuses) != LTR_OK)
            ret_val = TRES_T_FAIL;

        LTRAVR_Close(&hltr11avr);
    }
    else
    {
        ret_val = TRES_INIT_FAIL;
    }
    return ret_val;
}
/*-----------------------------------------------------------------------------------------------*/

typedef struct {
	int channel;
	double point;
	double max_error;
} sResumePoint;
sResumePoint ResumeArray[LTR11_ADC_RANGEQNT];

void InitResume() {
	int band;
	for (band=0;band<LTR11_ADC_RANGEQNT;++band) {
		ResumeArray[band].channel = -1;
	}
}

void AddResumePoint(int band, int chan, double point, double error) {
	sResumePoint *r = ResumeArray + band;
	if ((r->channel<0) || (fabs(r->max_error)<fabs(error))) {
		r->channel = chan;
		r->max_error = error;
		r->point = point;
	}
}

/*-----------------------------------------------------------------------------------------------*/
int configHW(TCONTEXT *tc, PTLTR11 hltr11);
char* VERIFY_MESSAGE[] = {"", "", ""};
int utspVerifyModule(TCONTEXT *tc)
{
#define VERIFY_CHANNELS 16
#define ADC_FRAME_SIZE   (1024*16)
#define VERIFYPOINTS_QNT (5)
#if VERIFYPOINTS_QNT < 3
#error Verify points quantity must be more than 3 (VERIFYPOINTS_QNT >= 3)
#elif VERIFYPOINTS_QNT % 2 == 0
#error Verify points quantity must be even
#endif

    int          band, chan, vchan;
    int          cnt_adcdata_err = 0;
    const int    divider = 749;
    /*double       du;*/
    double       e;
    int          err_code, ret_code, verify_code;
    double       frame[ADC_FRAME_SIZE];
    double       frame_avg[LTR11_MAX_CHANNEL];
    int          frame_size;
    int          j;
    /*     LTR-11  ,  % */
    double       ltr11_max_err[LTR11_ADC_RANGEQNT] = {0.03, 0.03, 0.06, 0.25};
    double       ltr11adc_val;
    /*double       max_band_err[LTR11_ADC_RANGEQNT];*/
    double       point;
    const int    prescaler = 1;
    TE01         *pte01;
    char         s[100];
    const double te01_max_sdac_err = 0.1;
    double       uadc_val;
    int          verifypoints_qnt = VERIFYPOINTS_QNT, ver_intervals;
    double       range, vrf_point;
    sResumePoint *r;
	char         ini_file_name[256], ini_val_buf[64], MaxErrorField[32];
	int useTE01A11 = 1;

 
 	ret_code = 0;
	/*  INI-. */
	sprintf(ini_file_name, "%s\\%s", tc->dllpath, "ltr11metr.ini");
	GetPrivateProfileString("VERIFY", "points", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) verifypoints_qnt = atoi(ini_val_buf);
	else verifypoints_qnt = VERIFYPOINTS_QNT;
	/*     3.*/
	if (verifypoints_qnt<3) verifypoints_qnt=3;
	/*     .*/
	if (verifypoints_qnt%2==0) ++verifypoints_qnt;
	/*    .*/
	ver_intervals = verifypoints_qnt - 1;

	GetPrivateProfileString("TE01", "useTE01A11", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) useTE01A11 = atoi(ini_val_buf);
	else useTE01A11 = 1;
	
	for(band=0;band<LTR11_ADC_RANGEQNT;++band) {
		sprintf(MaxErrorField, "MaxError%d", band);
		GetPrivateProfileString("VERIFY", MaxErrorField, "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
		if (*ini_val_buf) ltr11_max_err[band] = atof(ini_val_buf);
	}

    if (testPlolog(tc))
        return TRES_INIT_FAIL;
        

	/*      . */
	InitResume();
	
	pte01 = (TE01 *)tc->te01;

	if (useTE01A11) {
    	if (!Check_TE01A11(pte01))
    	{
        	sprintf(s, "Pluggable module not found\n");
        	if (tc->cbf != NULL)
            	tc->cbf(TFMSG_REPORT, s);
        	goto bad_ret;
    	}
    	SetAModule_Config(pte01, USPTOX_US0TOY_CONNECT, USPTOX_US0TOY_CONNECT, US0TOGND32_CONNECT,
        	SIGNALTOVOLTMETER_CONNECT);
    }
        
    TE01ReadCoeffFromEEPROM(pte01);
    
    /*    LTR-11 */
	if (useTE01A11) {/* .*/
	    hltr11.InpMode           = LTR11_INPMODE_INT;
    	hltr11.ADCRate.divider   = divider;
	    hltr11.ADCRate.prescaler = prescaler;
    	hltr11.LChQnt            = VERIFY_CHANNELS;
	    hltr11.ADCMode           = LTR11_ADCMODE_ACQ;
    	hltr11.StartADCMode      = LTR11_STARTADCMODE_INT;
	}
	else {/* .*/
	    if (configHW(tc, &hltr11))
        	return 1;
	}

	readINI(tc->dllpath, tc->mdescr);
	if (useTE01A11) {/* .*/
		hltr11.LChQnt = 1;
	}
	else {/* .*/
		hltr11.LChQnt = 0;
		for (chan=0; chan<LTR11_MAX_LCHANNEL; ++chan)
			if (IniSet.vchan[chan].onoff) ++hltr11.LChQnt;
	}
		
    for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
    {
        sprintf(s, " +- %.3f :", ranges[band]);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_REPORT, s);
       
#if 1
		if (useTE01A11) {/* .*/
			hltr11.LChTbl[0] = (band << 6) | (0 << 4) | 0;
		}
		else {/* .*/
			vchan = 0;
    	    for (chan=0; chan<LTR11_MAX_LCHANNEL; ++chan)
        		if (IniSet.vchan[chan].onoff)
        			hltr11.LChTbl[vchan++] = (band << 6) | (/*IniSet.vchan[chan].mode*/0 << 4) | chan;
        }
#endif
        SetUADC_Config(pte01, te01adc_band_ltr11[band]);
        
        /*
         *       :
         * vpoints[0]                                          -  0;
         * vpoints[1..VERIFYPOINTS_QNT/2]                      - ,    
         *                                                         ;
         * vpoints[VERIFYPOINTS_QNT/2 + 1..VERIFYPOINTS_QNT-1] - ,    
         *                                                         .
         */
		range         = ranges[band];
        /*     */
        for (j = 0; j < verifypoints_qnt; j++)
        {
        	vrf_point = range*0.99*((2.0*j)/ver_intervals - 1.);
            ProcessSystemEvents();

            if (tc->cbf != NULL)
            {
                double h;

                h  = 100.0 * (band * verifypoints_qnt + j);
                h /= (LTR11_ADC_RANGEQNT * verifypoints_qnt - 1);
                if (tc->cbf(TFMSG_PROGRESS(h), NULL))
                    goto uabort;
            }

            sprintf(s, " : %.3f", vrf_point);
            if (tc->cbf != NULL)
            {
                if (tc->cbf(TFMSG_REPORT, s))
                    goto uabort;
            }

            /*      TE-01 */
            point = vrf_point * 1000.0;    /*      TE-01 API */
            SetSDAC_Value(pte01, point, te01sdac_band_ltr11[band]);
            uadc_val = GetUADC_Val(pte01);
            e        = 100 * (uadc_val - point) / te01_uadc_ranges[te01adc_band_ltr11[band]];
            if (fabs(e) > te01_max_sdac_err)
            {
                sprintf(s, "     TE-01 ( %+.2f%%)", e);
                if (tc->cbf != NULL)
                    tc->cbf(TFMSG_REPORT, s);
                goto bad_ret;
            }
            point    /= 1000.0;                /*    */
            uadc_val /= 1000.0;
            
            /*       LTR-11 */
            frame_size       = ADC_FRAME_SIZE;
            err_code         = getframe/*async_getframe*/(tc, frame, TRUE, &frame_size);
            if (err_code != LTR_OK)
            {
                if (   err_code == LTR11_ERR_ADCDATA_CHNUM
                    || err_code == LTR11_ERR_ADCDATA_SLOTNUM
                    || err_code == LTR11_ERR_ADCDATA_CNT
                   )
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, "!!!     ");
                    if (++cnt_adcdata_err >= max_adcdata_err)
                    {
                        if (tc->cbf != NULL)
                            tc->cbf(TFMSG_REPORT, "      ");
                        goto bad_ret;
                    }
                }
                else
                {
                    if (tc->cbf != NULL)
                        tc->cbf(TFMSG_REPORT, ".    LTR-11");
                    goto bad_ret;
                }

            }
            getframe_avg(frame, frame_size, frame_avg);
			if (useTE01A11) {/* .*/
       	    	ltr11adc_val = frame_avg[0];
				e = 100 * (ltr11adc_val - uadc_val) / ranges[band];
        	    sprintf(s, " %.6f ,  %.6f ,  %+.3f%%", uadc_val,
	                	ltr11adc_val, e);
		        if (tc->cbf != NULL)
    		    	tc->cbf(TFMSG_REPORT, s);
            	AddResumePoint(band, 0, vrf_point, e);
			}
			else {/* .*/
	            vchan = 0;
		        for (chan=0; chan<LTR11_MAX_LCHANNEL; ++chan) {
	    	    	if (IniSet.vchan[chan].onoff){
	        	    	ltr11adc_val = frame_avg[vchan];
    	        		e = 100 * (ltr11adc_val - uadc_val) / ranges[band];
        	    		sprintf(s, " %.6f ,  %.6f ,  %+.3f%%", uadc_val,
	            	    	ltr11adc_val, e);
		                if (tc->cbf != NULL)
    		                tc->cbf(TFMSG_REPORT, s);
            	    	AddResumePoint(band, chan, vrf_point, e);
                		++vchan;
	                }
    	        }
            }
        } /*for (j = 0; j < VERIFYPOINTS_QNT; j++)*/
    }     /*for (band = 0; band < LTR11_ADC_RANGEQNT; band++)*/

#if 0
    if (tc->cbf != NULL)
    {
        tc->cbf(TFMSG_REPORT, "   ");
        for (band = 0; band < LTR11_ADC_RANGEQNT; band++)
        {
            sprintf(s, " +- %.3f : %+.3f%%", ranges[band], max_band_err[band]);
            if (tc->cbf != NULL)
                tc->cbf(TFMSG_REPORT, s);
        }
    }
#endif
	for(band=0;band<LTR11_ADC_RANGEQNT;++band) {
		r = ResumeArray + band;
		if (fabs(r->max_error)>ltr11_max_err[band]) ret_code = TRES_T_FAIL;
	}
	if(tc->cbf) {
		if(tc->cbf(TFMSG_REPORT, "===================================")) {
			ret_code = 1;
		}
		sprintf(s, " .  %s.", ret_code ? "" : "");
		if(tc->cbf(TFMSG_REPORT, s)) {
			ret_code = 1;
		}
	}
	for(band=0;band<LTR11_ADC_RANGEQNT;++band) {
		r = ResumeArray + band;
		if (fabs(r->max_error)<ltr11_max_err[band]/2) verify_code = 0;
		else if (fabs(r->max_error)<ltr11_max_err[band]) verify_code = 1;
		     else verify_code = 2;
		sprintf(s, "Band %d:  chan %2d    point %7.3f    error %6.3f     %s",
		        band+1, r->channel, r->point, r->max_error, VERIFY_MESSAGE[verify_code]);
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, s)) {
				ret_code = 1;
				break;
			}
		}
	}

 
    return ret_code;
 
bad_ret:
    return TRES_T_FAIL;

uabort:
    return TRES_ABORT;
    
#undef VERIFYPOINTS_QNT
#undef ADC_FRAME_SIZE
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspWriteData(TCONTEXT *tc, void *p)
{
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspWriteFile(TCONTEXT *tc, void *p)
{
    return 0;
}
/*-----------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int utspWriteMCUFlash(TCONTEXT *tc, void *p)
{ 
    int            j;
    burnMcuContext *pbc = p;
    ltrTContext    *tclocal = tc->tcontext;

  
    if (!needOpen)
        LTR11_Close(&hltr11);
    needOpen = 1;
    
    j = strlen(pbc->serial);
    if ((j >= sizeof hltr11.ModuleInfo.Serial / sizeof hltr11.ModuleInfo.Serial[0] - 1) || (j == 0))
    {
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_LOG, "Serial number is not valid");
        return TRES_INIT_FAIL;
    }
    
    hltr11.size = sizeof hltr11;
    LTR11_Init(&hltr11);

    if (LTR11_Open(&hltr11, tclocal->serverAddr, tclocal->serverPort, tclocal->crateSerial,
            tclocal->slot) != LTR_OK
       )
    {
        LTR11_Close(&hltr11);
        if (tc->cbf != NULL)
            tc->cbf(TFMSG_INIT_END, NULL);
        return TRES_INIT_FAIL;
    }
    needOpen = 0;

    for (j = 0; j < LTR11_ADC_RANGEQNT; j++)
    {
        hltr11.ModuleInfo.CbrCoef[j].Offset = 0.0;
        hltr11.ModuleInfo.CbrCoef[j].Gain   = 1.0;
    }
    memset(hltr11.ModuleInfo.Serial, '\0',
        sizeof hltr11.ModuleInfo.Serial / sizeof hltr11.ModuleInfo.Serial[0]);
    strcpy(hltr11.ModuleInfo.Serial, pbc->serial);
    
    if (tc->cbf != NULL)
        tc->cbf(TFMSG_LOG, "Start burning...");
    write_coeffs(tc, pbc->fname, 1);

    
    LTR11_Close(&hltr11);
    
    needOpen = 1;
    
    return utspGetModuleInfo(tc, NULL);
}
/*------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
int utspWriteMCUFuses(TCONTEXT *tc, void *p)
{ 
    TLTRAVR        hltr11avr;
    burnMcuContext *pbc = p;
    int            ret_val = TRES_T_OK;
    ltrTContext    *tclocal = tc->tcontext;
    
    
    if (!needOpen)
        LTR11_Close(&hltr11);
    needOpen = 1;
    
    LTRAVR_Init(&hltr11avr);
    if (LTRAVR_Open(&hltr11avr, tclocal->serverAddr, tclocal->serverPort, tclocal->crateSerial,
            CC_MODULE1 + tclocal->slot) == LTR_OK)
    {
        int write_fuse_success = 0;
        
        write_fuse_success = LTRAVR_SetSpeedFlag(&hltr11avr, TRUE) == LTR_OK &&
            LTRAVR_WriteFuseBits(&hltr11avr, (BYTE*)&pbc->fuses) == LTR_OK;
        if (!write_fuse_success)
            ret_val = TRES_T_FAIL;


        LTRAVR_Close(&hltr11avr);
    }
    else
    {
        ret_val = TRES_INIT_FAIL;
    }
    return ret_val;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
static int write_coeffs(TCONTEXT *tc, const char *fname, int need_progress)
{
    int         band;
    char        *buf;
    int         crc;
    TLTRAVR     hltr11avr;
    int         i;
    char        *pbuf;
    int         size;
    ltrTContext *tclocal = tc->tcontext;
    int         wnd_size;

	
    if ((size = ReadAVR_PM(fname, &buf)) <= 0)
        return size;

    memcpy(buf + 0x99, hltr11.ModuleInfo.Serial,
        sizeof hltr11.ModuleInfo.Serial / sizeof hltr11.ModuleInfo.Serial[0]);
    for (band = 0; band < LTR11BANDS; band++)
    {
		*(float *)(buf + 0xA9 + band * 2 * sizeof(float)) = hltr11.ModuleInfo.CbrCoef[band].Offset;
		*(float *)(buf + 0xA9 + (band * 2 + 1) * sizeof(float)) =
		    hltr11.ModuleInfo.CbrCoef[band].Gain;
	}

	crc       = eval_crc16(0, (unsigned char *)buf + 0x80, CONFIG_DATA_SIZE - 2);
	buf[0xC9] = (crc >> 8) & 0xFF;
	buf[0xCA] = crc & 0xFF;
	
	LTRAVR_Init(&hltr11avr);
	LTRAVR_Open(&hltr11avr, tclocal->serverAddr, tclocal->serverPort, tclocal->crateSerial,
        tclocal->slot);
    LTRAVR_SetSpeedFlag(&hltr11avr, FALSE);
	LTRAVR_ChipErase(&hltr11avr);

	if (!need_progress)
	{
	    LTRAVR_WriteProgrammMemory(&hltr11avr, (WORD *)buf, size / 2, 0);
	}
	else
	{
    	wnd_size = size / 8;
    	for (pbuf = buf, i = 0; i < 8; pbuf += wnd_size, i++)
    	{
    	    if (tc->cbf != NULL)
    	        tc->cbf(TFMSG_PROGRESS(100.0 * i / 7), NULL);
    	    LTRAVR_WriteProgrammMemory(&hltr11avr, (WORD *)pbuf, wnd_size / 2,
    	        i * wnd_size / 2);
        }
    }
    
	
	LTRAVR_Close(&hltr11avr);
	
	free(buf);
	
	return 0;
}
/*-----------------------------------------------------------------------------------------------*/
/*===============================================================================================*/
