#include <ansi_c.h>
#include <userint.h>
#include <inifile.h>
//#include <utility.h>

#include "ltr/include/ltrapi.h"
#include "uts/systems/ltr/common/include/ltrspecific.h"
#include "ltr/include/ltr212api.h"
#include "localdefs.h"
#include "ltr212clb_coeffs.h"
#include "uts/instruments/te01/te01api.h"
#include "uts/instruments/te01/te01func.h"


#define LTR212VERPOINTS 5
const double LTR212RANGES[MAX_BAND][2] = {{-10.,10}, {-20.,20}, {-40.,40}, {-80.,80},
											{0.,10}, {0.,20}, {0.,40}, {0.,80}};
const int PhisChanOrder[MAX_CHANNEL] = {1,2,3,4,5,6,7,8};

extern descr_ltr212 hltr212;
extern tDataRecord AmpFreqDescriptor;
extern IniSet_XXX IniSet;

int configHW(TCONTEXT *tc, descr_ltr212 *hltr212);
void closeModule(void);
int testPlolog(TCONTEXT *tc);

typedef struct {
	int channel;
	double point;
	double max_error;
} sResumePoint;
sResumePoint ResumeArray[MAX_BAND];

void InitResume() {
	int band;
	for (band=0;band<MAX_BAND;++band) {
		ResumeArray[band].channel = -1;
	}
}

void AddResumePoint(int band, int chan, double point, double error) {
	sResumePoint *r = ResumeArray + band;
	if ((r->channel<0) || (fabs(r->max_error)<fabs(error))) {
		r->channel = chan;
		r->max_error = error;
		r->point = point;
	}
}


// ****************************************************************************
// ****************************************************************************

/**
 *      .
 * 
 * \param tc 
 * \param te01   TE-01
 * \param VerPoints   
 * \param band  
 * \param bottom_value   
 * \param top_value   
 * \param PhisChan        
 * \param coeffs  
 * \param HardClb 1 -    , 0 -    .
 *
 * \return  .
 */
int verify_side(TCONTEXT *tc, TE01 *te01, int VerPoints, int band, double bottom_value, double top_value,
								int *PhisChan, LTR212clb_coeffs** coeffs, int HardClb) {
	int iverpoint, chan;
	char msg[512];
	int err_code, ret_code = 0;
	double ltr_res[MAX_CHANNEL];
	double range_size;
	double value, corr_value;
	double te01_dac_value, te01_adc_val;

	range_size = top_value - bottom_value;
	top_value -= range_size*0.01;
	bottom_value += range_size*0.01;
	/*   . */
	for(iverpoint=0;iverpoint<VerPoints;++iverpoint) {
		ProcessSystemEvents();
		te01_dac_value = (top_value-bottom_value)/(VerPoints-1)*iverpoint + bottom_value;

		SetSDAC_Value(te01, te01_dac_value, TE01DAC_BAND_for_LTR212);
		/*   ()*/
		te01_adc_val = GetUADC_Val(te01);

		sprintf(msg, "point=%10.3f, etalon=%10.3f", te01_dac_value, te01_adc_val);
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, msg)) {
				ret_code = 1;
				break;
			}
		}

		if (err_code=GetFrame(tc, ltr_res)) {
			ret_code = err_code;
			sprintf(msg, "GetFrame:%d", err_code);
			if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);
			if(tc->cbf!=NULL) {
				if(tc->cbf(TFMSG_PROGRESS(100.0*band/MAX_BAND), NULL)) {
					ret_code = 1;
					break;
				}
			};
			break;
		}

		for(chan=0;chan<hltr212.LChQnt;++chan) {
			value = ltr_res[chan]*1000.;
			corr_value = HardClb ? value : correct_value(coeffs, band, PhisChan[chan], value);
			AddResumePoint(band, PhisChan[chan], te01_dac_value, (corr_value-te01_adc_val)/LTR212RANGES[band][1]*100);
#if 0
			sprintf(msg, "Chan %2d  %10.3f %10.3f  %10.3f %10.3f  (%d %d)", PhisChan[chan],
							value, (value-te01_adc_val)/LTR212RANGES[band][1]*100,
							corr_value, (corr_value-te01_adc_val)/LTR212RANGES[band][1]*100,
							coeffs[band][PhisChan[chan]-1].offset, coeffs[band][PhisChan[chan]-1].scale);
#else
			sprintf(msg, "Chan %2d  %10.3f %10.3f", PhisChan[chan],
							corr_value, (corr_value-te01_adc_val)/LTR212RANGES[band][1]*100);
#endif
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = 1;
					break;
				}
			}
		}
	}

	return ret_code;
}

/**
 *     .
 *
 * \param tc 
 * \param te01   TE-01
 * \param VerPoints        
 * \param PhisChan    
 * \param coeffs   
 * \param HardClb 1 --    , 0 --    PC
 *
 * \return  .
 */
int bands_loop_verify(TCONTEXT *tc, TE01 *te01, int VerPoints, int *PhisChan, LTR212clb_coeffs** coeffs, int HardClb) {
	int chan, band;
	char msg[512];
	int err_code, ret_code = 0;

	for (band=0;band<MAX_BAND;++band) {
		if (HardClb)
			if (copy_coeffs(&hltr212, band))
				return 1;
		
		SetADC_Range(te01, TE01ADC_BAND_for_LTR212);
		sprintf(msg, "Range=[%7.3f,%7.3f]  *** *** ***", LTR212RANGES[band][0], LTR212RANGES[band][1]);
		if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);
		if(tc->cbf!=NULL) {
			if(tc->cbf(TFMSG_PROGRESS(100.0*band/MAX_BAND), NULL)) {
				ret_code = 1;
				break;
			}
		};

		hltr212.LChQnt = AmpFreqDescriptor.channels;
		sprintf(msg, "hltr212.LChQnt=%d", hltr212.LChQnt);
		if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);
		for(chan=0;chan<hltr212.LChQnt;++chan){
			hltr212.LChTbl[chan]=LTR212_CreateLChannel(PhisChan[chan], band);
		}

		err_code = LTR212_SetADC(&hltr212);
		if (err_code) {
			ret_code = err_code;
			sprintf(msg, "LTR212_SetADC:%d", err_code);
			if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);
			break;
		}

		if (band<4) { /*   . */
			/*       . */
			ret_code = verify_side(tc, te01, VerPoints, band, LTR212RANGES[band][0], 0, PhisChan, coeffs, HardClb);
			if (ret_code) break;
		}
		/*   . */
		/*       . */
		ret_code = verify_side(tc, te01, VerPoints, band, 0, LTR212RANGES[band][1], PhisChan, coeffs, HardClb);
		if (ret_code) break;
	}

	return ret_code;
}
// ****************************************************************************

/**
 *       .
                                     +-80  +-40  +-20  +-10  +80   +40   +20  +10 */
double LTR212MAX_ERRORS[MAX_BAND] = {0.05, 0.05, 0.07, 0.1,  0.05, 0.07, 0.1, 0.1};
char* VERIFY_MESSAGE[] = {"", "", ""};

/**
 *  .
 *
 * \param tc 
 *
 * \return  .
 */
int VerifyModule(TCONTEXT *tc) {
	ltrTContext *tclocal= tc->tcontext;
	TE01 *te01;
	int band, chan;
	int PhisChan[MAX_CHANNEL];
	char msg[512];
	int err_code, ret_code = 0, verify_code;
	LTR212clb_coeffs** coeffs;
	unsigned char buffer[7];
	int i;
	int VerPoints;
	sResumePoint *r;
	char ini_file_name[256], ini_val_buf[64], MaxErrorField[32];
	int HardClb = 0;/*  -  INI*/
	int tuning = 0;/*  -  INI*/

	/*  INI-. */
	sprintf(ini_file_name, "%s\\%s", tc->dllpath, "ltr212metr.ini");
	GetPrivateProfileString("VERIFY", "points", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) VerPoints = atoi(ini_val_buf);
	else VerPoints = LTR212VERPOINTS;
		
	GetPrivateProfileString("VERIFY", "HardClb", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) HardClb = atoi(ini_val_buf);
	
	for(band=0;band<MAX_BAND;++band) {
		sprintf(MaxErrorField, "MaxError%d", band);
		GetPrivateProfileString("VERIFY", MaxErrorField, "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
		if (*ini_val_buf) LTR212MAX_ERRORS[band] = atof(ini_val_buf);
	}

	GetPrivateProfileString("COMMON", "tuning", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) tuning = atoi(ini_val_buf);
	
	if(testPlolog(tc)) return TRES_INIT_FAIL;

	if(configHW(tc, &hltr212)) {
		closeModule();
		return 1;
	};

	te01 = (TE01 *)tc->te01;


	/*    . */
	coeffs = create_coeffs_table();
	if(coeffs==NULL)
		return 1;

	/*   . */
	if (read_coeffs(&hltr212, coeffs)) {
		delete_coeffs_table(coeffs);
		return 1;
	}

	/*     . */
	/*    . */
	AmpFreqDescriptor.update_func = LC212DataUpdate;

	/*  .*/
	if (tuning) {/*  .*/
		AmpFreqDescriptor.channels = 8;
		for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 1,3,5,7/2,4,6,8
			PhisChan[chan] = PhisChanOrder[chan];
		};
		/*    . */
		AmpFreqDescriptor.buf_size = 3;

		hltr212.UseClb=1;/*  -  .*/
		hltr212.LChQnt = 8;/*  - 8.*/
		hltr212.AcqMode = 2;/* - 8 .*/
	}
	else { /*   */
	/*    . */
		AmpFreqDescriptor.channels = 0;
		for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 1,3,5,7/2,4,6,8
			if (IniSet.chan[chan].onoff) {
				PhisChan[AmpFreqDescriptor.channels++] = PhisChanOrder[chan];
			}
		};
		/*    . */
		AmpFreqDescriptor.buf_size = IniSet.point;

		hltr212.UseClb=HardClb;/*  -  INI-.*/
		hltr212.LChQnt = AmpFreqDescriptor.channels;/*  -  .*/
		hltr212.AcqMode = IniSet.common.mode;/**/
	}

	/*    . */
	AmpFreqDescriptor.mode = VALUE_DATA_MODE;
	/*   . */
	AmpFreqDescriptor.buffer = (double**)malloc(AmpFreqDescriptor.channels*sizeof(double*));
	/*     . */
	AmpFreqDescriptor.stored = (int*)malloc(AmpFreqDescriptor.channels*sizeof(int));
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		AmpFreqDescriptor.buffer[chan] = (double*)malloc(AmpFreqDescriptor.buf_size*sizeof(double));
	}

	TE01ReadCoeffFromEEPROM(te01);

	InitResume();
	ret_code = bands_loop_verify(tc, te01, VerPoints, PhisChan, coeffs, hltr212.UseClb);
	if (ret_code) {
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, "   ")) {
				ret_code = 1;
			}
		}
	}
	else{
		for(band=0;band<MAX_BAND;++band) {
			r = ResumeArray + band;
			if (fabs(r->max_error)>LTR212MAX_ERRORS[band]) ret_code = TRES_T_FAIL;
		}
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, "===================================")) {
				ret_code = 1;
			}
			sprintf(msg, " .  %s.", ret_code ? "" : "");
			if(tc->cbf(TFMSG_REPORT, msg)) {
				ret_code = 1;
			}
		}
		for(band=0;band<MAX_BAND;++band) {
			r = ResumeArray + band;
			if (fabs(r->max_error)<LTR212MAX_ERRORS[band]/2) verify_code = 0;
			else if (fabs(r->max_error)<LTR212MAX_ERRORS[band]) verify_code = 1;
			     else verify_code = 2;
			sprintf(msg, "Band %d:  chan %2d    point %7.3f    error %6.3f     %s",
			        band+1, r->channel, r->point, r->max_error, VERIFY_MESSAGE[verify_code]);
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = 1;
					break;
				}
			}
		}
	}
	
	if(tc->cbf!=NULL) {
		if(tc->cbf(TFMSG_PROGRESS(100.0), NULL)) ret_code = 1;
	};


 /*__uabort:*/
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		free(AmpFreqDescriptor.buffer[chan]);
	}
	/*   . */
	free(AmpFreqDescriptor.buffer);
	/*     . */
	free(AmpFreqDescriptor.stored);

	/*    . */
	delete_coeffs_table(coeffs);

	return ret_code;

	/*return TRES_ABORT;*/
};

// ****************************************************************************
#define LTR212MODE_NUMBER 3
struct {
	int mode;
	int channels;
	int divider;
	int points;
	int chop, skip, ac, fast, delay/*, iir, fir*/;
} ModeDescription[LTR212MODE_NUMBER] =
	{/*  */
		{0,/*mode*/
		 4,/*channels*/
		 40,/*divider*/
		 75,/*points*/
		 0,/*chop*/
		 1,/*skip*/
		 0,/*ac*/
		 0,/*fast*/
		 0/*delay*/},
		/*  4 */
		{0,/*mode*/
		 4,/*channels*/
		 682,/*divider*/
		 75,/*points*/
		 1,/*chop*/
		 0,/*skip*/
		 0,/*ac*/
		 0,/*fast*/
		 0/*delay*/},
		/*  8 */
		{1,/*mode*/
		 8,/*channels*/
		 682,/*divider*/
		 75,/*points*/
		 1,/*chop*/
		 0,/*skip*/
		 0,/*ac*/
		 0,/*fast*/
		 0/*delay*/}
	};

int ModuleStatistics(TCONTEXT *tc) {
	ltrTContext *tclocal= tc->tcontext;
	TE01 *te01;
	int test_mode, chan;
	int PhisChan[MAX_CHANNEL];
	char msg[512];
	int err_code, ret_code = 0;
	LTR212clb_coeffs** coeffs;
	unsigned char buffer[7];
	int i;
	time_t current_time;
	struct tm * current_date;
	char filename[64];
	FILE *f;
	int HardClb = 0;

	if(configHW(tc, &hltr212)) {
		closeModule();
		return 1;
	};

	te01= (TE01 *)tc->te01;

	if(testPlolog(tc)) return TRES_INIT_FAIL;

	LTR212_Stop(&hltr212);
	Sleep(1000.0);

#if 1
	/*    . */
	coeffs = create_coeffs_table();
	if(coeffs==NULL)
		return 1;

	/*   . */
	if (read_coeffs(&hltr212, coeffs))
		return 1;
#endif

	/*     . */
	/*    . */
	AmpFreqDescriptor.update_func = LC212DataUpdate;
	/*    . */
	AmpFreqDescriptor.buf_size = /*256*/IniSet.point;
	/*    . */
	AmpFreqDescriptor.mode = VALUE_DATA_MODE;
	/*   . */
	AmpFreqDescriptor.buffer = (double**)malloc(MAX_CHANNEL*sizeof(double*));
	/*     . */
	AmpFreqDescriptor.stored = (int*)malloc(MAX_CHANNEL*sizeof(int));
	/*  . */
	for (chan=0;chan<MAX_CHANNEL;++chan) {
		AmpFreqDescriptor.buffer[chan] = (double*)malloc(AmpFreqDescriptor.buf_size*sizeof(double));
	}

	hltr212.UseClb=0;

	TE01ReadCoeffFromEEPROM(te01);

	/*   . */
	time(&current_time);
	current_date = localtime(&current_time);
	sprintf(filename, "ltr212_%4d%02d%02d_%02d%02d%02d.tst",
					current_date->tm_year+1900, current_date->tm_mon, current_date->tm_mday,
					current_date->tm_hour, current_date->tm_min, current_date->tm_sec);
	if (!(f=fopen(filename, "w")))
		ret_code = 1;

	for (test_mode=0;test_mode<LTR212MODE_NUMBER;++test_mode) {
		if (ret_code) break;

		sprintf(msg, "** %s ** mode = %d **", filename, test_mode);
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, msg)) {
				ret_code = 1;
				break;
			}
		}
		//hltr212.Mode=ModeDescription[test_mode].mode;
		hltr212.AcqMode = test_mode;
#if 0
		hltr212.codec.CHOP=ModeDescription[test_mode].chop;
		hltr212.codec.SKIP=ModeDescription[test_mode].skip;
		hltr212.codec.FAST=ModeDescription[test_mode].fast;
		hltr212.codec.AC=ModeDescription[test_mode].ac;
		hltr212.codec.Delay=ModeDescription[test_mode].delay;
#endif
		AmpFreqDescriptor.channels = ModeDescription[test_mode].channels;
		for(chan=0; chan<AmpFreqDescriptor.channels; ++chan) {  //  : 1,3,5,7/2,4,6,8
			PhisChan[chan] = PhisChanOrder[chan];
		};
		IniSet.ppoint = ModeDescription[test_mode].points;
#if 0
		hltr212.codec.Fdivider = ModeDescription[test_mode].divider;
#endif
		ret_code = bands_loop_verify(tc, te01, LTR212VERPOINTS, PhisChan, coeffs, HardClb);
		if (ret_code) break;
	}

	if(tc->cbf!=NULL) {
		if(tc->cbf(TFMSG_PROGRESS(100.0), NULL)) ret_code = 1;;
	};

	fclose(f);

 /*__uabort:*/
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		free(AmpFreqDescriptor.buffer[chan]);
	}
	/*   . */
	free(AmpFreqDescriptor.buffer);
	/*     . */
	free(AmpFreqDescriptor.stored);

#if 1
	/*    . */
	delete_coeffs_table(coeffs);
#endif

	return ret_code;

	/*return TRES_ABORT;*/
};

// ****************************************************************************
#if 0
int CoeffsTest(TCONTEXT *tc) {
	ltrTContext *tclocal= tc->tcontext;
	TE01 *te01;
	int chan, band, iverpoint;
	int PhisChan[MAX_CHANNEL];
	double ltr_res[MAX_CHANNEL];
	double value, corr_value;
	double te01_dac_value, te01_adc_val;
	char msg[512];
	int err_code, ret_code;
	LTR212clb_coeffs** coeffs;
	unsigned char buffer[7];
	int i;

	if(configHW(tc, &hltr212)) {
		closeModule();
		return 1;
	};

	te01= (TE01 *)tc->te01;

	if(testPlolog(tc)) return TRES_INIT_FAIL;

	LTR212_Stop(&hltr212);
	Sleep(1000.0);

	/*     . */
	/*    . */
	AmpFreqDescriptor.update_func = LC212DataUpdate;
	/*    . */
	AmpFreqDescriptor.channels = 0;
	for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 1,3,5,7/2,4,6,8
		PhisChan[AmpFreqDescriptor.channels] = PhisChanOrder[chan];
		AmpFreqDescriptor.channels += IniSet.chan[chan].onoff!=0;
	};
	/*    . */
	AmpFreqDescriptor.buf_size = /*256*/IniSet.point;
	/*    . */
	AmpFreqDescriptor.mode = VALUE_DATA_MODE;
	/*   . */
	AmpFreqDescriptor.buffer = (double**)malloc(AmpFreqDescriptor.channels*sizeof(double*));
	/*     . */
	AmpFreqDescriptor.stored = (int*)malloc(AmpFreqDescriptor.channels*sizeof(int));
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		AmpFreqDescriptor.buffer[chan] = (double*)malloc(AmpFreqDescriptor.buf_size*sizeof(double));
	}

	hltr212.UseClb=1;
	hltr212.codec.CHOP=1;
	hltr212.codec.SKIP=0;
	hltr212.codec.FAST=0;
	hltr212.codec.AC=0;
	hltr212.codec.Delay=0;

	TE01ReadCoeffFromEEPROM(te01);
#if 0
	for (chan=0;chan<MAX_CHANNEL;++chan) {
	
		LTR212_ReadEEPROM(&hltr212, 7, 7*chan, buffer);
		for (i=2;i>=0;--i) {
			printf("%02X", buffer[i]);
		}
		printf(" ");
		for (i=5;i>=3;--i) {
			printf("%02X", buffer[i]);
		}
		printf(" %02X\n", buffer[6]);
		buffer[2] = 0x80;
		buffer[1] = 0x00;
		buffer[0] = 0x00;
		
		buffer[5] = 0x59;
		buffer[4] = 0x3C;
		buffer[3] = 0xEA;

		buffer[6] = 0x20;
		LTR212_WriteEEPROM(&hltr212, 7, 7*chan, buffer);
	}
#endif
	for (band=0;band<0;++band) {
#if 0
		if (copy_coeffs(&hltr212, band)) {
			ret_code = 1;
			break;
		}
		printf("band=%d\n", band);
		for (chan=0;chan<MAX_CHANNEL;++chan) {
			LTR212_ReadEEPROM(&hltr212, 7, 7*chan, buffer);
			for (i=2;i>=0;--i) {
				printf("%02X", buffer[i]);
			}
			printf(" ");
			for (i=5;i>=3;--i) {
				printf("%02X", buffer[i]);
			}
			printf(" %02X\n", buffer[6]);
		}
#endif
		SetADC_Range(te01, TE01ADC_BAND_for_LTR212);
		sprintf(msg, "Range=[%10.3f,%10.3f]", LTR212RANGES[band][0], LTR212RANGES[band][1]);
		if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);
		if(tc->cbf!=NULL) {
			if(tc->cbf(TFMSG_PROGRESS(100.0*band/MAX_BAND), NULL)) goto __uabort;
		};
		/*   . */
		for(iverpoint=0;iverpoint<0;++iverpoint) {
			ProcessSystemEvents();
			/*te01_dac_value = ((LTR212RANGES[band][1]-LTR212RANGES[band][0])/(LTR212VERPOINTS-1)*iverpoint
												+LTR212RANGES[band][0]);
			if (iverpoint==0) {
				te01_dac_value = ((LTR212RANGES[band][1]-LTR212RANGES[band][0])/(LTR212VERPOINTS-1)*0.5
													+LTR212RANGES[band][0]);
			}
			if (iverpoint==(LTR212VERPOINTS-1)) {
				te01_dac_value = ((LTR212RANGES[band][1]-LTR212RANGES[band][0])/(LTR212VERPOINTS-1)*(LTR212VERPOINTS-1.5)
													+LTR212RANGES[band][0]);*/
			}

			te01_dac_value = 0.0;
			SetSDAC_Value(te01, te01_dac_value, TE01DAC_BAND_for_LTR212);
			/*   ()*/
			te01_adc_val = GetUADC_Val(te01);

			sprintf(msg, "point=%10.3f, etalon=%10.3f", te01_dac_value, te01_adc_val);
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) goto __uabort;
			}

			hltr212.LChQnt=4;
			for(chan=0;chan<hltr212.LChQnt;++chan){
				hltr212.VChTbl[chan]=LTR212_CreateVChannel(PhisChan[chan], band);
			}

			err_code = LTR212_SetADC(&hltr212);
			if (err_code) {
				ret_code = err_code;
				break;
			}

			if (GetFrame(tc, ltr_res))
				return 1;

			for(chan=0;chan<hltr212.LChQnt;++chan) {
				value = ltr_res[chan]*1000.;
				sprintf(msg, "Chan %2d  %10.3f, %10.3f%%", PhisChan[chan],
								value, (value-te01_adc_val)/LTR212RANGES[band][1]*100);
				if(tc->cbf) {
					if(tc->cbf(TFMSG_REPORT, msg)) goto __uabort;
				}
			}
		}
	}
	if(tc->cbf!=NULL) {
		if(tc->cbf(TFMSG_PROGRESS(100.0), NULL)) goto __uabort;
	};

 __uabort:
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		free(AmpFreqDescriptor.buffer[chan]);
	}
	/*   . */
	free(AmpFreqDescriptor.buffer);
	/*     . */
	free(AmpFreqDescriptor.stored);

#if 1
	/*    . */
	delete_coeffs_table(coeffs);
#endif

	return 0;

	return TRES_ABORT;
};
#endif
// ****************************************************************************
int CalibrateModule(TCONTEXT *tc) {
	ltrTContext *tclocal= tc->tcontext;
	TE01 *te01;
	int chan, band, iverpoint;
	int PhisChan[MAX_CHANNEL];
	double ltr_res[MAX_CHANNEL], val_max[MAX_CHANNEL], val_min[MAX_CHANNEL], val_zero[MAX_CHANNEL];
	double te01_dac_value, te01_adc_val;
	double max_etalon, min_etalon, zero_etalon;
	char msg[512];
	int err_code, ret_code = 0;
	LTR212clb_coeffs** coeffs;
	char ini_file_name[256], ini_val_buf[64];
	int tuning = 0;/* -  INI*/

	/*  INI-. */
	sprintf(ini_file_name, "%s\\%s", tc->dllpath, "ltr212metr.ini");
	GetPrivateProfileString("COMMON", "tuning", "", ini_val_buf, sizeof(ini_val_buf)-1, ini_file_name);
	if (*ini_val_buf) tuning = atoi(ini_val_buf);

	if(testPlolog(tc))
		return TRES_INIT_FAIL;

	if(configHW(tc, &hltr212)) {
		closeModule();
		return 1;
	};

	te01 = (TE01 *)tc->te01;

	/*    . */
	coeffs = create_coeffs_table();
	if(coeffs==NULL)
		return 1;
	/*   . */
	if (read_coeffs(&hltr212, coeffs))
		return 1;

	/*     . */
	/*    . */
	AmpFreqDescriptor.update_func = LC212DataUpdate;
	hltr212.UseClb=0;

	/*  .*/
	if (tuning) {/*  .*/
		AmpFreqDescriptor.channels = 8;
		for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 1,3,5,7/2,4,6,8
			PhisChan[chan] = PhisChanOrder[chan];
		};
		/*    . */
		AmpFreqDescriptor.buf_size = 3;

		hltr212.LChQnt = 8;/*  - 8.*/
		hltr212.AcqMode = 2;/* - 8 .*/
	}
	else { /*   */
	/*    . */
		AmpFreqDescriptor.channels = 0;
		for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 1,3,5,7/2,4,6,8
			if (IniSet.chan[chan].onoff) {
				PhisChan[AmpFreqDescriptor.channels++] = PhisChanOrder[chan];
			}
		};
		/*    . */
		AmpFreqDescriptor.buf_size = IniSet.point;

		hltr212.LChQnt = AmpFreqDescriptor.channels;/*  -  .*/
		hltr212.AcqMode = IniSet.common.mode;/**/
	}

	/*    . */
	AmpFreqDescriptor.mode = CODE_DATA_MODE;
	/*   . */
	AmpFreqDescriptor.buffer = (double**)malloc(AmpFreqDescriptor.channels*sizeof(double*));
	/*     . */
	AmpFreqDescriptor.stored = (int*)malloc(AmpFreqDescriptor.channels*sizeof(int));
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		AmpFreqDescriptor.buffer[chan] = (double*)malloc(AmpFreqDescriptor.buf_size*sizeof(double));
	}
#if 0
	/*     . */
	/*    . */
	AmpFreqDescriptor.update_func = LC212DataUpdate;
	/*    . */
	AmpFreqDescriptor.channels = 0;
	for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 1,3,5,7/2,4,6,8
		if (IniSet.chan[chan].onoff) {
			PhisChan[AmpFreqDescriptor.channels++] = PhisChanOrder[chan];
		}
	};
	/*    . */
	AmpFreqDescriptor.buf_size = IniSet.point/*256*/;
	/*   . */
	AmpFreqDescriptor.mode = CODE_DATA_MODE;
	/*   . */
	AmpFreqDescriptor.buffer = (double**)malloc(AmpFreqDescriptor.channels*sizeof(double*));
	/*     . */
	AmpFreqDescriptor.stored = (int*)malloc(AmpFreqDescriptor.channels*sizeof(int));
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		AmpFreqDescriptor.buffer[chan] = (double*)malloc(AmpFreqDescriptor.buf_size*sizeof(double));
	}

	hltr212.UseClb = 0;
	hltr212.LChQnt = AmpFreqDescriptor.channels;
	hltr212.AcqMode = IniSet.common.mode;
#endif

	TE01ReadCoeffFromEEPROM(te01);
	for (band=0;band<MAX_BAND;++band) {
		SetADC_Range(te01, TE01ADC_BAND_for_LTR212);
		sprintf(msg, "Range=[%10.3f,%10.3f]", LTR212RANGES[band][0], LTR212RANGES[band][1]);
		if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);
		if(tc->cbf!=NULL) {
			if(tc->cbf(TFMSG_PROGRESS(100.0*band/MAX_BAND), NULL)) {
				ret_code = TRES_ABORT;
				break;
			}
		};


		for(chan=0;chan<hltr212.LChQnt;++chan){
			hltr212.LChTbl[chan]=LTR212_CreateLChannel(PhisChan[chan], band);
		}

		err_code = LTR212_SetADC(&hltr212);
		if (err_code) {
			ret_code = err_code;
			break;
		}
		

		/* MAX */
		ProcessSystemEvents();

		te01_dac_value = LTR212RANGES[band][1]*0.95;
		SetSDAC_Value(te01, te01_dac_value, TE01DAC_BAND_for_LTR212);
		/*   ()*/
		max_etalon = GetUADC_Val(te01);
		if (fabs(max_etalon-te01_dac_value)>LTR212RANGES[band][1]*0.1) {
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, "   ")) {
					ret_code = TRES_ABORT;
					break;
				}
			}
			ret_code = TRES_ABORT;
			break;
		}

		sprintf(msg, "point=%10.3f, etalon=%10.3f", te01_dac_value, max_etalon);
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, msg)) {
				ret_code = TRES_ABORT;
				break;
			}
		}


		if (GetFrame(tc, ltr_res)) {
			ret_code = 1;
			break;
		}

		
		for(chan=0;chan<hltr212.LChQnt;++chan){
			val_max[chan] = ltr_res[chan];
			if (fabs(code2value(val_max[chan], band)-max_etalon)>LTR212RANGES[band][1]*0.2) {
				if(tc->cbf) {
					if(tc->cbf(TFMSG_REPORT, "   ")) {
						ret_code = TRES_ABORT;
						break;
					}
				}
				ret_code = TRES_ABORT;
				break;
			}
		}
		if (ret_code) break;

		/* MIN */
		ProcessSystemEvents();

		if (band<4) {
			te01_dac_value = LTR212RANGES[band][0]*0.95;
		}
		else {
			te01_dac_value = LTR212RANGES[band][1]*0.05;
		}
		SetSDAC_Value(te01, te01_dac_value, TE01DAC_BAND_for_LTR212);
		/*   ()*/
		min_etalon = GetUADC_Val(te01);
		if (fabs(min_etalon-te01_dac_value)>LTR212RANGES[band][1]*0.1) {
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, "   ")) {
					ret_code = TRES_ABORT;
					break;
				}
			}
			ret_code = TRES_ABORT;
			break;
		}
		if (ret_code) break;

		sprintf(msg, "point=%10.3f, etalon=%10.3f", te01_dac_value, min_etalon);
		if(tc->cbf) {
			if(tc->cbf(TFMSG_REPORT, msg)) {
				ret_code = TRES_ABORT;
				break;
			}
		}

		if (GetFrame(tc, ltr_res)) {
			ret_code = 1;
			break;
		}

		for(chan=0;chan<hltr212.LChQnt;++chan) {
			val_min[chan] = ltr_res[chan];
			if (fabs(code2value(val_min[chan], band)-min_etalon)>LTR212RANGES[band][1]*0.2) {
				if(tc->cbf) {
					if(tc->cbf(TFMSG_REPORT, "   ")) {
						ret_code = TRES_ABORT;
						break;
					}
				}
				ret_code = TRES_ABORT;
				break;
			}
		}
		if (ret_code) break;

		/* ZERO */
		if (band>=4) { /*       . */
			zero_etalon = min_etalon;
			for(chan=0;chan<hltr212.LChQnt;++chan) {
				val_zero[chan] = val_min[chan];
			}
		}
		else {
			ProcessSystemEvents();

			te01_dac_value = 0;
			SetSDAC_Value(te01, te01_dac_value, TE01DAC_BAND_for_LTR212);
			/*   ()*/
			zero_etalon = GetUADC_Val(te01);
			if (fabs(zero_etalon-te01_dac_value)>LTR212RANGES[band][1]*0.1) {
				if(tc->cbf) {
					if(tc->cbf(TFMSG_REPORT, "   ")) {
						ret_code = TRES_ABORT;
						break;
					}
				}
				ret_code = TRES_ABORT;
				break;
			}

			sprintf(msg, "point=%10.3f, etalon=%10.3f", te01_dac_value, zero_etalon);
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = TRES_ABORT;
					break;
				}
			}

			if (GetFrame(tc, ltr_res)) {
				ret_code = 1;
				break;
			}

			for(chan=0;chan<hltr212.LChQnt;++chan) {
				val_zero[chan] = ltr_res[chan];
				if (fabs(code2value(val_zero[chan], band)-zero_etalon)>LTR212RANGES[band][1]*0.2) {
					if(tc->cbf) {
						if(tc->cbf(TFMSG_REPORT, "   ")) {
							ret_code = TRES_ABORT;
							break;
						}
					}
					ret_code = TRES_ABORT;
					break;
				}
			}
			if (ret_code) break;
		}

		for(chan=0;chan<hltr212.LChQnt;++chan) {
			calc_coeffs(coeffs, band, PhisChan[chan],
									max_etalon, min_etalon, val_max[chan], val_min[chan],
									zero_etalon, val_zero[chan]);
			sprintf(msg, "band=%2d, chan=%2d min=%f, max=%f, zero=%f, coeffs %10d, %10d: ",
			        band, PhisChan[chan],
							code2value(val_min[chan], band),
							code2value(val_max[chan], band),
							code2value(val_zero[chan], band),
							coeffs[band][PhisChan[chan]-1].offset, coeffs[band][PhisChan[chan]-1].scale);
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = TRES_ABORT;
				}
			}
#if	0
			sprintf(msg, "                     min=%f, max=%f, zero=%f", val_min[chan], val_max[chan], val_zero[chan]);
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = TRES_ABORT;
				}
			}
			sprintf(msg, "band=%2d, chan=%2d code=%f zero=%f, corrected: code=%f, value=%f",
							band, PhisChan[chan], val_zero[chan], code2value(val_zero[chan], band),
							correct_code(coeffs, band, PhisChan[chan], val_zero[chan]),
							code2value(correct_code(coeffs, band, PhisChan[chan], val_zero[chan]), band));
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = TRES_ABORT;
				}
			}
			sprintf(msg, "band=%2d, chan=%2d code=%f min=%f, corrected: code=%f, value=%f",
							band, PhisChan[chan], val_min[chan], code2value(val_min[chan], band),
							correct_code(coeffs, band, PhisChan[chan], val_min[chan]),
							code2value(correct_code(coeffs, band, PhisChan[chan], val_min[chan]), band));
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = TRES_ABORT;
				}
			}
			sprintf(msg, "band=%2d, chan=%2d code=%f max=%f, corrected: code=%f, value=%f",
							band, PhisChan[chan], val_max[chan], code2value(val_max[chan], band),
							correct_code(coeffs, band, PhisChan[chan], val_max[chan]),
							code2value(correct_code(coeffs, band, PhisChan[chan], val_max[chan]), band));
			if(tc->cbf) {
				if(tc->cbf(TFMSG_REPORT, msg)) {
					ret_code = TRES_ABORT;
				}
			}
#endif
		}
	}

	if(tc->cbf!=NULL) {
		if(tc->cbf(TFMSG_PROGRESS(100.0), NULL)) {
			ret_code = TRES_ABORT;
		}
	};

	/*    . */
	ret_code = write_coeffs(&hltr212, coeffs);

	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		free(AmpFreqDescriptor.buffer[chan]);
	}
	/*   . */
	free(AmpFreqDescriptor.buffer);
	/*     . */
	free(AmpFreqDescriptor.stored);

	/*    . */
	delete_coeffs_table(coeffs);

	return ret_code;

};
