#include <ansi_c.h>
#include <userint.h>
#include <analysis.h>

#include "localdefs.h"
#include "ltr/include/ltrapi.h"
#include "ltr/include/ltr212api.h"
#include "uts/instruments/te01/te01api.h"
#include "uts/instruments/te01/te01func.h"
#include "amp_freq.h"
#include "ltr212clb_coeffs.h"

tDataRecord AmpFreqDescriptor;
extern descr_ltr212 hltr212;
extern IniSet_XXX IniSet;

int configHW(TCONTEXT *tc, descr_ltr212 *hltr212);
void readINI(char *xdir, char *mdescr);
extern int testPlolog(TCONTEXT *tc);
extern void closeModule(void);

INT LTR212_WriteEEPROM_UTS(descr_ltr212 *hnd, INT n, INT Address, BYTE *image);
INT LTR212_ReadEEPROM_UTS(descr_ltr212 *hnd, INT n, INT Address, BYTE *image);

static TCONTEXT theTCONTEXT;
/*static char thePATH[FNAMELEN+1];*/

static HANDLE hThread;
static int (*dataUpdateCBF)(int nchan, serviceStrucData X[], char *info)= NULL;
static UINT32 rd_buf[MAXNPOINTS * MAX_CHANNEL];
static double rd_buf_double[MAXNPOINTS * MAX_CHANNEL];
static double rd_buf_double_u[MAXNPOINTS * MAX_CHANNEL];
static double ibuf[MAX_CHANNEL][MAXNPOINTS];
static double dbuf[MAX_CHANNEL][MAXNPOINTS];
static serviceStrucData serviceDATA[MAX_CHANNEL];

static int runFlag;

static int AF_Pnl= -1;

/*****************************************************************************************/

/**     .

* @param nchan  .
* @param X     .
* @param descriptor   tDataRecord.

* @return 0 -   , 1 -   .
*/
int LC212DataUpdate(int nchan, serviceStrucData X[], void* descriptor) {
	tDataRecord *descr = descriptor;
	int chan, copy_size, done_flag;
	double *buf;

	/*   ,     .*/
	done_flag = 1;
	/*printf("LC212DataUpdate: %f %f\n", X[0].iarray[0], X[0].darray[0]);*/
	/*  . */
	for(chan= 0; chan< nchan; chan++) {
		/*         ,    descr->mode */
		buf = descr->mode==CODE_DATA_MODE ? X[chan].iarray : X[chan].darray;
		/* ,         . */
		if (descr->stored[chan]+X[chan].point>=descr->buf_size) {/*  .*/
			/*   ,    . */
			copy_size = descr->buf_size - descr->stored[chan];
		}
		else {/*   .*/
			/*   . */
			copy_size = X[chan].point;
			/*   ,    . */
			done_flag = 0;
		}
		/*  .  */
		memcpy(descr->buffer[chan]+descr->stored[chan], buf, copy_size*sizeof(double));
		/*       . */
		descr->stored[chan] += copy_size;
	};

	/*   . */
	return done_flag;
};


static void calcFS(double *fsBase, double *fs, int mode, int decimation) {
	int d= 1;
	double xtime;
	double fs_AcqMode[3]={7680.0, 150.15, 150.15};

	*fsBase=fs_AcqMode[mode];
	if(IniSet.common.fir) 
	{
		d= decimation;
		*fs= *fsBase / d;
	} 
	else   
		*fs= *fsBase;

	if(IniSet.common.mode==2)
		*fs=3.4;
};

static int calcTimeOutMs(int points, int mode, int decimation) {
	double to, fbase, fs;

	calcFS(&fbase, &fs, mode, decimation);
	to= 4 * 1e3 * points / fs + 0.5;

	if(to < 10000.0) to= 10000.0;
	return (int)to;
};


/*****************************************************************************************/
/**    .

*    . ,  params->runFlag
*    ( )   params->update_func
*    (   ).

* @param par   tDataRecord.
*/
void LC212DataRecorder(void *par) {
	int    cnt, j, r, p;
	int    channels, points, to, xp, xp2;
	char   tmp[512];
	int    q, dval, nline, tx;
	double fb;
	int bad_sample_num;
	tDataRecord *params = par;
	DataUpdateFunc func = params->update_func;
	time_t begin_time, current_time, end_time;

	points = params->buf_size;
	to= calcTimeOutMs(points, params->mode, hltr212.filter.Decimation);
	time(&begin_time);
	end_time = begin_time + params->timeout;
	params->errorFlag = 0;
	channels = params->channels;
	while(params->runFlag) {
		time(&current_time);
		if (current_time>end_time) {
			params->errorFlag = 1;
			break;
		}

		ProcessSystemEvents();
		/*      . */
		cnt= LTR_Recv(&hltr212.channel, rd_buf, NULL, 2*channels*points, 3*to);

		for(j=0; j<channels; j++) {
			serviceDATA[j].point= 0;
			serviceDATA[j].iarray= NULL;
			serviceDATA[j].darray= NULL;
			serviceDATA[j].fs= 0;
		};

		if(cnt<= 0) goto __skip_processing;

		xp= cnt;
		xp2= xp;

		/*   . */
		// By Milovanov for debug!!!
		r= LTR212_ProcessData (&hltr212, rd_buf, rd_buf_double, &xp, params->mode);
#if 0
		r|= LTR212_ProcessData24 (&hltr212, rd_buf, rd_buf_double_u, &xp2, 1); // !!! tmp    
#endif
		if (r) {
			printf("LTR212_ProcessData24 error:%d\n", r);
			ExitThread(1);
		}
		q= xp/channels;

		if(q!= IniSet.ppoint) if(!q) goto __skip_processing;

		for(j=0; j<channels; j++) {
			serviceDATA[j].point=  q;
			serviceDATA[j].iarray= ibuf[j];
			serviceDATA[j].darray= dbuf[j];
			calcFS(&fb, &serviceDATA[j].fs, params->mode, hltr212.filter.Decimation);
			for(p= 0; p< q; p++) {
				if (params->mode) {
					dbuf[j][p]= rd_buf_double[j + p*channels];  // !!! tmp
				} else {
					ibuf[j][p]= rd_buf_double[j + p*channels];
				}
			};
		};

		/*     .    !=0,   . */
		if (func(channels, serviceDATA, par)) break;
	__skip_processing:

		Sleep(1);
	};


	// runFlag= 2;
	ExitThread(0);

};

// *****************************************************************************************
// *****************************************************************************************

// *****************************************************************************
int GetFrame(TCONTEXT *tc, double *buf) {
	int chan, isample;
	DWORD TmpDWord;
	int ret_code = 0;

	//printf("GetFrame: start!!!  %d\n", AmpFreqDescriptor.channels);
	/*  . */
	if(LTR212_Start(&hltr212)) {
		closeModule();
		return 2;
	};

	/*      . */
	AmpFreqDescriptor.runFlag = 1;
	AmpFreqDescriptor.timeout = 60;
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		AmpFreqDescriptor.stored[chan] = 0;
	}
	/*    . */
	hThread= CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)LC212DataRecorder,
												(void*)&AmpFreqDescriptor, CREATE_SUSPENDED, (LPDWORD)&TmpDWord);

	/*    . */
	ResumeThread(hThread);        // start thread here

	/*     . */
	WaitForMultipleObjects(1, &hThread, TRUE, INFINITE);
	/*    . */
	CloseHandle(hThread);
	/*  . */
	if (ret_code=LTR212_Stop(&hltr212)) {
		closeModule();
		return ret_code;
	};
	if (AmpFreqDescriptor.errorFlag)
		return 3;

	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		buf[chan] = 0.;
		/*  .  . */
		for (isample=0;isample<AmpFreqDescriptor.stored[chan];++isample) {
			buf[chan] += AmpFreqDescriptor.buffer[chan][isample];
		}
		/*  . */
		if (AmpFreqDescriptor.stored[chan])
			buf[chan] /= AmpFreqDescriptor.stored[chan];
	}

	return ret_code;
}


double calculate_freq(double *data, int size) {
	int i, first, last, count, s, sign;

	/*average=0;
	for(i=0;i<size;++i) average+=data[i];
	average/=size;*/
	
	sign = data[0]>=0 ? 1 : -1;
	for(i=1;i<size;++i) {
		s = data[i]>=0 ? 1 : -1;
		if (s!=sign) {
			sign = s;
			break;
		}
	}

	count = 0;
	first = i;
	for(;i<size;++i) {
		s = data[i]>=0 ? 1 : -1;
		if (s!=sign) {
			sign = s;
			++count;
			if (count%2==0) last = i;
		}
	}
	
	count/=2;
	return count ? (double)(last-first)/count : 0.;
}


/** - .
 *
 */
int AmpFreqFunc() {
	HANDLE hThread;
	double ltr_res[MAX_CHANNEL];
	double te01_dac_value;
	int chan, band, ipoint, isample;
	int PhisChan[MAX_CHANNEL];
	int num_points = 10, shift=512;
	double max_freq, freq, freq0;
	/*int image_size = 2000, base_freq_index, step;*/
	TE01 *te01;
	double chan_buffer[1024+512];
	double acEstimate, dcEstimate, min, max;
	char mess[256];
	char file_name[256], bin_name[256];
	/*FILE *res_file = NULL, *bin_file = NULL;*/
	LTR212clb_coeffs** coeffs;
	int err_code;
	double fb, fs, ratio, freq212;
	
	if(testPlolog(&theTCONTEXT)) return 1;  // !!! ??? tmp
	if(configHW(&theTCONTEXT, &hltr212)) {
		closeModule();
		return 1;
	};

	ResetTextBox (AF_Pnl, AF_PANEL_MESSAGES, "");
	DeleteGraphPlot (AF_Pnl, AF_PANEL_AFC_GRAPH, -1, VAL_IMMEDIATE_DRAW);
	GetCtrlVal (AF_Pnl, AF_PANEL_MAX_FREQ, &max_freq);
	GetCtrlVal (AF_Pnl, AF_PANEL_POINTS, &num_points);
	GetCtrlVal (AF_Pnl, AF_PANEL_FILENAME, file_name);
	/*if (*file_name) {
		res_file = fopen(file_name, "wb");
	}*/
	
	te01 = (TE01 *)theTCONTEXT.te01;	
	TE01ReadCoeffFromEEPROM(te01);

	/*    . */
	coeffs = create_coeffs_table();
	if(coeffs==NULL)
		return 1;

	/*   . */
	if (read_coeffs(&hltr212, coeffs)) {
		delete_coeffs_table(coeffs);
		return 1;
	}

	/*     . */
	/*    . */
	AmpFreqDescriptor.update_func = LC212DataUpdate;
	/*    . */
	AmpFreqDescriptor.channels = 0;
	for(chan=0; chan<MAX_CHANNEL; ++chan) {  //  : 
		if (IniSet.chan[chan].onoff) {
			PhisChan[AmpFreqDescriptor.channels++] = chan+1;
		}
	};

	if (AmpFreqDescriptor.channels==0)
		return 1;

	band = 3;
	hltr212.LChQnt = AmpFreqDescriptor.channels;
	/*sprintf(mess, "hltr212.LChQnt=%d", hltr212.LChQnt);
	if(tc->cbf) tc->cbf(TFMSG_REPORT, msg);*/
	for(chan=0;chan<hltr212.LChQnt;++chan){
		hltr212.LChTbl[chan]=LTR212_CreateLChannel(PhisChan[chan], band);
	}

	err_code = LTR212_SetADC(&hltr212);
	if (err_code) {
		sprintf(mess, "LTR212_SetADC:%d", err_code);
		InsertTextBoxLine (AF_Pnl, AF_PANEL_MESSAGES, -1, mess);
	}
	
	/*    . */
	AmpFreqDescriptor.buf_size = sizeof(chan_buffer)/sizeof(chan_buffer[0]);
	/*    . */
	AmpFreqDescriptor.mode = VALUE_DATA_MODE/*VALUE_DATA_MODE*/;
	/*   . */
	AmpFreqDescriptor.buffer = (double**)malloc(AmpFreqDescriptor.channels*sizeof(double*));
	/*     . */
	AmpFreqDescriptor.stored = (int*)malloc(AmpFreqDescriptor.channels*sizeof(int));
	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		AmpFreqDescriptor.buffer[chan] = (double*)malloc(AmpFreqDescriptor.buf_size*sizeof(double));
	}

	hltr212.UseClb=0;
	hltr212.AcqMode = 0;
#if 0
	hltr212.codec.CHOP=0;
	hltr212.codec.SKIP=1;
	hltr212.codec.FAST=0;
	hltr212.codec.AC=0;
	hltr212.codec.Delay=0;
#endif

	AmpFreqDescriptor.runFlag = 1;

	/*   . */
	te01_dac_value = 50;
	for (ipoint=0;ipoint<num_points;++ipoint) {
		ProcessSystemEvents();
		if (!AmpFreqDescriptor.runFlag) break;

		freq = (ipoint+1)*(max_freq/num_points);
		FDAC_SetSin(te01, &freq, te01_dac_value, /*7*/1);
		Sleep(1000);
#if 1
		if (err_code = GetFrame(&theTCONTEXT, ltr_res))
			break;

		/*  . */
		for (chan=0;chan<1/*AmpFreqDescriptor.channels*/;++chan) {
			ltr_res[chan] = 0.;
			/*  .  . */
			min = max = AmpFreqDescriptor.buffer[chan][0];
			for (isample=0;isample<AmpFreqDescriptor.stored[chan];++isample) {
				chan_buffer[isample] = correct_value(coeffs, band, PhisChan[chan], AmpFreqDescriptor.buffer[chan][isample]*1000.);
				if (min>chan_buffer[isample]) min=chan_buffer[isample];
				if (max<chan_buffer[isample]) max=chan_buffer[isample];
			}
			/* . */
			/*shift=0;*/
			/*sprintf(bin_name, "e:/prj/uts/systems/ltr/ltr212_metr/bin/%05d.dat", (int)freq);
			bin_file = fopen(bin_name, "wb");
			fwrite((void*)chan_buffer, sizeof(chan_buffer[0]), AmpFreqDescriptor.stored[0], bin_file);*/
			/*for (isample=0;isample<AmpFreqDescriptor.stored[chan];++isample) {
			}*/
			/*fclose(bin_file);*/
			ACDCEstimator (chan_buffer+shift, AmpFreqDescriptor.stored[chan]-shift, &acEstimate, &dcEstimate);
			calcFS(&fb, &fs, hltr212.AcqMode, hltr212.filter.Decimation);
			ratio = calculate_freq(chan_buffer+shift, AmpFreqDescriptor.stored[chan]-shift);
			freq212 = -1.;
			if (ratio)
				freq212=fs/ratio;
			sprintf(mess, "frequency=%8.2fHz    %8.5e f=%12.6f r=%9.6f",
			        freq, 20*log10(acEstimate*sqrt(2.)/te01_dac_value), freq212, ratio);
			InsertTextBoxLine (AF_Pnl, AF_PANEL_MESSAGES, -1, mess);
		}

		PlotPoint(AF_Pnl, AF_PANEL_AFC_GRAPH, freq, 20*log10(acEstimate*sqrt(2.)/te01_dac_value), VAL_SOLID_SQUARE, VAL_BLACK);
		/*if (res_file)
			fprintf(res_file, "%15.5f\t\t%15.5e\n", freq, 20*log10(acEstimate*sqrt(2.)/te01_dac_value));*/
#endif			
	}

	InsertTextBoxLine (AF_Pnl, AF_PANEL_MESSAGES, -1, "Complete");

	/*if (res_file)
		fclose(res_file);*/

	/*  . */
	for (chan=0;chan<AmpFreqDescriptor.channels;++chan) {
		free(AmpFreqDescriptor.buffer[chan]);
	}
	/*   . */
	free(AmpFreqDescriptor.buffer);
	/*     . */
	free(AmpFreqDescriptor.stored);
	/*    . */
	delete_coeffs_table(coeffs);

	return 0;
}

int CVICALLBACK stop_amp_freq (int panel, int control, int event,
															 void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			AmpFreqDescriptor.runFlag = 0;
			break;
		}
	return 0;
}

int CVICALLBACK quit_vdevice (int panel, int control, int event,
															void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			//if (manual_started) return 0;
			DiscardPanel(AF_Pnl);
			AF_Pnl= -1;
			break;
		}
	return 0;
}

int CVICALLBACK start_amp_freq (int panel, int control, int event,
																void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			SetCtrlAttribute (AF_Pnl, AF_PANEL_START_BUTTON, ATTR_DIMMED, 1);
			SetCtrlAttribute (AF_Pnl, AF_PANEL_STOP_BUTTON, ATTR_DIMMED, 0);
			SetCtrlAttribute (AF_Pnl, AF_PANEL_QUIT_BUTTON, ATTR_DIMMED, 1);

			AmpFreqFunc();

			SetCtrlAttribute (AF_Pnl, AF_PANEL_START_BUTTON, ATTR_DIMMED, 0);
			SetCtrlAttribute (AF_Pnl, AF_PANEL_STOP_BUTTON, ATTR_DIMMED, 1);
			SetCtrlAttribute (AF_Pnl, AF_PANEL_QUIT_BUTTON, ATTR_DIMMED, 0);
			
			break;
		}
	return 0;
}

int CVICALLBACK Load_From_Flash_CLB (int panel, int control, int event,
																		 void *callbackData, int eventData1, int eventData2)
{


	int EEPROM_Addr;		//  
	BYTE EEPROM_Data[7]; // ,       
	int err;
	int Phys_Channel;
	int i;
	BYTE Reg_LByte;
	BYTE Reg_MByte;
	BYTE Reg_HByte;

	int OffsetReg_Val;
	int GainReg_Val;

	switch (event)
		{
		case EVENT_COMMIT:


			for(Phys_Channel=0;Phys_Channel<=4;Phys_Channel+=2)

				{
					EEPROM_Addr=Phys_Channel*7;   //         

					LTR212_ReadEEPROM_UTS(&hltr212, 7, Phys_Channel*7, EEPROM_Data);

					Reg_LByte=EEPROM_Data[0];
					Reg_MByte=EEPROM_Data[1];
					Reg_HByte=EEPROM_Data[2];

					OffsetReg_Val=Reg_LByte|(Reg_MByte<<8)|(Reg_HByte<<16);

					Reg_LByte=EEPROM_Data[3];
					Reg_MByte=EEPROM_Data[4];
					Reg_HByte=EEPROM_Data[5];

					GainReg_Val=Reg_LByte|(Reg_MByte<<8)|(Reg_HByte<<16);
					/*
						DAC_Value=EEPROM_Data[6];

						reg_name=OFFSET_REG;
						err=LTR212_Write_24bitReg(h, reg_name, OffsetReg_Val);
						if(err) return(err);

						reg_name=GAIN_REG;
						err=LTR212_Write_24bitReg(h, reg_name, GainReg_Val);
						if(err) return(err);
					*/
				}

			break;
		}
	return 0;
}

int metrVirtualDevice(TCONTEXT *tc, void *p) {
	char ppath[FNAMELEN+1], tmp[FNAMELEN+1], *ch;

	memcpy(&theTCONTEXT, tc, sizeof(TCONTEXT));

	readINI(tc->dllpath, tc->mdescr);
	strcpy(tmp, tc->dllpath);
	ch = tmp + strlen(tmp);
	while(ch>tmp && *ch!='\\' && *ch!='/')
		--ch;
	*ch = '\0';
	sprintf(ppath, "%s\\%s\\%s", tmp, "ltr212_metr", "amp_freq.uir");
	/*strcpy(thePATH, ppath);*/

	AF_Pnl= LoadPanelEx (0, ppath, AF_PANEL, __CVIUserHInst);

	InstallPopup(AF_Pnl);
	return 0;
};
