#include <math.h>
#include "te01func.h"
#include "uts\\instruments\\te01\\te01api.h"


/*===============================================================================================*/
static const int  ext_aregaddr = 3;      /*   EXT (  ) 
                                          *  */
/*     TE-01   (     ) */
static const long te01_fdac_ramfreq[TE01_FDAC_FREQ_QNT] = {20000000, 2000000, 200000, 20000, 2000};
static const int  sel_aregaddr = 1;      /*   SEL   */
static const int  te01a11_bid  = 0x17;   /*     TE-01A-11 */
static const int  u_aregaddr   = 2;      /*   U   */
/*===============================================================================================*/

/*===============================================================================================*/
/*-----------------------------------------------------------------------------------------------*/
int Check_TE01A11(TE01 *te01)
{
    /*
     *     TE-01A-11.
     * 
     *         ,   
     *       .    
     *      .
     *        ,    
     *     TE-01A-11    .
     *  
     *            .
     * 
     *   te01 -     .
     *  
     *   0 -       ,   TE-01A-11;
     *   1 -   TE-01A-11   .
     */
     
     return te01->bid == te01a11_bid;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
double GetUADC_Val(TE01 *te01)
{
    double value;


    TE01UMeasurement(te01, &value, 8, 1000.0, NULL);
    value = TE01UModuleValue(te01, value, 1);
    /*      . */
    value = value * te01->Ku[te01->uadc_range] + te01->Bu[te01->uadc_range];

    return value;
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetADC_Range(TE01 *te01, int range)
{
    /*
     * \brief     TE-01.
     *
     *
     * \param range  
     *  - 0 10 
     *  - 1 2.5  ( !)
     *  - 2 500 
     *  - 3 100 
     */

    te01->uadc_range = range;
    TE01SetConfiguration(te01, FORCE_UPDATE_UMODULE);
    Sleep(3000);                  /* ToDo: ???? */
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetAModule_Config(TE01 *te01, int ch1_connection, int ch2_connection, int gnd32_connection,
                       int voltmeter_connection)
{
    /*
     *    .
     * 
     *            
     *     .
     * 
     *   te01             -    ;
     *   ch1_connection   -    1, 5, 9, 13  LTR11;
     *   ch2_connection   -    2, 3, 4, 6, 7, 8, 10, 11, 12, 14, 15, 16
     *                       LTR11;
     *   gnd32_connection -    GND32  LTR11.
     */
    
    /*        SEL   
     *  LTR-11
     */
    te01->aregs[sel_aregaddr] = ch1_connection & 0x03 | ((gnd32_connection & 0x03) << 4);
    if (ch2_connection == USPTOX_US0TOY_CONNECT)  /*   SEL    */
        ch2_connection = 0x00;
    te01->aregs[sel_aregaddr] |= (ch2_connection & 0x03) << 2;
    te01->aregs[u_aregaddr]    = voltmeter_connection & 0x03;
    TE01SetConfiguration(te01, FORCE_UPDATE_AMODULE);
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetExt_Inputs(TE01 *pte01, int inputs)
{
    /*
     *   .
     * 
     *            .
     * 
     *   pte01  -    ;
     *   inputs -    .
     */
    
    pte01->aregs[ext_aregaddr] = inputs;
    TE01SetConfiguration(pte01, FORCE_UPDATE_AMODULE);
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetFDAC_DCValue(TE01 *pte01, double val, int range)
{
    /*
     *       .
     * 
     *        .
     *         :
     *     -  RAM  1 (      );
     *     -    0;
     *     -          ;
     *     -    2  (  );
     *     -      400  (  ..  ).
     * 
     *   pte01 -    ;
     *   val   -      .
     *   range -    (.   ).
     */
     
     /*        */
     val = -pte01->Kf2[range] * val + pte01->Bf2[range];
     TE01SetFDACConfiguration(pte01, 1, 0, 4, range, 3, val, val, FORMAT_VALUE);
     TE01SetConfiguration(pte01, FORCE_UPDATE_FDAC);
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetFDAC_Sinus(TE01 *pte01, double *freq, double amp, int filt, int range)
{
    /*
     *    TE-01      .
     * 
     *           EEPROM TE-01  RAM 
     *     .
     *        :
     *     -           
     *       :
     *         f = ram_freq / (ram_size / ram_step)
     *       ..
     *         ram_step = f * ram_size / ram_freq
     *        ram_step  ram_freq     ram_freq, 
     *             ram_step >= 1  ram_step <= max_ram_step
     *       (     ,        1);
     *     -        .
     * 
     *   pte01 -     TE-01;
     *   freq  -      ,     , 
     *                 ;
     *   amp   -     ;
     *   filt  -    (.   TE01API);
     *   range -    (.   TE01API).
     */

    double    f;
    const int filt_off = 0;         /*     */
    double    ip;
    const int max_ram_step = 100;
    int       ram_freq;
    int       ram_step;
    const int signal_ramsize = 20000;
    
    
    /*      */
    for (ram_freq = 0; ram_freq < TE01_FDAC_FREQ_QNT; ram_freq++)
    {
        ram_step = *freq * signal_ramsize / te01_fdac_ramfreq[ram_freq];
        f        = te01_fdac_ramfreq[ram_freq] * ram_step / signal_ramsize;
        if (ram_step >= 1 && modf(f, &ip) == 0.0 || ram_step >= max_ram_step)
            break;
    }
    if (ram_freq >= TE01_FDAC_FREQ_QNT)   /*     (  ) */
    {
        /*     */
        ram_freq = TE01_FDAC_FREQ_QNT - 1;
        ram_step = max_ram_step;
    }
    /*     */
    *freq = te01_fdac_ramfreq[ram_freq] * ram_step / signal_ramsize;
    
    /*           */
    if (filt == filt_off)                 /*      */
        amp = -pte01->Kf1[range] * amp + pte01->Bf1[range];
    else
        amp = -pte01->Kf2[range] * amp + pte01->Bf2[range];
        
    /*     RAM TE-01 */
    TE01LoadRAMFromEEPROM(pte01, 0, 0, signal_ramsize);
    
    /*        */
    TE01SetFDACConfiguration(pte01, signal_ramsize, ram_step, ram_freq, range, filt, amp, -amp,
        FORMAT_VALUE);
    TE01SetConfiguration(pte01, FORCE_UPDATE_FDAC);
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetSDAC_Value(TE01 *te01, double t_val, char range)
{
    double value;


    te01->sdac_range = range;
    value            = te01->Ks1[range] * t_val + te01->Bs1[range];
    TE01SetSDACConfiguration(te01, range, value, 1);
    /* ToDo: FORCE_UPDATE_UMODULE      API    ADC */
    TE01SetConfiguration(te01, FORCE_UPDATE_SDAC | FORCE_UPDATE_UMODULE);
    if (range == 0) Sleep(1000); /*      0  */
    else            Sleep(1000); /* ToDo: ??? */
}
/*-----------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------*/
void SetUADC_Config(TE01 *te01, int range)
{
    /*
     *   .
     * 
     *       ,      
     *   .
     * 
     *   te01  -     TE-01;
     *   range -    (.    TE-01).
     */
     
    TE01SetUADCConfiguration(te01, 9, range, 4000);//ADC_FILTER_CONST / 10);
    TE01SetConfiguration(te01, FORCE_UPDATE_AMODULE | FORCE_UPDATE_UMODULE);
}
/*-----------------------------------------------------------------------------------------------*/
/*===============================================================================================*/
