/*
 *      LTR11.
 *     (, ,   ..)   LTR11.
 *              .
 *          .
 *          .
 */


#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include "ltr11api.h"
#include "ltr\\include\\ltrapi.h"
#include "ltr\\include\\ltrapidefine.h"
#include "lib\\crc\\crc.h"


/*================================================================================================*/
/*    (  ,  0 -  -1000, 1 - -1001  ..) */
const LPCSTR err_str[] =
{
    "     NULL",
    "   ",
    "   ",
    "     ",
    "     ",
    "      ",
    "    ",
    "    ",
    "      ",
    "    ",
    "    NULL",
    "        ",
    "        NULL",
    "    ",
    "    LTR11",
    "   LTR11",
    "    LTR11",
    "        ",
    "        ",
    "    "
};

/*      LTR-11 */
static const int min_err_id = 1000;                   /*   () */

/*     LTR11 */
/* 
 *         (0x8000)  
 *  (6-).
 */
static const unsigned long ack_mask = 0x803F;         /*     */
static const unsigned long modulemode_ack = 0x8012;   /*    */
static const unsigned long start_ack = 0x8011;        /*     */
static const unsigned long stop_ack = 0x8010;         /*     */

/*     */
static const unsigned dummy_cmd = 0x00;               /*    */
static const unsigned getconfig_cmd = 0x0C;           /*    */
static const unsigned modulemode_cmd = 0x18;          /*    */
static const unsigned start_cmd = 0x0A;               /*    */
static const unsigned stop_cmd = 0x05;                /*     */

/* -   */
static const DWORD ack_tm_out = 6000;                 /*    */
static const DWORD send_tm_out = 2000;                /*   */

/* ,      */
static const long max_divider = 65535;                /*    */
static const long min_divider = 2;                    /*    */
/*     ,    */
static const int prescalers[] = {1, 8, 64, 256, 1024};
/*================================================================================================*/

/*================================================================================================*/
static double eval_adcrate(int p, long d);
static int eval_parity(DWORD cmd);
static DWORD f_elapsed_time(DWORD start_time);
static DWORD fill_command(unsigned cmd, unsigned param);
static INT start_ltr11(PTLTR11 hnd, int frame);
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
int WINAPI DllEntryPoint
    (
    HINSTANCE hinst,
    unsigned long reason,
    void* lpReserved
    )
    {
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static double eval_adcrate
    (
    int p,
    long d
    )
    {
    /*
     *    .
     * 
     *          .
     * 
     *   p -  ;
     *   d -  .
     *  
     *     .     - 0.
     */

    double denom;
    double ret_val = 0.0;


    if ((p == 1) && (d == 36))                /*   -  400  */
        {
        ret_val = 400.0;
        }
    else if ((denom = (double)p * ((double)d + 1)) > 0)
        {
        ret_val = LTR11_CLOCK / denom;
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int eval_parity
    (
    DWORD cmd
    )
    {
	unsigned long index;
	int parity;
	
	
	parity = 0;
	for (index = 0x01UL; (index <= 0x10UL); index <<= 1)
        {
		parity ^= (cmd & index) != 0;
        }
		
	for (index = 0x80000000UL; (index >= 0x010000UL); index >>= 1)
        {
		parity ^= (cmd & index) != 0;
        }
	
	
	return parity;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static DWORD f_elapsed_time
    (
    DWORD start_time
    )
    {
    DWORD elapsed_time;
    DWORD sys_time;


    sys_time = GetTickCount();
    if (sys_time >= start_time) elapsed_time = sys_time - start_time;
    else                        elapsed_time = (65535 - (start_time - sys_time)) + 1;

    return elapsed_time;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static DWORD fill_command
    (
    unsigned cmd,
    unsigned param
    )
    {
	DWORD ret_cmd;
	
	
	ret_cmd = 0x80C0U | (DWORD)param << 16 | (DWORD)cmd & 0x1F;
	ret_cmd |= eval_parity(ret_cmd) << 5;
	
	return ret_cmd;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_Close
    (
    PTLTR11 hnd
    )
    {
    /*
     *     LTR11.
     * 
     *       .
     * 
     *   hnd -    .
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else
        {
        DWORD cmd = LTR010CMD_STOP;
        
        LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out);		
        ret_val = LTR_Close(&hnd->Channel);
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_GetConfig
    (
    PTLTR11 hnd
    )
    {
    /*
     *      LTR11.
     * 
     *             
     *          ,  .
     *      (   2-   ):
     *                        
     *      0              
     *      1-2          
     *      3-16           
     *      17-24         
     *      25-40          
     *      41-44            
     *      45-48            
     *      49-52            
     *      53-56            
     *      57-60            
     *      61-64            
     *      65-68            
     *      69-72            
     *      73-74         
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */

#define CONFIG_DATA_SIZE (75)                    /*      */

    BYTE cfg_arr[CONFIG_DATA_SIZE];
    /*     */
    const int cfg_signature = 0x5A;
    /* -          */
    const DWORD cmd_tmout = 10;
    int i;
    DWORD rd_buf[CONFIG_DATA_SIZE * 4];          /*    4-  */
    INT ret_val = LTR_OK;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else
        {
        DWORD cmd;
        long n;

        cmd = fill_command(getconfig_cmd, 0);
        if ((n = LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out)) < 0)
            {
            ret_val = n;
            }
        else if (n != 1)
            {
            ret_val = LTR_ERROR_SEND;
            }
        else if ((n = LTR_Recv(&hnd->Channel, rd_buf, NULL, CONFIG_DATA_SIZE * 4,
                     CONFIG_DATA_SIZE * 4 * cmd_tmout + ack_tm_out)) < 0
                )
            {
            ret_val = n;
            }
        else if (n != CONFIG_DATA_SIZE * 4)           /*      */
            {
            ret_val = LTR11_ERR_GETCFG;
            }
        else
            {
            /*         */
            if ((rd_buf[0] & 0x803CUL) != 0x801CUL)   /*    */
                {
                ret_val = LTR11_ERR_CFGDATA;
                }
            else
                {
                /*   */
                int cnt = (rd_buf[0] >> 6) & 0x03UL;
                int j;
                DWORD *pbuf = rd_buf;

                rd_buf[0] &= ~0x10UL;                 /*     */
                for (i = 0; ((i < CONFIG_DATA_SIZE) && (ret_val == LTR_OK)); i++)
                    {
                    cfg_arr[i] = 0;
                    for (j = 6; ((j >= 0) && (ret_val == LTR_OK)); j-= 2)
                        {
                        if ((*pbuf & 0x803CUL) != 0x800CUL)
                            {
                            ret_val = LTR11_ERR_CFGDATA;
                            }
                        else
                            {
                            cfg_arr[i] |= (*pbuf & 0x03UL) << j;
                            cnt = (cnt + 1) & 0x03;
                            }
                        pbuf++;
                        }
                    }
                }

            /*        */
            if (ret_val == LTR_OK)
                {
                if (cfg_arr[0] != cfg_signature)
                    {
                    ret_val = LTR11_ERR_CFGSIGNATURE;
                    }
                else if (eval_crc16(0, cfg_arr, CONFIG_DATA_SIZE - 2) !=
                             (((unsigned short)cfg_arr[CONFIG_DATA_SIZE - 2] << 8 |
                             (unsigned short)cfg_arr[CONFIG_DATA_SIZE - 1]) & 0xFFFFU)
                        )
                    {
                    ret_val = LTR11_ERR_CFGCRC;
                    }
                else
                    {
                    float *pcfg;

                    hnd->ModuleInfo.Ver = (unsigned)cfg_arr[1] << 8 | (unsigned)cfg_arr[2];
                    (void)memcpy((char *)hnd->ModuleInfo.Date, (char *)(cfg_arr + 3), 14);
                    (void)memcpy((char *)hnd->ModuleInfo.Name, (char *)(cfg_arr + 17), 8);
                    (void)memcpy((char *)hnd->ModuleInfo.Serial, (char *)(cfg_arr + 25), 16);
                    pcfg = (float *)(cfg_arr + 41);
                    for (i = 0; (i < LTR11_ADC_RANGEQNT); i++)
                        {
                        hnd->ModuleInfo.CbrCoef[i].Offset = *pcfg++;
                        hnd->ModuleInfo.CbrCoef[i].Gain = *pcfg++;
                        }
                    }
                }
            } /*else @else if (n != CONFIG_DATA_SIZE * 4)@*/
        } /*else @if (hnd->Channel == NULL)@*/

    return ret_val;
#undef CONFIG_DATA_SIZE
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(LPCSTR) LTR11_GetErrorString
    (
    INT err
    )
    {
    /*
     *      ,    .
     * 
     * 
     *   err -   (  ),     LTR11.
     *  
     *         ,   .
     *   NULL -   .
     */
     
    const int err_qnt = sizeof err_str / sizeof err_str[0];
    LPCSTR ret_val = NULL;

    
    if ((err > -min_err_id) || (err <= -(min_err_id + err_qnt)))
        {
        /*    (     ) */
        /*         */
        ret_val = LTR_GetErrorString(err);
        }
    else
        {
        /*          LTR-11 */
        ret_val = err_str[-err - min_err_id];
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_GetFrame
    (
    PTLTR11 hnd,
    DWORD *buf
    )
    {
    /*
     *       LTR11.
     * 
     *    LTR11       .
     *          .    
     *           - ( -
     *      ).
     * 
     *   hnd -     ;
     *   buf -   ,        .
     *  
     *      0 -     .
     *           -   (.  ).
     */

    DWORD rd_buf[LTR11_MAX_LCHANNEL + 10];
    INT ret_val = 0;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else if ((hnd->LChQnt <= 0) || (LTR11_MAX_LCHANNEL < hnd->LChQnt))
        {
        ret_val = LTR11_ERR_INVALID_ADCLCHQNT;
        }
    else if (hnd->ChRate <= 0.0)
        {
        ret_val = LTR11_ERR_INVALID_ADCRATE;
        }
    else if ((ret_val = start_ltr11(hnd, 1)) == LTR_OK)
        {
        long n;
        DWORD *pbuf;
        DWORD tick;
        DWORD tm_out;

        tick = GetTickCount();
        /*   -       */
        tm_out = (DWORD)(1 / hnd->ChRate + ack_tm_out + 0.5);
        pbuf = rd_buf;
        do
            {
            if ((n = LTR_Recv(&hnd->Channel, pbuf, NULL, hnd->LChQnt + 1, tm_out)) < 0)
                {
                ret_val = n;
                }
            else if (n > 0)
                {
                unsigned a;

                /*     (  ) */
                pbuf += n;
                a = pbuf[-1] & ack_mask;
                if (a == stop_ack)                   /*     */
                    {
                    ret_val = pbuf - rd_buf;
                    if (--ret_val <= 0)
                        {
                        ret_val = LTR11_ERR_GETFRAME;
                        }
                    else
                        {
                        (void)memcpy(buf, rd_buf, ret_val * sizeof(DWORD));
                        }
                    }
                else if (n < hnd->LChQnt + 1)        /*      */
                    {
                    ret_val = LTR11_ERR_GETFRAME;
                    }
                }
            } while ((f_elapsed_time(tick) <= tm_out) && (ret_val == LTR_OK) && (n > 0));
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_Init
    (
    PTLTR11 hnd
    )
    {
    /*
     *    LTR11.
     * 
     *         " ".
     * 
     *   hnd -    .
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else if ((ret_val = LTR_Init(&hnd->Channel)) == LTR_OK)
        {
        const char date_def[14] = "\0\0\0\0\0\0\0\0\0\0\0\0\0";
        int i;
        const char name_def[16]  = "LTR11\0\0\0\0\0\0\0\0\0\0";
        const char serial_def[24] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";

        hnd->size = sizeof(TLTR11);
        hnd->StartADCMode = LTR11_STARTADCMODE_INT;
        hnd->InpMode = LTR11_INPMODE_INT;
        hnd->LChQnt = 0;
        for (i = 0; (i < LTR11_MAX_LCHANNEL); i++)
            {
            hnd->LChTbl[i] = 0;
            }
        hnd->ADCMode = LTR11_ADCMODE_ACQ;
        hnd->ADCRate.divider = 36;               /* 400  */
        hnd->ADCRate.prescaler = 1;
        hnd->ChRate = eval_adcrate(hnd->ADCRate.prescaler, hnd->ADCRate.divider);
        for (i = 0; (i < LTR11_ADC_RANGEQNT); i++)
            {
            hnd->ModuleInfo.CbrCoef[i].Offset = 0.0;
            hnd->ModuleInfo.CbrCoef[i].Gain   = 1.0;
            }
        hnd->ModuleInfo.Ver = 0;
        (void)strcpy((char *)hnd->ModuleInfo.Date, date_def);
        (void)strcpy((char *)hnd->ModuleInfo.Name, name_def);
        (void)strcpy((char *)hnd->ModuleInfo.Serial, serial_def);
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_Open
(
 PTLTR11 hnd,
 DWORD net_addr,
 WORD net_port,
 CHAR *crate_sn,
 INT slot_num
 )
{
	/*
	*     LTR11.
	* 
	*            .
	* 
	*   hnd      -    ;
	*   net_addr -    (  )   (hex):
	*              MsbAABBCCDDLsb - AA.BB.CC.DD;
	*   net_port -   ;
	*   crate_sn -   ,    ;
	*   slot_num -   ,    .
	*  
	*     (.  ).
	*/

	const int max_slotnum = 16;
	const int min_slotnum = 1;
	INT ret_val = LTR_OK;
	int WarningRes = LTR_OK;


	if (hnd == NULL)
	{
		ret_val = LTR11_ERR_INVALID_DESCR;
	}
	else if (crate_sn == NULL)
	{
		ret_val = LTR11_ERR_INVALID_CRATESN;
	}
	else if ((slot_num < min_slotnum) || (max_slotnum < slot_num))
	{
		ret_val = LTR11_ERR_INVALID_SLOTNUM;
	}
	else
	{
		if (LTR_IsOpened(&hnd->Channel) == LTR_OK)
			LTR_Close(&hnd->Channel);

		hnd->Channel.saddr = net_addr;
		hnd->Channel.sport = net_port;
		(void)strncpy((char *)hnd->Channel.csn,(char *)crate_sn, 16);
		hnd->Channel.cc = slot_num;                   /*   -    */

		WarningRes =LTR_Open(&hnd->Channel);
		if(WarningRes==LTR_OK||WarningRes==LTR_WARNING_MODULE_IN_USE) ret_val=LTR_OK;
		else ret_val = WarningRes;

		if (ret_val == LTR_OK)
		{
#define CMD_BUFSZ (21)
			DWORD cmd[CMD_BUFSZ];			
			long n;

			cmd[0] = LTR010CMD_STOP;
			cmd[1] = LTR010CMD_RESET;			
			cmd[2] = LTR010CMD_STOP;
			if ((n = LTR_Send(&hnd->Channel, cmd, 3, send_tm_out)) < 0)
			{
				ret_val = n;
			}
			else if (n != 3)
			{
				ret_val = LTR_ERROR_SEND;
			}
			else
			{
				int ack_received;
				const unsigned long ltr11_id = 0x0B0B;  /*  LTR-11   */
				DWORD rd_buf;                
				DWORD start_systick;


				ack_received = 0;
				start_systick = GetTickCount();
				while ((ret_val == LTR_OK) && !ack_received)
				{
					if ((n = LTR_Recv(&hnd->Channel, &rd_buf, NULL, 1, ack_tm_out)) < 0)
					{
						ret_val = n;
					}
					else if (n == 0)
					{
						ret_val = LTR11_ERR_NOACK;          /*      */
					}
					else if ((rd_buf & 0xF0FF) == LTR010CMD_RESET) 
					{
						if (((rd_buf >> 16) & 0xFFFFUL) != ltr11_id)
						{
							ret_val = LTR11_ERR_MODULEID;
						}
						else
						{
							ack_received = 1;
						}
					}

					/*   -     */
					if (!ack_received)
					{
						int timeout_expired;

						timeout_expired = (f_elapsed_time(start_systick) > ack_tm_out);
						if (timeout_expired)
							ret_val = LTR11_ERR_INVALIDACK;
					}
				}
			} /*else @else if (n != 2)@*/
			Sleep(200); // ,       
			/*          */
			/*
			for (cmd_index = 0; (cmd_index < CMD_BUFSZ - 1); cmd_index++)
			{
			cmd[cmd_index] = LTR010CMD_STOP;
			}
			cmd[CMD_BUFSZ-1] = fill_command(dummy_cmd, 0);
			if ((n = LTR_Send(&hnd->Channel, cmd, CMD_BUFSZ, send_tm_out)) < 0)
			{
			ret_val = n;
			}
			else if (n != CMD_BUFSZ)
			{
			ret_val = LTR_ERROR_SEND;
			}
			*/
#undef CMD_BUFSZ
		} /*if ((ret_val = LTR_Open(&hnd->Channel)) == LTR_OK)*/
	} /*else @else if ((slot_num < min_slotnum) || (max_slotnum < slot_num))@*/	

	if(WarningRes==LTR_WARNING_MODULE_IN_USE&&ret_val==LTR_OK) ret_val=WarningRes;
	return ret_val;
}
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_ProcessData
    (
    PTLTR11 hnd,
    DWORD *src,
    double *dest,
    INT *size,
    BOOL calibr,
    BOOL volt
    )
    {
    /*
     *     .
     * 
     *      (    
     *     ) ,  ,    .
     * 
     *   hnd    -    ;
     *   src    -       ;
     *   dest   -   ,      ;
     *   size   -       (..   ),   
     *                     
     *            ;
     *   calibr -     ;
     *   volt   -        (   ).
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else if ((hnd->LChQnt <= 0) || (LTR11_MAX_LCHANNEL < hnd->LChQnt))
        {
        ret_val = LTR11_ERR_INVALID_ADCLCHQNT;
        }
    else if ((src == NULL) || (dest == NULL))
        {
        ret_val = LTR11_ERR_INVALID_ARRPOINTER;
        }
    else
        {
        /*         */
        struct
            {
            double adc_gain;
            double adc_offset;
            int num;
            double range;
            } ch_tbl[LTR11_MAX_LCHANNEL];
        /*       */
        int cnt;
        /*    */
        SHORT frame[LTR11_MAX_LCHANNEL];
        /*     */
        const double ranges[LTR11_ADC_RANGEQNT] = {10.0, 2.5, 0.625, 0.156};
        /*   (.   ) */
        const int resolution = 8192;
        double *pdest = dest;
        DWORD *psrc = src;
        int slot_num = hnd->Channel.cc - CC_MODULE1;
        int vch_ind;
        int vch_qnt;                             /*    */


        /*       */
        vch_qnt = hnd->LChQnt;
        for (vch_ind = 0; (vch_ind < vch_qnt); vch_ind++)
            {
            ch_tbl[vch_ind].num = hnd->LChTbl[vch_ind] & 0x0F;
            cnt = (hnd->LChTbl[vch_ind] >> 6) & 0x03;
            ch_tbl[vch_ind].range = ranges[cnt];
            ch_tbl[vch_ind].adc_offset = hnd->ModuleInfo.CbrCoef[cnt].Offset;
            ch_tbl[vch_ind].adc_gain = hnd->ModuleInfo.CbrCoef[cnt].Gain;
            }

        cnt = (*psrc >> 6) & 0x03;
        vch_ind = 0;
        for (; (psrc < src + *size); psrc++)
            {
            /* :
             *  1)     ( 0x80)    (0x0F)
             *  2)   
             */
            if (((*psrc >> 8) & 0x8F) != slot_num)
                ret_val = LTR11_ERR_ADCDATA_SLOTNUM;
            if (cnt != ((*psrc >> 6) & 0x03))
                ret_val = LTR11_ERR_ADCDATA_CNT;

            if ((*psrc & 0x0F) != ch_tbl[vch_ind].num)   /*      */
                {                                        /*   */
                ret_val = LTR11_ERR_ADCDATA_CHNUM;
                vch_ind = 0;                             /*     */
                }
            else
                {
                /*    */
                frame[vch_ind] = (SHORT)(*psrc >> 16 & 0xFFFFU);
                if (++vch_ind >= vch_qnt)                /*     */
                    {
                    /*        */
                    for (vch_ind = 0; (vch_ind < vch_qnt); vch_ind++)
                        {
                        double dt;

                        dt = frame[vch_ind];
                        /*    */
                        if (calibr == TRUE)
                            dt = (dt + ch_tbl[vch_ind].adc_offset) * ch_tbl[vch_ind].adc_gain;
                        if (volt == TRUE)                /*    */
                            dt = dt * ch_tbl[vch_ind].range / resolution;
                        *pdest++ = dt;
                        }
                    vch_ind = 0;
                    }
                }
            cnt = (cnt + 1) & 0x03;
            }
        *size = pdest - dest;
        } /*else @else if ((src == NULL) || (dest == NULL))@*/

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_SetADC
    (
    PTLTR11 hnd
    )
    {
    /*
     *     LTR11.
     * 
     *        ,     
     *          .
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */

#if (LTR11_MAX_LCHANNEL % 2 != 0)
#error Channel quantity must be even
#endif
#define PARAM_ARR_SIZE (LTR11_MAX_LCHANNEL / 2 + 3)

    INT ret_val = LTR_OK;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else if ((hnd->LChQnt <= 0) || (LTR11_MAX_LCHANNEL < hnd->LChQnt))
        {
        ret_val = LTR11_ERR_INVALID_ADCLCHQNT;
        }
    else
        {
        /*     
         * 1 -      ;
         * 2 -   ;
         * 3 -          
         *      .
         */
        DWORD cmdbuf[PARAM_ARR_SIZE];            /*      */
        unsigned tx_data;
        long divider;
        int i;
        DWORD *pcmdbuf;
        int presc;

        pcmdbuf = cmdbuf;                                                                     /*1*/
        tx_data = (((unsigned)PARAM_ARR_SIZE * 2) << 8) & 0xFFFFU;
        *pcmdbuf++ = fill_command(modulemode_cmd | 0x01, tx_data);

        for (i = 0; (i < LTR11_MAX_LCHANNEL); i += 2)                                         /*2*/
            {
            tx_data = (((unsigned)hnd->LChTbl[i] & 0xFF) << 8 |
                ((unsigned)hnd->LChTbl[i + 1] & 0xFF)) & 0xFFFFU;
            *pcmdbuf++ = fill_command(modulemode_cmd, tx_data);
            }
	
	    presc = 0;                                                                            /*3*/
        switch (hnd->InpMode)
            {
            case LTR11_INPMODE_EXTRISE:
            case LTR11_INPMODE_EXTFALL:
                divider = hnd->InpMode;
                break;
            case LTR11_INPMODE_INT:
                /*        */
                
                while (   (presc < sizeof prescalers / sizeof prescalers[0])
                       && (prescalers[presc] != hnd->ADCRate.prescaler)
                      )
                    {
                    presc++;
                    }
                divider = hnd->ADCRate.divider;
                if (   (presc >= sizeof prescalers / sizeof prescalers[0])
                    || (divider < min_divider)
                    || (max_divider < divider)
                    || ((hnd->ChRate = eval_adcrate(hnd->ADCRate.prescaler, divider)) > 400.0)
                   )
                    {
                    ret_val = LTR11_ERR_INVALID_ADCRATE;
                    }
                else
                    {
                    hnd->ChRate /= hnd->LChQnt;
                    }
                break;
            default:
                ret_val = LTR11_ERR_INVALID_ADCSTROBE;
            }

        if (ret_val == LTR_OK)
            {
            DWORD ack;
            long n;
        
            /*         */
            tx_data = ((unsigned)divider & 0xFF00U) | ((unsigned)divider & 0xFF);
            *pcmdbuf++ = fill_command(modulemode_cmd, tx_data);
            tx_data = (((unsigned)presc & 0xFF) << 8) | (((unsigned)hnd->LChQnt - 1) & 0xFF);
            *pcmdbuf = fill_command(modulemode_cmd, tx_data);
            if ((n = LTR_Send(&hnd->Channel, cmdbuf, PARAM_ARR_SIZE, send_tm_out * PARAM_ARR_SIZE))
                    < 0
               )
                {
                ret_val = n;
                }
            else if (n != PARAM_ARR_SIZE)
                {
                ret_val = LTR_ERROR_SEND;
                }
            else if ((n = LTR_Recv(&hnd->Channel, &ack, NULL, 1, ack_tm_out)) < 0)
                {
                ret_val = n;
                }
            else if (n < 1)
                {
                ret_val = LTR11_ERR_NOACK;
                }
            else
                {
                ack &= ack_mask;
                if (ack != modulemode_ack)
                    ret_val = LTR11_ERR_INVALIDACK;
                }
            }
        } /*else @else if ((hnd->LChQnt <= 0) || (LTR11_MAX_LCHANNEL < hnd->LChQnt)@*/

    return ret_val;
#undef PARAM_ARR_SIZE
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_Start
    (
    PTLTR11 hnd
    )
    {
    /*
     *     LTR11.
     * 
     *           .
     * 
     *   hnd -    .
     *  
     *     (.  ).
     */

    return start_ltr11(hnd, 0);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR11API_DllExport(INT) LTR11_Stop
    (
    PTLTR11 hnd
    )
    {
    /*
     *   LTR11   .
     * 
     *            
     *    .
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */

#define RD_BUFSZ (2048)
    DWORD ack;
    DWORD cmd;
    DWORD rd_buf[RD_BUFSZ];
    long rd_cnt;
    INT ret_val = LTR_OK;
    DWORD tick;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else
        {
        long n;

        cmd = fill_command(stop_cmd, 0);
        if ((n = LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out)) < 0)
            {
            ret_val = n;
            }
        if (n != 1)
            {
            ret_val = LTR_ERROR_SEND;
            }
        else
            {
            int data_received = 0;
            DWORD elapsed_time;
            const DWORD rx_time_out = 100;

            /*        */
            tick = GetTickCount();
            ack = ~stop_ack;
            do
                {
                rd_cnt = (long)LTR_Recv(&hnd->Channel, rd_buf, NULL, RD_BUFSZ, rx_time_out);
                if (rd_cnt > 0)                     /*    */
                    {
                    /*   */
                    ack = rd_buf[rd_cnt - 1] & ack_mask;
                    data_received = 1;
                    }
                elapsed_time = f_elapsed_time(tick);
                } while ((elapsed_time <= ack_tm_out) && (rd_cnt >= 0) && (ack != stop_ack));
            if (!data_received)                     /*      */
                {
                ret_val = LTR11_ERR_NOACK;
                }
            else if (ack != stop_ack)               /*   */
                {
                ret_val = LTR11_ERR_INVALIDACK;
                }
            }
        } /*else @if (hnd->Channel == NULL)@*/

    return ret_val;
#undef RD_BUFSZ
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static INT start_ltr11
    (
    PTLTR11 hnd,
    int frame
    )
    {
    /*
     *     LTR11.
     * 
     *             (
     *   -)  .
     * 
     *   hnd   -    ;
     *   frame -        .
     *  
     *     (.  ).
     */

    DWORD ack;
    DWORD cmd;
    int mode;
    long n;
    INT ret_val = LTR_OK;


    if (hnd == NULL)
        {
        ret_val = LTR11_ERR_INVALID_DESCR;
        }
    else
        {
        switch (mode = hnd->ADCMode)
            {
            case LTR11_ADCMODE_ACQ:
            case LTR11_ADCMODE_TEST_U1P:
            case LTR11_ADCMODE_TEST_U1N:
            case LTR11_ADCMODE_TEST_U2N:
            case LTR11_ADCMODE_TEST_U2P:
                if (   (hnd->StartADCMode != LTR11_STARTADCMODE_INT)
                    && (hnd->StartADCMode != LTR11_STARTADCMODE_EXTRISE)
                    && (hnd->StartADCMode != LTR11_STARTADCMODE_EXTFALL)
                   )
                    {
                    ret_val = LTR11_ERR_INVALID_STARTADCMODE;
                    }
                else
                    {
                    mode |= hnd->StartADCMode << 6;
                    if (frame)
                        mode |= 0x8000U;
                    cmd = fill_command(start_cmd, mode);
                    if ((n = LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out)) < 0)
                        {
                        ret_val = n;
                        }
                    else if (n != 1)
                        {
                        ret_val = LTR_ERROR_SEND;
                        }
                    else if ((n = LTR_Recv(&hnd->Channel, &ack, NULL, 1, ack_tm_out)) < 0)
                        {
                        ret_val = n;
                        }
                    else if (n < 1)
                        {
                        ret_val = LTR11_ERR_NOACK;
                        }
                    else
                        {
                        ack &= ack_mask;
                        if (ack != start_ack)
                            ret_val = LTR11_ERR_INVALIDACK;
                        }
                    }
                break;
            default:                          /*   */
                ret_val = LTR11_ERR_INVALID_ADCMODE;
            }
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
