/*
 *      LTR114.
 *     (, ,   ..)   LTR114.
 *              .
 *          .
 *          .
 */
#include <math.h>
#define CBR_MODE_3T       1
#define CBR_MODE_3T_INTER  2
#define CBR_MODE_5T       3

#define CUR_CBR_MODE  CBR_MODE_5T

#define LTR114API_EXPORTS

#include <windows.h>
#include <string.h>
#include <stdlib.h>


#include "ltr\\include\\ltr114api.h"
#include "ltr\\include\\ltrapi.h"
#include "ltr\\include\\ltrapidefine.h"
#include "lib\\crc\\crc.h"



#define LTR114_GAIN_10                (2)
#define LTR114_GAIN_2                 (0)
#define LTR114_GAIN_04                (1)


/*================================================================================================*/
/*    (  ,  0 -  -1000, 1 - -1001  ..) */
const LPCSTR err_str[] =
{
    "     NULL",
    "   ",
    "   ",
    "     ",
    "      ",
    "    ",
    "    ",
    "      ",
    "    ",
    "    NULL",
    "        ",
    "        NULL",
    "    ",
    "    LTR114",
    "   LTR114",
    "    LTR114",
    "        ",
    "        ",
    "  :     ",
    "   "
};

/*      LTR-114 */
static const int min_err_id = 10000;                   /*   () */

/*     LTR114 */
/* 
 *         (0x8000)  
 *  (6-).
 */
static const unsigned long ack_mask = 0x803F;         /*     */
static const unsigned long modulemode_ack = 0x8012;   /*    */
static const unsigned long start_ack = 0x8011;        /*     */
static const unsigned long stop_ack = 0x8010;         /*     */
static const unsigned long test_interface_ack = 0x8013; /*     */

/*     */
static const unsigned dummy_cmd = 0x00;               /*    */
static const unsigned getconfig_cmd = 0x0C;           /*    */
static const unsigned modulemode_cmd = 0x18;          /*    */
static const unsigned modulemode_first_cmd = 0x19;     /*     -  */
static const unsigned start_cmd = 0x0A;               /*    */
static const unsigned stop_cmd = 0x05;                /*     */
static const unsigned test_cmd = 0x06;                /*     */

/* -   */
static const DWORD ack_tm_out = 6000;                 /*    */
static const DWORD send_tm_out = 2000;                /*   */
static const DWORD cmd_tm_out  = 10;

static const DWORD recv_tm_out  = 1000;

/* ,      */
static const long max_divider = 8000;                /*    */
static const long min_divider = 2;                    /*    */
/*     ,    */
static const int prescalers[] = {1, 8, 64, 256, 1024};

static double theScalesVal[] = {10, 2, 0.4}; 
static int theRScalesVal[] = {150, 400, 1000, 2500};

static const int cfg_signature = 0x5A;
/*================================================================================================*/

/*================================================================================================*/
static double eval_adcrate(long d);
static int eval_parity(DWORD cmd);
static DWORD f_elapsed_time(DWORD start_time);
static DWORD fill_command(WORD cmd, WORD param);
//static INT start_ltr11(PTLTR11 hnd, int frame);
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
int WINAPI DllEntryPoint
    (
    HINSTANCE hinst,
    unsigned long reason,
    void* lpReserved
    )
    {
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/


#define T_AVR 0.03 //  AVR   

int powr (int n, int p)
{
    int i;
    int res = 1;
    for (i=0; i < p; i++)
        res *= n;
    return res;
}
/*------------------------------------------------------------------------------------------------*/
BYTE eval_OSR(double fx)
{
    /*     OSR   
     * :
     *   fx -    
     *  
     *    OSR, .  .
     */
    double Tadc = 1000/fx;
    double Tconv = Tadc+1;
    BYTE osr = 0x8F;
    Tconv = ((double)40*32768 + 170)/(8*1000); 

    if ((Tadc - Tconv) >= T_AVR)
        return osr;

    osr = 0xF;
    Tconv = ((double)40*32768 + 170)/(15*1000);

    while (Tadc - Tconv < T_AVR)
    {
        osr = (osr==0xF) ? 9:osr-1;
        Tconv = ((double)40*32*powr(2,osr) + 170)/(15*1000);
    }
    return osr;
}
/*------------------------------------------------------------------------------------------------*/
static double eval_adcrate
    (
   long d
    )
{
    /*
     *    .
     * 
     *          .
     * 
     *   p -  ;
     *   d -  .
     *  
     *     .     - 0.
     */

    double denom;
    double ret_val = 0.0;
    
    if ((denom = ((double)d)) > 0)
    {
        ret_val = LTR114_CLOCK / denom;
    }

    return ret_val;
}

/*------------------------------------------------------------------------------------------------*/
static int eval_parity
    (
    DWORD cmd
    )
    {

    /*
    *   
    * 
    *          22-0    
    * 
    *   cmd - ;
    *  
    *    (0  1)
    */
    unsigned long index;
    int parity;

    parity = 0;
    for (index = 0x01UL; (index <= 0x10UL); index <<= 1)
        {
        parity ^= (cmd & index) != 0;
        }

    for (index = 0x80000000UL; (index >= 0x010000UL); index >>= 1)
        {
        parity ^= (cmd & index) != 0;
        }
    return parity;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static DWORD f_elapsed_time
    (
    DWORD start_time
    )
     /*
     *  ,    start_time
     * 
     *   start_time -  ,     ;
     *  
     *     ( )
     */

    {
    DWORD elapsed_time;
    DWORD sys_time;


    sys_time = GetTickCount();
    if (sys_time >= start_time) elapsed_time = sys_time - start_time;
    else                        elapsed_time = (65535 - (start_time - sys_time)) + 1;

    return elapsed_time;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/

static DWORD fill_command
    (
    WORD cmd,
    WORD param
    )
     /*
     *  4-              
     * 
     *   cmd -  ;
     *   param - 2-   
     *  
     *   32-.   
     */

    {
    DWORD ret_cmd;
    ret_cmd = 0x80C0U | (DWORD)param << 16 | (DWORD)cmd & 0x1F;
    ret_cmd |= eval_parity(ret_cmd) << 5;  //   ( P)

    return ret_cmd;
    }
/*------------------------------------------------------------------------------------------------*/

#define IS_R_SWMODE(w) ((LTR114_TABLE_SWMODE(w) & 0xF0) == 0x20)

#define IS_MODE_R(mode) ((mode& 0xF0) == 0x20)
#define IS_MODE_CHANNEL(mode) ((IS_MODE_R(mode))||(mode==LTR114_MEASMODE_U))
#define IS_TEST_MODE(mode) ((mode == LTR114_MEASMODE_X0Y0)||(mode==LTR114_MEASMODE_X5Y0)||(mode==LTR114_MEASMODE_X0Y5))

#define INVALID_CH 0xFFFF


/* .      
  LCHANNEL     WORD ( ,  )
      INVALID_CH*/
WORD create_table_lch(LCHANNEL ch)
{
    WORD out = 0;

    // 
    if (IS_MODE_R(ch.MeasMode))
    {
        if (ch.Range >= LTR114_R_RANGEQNT)
            return INVALID_CH;
        else
            out |= (WORD)ch.Range<<10;

        if (ch.Range == 0)
            out |= 0x40;    //1   0-400
    }
    else
    {
        switch (ch.Range)
        {
            case LTR114_URANGE_10: out |= LTR114_GAIN_10<<8; break;
            case LTR114_URANGE_2: out |= LTR114_GAIN_2<<8; break;
            case LTR114_URANGE_04: out |= LTR114_GAIN_04<<8; break;
            default: return INVALID_CH;
        }
    }
    
    //  
    if (IS_MODE_R(ch.MeasMode))
    {
        if ((ch.Channel < 0)||(ch.Channel > 7))  // 
            return INVALID_CH;
        else out |= ch.Channel;
    }

    if ((ch.MeasMode == LTR114_MEASMODE_U)||(IS_TEST_MODE(ch.MeasMode))) 
    {
        if ((ch.Channel < 0) || (ch.Channel > 15))
            return INVALID_CH;
        else out |= ch.Channel;
    }

    if ((ch.MeasMode==LTR114_MEASMODE_DAC12_CBR) || (ch.MeasMode==LTR114_MEASMODE_NDAC12_CBR)
        || (ch.MeasMode==LTR114_MEASMODE_DAC12_INTR_CBR) || (ch.MeasMode==LTR114_MEASMODE_NDAC12_INTR_CBR))
    {
        if ((ch.Channel < 8) || (ch.Channel > 15))
            return INVALID_CH;
        out |= (ch.Channel-8);
    }
    out |= ch.MeasMode;
    return out;
}






/*------------------------------------------------------------------------------------------------*/
static INT start_ltr114
    (
    PTLTR114 hnd,
    int frame
    )
    {
    /*
     *     LTR114.
     * 
     *             (
     *   -)  .
     * 
     *   hnd   -    ;
     *   frame -        .
     *  
     *     (.  ).
     */

    DWORD ack;
    DWORD cmd;
    int mode = 0;
    long n;
    INT ret_val = LTR_OK;


    if (hnd == NULL)
        return LTR114_ERR_INVALID_DESCR;
    

    //filter_data_clear(hnd);
    if (frame)
        mode |= 0x8000U;
    cmd = fill_command(start_cmd, mode);


    //  RUN
    if ((n = LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out)) < 0)
        return n;
    if (n != 1)
        return LTR_ERROR_SEND;
    
    // 
    if ((n = LTR_Recv(&hnd->Channel, &ack, NULL, 1, ack_tm_out)) < 0)
        return n;
    if (n < 1)
        return LTR114_ERR_NOACK;
    // 
    ack &= ack_mask;
    if (ack != start_ack)
        return LTR114_ERR_INVALIDACK;

    return ret_val;
    }

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_Close
    (
    PTLTR114 hnd
    )
    {
    /*
     *     LTR114.
     * 
     *       .
     * 
     *   hnd -    .
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;


    if (hnd == NULL)
    {
        ret_val = LTR114_ERR_INVALID_DESCR;
    }
    else
    {
        DWORD cmd = LTR010CMD_STOP;  //  -   STOP
        int i;

        LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out);    

        for (i=0; i < LTR114_ADC_RANGEQNT; i++)  /*  ,   */
        {
            if (hnd->AutoCalibrInfo[i].TempScale!=NULL)
            {
                free(hnd->AutoCalibrInfo[i].TempScale);
                hnd->AutoCalibrInfo[i].TempScale = NULL;
            }
        }

        /*        if (hnd->FilterData!=NULL)
        {
            free(hnd->FilterData);
            hnd->FilterData = NULL;
        }*/
        ret_val = LTR_Close(&hnd->Channel);
    }

    return ret_val; 
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_GetConfig
    (
    PTLTR114 hnd
    )
    {
    /*
     *      LTR114.
     * 
     *             
     *          ,  .
     *      (   2-   ):
     *                        
     *      0              
     *      1-2          
     *      3-16           
     *      17-24         
     *      25-40          
     *      41-44          
     *      45-48          1  2    2  
     *      49-52          1  2    0.4  
     *      53-56               
     *      57-60        --//--   
     *      61-64        --//--   
     *      65-66         
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */

    BYTE cfg_arr[LTR114_CONFIG_DATA_SIZE];
    int i;
    DWORD rd_buf[LTR114_CONFIG_DATA_SIZE * 4];          /*    4-  */
    INT ret_val = LTR_OK;

    DWORD cmd;
    long n;

    int j;
    DWORD *pbuf = rd_buf;
    float *pcfg;

    if (hnd == NULL)
        return LTR114_ERR_INVALID_DESCR;

    cmd = fill_command(getconfig_cmd, 0);
    if ((n = LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out)) < 0)  //   GET_CONFIG
        return n;
            
    if (n != 1)
        return LTR_ERROR_SEND;
           
    if ((n = LTR_Recv(&hnd->Channel, rd_buf, NULL, LTR114_CONFIG_DATA_SIZE * 4,
        LTR114_CONFIG_DATA_SIZE * 4 * cmd_tm_out + ack_tm_out)) < 0)  //  
            return n;
           
    if (n != LTR114_CONFIG_DATA_SIZE * 4)           /*      */
        return LTR114_ERR_GETCFG;
            
    /*         */
    if ((rd_buf[0] & 0x80FCUL) != 0x80DCUL)   /*    */
        return LTR114_ERR_CFGDATA;
        
    rd_buf[0] &= ~0x10UL;                 /*     */
    for (i = 0; ((i < LTR114_CONFIG_DATA_SIZE) && (ret_val == LTR_OK)); i++)
    {
        cfg_arr[i] = 0;
        for (j = 6; ((j >= 0) && (ret_val == LTR_OK)); j-= 2)
        {
            if ((*pbuf & 0x803CUL) != 0x800CUL)
            {
                ret_val = LTR114_ERR_CFGDATA;
            }
            else
            {
                cfg_arr[i] |= (*pbuf & 0x03UL) << j;
            }
            pbuf++;
        }
    }

    /*        */
    if (ret_val != LTR_OK)
        return ret_val;
       
    if (cfg_arr[0] != cfg_signature)      // 
       return LTR114_ERR_CFGSIGNATURE;
       
    if (eval_crc16(0, cfg_arr, LTR114_CONFIG_DATA_SIZE - 2) !=
        (((unsigned short)cfg_arr[LTR114_CONFIG_DATA_SIZE - 2] << 8 |
        (unsigned short)cfg_arr[LTR114_CONFIG_DATA_SIZE - 1]) & 0xFFFFU)  //  
        )
        return LTR114_ERR_CFGCRC;
       
    hnd->ModuleInfo.Ver = (unsigned)cfg_arr[1] << 8 | (unsigned)cfg_arr[2];
    (void)memcpy((char *)hnd->ModuleInfo.Date, (char *)(cfg_arr + 3), 14);
    (void)memcpy((char *)hnd->ModuleInfo.Name, (char *)(cfg_arr + 17), 8);
    (void)memcpy((char *)hnd->ModuleInfo.Serial, (char *)(cfg_arr + 25), 16);
       
    pcfg = (float *)(cfg_arr + 41);

    for (i = 0; i < LTR114_ADC_RANGEQNT; i++)
       hnd->ModuleInfo.CbrCoef.U[i] = *pcfg++;
    for (i = 0; (i < LTR114_R_RANGEQNT); i++)
    {
     hnd->ModuleInfo.CbrCoef.I[i] = *pcfg++;
    }
    for (i = 0; i < LTR114_ADC_RANGEQNT; i++)
       hnd->ModuleInfo.CbrCoef.UIntr[i] = *pcfg++;
    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(LPCSTR) LTR114_GetErrorString
    (
    INT err
    )
    {
    /*
     *      ,    .
     * 
     * 
     *   err -   (  ),     LTR114.
     *  
     *         ,   .
     *   NULL -   .
     */
     
    const int err_qnt = sizeof err_str / sizeof err_str[0];
    LPCSTR ret_val = NULL;

    
    if ((err > -min_err_id) || (err <= -(min_err_id + err_qnt)))
        {
        /*    (     ) */
        /*         */
        ret_val = LTR_GetErrorString(err);
        }
    else
        {
        /*          LTR-114 */
        ret_val = err_str[-err - min_err_id];
        }

    return ret_val;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_GetFrame
    (
    PTLTR114 hnd,
    DWORD *buf
    )
{
    /*
     *       LTR11.
     * 
     *    LTR11       .
     *          .    
     *           - ( -
     *      ).
     * 
     *   hnd -     ;
     *   buf -   ,        .
     *  
     *      0 -     .
     *           -   (.  ).
     */
    DWORD cmd;
    DWORD rd_buf[LTR114_MAX_LCHANNEL*2 + 2];
    INT ret_val = 0;
    double adcrate;

    if (hnd == NULL)
        return LTR114_ERR_INVALID_DESCR;
        
    if ((hnd->LChQnt <= 0) || (LTR114_MAX_LCHANNEL < hnd->LChQnt))
        return LTR114_ERR_INVALID_ADCLCHQNT;
    
    if ((adcrate = eval_adcrate(hnd->FreqDivider)) <= 0.0)
       return LTR114_ERR_INVALID_ADCRATE;

    if ((ret_val = start_ltr114(hnd, 1)) == LTR_OK)
    {
        long n;
        DWORD *pbuf;
        DWORD tick;
        DWORD tm_out;

        tick = GetTickCount();
        /*   -       */
        tm_out = (DWORD)((double)1 / adcrate + ack_tm_out + 0.5);
        pbuf = rd_buf;
        do
        {
            if ((n = LTR_Recv(&hnd->Channel, pbuf, NULL, 2*hnd->LChQnt + 1, tm_out)) < 0)
            {
                ret_val = n;
            }
            else if (n > 0)
            {
                unsigned a;

                /*     (  ) */
               pbuf += n;
                a = pbuf[-1] & ack_mask;
                if (a == stop_ack)                   /*     */
                {
                    ret_val = pbuf - rd_buf;
                    if (--ret_val <= 0)
                    {
                        ret_val = LTR114_ERR_GETFRAME;
                    }
                    else
                    {
                        (void)memcpy(buf, rd_buf, ret_val * sizeof(DWORD));
                    }
                }
                else if (n < hnd->LChQnt + 1)        /*      */
                {
                    ret_val = LTR114_ERR_GETFRAME;
                }
             }
        } while ((f_elapsed_time(tick) <= tm_out) && (ret_val == LTR_OK) && (n > 0));
    }

    cmd = LTR010CMD_STOP;  //  -   STOP
        
    LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out);    

    return ret_val;
}
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_Init
    (
    PTLTR114 hnd
    )
{
    /*
     *    LTR114.
     * 
     *         "-".
     * 
     *   hnd -    .
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;

    if (hnd == NULL)
     return LTR114_ERR_INVALID_DESCR;
    hnd->size = sizeof(TLTR114);

    if ((ret_val = LTR_Init(&hnd->Channel)) == LTR_OK)
    {
        const char date_def[14] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
        int i,j;
        const char name_def[8]  = "LTR114\0\0";
        const char serial_def[16] = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";

        hnd->LChQnt = 1;               //  
        //      
        for (i = 0; (i < LTR114_MAX_LCHANNEL); i++)
        {
            hnd->LChTbl[i].Channel = 0; hnd->LChTbl[i].Range =0; hnd->LChTbl[i].MeasMode=0;
        }

        //  -
        hnd->FreqDivider = LTR114_DEF_DIVIDER;         // 4000       
        hnd->Interval = LTR114_DEF_INTERVAL;           //  
        hnd->SpecialFeatures = 0;                      //  
        hnd->AdcOsr = LTR114_DEF_OSR;                  //OSR   
        hnd->SyncMode = LTR114_DEF_SYNC_MODE;          // 


        hnd->ModuleInfo.Ver = 0;
        (void)strcpy((char *)hnd->ModuleInfo.Date, date_def);
        (void)strcpy((char *)hnd->ModuleInfo.Name, name_def);
        (void)strcpy((char *)hnd->ModuleInfo.Serial, serial_def);

        for (i=0; i < LTR114_ADC_RANGEQNT; i++)
        {
            hnd->AutoCalibrInfo[i].TempScale=NULL;
            for (j=0;j<LTR114_SCALE_INTERVALS; j++)
            {
                hnd->AutoCalibrInfo[i].Coef[j].Gain = 1;
                hnd->AutoCalibrInfo[i].Coef[j].Offset = 0;
            }
        }

//        hnd->FilterData = NULL;
    }  //if ((ret_val = LTR_Init(&hnd->Channel)) == LTR_OK)

    return ret_val;
}
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_Open
(
 PTLTR114 hnd,
 DWORD net_addr,
 WORD net_port,
 CHAR *crate_sn,
 INT slot_num
 )
{
    /*
    *     LTR114.
    * 
    *            .
    * 
    *   hnd      -    ;
    *   net_addr -    (  )   (hex):
    *              MsbAABBCCDDLsb - AA.BB.CC.DD;
    *   net_port -   ;
    *   crate_sn -   ,    ;
    *   slot_num -   ,    .
    *  
    *     (.  ).
    */

    INT ret_val = LTR_OK;
    int WarningRes = LTR_OK;


    if (hnd == NULL)
    {
        ret_val = LTR114_ERR_INVALID_DESCR;
    }
    else if (crate_sn == NULL)
    {
        ret_val = LTR114_ERR_INVALID_CRATESN;
    }
    else if ((slot_num < 1) || (MODULE_MAX < slot_num))
    {
        ret_val = LTR114_ERR_INVALID_SLOTNUM;
    }
    else
    {
        //if ((WarningRes = LTR_IsOpened(&hnd->Channel)) == LTR_OK)
        //    LTR_Close(&hnd->Channel);

        hnd->Channel.saddr = net_addr;
        hnd->Channel.sport = net_port;
        (void)strncpy((char *)hnd->Channel.csn,(char *)crate_sn, 16);
        hnd->Channel.cc = slot_num;                   /*   -    */

        WarningRes = LTR_Open(&hnd->Channel);
        if(WarningRes==LTR_OK||WarningRes==LTR_WARNING_MODULE_IN_USE) 
            ret_val=LTR_OK;
        else ret_val = WarningRes;

        if (ret_val == LTR_OK)
        {
            DWORD cmd[3];
            long n;
            /*       STOP - RESET - STOP*/
            cmd[0] = LTR010CMD_STOP;
            cmd[1] = LTR010CMD_RESET;
            cmd[2] = LTR010CMD_STOP;
            if ((n = LTR_Send(&hnd->Channel, cmd, 3, send_tm_out)) < 0)
            {
                ret_val = n;
            }
            else if (n != 3)
           {
               ret_val = LTR_ERROR_SEND;
           }
           else
           {
               DWORD rd_buf;  //      RESET
               if ((n = LTR_Recv(&hnd->Channel, &rd_buf, NULL, 1, ack_tm_out)) < 0)
               {
                   ret_val = n;
               }
               else if (n == 0)
               {
                   ret_val = LTR114_ERR_NOACK;          
               }
               else if ((rd_buf & 0xF0FF) == LTR010CMD_RESET)  //  
               {
                   if (((rd_buf >> 16) & 0xFFFFUL) != LTR114_MID)
                   {
                       ret_val = LTR114_ERR_MODULEID;
                   }
                   else
                   {
                        ret_val = LTR_OK;
                   }
                }
            } /*else @else if (n != 3)@*/
            Sleep(200); // ,       

        } /*if ((ret_val = LTR_Open(&hnd->Channel)) == LTR_OK)*/
    } /*else @else if ((slot_num < min_slotnum) || (max_slotnum < slot_num))@*/    

    if(WarningRes==LTR_WARNING_MODULE_IN_USE&&ret_val==LTR_OK) ret_val=WarningRes;
    return ret_val;
}
/*------------------------------------------------------------------------------------------------*/

/*,     (.   )*/
#define RECV_IS_FIRST_WORD(RecWord) !(RecWord&0x8)                   //    ?
#define RECV0_SWITCH_MODE(RecWord)    (BYTE)((RecWord>>24)&0xFF)     //     
#define RECV0_GAIN(RecWord)           (BYTE)((RecWord&0x3))          //     
#define RECV_COUNT(RecWord)          (BYTE)((RecWord>>4)&0xF)        //   (  )
#define RECV1_N(RecWord)              (BYTE)((RecWord&0x7))          //     
#define RECV0_DATA2(RecWord)          (BYTE)((RecWord>>16)&0xFF)     //      
#define RECV1_DATA10(RecWord)          (WORD)((RecWord>>16)&0xFFFF)  //       

//          ?
//#define IS_AUTO_CALIBR_MODE(RecWord)   \
//   ((RECV0_SWITCH_MODE(RecWord) >= LTR114_SWMODE_NULL) && ((RECV0_SWITCH_MODE(RecWord)) <= LTR114_SWMODE_REF)) 
#define IS_AUTO_CALIBR_MODE(RecWord)   \
   ((RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_DAC12) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_NDAC12) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_NDAC12_CBR) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_DAC12_CBR)) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_DAC12_INTR) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_NDAC12_INTR) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_DAC12_INTR_CBR) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_NDAC12_INTR_CBR) || \
    (RECV0_SWITCH_MODE(RecWord) == LTR114_MEASMODE_NULL)



#define IS_THERM(RecWord) ((RecWord&0xE) == 4)  //       ?

#define IS_THERM_VALID(RecWord) ((RecWord&0xF) == 5)  //    ?

#define LTR114_CYCLE_CNT_MODE 0xD //  

//    .   
#define CYCLE_COUNTER_INC(cnt) \
  if (cnt == (LTR114_CYCLE_CNT_MODE-1)) \
    cnt = 0;\
  else\
    (cnt)++

//    3-     DWORD  4-  INT  
#define SCALE_TO_INT(scale) (scale & 0x800000)? (INT)(scale | 0xFF000000):(INT)scale

int coef_calc(PTLTR114 hnd, INT gain)
{
    /*      (GAIN, OFFSET)
     *       , +, - 
     * (   hnd->AutoCalibrInfo[gain].TempScale[0])
     *       
     *   
     *
     * 
     *   hnd      -    ;
     *   gain     -  
    *  
    *     (.  ).
    */
    if (gain != LTR114_URANGE_10)
    {
        double x1 = hnd->AutoCalibrInfo[gain].TempScale[0].NRef;
        double x2 = hnd->AutoCalibrInfo[gain].TempScale[0].NInterm;
        double x3 = hnd->AutoCalibrInfo[gain].TempScale[0].Null;
        double x4 = hnd->AutoCalibrInfo[gain].TempScale[0].Interm;
        double x5 = hnd->AutoCalibrInfo[gain].TempScale[0].Ref;

        double u =  (double)hnd->ModuleInfo.CbrCoef.U[gain]*LTR114_MAX_SCALE_VALUE/theScalesVal[gain];
        double u_int =  (double)hnd->ModuleInfo.CbrCoef.UIntr[gain]*LTR114_MAX_SCALE_VALUE/theScalesVal[gain];
        double o1 = (x4+x2)/2;


        if (CUR_CBR_MODE == CBR_MODE_5T)
        {

            hnd->AutoCalibrInfo[gain].Coef[1].Gain = 2*u_int/(x4-x2);
            hnd->AutoCalibrInfo[gain].Coef[1].Offset = x3;

            hnd->AutoCalibrInfo[gain].Coef[2].Gain = (u-u_int)/(x5-x4);
            hnd->AutoCalibrInfo[gain].Coef[2].Offset = x4 - (1/hnd->AutoCalibrInfo[gain].Coef[2].Gain) * u_int + (x3-o1);

            hnd->AutoCalibrInfo[gain].Coef[0].Gain = (u-u_int)/(x2-x1);
            hnd->AutoCalibrInfo[gain].Coef[0].Offset = x2 - (1/hnd->AutoCalibrInfo[gain].Coef[0].Gain) * (-u_int) + (x3-o1);
        }

        if (CUR_CBR_MODE == CBR_MODE_3T)
        {
            hnd->AutoCalibrInfo[gain].Coef[0].Gain = hnd->AutoCalibrInfo[gain].Coef[1].Gain =  hnd->AutoCalibrInfo[gain].Coef[2].Gain = 2*u/(x5-x1);
            hnd->AutoCalibrInfo[gain].Coef[0].Offset = hnd->AutoCalibrInfo[gain].Coef[1].Offset = hnd->AutoCalibrInfo[gain].Coef[2].Offset = x3;
        }

        if (CUR_CBR_MODE == CBR_MODE_3T_INTER)
        {
            hnd->AutoCalibrInfo[gain].Coef[0].Gain = hnd->AutoCalibrInfo[gain].Coef[1].Gain =  hnd->AutoCalibrInfo[gain].Coef[2].Gain = 2*u_int/(x4-x2);
            hnd->AutoCalibrInfo[gain].Coef[0].Offset = hnd->AutoCalibrInfo[gain].Coef[1].Offset = hnd->AutoCalibrInfo[gain].Coef[2].Offset = x3;
        }


        hnd->AutoCalibrInfo[gain].HVal = x4;
        hnd->AutoCalibrInfo[gain].LVal = x2;
        

    }
    else
    {

        double x4 = hnd->AutoCalibrInfo[gain].TempScale[0].Ref;
        double x3 = hnd->AutoCalibrInfo[gain].TempScale[0].Null;
        double x2 = hnd->AutoCalibrInfo[gain].TempScale[0].NRef;
        double o1 = (x4+x2)/2;

        if (CUR_CBR_MODE == CBR_MODE_5T)
        {
            double u_int =  (double)hnd->ModuleInfo.CbrCoef.U[gain]*LTR114_MAX_SCALE_VALUE/theScalesVal[gain];
            double u_int2;
            double u_int1;

            hnd->AutoCalibrInfo[gain].Coef[1].Gain =  2*u_int/(x4-x2);
            hnd->AutoCalibrInfo[gain].Coef[1].Offset = x3;

            x2 = hnd->AutoCalibrInfo[LTR114_URANGE_2].TempScale[0].NInterm;
            x4 = hnd->AutoCalibrInfo[LTR114_URANGE_2].TempScale[0].Interm;
            u_int1 = u_int * x2/hnd->AutoCalibrInfo[gain].TempScale[0].NRef; 
            u_int2 = u_int * x4/hnd->AutoCalibrInfo[gain].TempScale[0].Ref; 

            hnd->AutoCalibrInfo[gain].Coef[0].Gain = hnd->AutoCalibrInfo[gain].Coef[1].Gain * hnd->AutoCalibrInfo[LTR114_URANGE_2].Coef[0].Gain/hnd->AutoCalibrInfo[LTR114_URANGE_2].Coef[1].Gain;
            hnd->AutoCalibrInfo[gain].Coef[0].Offset = x2 - (1/hnd->AutoCalibrInfo[gain].Coef[0].Gain) * (-u_int1) + (x3-o1);

            hnd->AutoCalibrInfo[gain].Coef[2].Gain = hnd->AutoCalibrInfo[gain].Coef[1].Gain * hnd->AutoCalibrInfo[LTR114_URANGE_2].Coef[2].Gain/hnd->AutoCalibrInfo[LTR114_URANGE_2].Coef[1].Gain;
            hnd->AutoCalibrInfo[gain].Coef[2].Offset = x4 - (1/hnd->AutoCalibrInfo[gain].Coef[2].Gain) * u_int2 + (x3-o1);        
        }
        else
        {
            double u =  (double)hnd->ModuleInfo.CbrCoef.U[gain]*LTR114_MAX_SCALE_VALUE/theScalesVal[gain];

     //   hnd->AutoCalibrInfo[gain].Coef.Gain = u * (x1 - x3)/((x1*x1 + x2*x2 + x3*x3) - pow(x1 + x2 + x3,2)/3);
    //    hnd->AutoCalibrInfo[gain].Coef.Offset = - hnd->AutoCalibrInfo[gain].Coef.Gain * (x1+x2+x3)/3;

            hnd->AutoCalibrInfo[gain].Coef[0].Gain = hnd->AutoCalibrInfo[gain].Coef[1].Gain =  hnd->AutoCalibrInfo[gain].Coef[2].Gain = 2*u/(x4-x2);
      // hnd->AutoCalibrInfo[gain].Coef.Offset = (x1+x2+x3)/3;
            hnd->AutoCalibrInfo[gain].Coef[0].Offset = hnd->AutoCalibrInfo[gain].Coef[1].Offset = hnd->AutoCalibrInfo[gain].Coef[2].Offset = x3;

            
        }
        hnd->AutoCalibrInfo[gain].HVal = x4;
        hnd->AutoCalibrInfo[gain].LVal = x2;
    }
    hnd->AutoCalibrInfo[gain].LastVals = hnd->AutoCalibrInfo[gain].TempScale[0];

    return LTR_OK;

}

int get_autocbr_cnt(PTLTR114 hnd)
{
    /*      
     *      (   )
     *
     * 
     *   hnd      -    ;
     *  
     *   -   .
     */
    int res = (int)LTR114_FREQ((*hnd))/10;
    if (res < 2)
        res = 2;
    return res;
}

int proc_autocalibr(PTLTR114 hnd, DWORD** psrc1, BYTE *cnt, WORD cur_int)
{
     /*      
     *            
     *   
     * 
     *   hnd       -    ;
     *   psrc1     -      
     *   cnt       -   
     *   cur_int   -      
     *  
     *     (.  ).
     */

    DWORD* psrc = *psrc1;
    BYTE sw_mode = RECV0_SWITCH_MODE(*psrc);
    BYTE gain = RECV0_GAIN(*psrc);
     INT data, index;

     switch (gain)
     {
         case LTR114_GAIN_10: gain = LTR114_URANGE_10; break;
         case LTR114_GAIN_2: gain = LTR114_URANGE_2; break;
         case LTR114_GAIN_04: gain = LTR114_URANGE_04; break;
         default: return LTR114_ERR_INVALID_LCH;
     }
    
   

    data = (INT)RECV0_DATA2(*psrc)<< 16; //   

    //  
    CYCLE_COUNTER_INC(*cnt);
    psrc++;

    if (((*psrc >> 8) & 0x8F) != (hnd->Channel.cc - CC_MODULE1))
        return LTR114_ERR_ADCDATA_SLOTNUM;
    if (*cnt != RECV_COUNT(*psrc))
        return LTR114_ERR_ADCDATA_CNT;

    data |= RECV1_DATA10(*psrc); 

    data = SCALE_TO_INT(data);
    //            
    switch(sw_mode)
    {
        case LTR114_MEASMODE_NULL:
            hnd->AutoCalibrInfo[gain].TempScale[hnd->AutoCalibrInfo[gain].Index.Null++].Null = data;
            break;
        case LTR114_MEASMODE_NDAC12_INTR:
            hnd->AutoCalibrInfo[gain].TempScale[hnd->AutoCalibrInfo[gain].Index.NInterm++].NInterm = data;
            break;
        case LTR114_MEASMODE_DAC12_INTR:
            hnd->AutoCalibrInfo[gain].TempScale[hnd->AutoCalibrInfo[gain].Index.Interm++].Interm = data;
            break;
        case LTR114_MEASMODE_NDAC12:
            hnd->AutoCalibrInfo[gain].TempScale[hnd->AutoCalibrInfo[gain].Index.NRef++].NRef = data;
            break;
        case LTR114_MEASMODE_DAC12:
            hnd->AutoCalibrInfo[gain].TempScale[hnd->AutoCalibrInfo[gain].Index.Ref++].Ref = data;

            /*    NRef      
              =>   */
      /*      if (cur_int <= LTR114_ADC_RANGEQNT*LTR114_AUTOCALIBR_STEPS)
            {
                //-      - e -   3- 
                index = min(min(hnd->AutoCalibrInfo[gain].Index.Null, hnd->AutoCalibrInfo[gain].Index.Ref), 
                    hnd->AutoCalibrInfo[gain].Index.NRef);
                if (index > 1) //    -  
                {
                    long SNull = 0, SRef = 0, SNRef = 0; 
                    INT i;
                    for (i = 0; i < index; i++) 
                    {
                        SNull += hnd->AutoCalibrInfo[gain].TempScale[i].Null;
                        SRef += hnd->AutoCalibrInfo[gain].TempScale[i].Ref;
                        SNRef += hnd->AutoCalibrInfo[gain].TempScale[i].NRef;
                    }
                    hnd->AutoCalibrInfo[gain].TempScale[0].Null = SNull/index;
                    hnd->AutoCalibrInfo[gain].TempScale[0].Ref = SRef/index; 
                    hnd->AutoCalibrInfo[gain].TempScale[0].NRef = SNRef/index; 
                }
                coef_calc(hnd, gain);  // 
                //  
                hnd->AutoCalibrInfo[gain].Index.Null = hnd->AutoCalibrInfo[gain].Index.Ref = hnd->AutoCalibrInfo[gain].Index.NRef = 0;
            }*/

            //-      - e -   3- 
            index = min(min(hnd->AutoCalibrInfo[gain].Index.Null, hnd->AutoCalibrInfo[gain].Index.Ref), 
                hnd->AutoCalibrInfo[gain].Index.NRef);
            if (index == get_autocbr_cnt(hnd)) // -  =  -     .
            {
                __int64 SNull = 0, SRef = 0, SNRef = 0, SInter = 0, SNInter = 0; 
                INT i;
                for (i = 0; i < index; i++) 
                {
                    SNull += hnd->AutoCalibrInfo[gain].TempScale[i].Null;
                    SRef += hnd->AutoCalibrInfo[gain].TempScale[i].Ref;
                    SNRef += hnd->AutoCalibrInfo[gain].TempScale[i].NRef;

                    if (gain != LTR114_URANGE_10)
                    {
                        SInter += hnd->AutoCalibrInfo[gain].TempScale[i].Interm;
                        SNInter += hnd->AutoCalibrInfo[gain].TempScale[i].NInterm;
                    }
                    
                }
                hnd->AutoCalibrInfo[gain].TempScale[0].Null = SNull/index;
                hnd->AutoCalibrInfo[gain].TempScale[0].Ref = SRef/index; 
                hnd->AutoCalibrInfo[gain].TempScale[0].NRef = SNRef/index; 

                hnd->AutoCalibrInfo[gain].TempScale[0].Interm = SInter/index;
                hnd->AutoCalibrInfo[gain].TempScale[0].NInterm = SNInter/index;
            
                coef_calc(hnd, gain);  // 
                

                // 
                for (i=0; i< index-1; i++)
                {
                    hnd->AutoCalibrInfo[gain].TempScale[i].Null = hnd->AutoCalibrInfo[gain].TempScale[i+1].Null;
                    hnd->AutoCalibrInfo[gain].TempScale[i].Ref  = hnd->AutoCalibrInfo[gain].TempScale[i+1].Ref;
                    hnd->AutoCalibrInfo[gain].TempScale[i].NRef = hnd->AutoCalibrInfo[gain].TempScale[i+1].NRef;
                    if (gain != LTR114_URANGE_10)
                    {
                        hnd->AutoCalibrInfo[gain].TempScale[i].Interm = hnd->AutoCalibrInfo[gain].TempScale[i+1].Interm;
                        hnd->AutoCalibrInfo[gain].TempScale[i].NInterm = hnd->AutoCalibrInfo[gain].TempScale[i+1].NInterm;
                    }
                }

                hnd->AutoCalibrInfo[gain].Index.Null = hnd->AutoCalibrInfo[gain].Index.Ref = hnd->AutoCalibrInfo[gain].Index.NRef = index-1;
                if (gain != LTR114_URANGE_10)
                {
                    hnd->AutoCalibrInfo[gain].Index.Interm = hnd->AutoCalibrInfo[gain].Index.NInterm = index-1;
                }
            }
            break;
    }
    *psrc1 = psrc;
    return LTR_OK;    
}




void make_ch_table(PTLTR114 hnd, int flags, int* ch_table, int *ch_cnt)
{
    int skipavgr = 0;
    int cur_ch, ch_num=1;


    *ch_cnt = hnd->LChQnt;

    ch_table[0] = 0;
    

    for (cur_ch = 1; (cur_ch < hnd->LChQnt); cur_ch++)
    {
      //     R  +I  -I
        if ((IS_MODE_R(hnd->LChTbl[cur_ch].MeasMode))&&(flags & LTR114_PROCF_AVGR)&&(!skipavgr))
        {
            DWORD mask =~(LTR114_MEASMODE_R ^ LTR114_MEASMODE_NR);
             //       +I   -I
            if ((((hnd->LChTbl[cur_ch].MeasMode == LTR114_MEASMODE_R) && (hnd->LChTbl[cur_ch-1].MeasMode == LTR114_MEASMODE_NR))
                    ||((hnd->LChTbl[cur_ch].MeasMode == LTR114_MEASMODE_NR) && (hnd->LChTbl[cur_ch-1].MeasMode == LTR114_MEASMODE_R)))
                    &&(hnd->LChTbl[cur_ch].Channel == hnd->LChTbl[cur_ch-1].Channel)
                    &&(hnd->LChTbl[cur_ch].Range == hnd->LChTbl[cur_ch-1].Channel)
                    )
            {
            
                skipavgr = 1; //.   -       R
                *ch_cnt = *ch_cnt-1;
                continue;  //  . 
            }
        }
        ch_table[ch_num++] = cur_ch;
        skipavgr = 0;
    }
}

/*LTR114API_DllExport(INT) LTR114_ConvertToValue(PTLTR114 hnd, double *data, INT size, INT correction_mode, INT flags)
{
    int i, cur_ch;
    INT ch_table[LTR114_MAX_LCHANNEL];
    INT ch_cnt;

    make_ch_table(hnd, flags, ch_table, &ch_cnt);
    for (i=0; i <size; i+=ch_cnt)
    {
        for (cur_ch=0; cur_ch < ch_cnt; cur_ch++)
        {
            double dt = data[i+cur_ch];
            //  
            if (correction_mode != LTR114_CORRECTION_MODE_NONE)
                if (IS_MODE_R(hnd->LChTbl[ch_table[cur_ch]].MeasMode))
                    dt = (dt - hnd->AutoCalibrInfo[LTR114_URANGE_04].Coef.Offset) * hnd->AutoCalibrInfo[LTR114_URANGE_04].Coef.Gain;
                else
                    dt = (dt - hnd->AutoCalibrInfo[hnd->LChTbl[ch_table[cur_ch]].Range].Coef.Offset) * hnd->AutoCalibrInfo[hnd->LChTbl[ch_table[cur_ch]].Range].Coef.Gain; 
                      
             if (flags & LTR114_PROCF_VALUE) //   
             {
                if (IS_MODE_R(hnd->LChTbl[ch_table[cur_ch]].MeasMode))
                {
                    dt = (theScalesVal[LTR114_URANGE_04]*dt/LTR114_MAX_SCALE_VALUE)*1000/hnd->ModuleInfo.CbrCoef.I[hnd->LChTbl[ch_table[cur_ch]].Range];
                    if (dt < 0)
                        dt = -dt;
                    //dt = theRScalesVal[LTR114_TABLE_N(hnd->LChTbl[cur_ch])]*dt/LTR114_MAX_SCALE_VALUE;
                }
                else
                    dt = theScalesVal[hnd->LChTbl[ch_table[cur_ch]].Range]*dt/LTR114_MAX_SCALE_VALUE;
             }
             data[i+cur_ch] = dt;

             
        }
    }
    return 0;
}*/


LTR114API_DllExport(INT) LTR114_ProcessDataCb
    (
    PTLTR114 hnd,
    DWORD *src,
    double *dest,
    double *therm,
    INT *size,
    INT *tcnt,
    INT correction_mode,
    INT flags,
    INT (* cb_func)(PTLTR114 hnd, double* data)
    )
{
    /*
     *         cb-,   
            .
     * 
     *      ,        ,
     *      (    )
     *   ,  ,    .
     *             
     *             .
     *              
     *           
     * 
     *   hnd    -    ;
     *   src    -       ;
     *   dest   -   ,      ;
     *   therm  -   ,      
     *   size   -       (..   ),   
     *                     
     *            ;
     *   tcnt   -   therm
     *   correction_mode -      . . ;
     *   flags   - :
                    LTR114_PROCF_VALUE -       
                                         (, ) (   ).
                   LTR114_PROCF_AVGR -          
                                          
     *   cb_func - cb-,     . 
                            ,
                      .
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;
    INT res = LTR_OK;
    
    if (tcnt!=NULL)    *tcnt = 0;

    if (hnd == NULL)
        return LTR114_ERR_INVALID_DESCR;
        
    if ((hnd->LChQnt + hnd->Interval <= 0) || (LTR114_MAX_LCHANNEL < hnd->LChQnt))
        return LTR114_ERR_INVALID_ADCLCHQNT;

    if ((correction_mode!=LTR114_CORRECTION_MODE_NONE) 
        &&(correction_mode!=LTR114_CORRECTION_MODE_INIT)
        &&(correction_mode!=LTR114_CORRECTION_MODE_AUTO))
        return LTR114_ERR_CORRECTION_MODE;


    if ((src == NULL) || (dest == NULL))
        ret_val = LTR114_ERR_INVALID_ARRPOINTER;
    else
    {
        /*    */
        DWORD frame[LTR114_MAX_LCHANNEL];
        double d_frame[LTR114_MAX_LCHANNEL];
        int slot_num = hnd->Channel.cc - CC_MODULE1;
        double *pdest = dest;
        DWORD *psrc = src;
        BYTE cnt;           //      LTR114_CYCLE_CNT_MODE
        BYTE cur_ch = 0;    //  .   . 
        WORD cur_int = 0;   //    - - . 
        DWORD buff;

        cnt = RECV_COUNT(*psrc); //   
        
        if (hnd->LChQnt == 0)   // 0 .  =>     
            cur_int = hnd->Interval;
        
        if (!RECV_IS_FIRST_WORD(*psrc)) // -    .    
        {
            psrc++;
            CYCLE_COUNTER_INC(cnt);
        }

        for (; (psrc < src + *size); psrc++)
        {
            /* :
             *  1)     ( 0x80)    (0x0F)
             *  2)   
             */

            //  
            if (((*psrc >> 8) & 0x8F) != slot_num)
                ret_val = LTR114_ERR_ADCDATA_SLOTNUM;
            if (cnt != RECV_COUNT(*psrc))
                ret_val = LTR114_ERR_ADCDATA_CNT;

            // -       
            if (IS_THERM(*psrc) && (hnd->SpecialFeatures & LTR114_FEATURES_THERM)) 
            {
                if (IS_THERM_VALID(*psrc))  //  
                {
                    if (therm!=NULL)     //  ,    
                        *therm++ = (double)((INT)(*psrc & 0xFFFF0000)>>16)/16;
                    if (tcnt!=NULL) (*tcnt)++;
                }
                CYCLE_COUNTER_INC(cnt);
                continue;  //  . 
            }

            if (!cur_int && !hnd->LChQnt) // 0 .      -   
                cur_int = hnd->Interval;
            
            //       -  
            if ((cur_int) && (hnd->Interval > 0) && (hnd->SpecialFeatures & LTR114_FEATURES_STOPSW))
            {
                psrc++;
                CYCLE_COUNTER_INC(cnt);
                cur_int--;
                CYCLE_COUNTER_INC(cnt);
                continue;
            }
            if (cur_int)                                                            //  
                if ((hnd->Interval > 0) && (IS_AUTO_CALIBR_MODE(*psrc)))            //   
                {
                    if (correction_mode == LTR114_CORRECTION_MODE_AUTO)           // .  
                    {
                        res = proc_autocalibr(hnd, &psrc, &cnt, cur_int);     //    
                    }
                    else
                    {
                        psrc++;
                        CYCLE_COUNTER_INC(cnt);
                    }

                    cur_int--;
                    CYCLE_COUNTER_INC(cnt);
                    if (res!=LTR_OK)
                        ret_val = res;
                    continue;                  //  . 
                }
                else                  //    - ,   .  
                {
                    ret_val = LTR114_ERR_ADCDATA_CHNUM;
                    cur_ch = 0; 
                    cur_int = 0;
                    continue;
                }

            // -       
            if ((RECV0_SWITCH_MODE(*psrc) != LTR114_TABLE_SWMODE(create_table_lch(hnd->LChTbl[cur_ch])))
                ||(RECV0_GAIN(*psrc) != LTR114_TABLE_GAIN(create_table_lch(hnd->LChTbl[cur_ch])))
                )
            {

                ret_val = LTR114_ERR_ADCDATA_CHNUM; 
                if (IS_AUTO_CALIBR_MODE(*psrc))  //    -   . 
                {
                    cur_int = hnd->Interval;
                    if (correction_mode == LTR114_CORRECTION_MODE_AUTO)
                    {
                        res = proc_autocalibr(hnd, &psrc, &cnt, cur_int);         //    
                    }
                    else
                    {
                        psrc++;
                        CYCLE_COUNTER_INC(cnt);
                    }
                    cur_int--;
                }
                else
                {
                    cur_ch = 0;
                }
            }
            else        // . 
            {
                buff = (DWORD)RECV0_DATA2(*psrc)<< 16; //   
                //  
                CYCLE_COUNTER_INC(cnt);
                psrc++;

                if (((*psrc >> 8) & 0x8F) != slot_num)
                     ret_val = LTR114_ERR_ADCDATA_SLOTNUM;
                if (cnt != RECV_COUNT(*psrc))
                    ret_val = LTR114_ERR_ADCDATA_CNT;

                if (RECV1_N(*psrc) != LTR114_TABLE_SRC(create_table_lch(hnd->LChTbl[cur_ch])))
                {
                    ret_val = LTR114_ERR_ADCDATA_CHNUM;
                    cur_ch = 0;
                }
                else
                {
                    buff |= RECV1_DATA10(*psrc); 
                    frame[cur_ch] = buff;
                    if (++cur_ch >= hnd->LChQnt)                /*     */
                    {
                        int skipavgr = 0;
                        
                        for (cur_ch = 0; (cur_ch < hnd->LChQnt); cur_ch++)
                        {
                            d_frame[cur_ch] = SCALE_TO_INT(frame[cur_ch]);
                        }
                        if (cb_func!=NULL)
                            cb_func(hnd, d_frame);
                                                /*        */
                        for (cur_ch = 0; (cur_ch < hnd->LChQnt); cur_ch++)
                        {
                            double dt;
                            INT gain = (IS_MODE_R(hnd->LChTbl[cur_ch].MeasMode)) ? LTR114_URANGE_04 : hnd->LChTbl[cur_ch].Range;
                            dt = d_frame[cur_ch];
                            //  
                            if (correction_mode != LTR114_CORRECTION_MODE_NONE)
                            {
                                double k, b;
                                if (dt < hnd->AutoCalibrInfo[gain].LVal)
                                {
                                    k = hnd->AutoCalibrInfo[gain].Coef[0].Gain;
                                    b = hnd->AutoCalibrInfo[gain].Coef[0].Offset;
                                }
                                else if (dt > hnd->AutoCalibrInfo[gain].HVal)
                                {
                                    k = hnd->AutoCalibrInfo[gain].Coef[2].Gain;
                                    b = hnd->AutoCalibrInfo[gain].Coef[2].Offset;
                                }
                                else
                                {
                                    k = hnd->AutoCalibrInfo[gain].Coef[1].Gain;
                                    b = hnd->AutoCalibrInfo[gain].Coef[1].Offset;
                                }
                                dt = (dt - b) * k; 
                            }
                            if (flags & LTR114_PROCF_VALUE) //   
                            {
                                if (IS_R_SWMODE(hnd->LChTbl[cur_ch].MeasMode))
                                {
                                    dt = (theScalesVal[LTR114_URANGE_04]*dt/LTR114_MAX_SCALE_VALUE)*1000/hnd->ModuleInfo.CbrCoef.I[hnd->LChTbl[cur_ch].Range];
                                    if (dt < 0)
                                        dt = -dt;
                                    //dt = theRScalesVal[LTR114_TABLE_N(hnd->LChTbl[cur_ch])]*dt/LTR114_MAX_SCALE_VALUE;
                                }                                    
                                else
                                    dt = theScalesVal[hnd->LChTbl[cur_ch].Range]*dt/LTR114_MAX_SCALE_VALUE;
                            }

                            //     R  +I  -I
                            if ((IS_MODE_R(hnd->LChTbl[cur_ch].MeasMode))&&(flags & LTR114_PROCF_AVGR)&&(cur_ch > 0)&&(!skipavgr))
                            {
                                DWORD mask =~(LTR114_MEASMODE_R ^ LTR114_MEASMODE_NR);
                                 //       +I   -I
                                if ((((hnd->LChTbl[cur_ch].MeasMode == LTR114_MEASMODE_R) && (hnd->LChTbl[cur_ch-1].MeasMode == LTR114_MEASMODE_NR))
                                    ||((hnd->LChTbl[cur_ch].MeasMode == LTR114_MEASMODE_NR) && (hnd->LChTbl[cur_ch-1].MeasMode == LTR114_MEASMODE_R)))
                                    &&(hnd->LChTbl[cur_ch].Range == hnd->LChTbl[cur_ch-1].Range)
                                    &&(hnd->LChTbl[cur_ch].Channel == hnd->LChTbl[cur_ch-1].Channel))
                                {
                                    *(pdest-1) = ((fabs(*(pdest-1))) + fabs(dt))/2; //
                                    skipavgr = 1; //.   -       R
                                    
                                    continue;  //  . 
                                }
                            }
                            skipavgr = 0;
                         
                            *pdest++ = dt;
                        }


//                        first = 0;
                        cur_ch = 0;
                        cur_int = hnd->Interval;
                    }  //for (cur_ch = 0; (cur_ch < hnd->LChQnt); cur_ch++)
                }
            }
            CYCLE_COUNTER_INC(cnt);
        }
        *size = pdest - dest;
     } /*else @else if ((src == NULL) || (dest == NULL))@*/
     return ret_val;

}


LTR114API_DllExport(INT) LTR114_ProcessData
    (
    PTLTR114 hnd,
    DWORD *src,
    double *dest,
    INT *size,
    INT correction_mode,
    INT flags
    )
{
    /*
     *         
     *     LTR114_ProcessDataCb,   , 
     *          cb-
     */
    return LTR114_ProcessDataCb(hnd, src, dest, NULL, size, NULL, correction_mode, flags, NULL);
}





LTR114API_DllExport(INT) LTR114_ProcessDataTherm
    (
    PTLTR114 hnd,
    DWORD *src,
    double *dest,
    double *therm,
    INT *size,
    INT *tcnt,
    INT correction_mode,
    INT flags
    )
{
    /*
     *      
     *     LTR114_ProcessDataCb,   , 
     *     cb-
     */

    return LTR114_ProcessDataCb(hnd, src, dest, therm, size, tcnt, correction_mode, flags, NULL);
}



/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_SetADC
    (
    PTLTR114 hnd
    )
{
    /*
     *     LTR114.
     * 
     *        ,     
     *          .
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */

    INT ret_val = LTR_OK;

    if (hnd == NULL)
        return LTR114_ERR_INVALID_DESCR;
    if ((hnd->LChQnt + hnd->Interval <= 0) || (LTR114_MAX_LCHANNEL < hnd->LChQnt))
        return LTR114_ERR_INVALID_ADCLCHQNT;
    else
    {
        /*     
         * 1 -      ;
         * 2 -   ;
         * 3 -          
         *      .
         */

        WORD size =  (LTR114_MODULE_MODE_BASE_SIZE + hnd->LChQnt);
        DWORD cmdbuf[LTR114_MODULE_MODE_SIZE];            /*      */
        long divider;
        int i;
        DWORD *pcmdbuf;
        int presc;

        
        pcmdbuf = cmdbuf;
                                       //  -  
        *pcmdbuf++ = fill_command(modulemode_first_cmd, size*2);
        *pcmdbuf++ = fill_command(modulemode_cmd, hnd->LChQnt);

        
        for (i = 0; (i < hnd->LChQnt); i++)                                         /*2*/
        {
            WORD data = create_table_lch(hnd->LChTbl[i]);
            if (data != INVALID_CH)
                *pcmdbuf++ = fill_command(modulemode_cmd, data);
            else return LTR114_ERR_INVALID_LCH;
        }

        *pcmdbuf++ = fill_command(modulemode_cmd, hnd->Interval);

        
        //  
        presc = 0; 

        divider = hnd->FreqDivider;

        if ((divider < min_divider) || (max_divider < divider))
            //((hnd->ChRate = eval_adcrate(hnd->ADCRate.prescaler, divider)) > 400.0)
            return LTR114_ERR_INVALID_ADCRATE;
        if (!hnd->AdcOsr)
            hnd->AdcOsr = eval_OSR(1000*eval_adcrate(hnd->FreqDivider)/LTR114_ADC_DIVIDER);
        *pcmdbuf++ = fill_command(modulemode_cmd, hnd->FreqDivider);
        *pcmdbuf++ = fill_command(modulemode_cmd, (((WORD)hnd->SpecialFeatures << 8)&0xFF00) + ++presc);

        if ((hnd->SyncMode != LTR114_SYNCMODE_NONE)&&(hnd->SyncMode!=LTR114_SYNCMODE_INTERNAL)
            &&(hnd->SyncMode != LTR114_SYNCMODE_MASTER) && (hnd->SyncMode !=LTR114_SYNCMODE_EXTERNAL))
            return LTR114_ERR_INVALID_SYNCMODE;
        *pcmdbuf++ = fill_command(modulemode_cmd, (((WORD)hnd->SyncMode<<8)&0xFF00) + hnd->AdcOsr);

        if (ret_val == LTR_OK)
        {
            DWORD ack;
            long n;

            if ((n = LTR_Send(&hnd->Channel, cmdbuf, size, size*send_tm_out)) < 0)
                return n;
                
            if (n != size)
                return LTR_ERROR_SEND; 
                
            if ((n = LTR_Recv(&hnd->Channel, &ack, NULL, 1, ack_tm_out)) < 0)
                return n;
                
            if (n < 1)
                return LTR114_ERR_NOACK;
                
            ack &= ack_mask;
            if (ack != modulemode_ack)
               return LTR114_ERR_INVALIDACK;
        }

        /*    -        */
        
            ///int cnt = hnd->Interval/(LTR114_AUTOCALIBR_STEPS * LTR114_ADC_RANGEQNT) + 2;

       /* for (i=0; i < LTR114_ADC_RANGEQNT; i++)
        {
            if (hnd->AutoCalibrInfo[i].TempScale!=NULL)
            {
                free(hnd->AutoCalibrInfo[i].TempScale);
                hnd->AutoCalibrInfo[i].TempScale = NULL;
                hnd->AutoCalibrInfo[i].Index.Null = hnd->AutoCalibrInfo[i].Index.Ref = hnd->AutoCalibrInfo[i].Index.NRef = 
                    hnd->AutoCalibrInfo[i].Index.Interm = hnd->AutoCalibrInfo[i].Index.NInterm =0;
            }
            if (hnd->Interval > 0)
                hnd->AutoCalibrInfo[i].TempScale = (TSCALE_LTR114*)malloc((get_autocbr_cnt(hnd)+10)*sizeof(TSCALE_LTR114));//new TSCALE_LTR114[cnt];//
            
        }*/


        for (i=0; i < LTR114_ADC_RANGEQNT; i++)
        {
            if ((hnd->AutoCalibrInfo[i].TempScale!=NULL)&&(hnd->Interval > 0))
            {
                int scale_size;
                int index,j,k;

                TSCALE_LTR114* buff_scale = hnd->AutoCalibrInfo[i].TempScale;
                scale_size = get_autocbr_cnt(hnd)+10;

                hnd->AutoCalibrInfo[i].TempScale = (TSCALE_LTR114*)malloc(scale_size*sizeof(TSCALE_LTR114));
                
                index = hnd->AutoCalibrInfo[i].Index.Null;
                index = min(min(hnd->AutoCalibrInfo[i].Index.Null, hnd->AutoCalibrInfo[i].Index.Ref), 
                    min(hnd->AutoCalibrInfo[i].Index.NRef, min(hnd->AutoCalibrInfo[i].Index.NInterm,hnd->AutoCalibrInfo[i].Index.Interm)));
                
                
                for (j = index - 1, k=scale_size-1; (j>=0) && (j>= 0); j--,k--)
                    hnd->AutoCalibrInfo[i].TempScale[k] = buff_scale[j];
                
                hnd->AutoCalibrInfo[i].Index.Null = hnd->AutoCalibrInfo[i].Index.Ref = hnd->AutoCalibrInfo[i].Index.NRef = 
                    hnd->AutoCalibrInfo[i].Index.Interm = hnd->AutoCalibrInfo[i].Index.NInterm = min(index, scale_size);
                
                free(buff_scale);
            }
            else
            {
                if (hnd->AutoCalibrInfo[i].TempScale!=NULL)
                {
                    free(hnd->AutoCalibrInfo[i].TempScale);
                }
                hnd->AutoCalibrInfo[i].TempScale = NULL;
                hnd->AutoCalibrInfo[i].Index.Null = hnd->AutoCalibrInfo[i].Index.Ref = hnd->AutoCalibrInfo[i].Index.NRef = 
                hnd->AutoCalibrInfo[i].Index.Interm = hnd->AutoCalibrInfo[i].Index.NInterm =0;
                
                if (hnd->Interval > 0)
                    hnd->AutoCalibrInfo[i].TempScale = (TSCALE_LTR114*)malloc((get_autocbr_cnt(hnd)+10)*sizeof(TSCALE_LTR114));//new TSCALE_LTR114[cnt];//
            }
        }



        //filter_data_create(hnd);

        // -  ,    .      - 
        hnd->FrameLength = 2*(hnd->LChQnt + hnd->Interval);
        if (hnd->SpecialFeatures & LTR114_FEATURES_THERM)
            hnd->FrameLength++;



    } /*else @else if ((hnd->LChQnt <= 0) || (LTR11_MAX_LCHANNEL < hnd->LChQnt)@*/

    return ret_val;
}
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_Start
    (
    PTLTR114 hnd
    )
    {
    /*
     *     LTR114.
     * 
     *           .
     * 
     *   hnd -    .
     *  
     *     (.  ).
     */
    return start_ltr114(hnd, 0);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
LTR114API_DllExport(INT) LTR114_Stop
    (
    PTLTR114 hnd
    )
    {
    /*
     *   LTR114   .
     * 
     *            
     *    .
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */

#define RD_BUFSZ (100)
    DWORD ack;
    DWORD cmd;
    DWORD rd_buf[RD_BUFSZ];
    long rd_cnt;
    INT ret_val = LTR_OK;
    DWORD tick;


    if (hnd == NULL)
        {
        ret_val = LTR114_ERR_INVALID_DESCR;
        }
    else
        {
        long n;

        cmd = fill_command(stop_cmd, 0);
        if ((n = LTR_Send(&hnd->Channel, &cmd, 1, send_tm_out)) < 0)
            {
            ret_val = n;
            }
        if (n != 1)
            {
            ret_val = LTR_ERROR_SEND;
            }
        else
            {
            int data_received = 0;
            DWORD elapsed_time;
            const DWORD rx_time_out = 100;

            /*        */
            tick = GetTickCount();
            ack = ~stop_ack;
            do
                {
                rd_cnt = (long)LTR_Recv(&hnd->Channel, rd_buf, NULL, RD_BUFSZ, rx_time_out);
                if (rd_cnt > 0)                     /*    */
                    {
                    /*   */
                    ack = rd_buf[rd_cnt - 1] & ack_mask;
                    data_received = 1;
                    }
                elapsed_time = f_elapsed_time(tick);
                } while ((elapsed_time <= ack_tm_out) && (rd_cnt >= 0) && (ack != stop_ack));
            if (!data_received)                     /*      */
                {
                ret_val = LTR114_ERR_NOACK;
                }
            else if (ack != stop_ack)               /*   */
                {
                ret_val = LTR114_ERR_INVALIDACK;
                }
            }
        } /*else @if (hnd->Channel == NULL)@*/

    return ret_val;
#undef RD_BUFSZ
    }

/*------------------------------------------------------------------------------------------------*/

LTR114API_DllExport(INT) LTR114_Calibrate
    (
        PTLTR114 hnd
    )
    /*          
     *    ,  ,      
     * 
     *   hnd -     .
     *  
     *     (.  ).
     */
{
    DWORD cmd[3];
    INT res,i;
    DWORD *buf;
    double double_buf[9];
    int interv, qnt, features;
    int size = 2*LTR114_AUTOCALIBR_STEPS + 1;
    
    if (hnd == NULL)
        return LTR114_ERR_INVALID_DESCR;

    /*         -    */
    for (i=0; i < LTR114_ADC_RANGEQNT; i++)
    {
        if (hnd->AutoCalibrInfo[i].TempScale==NULL)
            hnd->AutoCalibrInfo[i].TempScale = (TSCALE_LTR114*)malloc((get_autocbr_cnt(hnd)+10) * sizeof(TSCALE_LTR114));
        hnd->AutoCalibrInfo[i].Index.NRef = hnd->AutoCalibrInfo[i].Index.Null = hnd->AutoCalibrInfo[i].Index.Ref =
            hnd->AutoCalibrInfo[i].Index.Interm = hnd->AutoCalibrInfo[i].Index.NInterm = 0;
    }


    //  -    
    cmd[0] = fill_command(test_cmd, LTR114_TEST_SELF_CALIBR);
    res = LTR_Send(&hnd->Channel, cmd, 1, send_tm_out);

    if (res < 0)
        return res;
    if (res != 1)
        return LTR_ERROR_SEND ;

    //  -  +  2   
    buf = (DWORD*)malloc(get_autocbr_cnt(hnd) * 2*LTR114_AUTOCALIBR_STEPS* sizeof(DWORD));
    size = get_autocbr_cnt(hnd) * 2*LTR114_AUTOCALIBR_STEPS;
    res = LTR_Recv(&hnd->Channel, buf, NULL, size, size*recv_tm_out);
    if (res < 0)
    {
        free(buf);
        return res;
    }
    if (res < size)
    {
        free(buf);
        return LTR_ERROR_RECV;
    }
  //  if ((buf[size - 1] & ack_mask)!=stop_ack)
 //       return LTR114_ERR_NOACK;

    res = LTR114_Stop(hnd);
    if (res!=LTR_OK)
    {
        free(buf);
        return res;
    }


    size--;
    interv = hnd->Interval;
    qnt = hnd->LChQnt;
    features = hnd->SpecialFeatures;

    /*         */
    hnd->Interval = LTR114_ADC_RANGEQNT*LTR114_AUTOCALIBR_STEPS;
    hnd->LChQnt = 0;
    hnd->SpecialFeatures  = 0;//|=LTR114_FEATURES_AUTOCBR;

    res = LTR114_ProcessData(hnd, buf, double_buf, &size, LTR114_CORRECTION_MODE_AUTO, 0);
    free(buf);
    if (res!=LTR_OK)
        return res;
    hnd->Interval = interv;
    hnd->LChQnt = qnt;
    hnd->SpecialFeatures = features;
    
    return LTR_OK;
    
}




LTR114API_DllExport(INT) LTR114_Recv(PTLTR114 hnd, DWORD *data, DWORD *tmark, DWORD size, DWORD timeout)
{
    return LTR_Recv(&hnd->Channel, data,tmark, size, timeout);
}

/*      */
LTR114API_DllExport(LCHANNEL) LTR114_CreateLChannel(INT MeasMode, INT Channel, INT Range)
{
    LCHANNEL ch;
    ch.Channel = Channel;
    ch.MeasMode = MeasMode;
    ch.Range = Range;
    return ch;
}
