unit ltr34api;

interface

uses windows, SysUtils;

const
        LTR_OK=0;
        LTR34_MAX_BUFFER_SIZE=2097151;
        LTR34_EEPROM_SIZE=2048;
        LTR34_USER_EEPROM_SIZE=1024;
        LTR34_DAC_NUMBER_MAX=8;

type

TLTR = packed record
        saddr:Cardinal;                       //   
        sport:word;                       //   
        csn:array [0..15] of byte;                     //   
        cc:word;                          //   
        flags:Cardinal;                       //   
        tmark:Cardinal;                       //    
        internal:Pointer;                   //   
end;

DAC_CHANNEL_CALIBRATION=packed record
	FactoryCalibOffset:array [0..LTR34_DAC_NUMBER_MAX] of real;
	FactoryCalibScale:array [0..LTR34_DAC_NUMBER_MAX] of real;

	UserCalibOffset:array [0..LTR34_DAC_NUMBER_MAX] of real;
	UserCalibScale:array [0..LTR34_DAC_NUMBER_MAX] of real;
end;

TDESCRIPTION_MODULE=packed record                        //                                                          //
  CompanyName:array[0..16] of byte;                                  //
  DeviceName:array[0..16] of byte;                                   //  
  SerialNumber:array[0..16] of byte;                                 //   
  Revision:byte;                                         //  
  Comment:array[0..256] of byte;                          //
end;

TDESCRIPTION_CPU=packed record                        //                                                          //
  Active:byte;                                      //     
  Name:array[0..16] of byte;                                        // 
  ClockRate:double;
  FirmwareVersion:Cardinal;                          //
  Comment:array[0..256] of byte;                          //
end;                                     //

//   ( )
TDESCRIPTION_LTR34=packed record                   //                                   //
  Description:TDESCRIPTION_MODULE;    //
  Cpu:TDESCRIPTION_CPU;       //
end;               //


// ,       E-440
TLTR34 = packed record
   //****         
  ltr:TLTR;							//         ltrapi.pdf
  //****  
  FrequencyDivisor:byte;            //    0..60 (31.25..500 )
  NumberOfChannel:byte;             //   0, 1, 2, 3  (1, 2, 4, 8)
  AcknowledgeType:bool;             //   false -    , true -     100 
  ExternalStart:bool;               //   true -  , false - 
  RingMode:bool;                    //    true -  , false -  
  BufferFull:bool;					//  -   - 
  BufferEmpty:bool;					//  -   - 
  DACRunning:bool;					//  -   
  FrequencyDAC:real;				//  -  -       
  DacCalibration:DAC_CHANNEL_CALIBRATION;
  ModuleDescription:TDESCRIPTION_LTR34;
end;
pTLTR34 = ^TLTR34;

Function LTR34_Init(module:pTLTR34) : Integer; stdcall;
Function LTR34_Open(module:pTLTR34; saddr:Cardinal; sport:Cardinal;csnArrayByte:Pointer;cc:short) : Integer; stdcall;
Function LTR34_Close(module:pTLTR34) : Integer; stdcall;
Function LTR34_IsOpened(module:pTLTR34) : Integer; stdcall;
Function LTR34_Recv(module:pTLTR34; dataCardinalArray:Pointer; tstampCardinal:Pointer; size:Cardinal; timeout:Cardinal) : Integer; stdcall;
Function LTR34_Send(module:pTLTR34; dataCardinalArray:Pointer; size:Cardinal; timeout:Cardinal) : Integer; stdcall;
Function LTR34_SendData(module:pTLTR34; dataDoubleArray:Pointer; size:Cardinal; timeout:Cardinal;calibrMainPset:bool;calibrExtravolts:bool) : Integer; stdcall;
Function LTR34_SetConfig(module:pTLTR34) : Integer; stdcall;
Function LTR34_DACStart(module:pTLTR34) : Integer; stdcall;
Function LTR34_DACStop(module:pTLTR34) : Integer; stdcall;
Function LTR34_Reset(module:pTLTR34) : Integer; stdcall;
Function LTR34_SetDescription(module:pTLTR34) : Integer; stdcall;
Function LTR34_GetDescription(module:pTLTR34) : Integer; stdcall;
Function LTR34_SetCalibrovka(module:pTLTR34) : Integer; stdcall;
Function LTR34_GetCalibrovka(module:pTLTR34) : Integer; stdcall;

Function LTR34_ReadFlash(module:pTLTR34; dataByteArray:Pointer; size:short; address:short) : Integer; stdcall;
Function LTR34_WriteFlash(module:pTLTR34; dataByteArray:Pointer; size:short; address:short) : Integer; stdcall;

Function LTR34_GetErrorString(Index:Integer) : string; stdcall;

implementation
Function LTR34_Init(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_Open(module:pTLTR34; saddr:Cardinal; sport:Cardinal;csnArrayByte:Pointer;cc:short) : Integer; external 'ltr34api.dll';
Function LTR34_Close(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_IsOpened(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_Recv(module:pTLTR34; dataCardinalArray:Pointer; tstampCardinal:Pointer; size:Cardinal; timeout:Cardinal) : Integer; external 'ltr34api.dll';
Function LTR34_Send(module:pTLTR34; dataCardinalArray:Pointer; size:Cardinal; timeout:Cardinal) : Integer; external 'ltr34api.dll';
Function LTR34_SendData(module:pTLTR34; dataDoubleArray:Pointer; size:Cardinal; timeout:Cardinal;calibrMainPset:bool;calibrExtravolts:bool) : Integer; external 'ltr34api.dll';
Function LTR34_SetConfig(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_DACStart(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_DACStop(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_Reset(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_SetDescription(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_GetDescription(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_SetCalibrovka(module:pTLTR34) : Integer; external 'ltr34api.dll';
Function LTR34_GetCalibrovka(module:pTLTR34) : Integer; external 'ltr34api.dll';

Function LTR34_ReadFlash(module:pTLTR34; dataByteArray:Pointer; size:short; address:short) : Integer; external 'ltr34api.dll';
Function LTR34_WriteFlash(module:pTLTR34; dataByteArray:Pointer; size:short; address:short) : Integer; external 'ltr34api.dll';

Function LTR34_GetErrorString(Index:Integer) : string; external 'ltr34api.dll';
end.
