// *******************    LTR34API **************************
// !!!      LTR34 :
//  1) LTRSERVER v1.4.x.x  
//  2)   - v3.0.2  
//  3)   - v1.5.x.x.  



#ifndef __LTR34API__
#define __LTR34API__

#define _DEBUG_MY

//#define APIENTRY __cdecl

#ifdef LTR34API_EXPORTS
  #define DllExport   __declspec(dllexport)  
#else
  #define DllExport   __declspec(dllimport)
#endif


#include "ltr\\include\\ltrapitypes.h"
#include "ltr\\include\\ltrapi.h"
#ifdef __cplusplus
extern "C" {
#endif
	// 
#define LTR34_ERROR_SEND_DATA      (-3000)
#define LTR34_ERROR_RECV_DATA      (-3001)
#define LTR34_ERROR_RESET_MODULE   (-3002)
#define LTR34_ERROR_NOT_LTR34      (-3003)
#define LTR34_ERROR_PARITY         (-3004)
#define LTR34_ERROR_OVERFLOW       (-3005)
#define LTR34_ERROR_INDEX          (-3006)
	//
#define LTR34_ERROR                (-3007)
#define LTR34_ERROR_EXCHANGE       (-3008)
#define LTR34_ERROR_FORMAT         (-3009)
#define LTR34_ERROR_CRC            (-3010)
#define LTR34_ERROR_PARAMETERS	   (-3011)
#define LTR34_ERROR_ANSWER		   (-3011)
	//
#define LTR34_MAX_BUFFER_SIZE		2097151
#define LTR34_EEPROM_SIZE			2048
#define LTR34_USER_EEPROM_SIZE		1024
#define LTR34_DAC_NUMBER_MAX		8
	//

	//
	typedef unsigned char byte;
	typedef unsigned short ushort;
	typedef unsigned int uint;

#ifndef __cplusplus
#define true 1
#define false 0
	typedef byte bool;
#endif
	//
#pragma pack(4)

	typedef struct 
	{
		float FactoryCalibOffset[LTR34_DAC_NUMBER_MAX];
		float FactoryCalibScale[LTR34_DAC_NUMBER_MAX];

		float UserCalibOffset[LTR34_DAC_NUMBER_MAX];
		float UserCalibScale[LTR34_DAC_NUMBER_MAX];

	} DAC_CHANNEL_CALIBRATION;

	//   ( )
	typedef struct                      //
	{                                   //
		TDESCRIPTION_MODULE    Description;    //
		TDESCRIPTION_CPU       Cpu;       //
	} TDESCRIPTION_LTR34;               //


	//****  
	typedef struct 
	{
		//****         
		TLTR ltr;							//         ltrapi.pdf  
		//****             
		byte FrequencyDivisor;            //    0..60 (31.25..500 )
		byte NumberOfChannel;             //   0, 1, 2, 3  (1, 2, 4, 8)
		bool AcknowledgeType;             //   false -    , true -     100 
		bool ExternalStart;               //   true -  , false - 
		bool RingMode;                    //    true -  , false -  
		bool BufferFull;					//  -   - 
		bool BufferEmpty;					//  -   - 
		bool DACRunning;					//  -   
		float FrequencyDAC;				//  -  -       
		DAC_CHANNEL_CALIBRATION	DacCalibration;
		TDESCRIPTION_LTR34 ModuleDescription;
	} TLTR34;                           
#pragma pack()


	//    TLTR34
	DllExport int APIENTRY LTR34_Init (TLTR34 *module);

	//     LTR34.
	//    STOP+RESET    .
	DllExport int APIENTRY LTR34_Open (TLTR34 *module, DWORD saddr, WORD sport, CHAR *csn, WORD cc);

	//    .
	DllExport int APIENTRY LTR34_Close (TLTR34 *module);

	//      .
	DllExport int APIENTRY LTR34_IsOpened (TLTR34 *module);

	/* 
	     .
	1)      ECHO -  , 
	,        
	  .  ,   ECHO 
	     .
	2)      PERIOD -  , 
	,    "".
	*/
	DllExport int APIENTRY LTR34_Recv (TLTR34 *module, DWORD *data, DWORD *tstamp, DWORD size, DWORD timeout);


	/*
	    
	1)  ltr-    
	   LTR34  . , 
	      (RingMode==0)
	       
	    LTR34. , 
	    -, ..
	     
	        
	   , ..   
	      
	     -.

	2)    LTR34    
	    ,   
	        (,
	 LTR34      , 
	          .
	  ,      ).
	       
	 RESET (       ltr-)
	     ( LTR34_Close()  LTR34_Open())
	*/
	DllExport int APIENTRY LTR34_Send (TLTR34 *module, DWORD *data, DWORD size, DWORD timeout);

	//    FIFO  
	DllExport int APIENTRY LTR34_SendData(TLTR34 *module, double *data, DWORD size, DWORD timeout, bool calibrMainPset, bool calibrExtraVolts);
	//   CONFIG
	DllExport int APIENTRY LTR34_SetConfig  (TLTR34 *module);

	//  .
	DllExport int APIENTRY LTR34_DACStart   (TLTR34 *module);

	//  .
	DllExport int APIENTRY LTR34_DACStop    (TLTR34 *module);
	DllExport int APIENTRY LTR34_Reset(TLTR34 *module);

	DllExport int APIENTRY LTR34_SetDescription(TLTR34 *module);
	DllExport int APIENTRY LTR34_GetDescription(TLTR34 *module);

	DllExport int APIENTRY LTR34_GetCalibrovka(TLTR34 *module);
	DllExport int APIENTRY LTR34_SetCalibrovka(TLTR34 *module);

	DllExport int APIENTRY LTR34_ReadFlash(TLTR34 *module, BYTE *data, WORD size, WORD Address);
	DllExport int APIENTRY LTR34_WriteFlash(TLTR34 *module, BYTE *data, WORD size, WORD Address);

	//   
	DllExport LPCSTR APIENTRY LTR34_GetErrorString(int error);

#ifdef _DEBUG_MY
	DllExport int APIENTRY LTR34_WriteFlash_FULL(TLTR34 *module, BYTE *data, WORD size, WORD Address);
#endif
#ifdef __cplusplus
}
#endif

#endif

