// ltr51api.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"

#include "ltr\\include\\ltr51api.h"
#include <stdio.h> 
#include <stdlib.h>


#ifdef _MANAGED
#pragma managed(push, off)
#endif

BOOL APIENTRY DllMain( HMODULE hModule,
					  DWORD  ul_reason_for_call,
					  LPVOID lpReserved
					  )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}



#define MODULE_ID (0x33)

//   AVR
#define ALTERA_LOAD                                   (0x80C1)
#define SET_FS                                        (0x80C2)
#define SET_BASE                                      (0x80C3) 
#define SET_THRESHOLD                                 (0x80C4)
#define READ_THRESHOLD								  (0x80C5)  
#define SET_EDGE_MODE                                 (0x80C6) 
#define START                                         (0x80C7)
#define STOP    						              (0x80C8)
#define READ_ID_RECORD                  			  (0x80C9)
#define WRITE_EEPROM                                  (0x80CA) 
#define READ_EEPROM                                   (0x80CB)



#define INIT                						  (0x80CF)
#define CHANNELS_ENA                                   (0x1A) 
#define PARITY_ERROR                                   (0x1B)        
#define TEST_REG_ERROR                                 (0x1C)  
#define TEST_REG_OK                                    (0x1E)
#define ALTERA_LOAD_OK                                 (0x1F)  


//  

#define STOP_TIMEOUT								  (5000)
#define CMD_TIMEOUT			     					  (2000)
#define BASE_1                                        (5000) 
#define BASE_2										  (5000)
#define FS                                            (500000)

//     
#define TIMEOUT_CMD_SEND							   (4000)
#define TIMEOUT_CMD_RECIEVE							   (6000)


#define F_OSC  (15000000) //    AVR 
#define F_REF  (20000000.0) //        



INT eval_parity(DWORD cmd);                                                                                
INT check_parity(DWORD cmd);                                                                               

int LTR51_ReadConfRecord(PTLTR51 hnd, CHAR *version, CHAR *date, CHAR *name, CHAR *serial, CHAR *FPGA_version); 
WORD CRC_Calc(BYTE *b, WORD N);                                                                     
void CRC_Update(WORD *crc, BYTE *b, int c) ;   
DWORD CalkTickInterval(DWORD PrevTick, DWORD ActTick);                                                                                                                         

int __stdcall DllEntryPoint (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	// Included for compatibility with Borland 

	return 1;
}

/****************************************************************************************************************

           

****************************************************************************************************************/

//              //
// :
// hnd -                                           //   

LTR51API_DllExport (INT) LTR51_Init(PTLTR51 hnd)
{
	int err;
	int i;
	double HighThreshold, LowThreshold;
	int ThresholdRange;


	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR; 

	hnd->size=sizeof(TLTR51);


	hnd->SetUserPars=0;

	hnd->Fs=500000;
	hnd->Base=70;
	hnd->LChQnt=16;

	/*     */
	HighThreshold=0.8;
	LowThreshold=-0.8;											
	ThresholdRange=1;

	/*   16   */

	for(i=0;i<16;i++)
		LTR51_CreateLChannel(i+1, &HighThreshold, &LowThreshold, ThresholdRange, 0);    


	strcpy(hnd->ModuleInfo.FirmwareVersion,"\0");
	strcpy(hnd->ModuleInfo.FirmwareDate,"\0");
	strcpy(hnd->ModuleInfo.FPGA_Version,"\0"); 
	strcpy(hnd->ModuleInfo.Serial,"\0");
	strcpy(hnd->ModuleInfo.Name,"\0");



	err=LTR_Init(&(hnd->Channel));		//   
	if(err==LTR_OK)
		return LTR51_NO_ERR;
	else return err;

}

/**************************************************************************************************

       LTR51.  RESET 

**************************************************************************************************/

//      ,   AVR       //
//                                                                                                 //
// :																					   //
// hnd -                                                    	   //
// net_addr -                                                                    //
// net_port -                                                                            //
// *crate_sn -    																   //
// slot_num -     ,                           //
// *ttf_name -    ,                               //
//                                                                                                 //
//   -                                                                //

LTR51API_DllExport (INT) LTR51_Open(PTLTR51 hnd, DWORD net_addr, WORD net_port, CHAR *crate_sn, INT slot_num, CHAR *ttf_name)
{

	DWORD command[2], stop[1], rbuf[1024]; //       / 
	DWORD *load_command;

	const int max_slot_num = 16;
	int read_cntr;
	int tick;
	int i;

	char *s;
	char *s_rew;
	char *s_begin;

	FILE *f;
	int tmp_x;
	int l;
	int j=0;
	int k=0;
	int FileSize;
	int cur_ftell;
	int ArraySize;

	BYTE *AlteraArray;
	DWORD AlteraArray_Size=0;

	int err=LTR_OK;
	int warning=0;

	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR; 



	//           

	f=fopen(ttf_name, "r");
	if(f==NULL)
	{
		err=LTR51_ERR_WRONG_FPGA_FILE;
		return err;
	}

	s=NULL;
	s_rew=0;
	s_begin=NULL;
	load_command=0;


	s=(char *) malloc(1000);


	if(s==NULL)
	{
		err=LTR_ERROR_MEMORY_ALLOC;
		goto exit;
	}

	s_rew=s;
	s_begin=s; //      


	//     
	fseek(f, 0, SEEK_END);
	FileSize=ftell(f);
	ArraySize=(int) FileSize/4;

	AlteraArray=(BYTE *) malloc(ArraySize); //     


	if( AlteraArray==NULL)
	{
		err=LTR_ERROR_MEMORY_ALLOC;
		goto exit;
	}

	rewind(f);


	while(fgets(s,1000,f)!=NULL)
	{

		cur_ftell=ftell(f);    

		if(cur_ftell==FileSize)
			break;


		//      
		while(l=(strlen(s)-1))
		{
			sscanf(s,"%d", &tmp_x); 
			s+=4;

			*(AlteraArray+j)=(BYTE) tmp_x;
			j++;
		}
		s=s_rew; 
		k++;

	}

	//    , ..  -   

	while(l=(strlen(s)))
	{
		sscanf(s,"%d", &tmp_x); 

		*(AlteraArray+j)=(BYTE) tmp_x;
		j++;

		s+=3;

		if(*s!=',')
			break;

		s++; 
	}

	AlteraArray_Size=j;

	//   

	hnd->Channel.saddr=net_addr; //  

	if(crate_sn==NULL)
	{
		err=LTR51_ERR_INVALID_CRATE_SN; //  
		goto exit;
	}

	sprintf(hnd->Channel.csn, crate_sn);   


	if((slot_num<1)||(slot_num)>max_slot_num)
	{
		err=LTR51_ERR_INVALID_SLOT_NUM;
		goto exit;
	} 

	hnd->Channel.cc=slot_num;   

	/*err=LTR_IsOpened(&(hnd->Channel));
	/*if(err)
	goto exit; */

	//       
	if((err=LTR_Open(&(hnd->Channel)))!=LTR_OK)   
		if(err<0)
			if(err==LTR_WARNING_MODULE_IN_USE)
			{
				warning=err;
				err=LTR_OK;
			}
			else 
				goto exit; 
		else
		{
			err=LTR51_ERR_CANT_OPEN;
			goto exit;
		}

		slot_num-=1;


		//  :  : , , 

		command[0]=LTR010CMD_STOP|(slot_num<<8);
		stop[0]=LTR010CMD_STOP|(slot_num<<8);



		//tmp_err=LTR_Send(&(hnd->Channel), stop, 1, 2000);	


		if((err=(LTR_Send(&(hnd->Channel), stop, 1, TIMEOUT_CMD_SEND))) != 1 )  //   "STOP"
			if(err<0)
				goto exit;
			else
			{
				err=LTR51_ERR_CANT_SEND_COMMAND;   
				goto exit;
			}

			command[0]=LTR010CMD_RESET|(slot_num<<8);  
			if((err=(LTR_Send(&(hnd->Channel), command, 1, TIMEOUT_CMD_SEND))) != 1 )  //  "RESET"
				if(err<0)
					goto exit; 
				else
				{ 
					err=LTR51_ERR_CANT_RESET_MODULE; 
					goto exit;
				} 

				tick=GetTickCount();

				while(1)
				{
					read_cntr=LTR_Recv(&(hnd->Channel), rbuf, NULL, 1024, 100); //  ,   FIFO

					if(read_cntr!=0)
						if((rbuf[read_cntr-1]&0xFFFF0000)==(0x33330000))
							break;



					if(CalkTickInterval(tick, GetTickCount())>=5000) //    5    
					{
						err=LTR51_ERR_CANT_RESET_MODULE; 
						goto exit;
					}
				}


				if((err=(LTR_Send(&(hnd->Channel), stop, 1, TIMEOUT_CMD_SEND))) != 1)  //   "STOP"
					if(err<0)
						goto exit; 
					else
					{
						err=LTR51_ERR_CANT_SEND_COMMAND;
						goto exit;
					}

					Sleep(500);
					command[0]=INIT|(slot_num<<8);

					if((err=(LTR_Send(&(hnd->Channel), command, 1, TIMEOUT_CMD_SEND))) != 1 )  //    INSTR
						if(err<0)
							goto exit;
						else
						{
							err=LTR51_ERR_CANT_SEND_COMMAND;
							goto exit; 
						}


						//           

						load_command=(DWORD *) malloc(((AlteraArray_Size)+1)*sizeof(DWORD)); //      

						if( load_command==NULL)
						{
							err=LTR_ERROR_MEMORY_ALLOC;
							goto exit;
						}


						j=0;
						for(i=0;i<AlteraArray_Size;i+=2)
						{
							load_command[j]=AlteraArray[i+1];
							load_command[j]<<=16;
							load_command[j]|=(AlteraArray[i]<<24);

							load_command[j]|=ALTERA_LOAD;
							load_command[j]|=(slot_num<<8);

							j++;

						} 

						if((err=(LTR_Send(&(hnd->Channel), load_command, j, TIMEOUT_CMD_SEND))) !=j )  //      
							if(err<0)
								goto exit; 
							else
							{
								err=LTR51_ERR_CANT_SEND_COMMAND;
								goto exit; 
							} 



							if((err=(LTR_Recv(&(hnd->Channel), rbuf, NULL, 3, TIMEOUT_CMD_RECIEVE))!=3))   //  
								if(err<0)
									goto exit; 
								else
								{
									err=LTR51_ERR_MODULE_NO_RESPONCE;
									goto exit; 
								}


								for(i=0;i<3;i++)   					   
									if(check_parity(rbuf[i]))
									{
										err=LTR51_ERR_PARITY_FROM_MODULE;
										goto exit; 
									} 

									if((rbuf[0]&0x0000FFDF)!=(load_command[0]&0x0000FFDF))
									{
										err=LTR51_ERR_CANT_LOAD_ALTERA;  
										goto exit; 
									} 


									if((rbuf[1]&0x0000001F)==TEST_REG_ERROR)
									{
										err=LTR51_ERR_ALTERA_TEST_FAILED;
										goto exit; 
									} 

									if((rbuf[1]&0x0000001F)!=TEST_REG_OK) 
									{
										err=LTR51_ERR_CANT_OPEN_MODULE;  
										goto exit; 
									}


									if((rbuf[2]&0x0000001F)!=CHANNELS_ENA)  
									{
										err=LTR51_ERR_CANT_OPEN_MODULE;  
										goto exit; 
									}

									hnd->ChannelsEna=rbuf[2]>>16;


									//   
									err=LTR51_ReadConfRecord(hnd, hnd->ModuleInfo.FirmwareVersion, hnd->ModuleInfo.FirmwareDate, hnd->ModuleInfo.Name, hnd->ModuleInfo.Serial, hnd->ModuleInfo.FPGA_Version); 
									if(err) return err;  



									if(warning)
										err=warning;
									else 
										err=LTR51_NO_ERR; 

exit:

									fclose(f);

									//      ,   

									if(s!=NULL)
									{
										s=s_begin;
										free(s); 
									}

									if(AlteraArray!=NULL)
										free(AlteraArray); 


									if(load_command!=NULL)
										free(load_command);

									return err;

}


/***************************************************************************************

 :   

****************************************************************************************/               


LTR51API_DllExport (INT) LTR51_IsOpened(PTLTR51 hnd)
{
	INT res;

	if(hnd!=NULL) {
		res=LTR_IsOpened(&hnd->Channel);
	} else res=LTR_ERROR_PARAMETRS;

	return res;
}


/***************************************************************************************

    

****************************************************************************************/               

LTR51API_DllExport (INT) LTR51_Close(PTLTR51 hnd)
{
	int err=0;

	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR; 


	if ((err=LTR_Close(&(hnd->Channel)))!=LTR_OK) 
		return(err);	

	return LTR51_NO_ERR; 
}


/*******************************************************************************************************

    

*******************************************************************************************************/
//                                         	 //
// :																                         //
// PhysChannel -    (1-16)                                                    //
// HighThreshold -     									             //
// LowThreshold -     	
// EdgeMode -    


LTR51API_DllExport (DWORD) LTR51_CreateLChannel(INT PhysChannel, double *HighThreshold, double *LowThreshold, int ThresholdRange, int EdgeMode)
{


	//  16                           
	//   24-31 -         
	//   16-23 -   
	//   8-15 -        (0- , 1- ) 
	//   0-7 -      ( 0)   PhysChannel -  
	//   ""       

#define Ku1  (-1.6737)
#define Ku2  (-0.2010)
#define Uref (2.048)

	DWORD ChannelWord;
	int HighThresholdCode, LowThresholdCode;  

	//   
	if(ThresholdRange)
	{
		HighThresholdCode=(int) (((128*(*HighThreshold*Ku1+Uref))/Uref)+0.5);

		if((HighThresholdCode>255)||(HighThresholdCode<0))
			HighThresholdCode=0; 

		/* if(HighThresholdCode<0)
		HighThresholdCode=0;      */

		ChannelWord=HighThresholdCode;//(DWORD) (((128*(*HighThreshold*Ku1+1))/Uref)+0.5);
		ChannelWord<<=8;
		LowThresholdCode=(int) (((128*(*LowThreshold*Ku1+Uref))/Uref)+0.5); 


		if((LowThresholdCode>255)||(LowThresholdCode<0))
			LowThresholdCode=255;

		/*if(LowThresholdCode<0)
		LowThresholdCode=0; */   

		ChannelWord|=(BYTE)LowThresholdCode; //(DWORD) (((128*(*LowThreshold*Ku1+1))/Uref)+0.5); 
		ChannelWord<<=16;

		*HighThreshold=(Uref*(2*HighThresholdCode-256))/(Ku1*256);
		*LowThreshold=(Uref*(2*LowThresholdCode-256))/(Ku1*256);  

	}
	//    
	else
	{
		HighThresholdCode=(int) (((128*(*HighThreshold*Ku2+Uref))/Uref)+0.5); 

		if((HighThresholdCode>255)||(HighThresholdCode<0))
			HighThresholdCode=0;

		ChannelWord=HighThresholdCode;//(DWORD) (((128*(*HighThreshold*Ku1+1))/Uref)+0.5);
		ChannelWord<<=8;
		LowThresholdCode=(int) (((128*(*LowThreshold*Ku2+Uref))/Uref)+0.5);  

		if((LowThresholdCode>255)||(LowThresholdCode<0))
			LowThresholdCode=255;

		ChannelWord|=(BYTE)LowThresholdCode; //(DWORD) (((128*(*LowThreshold*Ku1+1))/Uref)+0.5); 
		ChannelWord<<=16;

		*HighThreshold=(Uref*(2*HighThresholdCode-256))/(Ku2*256);
		*LowThreshold=(Uref*(2*LowThresholdCode-256))/(Ku2*256);
	}

	/*if(*HighThreshold<*HighThreshold)
	return LTR51_ERR_WRONG_THRESHOLD_VALUES;  */

	ChannelWord|=PhysChannel-1;
	ChannelWord|=(EdgeMode&0x1)<<8;

	return ChannelWord;

}

/*******************************************************************************************************

         

*******************************************************************************************************/

//       :   Fs,   Base	 //
// :																                         //
// hnd -      													 //
// PhysChannel -    (0-16)                                                    //
// HighThreshold -     									             //
// LowThreshold -     									             //


LTR51API_DllExport (INT) LTR51_GetThresholdVals(PTLTR51 hnd, INT LChNumber, double *HighThreshold, double *LowThreshold, INT ThresholdRange)
{

	//#define Ku1  (-1.6737)
	//#define Ku2  (-0.2010)
	//#define Uref (2.048)

	BYTE HighThresholdCode, LowThresholdCode; 

	HighThresholdCode=(hnd->LChTbl[LChNumber]&0xFF000000)>>24;
	LowThresholdCode=(hnd->LChTbl[LChNumber]&0x00FF0000)>>16;  

	if(ThresholdRange==1)
	{
		*HighThreshold=Uref*((2*HighThresholdCode-256)/(256*Ku1));
		*LowThreshold=Uref*((2*LowThresholdCode-256)/(256*Ku1));     
	}
	else
	{
		*HighThreshold=Uref*((2*HighThresholdCode-256)/(256*Ku2));
		*LowThreshold=Uref*((2*LowThresholdCode-256)/(256*Ku2));     
	}


	return LTR51_NO_ERR; 
}


/*******************************************************************************************************

      

*******************************************************************************************************/
//       :    Fs,   Base	 //
// :																                                 //
// hnd -      												             //


LTR51API_DllExport (INT) LTR51_Config(PTLTR51 hnd)
{

	//#define FS (40)



	DWORD command[32];
	DWORD command_ack[32];
	DWORD slot_num;
	WORD EdgeModeMask=0;
	WORD FsDiv;  //   
	WORD Base;     
	int i,j;
	int err;
	
	BYTE HighThresholdCode, LowThresholdCode;	


	double tmp1, tmp2, tmp3;


	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR; 


	for(i=0;i<hnd->LChQnt;i++)
	{
		HighThresholdCode=(hnd->LChTbl[i]&0xFF000000)>>24;
		LowThresholdCode=(hnd->LChTbl[i]&0x00FF0000)>>16;  

		if(HighThresholdCode>LowThresholdCode)
			return LTR51_ERR_WRONG_THRESHOLD_VALUES; 

	}   

	slot_num   =hnd->Channel.cc -1; 

	if(!hnd->SetUserPars)
	{
		FsDiv=(WORD) ((F_REF/FS)-1+0.5);  	

		hnd->Fs=FS;
		hnd->Base=BASE_1;
		Base = hnd->Base;
	} 
	else
	{

		FsDiv=(WORD) ((F_REF/hnd->Fs)-1+0.5);

		hnd->Fs=(F_REF/(FsDiv+1));
		Base=hnd->Base;
	}


	if(hnd->SetUserPars)
	{
		if(FsDiv<39)
			return LTR51_ERR_WRONG_FS_SETTINGS;   

		if(hnd->Base<70)
			return LTR51_ERR_WRONG_BASE_SETTINGS; 
	} 

	tmp1=((double) hnd->AcqTime/1000.0)*F_REF;
	tmp2=(FsDiv+1)*Base;
	tmp3=tmp1/tmp2;   

	hnd->TbaseQnt=(int) (tmp3+0.5);  
	if(hnd->TbaseQnt<2)
		return LTR51_ERR_NOT_ENOUGH_POINTS;

	tmp1=hnd->TbaseQnt*(FsDiv+1)*Base;

	hnd->AcqTime=(int) ((tmp1/F_REF)*1000+0.5); 
	hnd->F_Base=hnd->Fs/Base;

	for(i=0;i<hnd->LChQnt;i++)
		EdgeModeMask|=((hnd->LChTbl[i]&0x100)>>8)<<((hnd->LChTbl[i]&0xFF));

	//    Fs


	command[0]=FsDiv;
	command[0]<<=16;
	command[0]|=SET_FS;
	command[0]|=(slot_num<<8);
	command[0]|=(eval_parity(command[0]))<<5; 


	//   Base

	command[1]=Base;
	command[1]<<=16;
	command[1]|=SET_BASE;
	command[1]|=(slot_num<<8);
	command[1]|=(eval_parity(command[1]))<<5; 

	//      

	command[2]=EdgeModeMask;
	command[2]<<=16;
	command[2]|=SET_EDGE_MODE;
	command[2]|=(slot_num<<8);
	command[2]|=(eval_parity(command[2]))<<5; 

	if((err=(LTR_Send(&(hnd->Channel), command, 3, TIMEOUT_CMD_SEND))) !=3)  
		if(err<0)
			return err;
		else
			return LTR51_ERR_CANT_SEND_COMMAND;


	if((err=(LTR_Recv(&(hnd->Channel), command_ack, NULL, 3, TIMEOUT_CMD_RECIEVE))!=3))   //  
		if(err<0)
			return (err);
		else
			return LTR51_ERR_MODULE_NO_RESPONCE;


	for(i=0; i<3;i++)
	{
		if(check_parity(command_ack[i]))
			return LTR51_ERR_PARITY_FROM_MODULE;

		if((command_ack[i]&0x0000001F)==PARITY_ERROR) 
			return LTR51_ERR_PARITY_TO_MODULE; 
	}   

	if((command_ack[0]&0x0000FF1F)!=(command[0]&0x0000FF1F))
		return LTR51_ERR_CANT_SET_FS;

	if((command_ack[1]&0x0000FF1F)!=(command[1]&0x0000FF1F))
		return LTR51_ERR_CANT_SET_BASE;

	if((command_ack[2]&0x0000FF1F)!=(command[2]&0x0000FF1F))
		return LTR51_ERR_CANT_SET_EDGE_MODE;  

	//    -   
	//   :  4  -  ,  4  -     (1-High, 0-Low) 

	j=0;

	for(i=0;i<2*hnd->LChQnt;i+=2)												  
	{
		command[i]=(hnd->LChTbl[j]&0xFF000000)>>24;//hnd->HighThreshold[0];   //   ""
		command[i]<<=8; 
		command[i]|=1<<4;  //  
		command[i]|=hnd->LChTbl[j]&0xF;      //  
		command[i]<<=16;
		command[i]|=SET_THRESHOLD;     
		command[i]|=(slot_num<<8);
		command[i]|=(eval_parity(command[i]))<<5; 

		command[i+1]=(hnd->LChTbl[j]&0x00FF0000)>>16;//hnd->HighThreshold[0];   //   ""
		command[i+1]<<=8; 
		command[i+1]|=0<<4;  //  
		command[i+1]|=hnd->LChTbl[j]&0xF;      //  
		command[i+1]<<=16;
		command[i+1]|=SET_THRESHOLD;
		command[i+1]|=(slot_num<<8);
		command[i+1]|=(eval_parity(command[i+1]))<<5;

		j++;

	}

	if((err=(LTR_Send(&(hnd->Channel), command, 2*hnd->LChQnt, TIMEOUT_CMD_SEND))) !=2*hnd->LChQnt)  
		if(err<0)
			return err;
		else
			return LTR51_ERR_CANT_SEND_COMMAND;


	if((err=(LTR_Recv(&(hnd->Channel), command_ack, NULL, 2*hnd->LChQnt, TIMEOUT_CMD_RECIEVE))!=2*hnd->LChQnt))   //  
		if(err<0)
			return (err);
		else
			return LTR51_ERR_MODULE_NO_RESPONCE;


	for(i=0;i<2*hnd->LChQnt;i++)
	{

		if(check_parity(command_ack[i]))
			return LTR51_ERR_PARITY_FROM_MODULE;

		if((command_ack[i]&0x0000001F)==PARITY_ERROR) 
			return LTR51_ERR_PARITY_TO_MODULE; 

		if((command_ack[i]&0x0000FF1F)!=(command[i]&0x0000FF1F))
			return LTR51_ERR_CANT_SET_FS;  

	}  

	//      

	j=0;  
	for(i=0;i<2*hnd->LChQnt;i+=2)
	{

		if((command_ack[i]>>16)==0)
			hnd->LChTbl[j]&=0xFFFFFDFF;
		else
			hnd->LChTbl[j]|=0x00000200;  

		if((command_ack[i+1]>>16)==0)
			hnd->LChTbl[j]&=0xFFFFFEFF;
		else
			hnd->LChTbl[j]|=0x00000100;    


		j++;

	}  

	return LTR51_NO_ERR;  
}



/***************************************************************************************

   

****************************************************************************************/ 

//                                  //
//  - hnd,        //

LTR51API_DllExport (INT) LTR51_Start(PTLTR51 hnd)
{
	DWORD command;
	DWORD command_ack;
	DWORD slot_num; 
	int err;


	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR; 

	slot_num   =hnd->Channel.cc -1;


	command=START;
	command|=(slot_num<<8);
	command|=(eval_parity(command))<<5; 

	if((err=(LTR_Send(&(hnd->Channel), &command, 1, TIMEOUT_CMD_SEND))) != 1 )  
		if(err<0)
			return err;
		else
			return LTR51_ERR_CANT_SEND_COMMAND;



	if((err=(LTR_Recv(&(hnd->Channel), &command_ack, NULL, 1, TIMEOUT_CMD_RECIEVE)))!=1) 
		if(err<0)
			return err;
		else
			return LTR51_ERR_MODULE_NO_RESPONCE;  



	if(check_parity(command_ack))
		return LTR51_ERR_PARITY_FROM_MODULE;


	if((command_ack&0x0000001F)==PARITY_ERROR) 
		return LTR51_ERR_PARITY_TO_MODULE; 


	if((command_ack&0x0000FF1F)!=(command&0x0000FF1F))
		return LTR51_ERR_CANT_START_DATA_AQC; 


	return LTR51_NO_ERR;      

}

/***************************************************************************************

   

****************************************************************************************/ 
//                               //
//  - hnd,       //

LTR51API_DllExport (INT) LTR51_Stop(PTLTR51 hnd)
{
#define STOP_BUF_SIZE (10000)

	DWORD command;
	DWORD *stop_buf;
	DWORD slot_num; 
	int err=LTR51_NO_ERR;
	int StopDetected=0;
	DWORD StopAcqCommand;

	INT read_cntr;                                                                                 
	DWORD tick;   //            



	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR;

	slot_num=hnd->Channel.cc-1;

	stop_buf=(DWORD *) calloc(15000, sizeof(DWORD));

	if(stop_buf==NULL)
	{
		err=LTR_ERROR_MEMORY_ALLOC;
		goto exit;
	}


	command=STOP;
	command|=(slot_num<<8);
	command|=(eval_parity(command))<<5;


	if((err=(LTR_Send(&(hnd->Channel), &command, 1, TIMEOUT_CMD_SEND))) != 1 )
		if(err<0)
			goto exit;
		else
		{
			err=LTR51_ERR_CANT_SEND_COMMAND;
			goto exit;
		}

		err=0; 

		read_cntr=LTR_Recv(&(hnd->Channel), stop_buf, NULL, 1, TIMEOUT_CMD_RECIEVE); 
		if(read_cntr<0)
		{
			err=read_cntr;
			goto exit;
		}
		else
			if(read_cntr==0)
			{
				err=LTR51_ERR_MODULE_NO_RESPONCE;
				goto exit; 
			}

			StopDetected=(stop_buf[read_cntr-1]&0xFFDF)==(STOP|(slot_num<<8)); 
			if(StopDetected)
				StopAcqCommand=stop_buf[read_cntr-1];

			tick=GetTickCount();



			while(1)
			{

				read_cntr=LTR_Recv(&(hnd->Channel), stop_buf, NULL, 15000, 100); //  ,   FIFO



				if(read_cntr!=0)
					if((stop_buf[read_cntr-1]&0xFFDF)==(STOP|(slot_num<<8)))
					{
						StopAcqCommand=stop_buf[read_cntr-1];  
						break;
					} 



					if(CalkTickInterval(tick, GetTickCount())>=TIMEOUT_CMD_RECIEVE+(DWORD) ((1/(hnd->F_Base))*1000)) //    5    
						return LTR51_ERR_CANT_STOP_DATA_AQC; 

			} 



			if(check_parity(StopAcqCommand))
			{
				err=LTR51_ERR_PARITY_FROM_MODULE;
				goto exit;
			}  

			if((StopAcqCommand&0x0000001F)==PARITY_ERROR)
			{
				err=LTR51_ERR_PARITY_TO_MODULE; 
				goto exit;
			} 

			if((StopAcqCommand&0x0000FFDF)!=(command&0x0000FFDF))
			{
				err=LTR51_ERR_CANT_STOP_DATA_AQC; 
				goto exit; 
			}  

exit:

			//read_cntr=LTR_Recv(&(hnd->Channel), rubbish_buf, NULL, 32, CMD_TIMEOUT);  

			if(stop_buf!=NULL)
				free(stop_buf);

			return err;     




}


/****************************************************************************************

       

****************************************************************************************/ 

// :
// hnd -     
// *data -   
// *tmark -   
// size - -  
// timeout -      

LTR51API_DllExport (INT) LTR51_Recv(PTLTR51 hnd, DWORD *data, DWORD *tmark, DWORD size, DWORD timeout)
{
	INT res;

	res=LTR_Recv(&(hnd->Channel), data, tmark, size, timeout); 


	if(res>0)
		if(hnd->Channel.flags&FLAG_RBUF_OVF)
			res=LTR51_ERR_ERROR_OVERFLOW; 

	return res;
} 



/****************************************************************************************

        

****************************************************************************************/ 


LTR51API_DllExport (INT) LTR51_ProcessData(PTLTR51 hnd, DWORD *src, DWORD *dest, double *Frequency, DWORD *size)
{

	//    ,          
	// :
	// hnd -      												             
	// *src -    
	// *dest -       N  M
	// *Frequency -              . 
	// *size -    -    src.   -   dest.


	WORD Base;
	WORD FsDiv;//   
	int i,j,k,l,dest_qnt; // 
	int Tp_Qnt;
	int Tp_Cntr;
	int Empty_Tp_Cntr;
	int Empty_Tp_Flag=0;
	BYTE PhysChannel;
	int RelCntr; //  ,   
	int Index=0;
	double FreqS;
	BYTE WordsCntr;
	int CurSampleIndex; //   
	int N, Nm;
	int M1, Mk;
	int denominator; 


	if((*size)%32)
		return LTR51_ERR_WRONG_SRC_SIZE;

	if(!hnd->SetUserPars)
	{
		FsDiv=(WORD) ((F_REF/FS)-1+0.5);  

		if(hnd->AcqTime<1000) 
			Base=BASE_1;
		else
			Base=BASE_2;

		//hnd->Fs=FS;
		//hnd->Base=Base;
	} 
	else
	{

		FsDiv=(WORD) ((F_REF/hnd->Fs)-1+0.5);

		//hnd->Fs=(F_REF/(FsDiv+1));
		Base=hnd->Base;
	}

	//    
	// WordsCntr    
	WordsCntr=(src[0]&0x000000E0)>>5;

	//      
	j=0;
	k=15;

	for(i=0;i<*size;i++)
	{
		if(WordsCntr!=((src[i]&0x000000E0)>>5))
			return LTR51_WRONG_DATA;

		WordsCntr++; 
		if(WordsCntr>7)
			WordsCntr=0;

	}  //    for(i)

	dest_qnt=*size; 

	if(dest!=NULL) 
	{
		dest_qnt=0;

		for(i=0;i<hnd->LChQnt;i++)  
		{
			l=0;
			PhysChannel=(hnd->LChTbl[i])&0xF;

			//         15-PhysChannel         
			RelCntr=2*(15-PhysChannel)+1;

			Index=0;

			for(k=0;(Index+32)<*size;k++)
			{
				Index=k*32+RelCntr;
				dest[l+i]=src[Index]&0xFFFF0000; //        
				dest[l+i]|=(src[Index-1])>>16;

				l+=hnd->LChQnt;

				dest_qnt++;

			}
		}


	}   //  if dest!=NULL

	FreqS=F_REF/(FsDiv+1);
	//     size/32
	Tp_Qnt=(int) *size/32;

	//    

	for(i=0;i<hnd->LChQnt;i++)
	{
		//         . .  <0..7>

		PhysChannel=(hnd->LChTbl[i])&0xF;

		//         15-PhysChannel         
		RelCntr=2*(15-PhysChannel)+1;

		//   N
		N=0;
		M1=0;
		Mk=0;
		Nm=0;
		Empty_Tp_Cntr=0;
		Empty_Tp_Flag=1; 

		//     
		for(Tp_Cntr=0/*1*/; Tp_Cntr<Tp_Qnt; Tp_Cntr++)
		{   
			CurSampleIndex=32*Tp_Cntr+RelCntr;

			//     
			if((src[CurSampleIndex]&0xF)!=PhysChannel)
			{
				;
				return LTR51_WRONG_DATA;
			}
			//        
			if(((src[CurSampleIndex]&0x10)>>4)!=1)
			{
				;
				return LTR51_WRONG_DATA; 
			}
			//       

			Nm=((src[CurSampleIndex]&0xFFFF0000)>>16);  



			//   "" 
			if(Empty_Tp_Flag)
			{
				if(Nm==0)
					Empty_Tp_Cntr++;
				else
				{
					//     "" ,  M1     

					M1=(src[CurSampleIndex-1]&0xFFFF0000)>>16;   
					Empty_Tp_Flag=0;
					//     N   ,  !
				}

			} //   "",    N

			else 
				N+=Nm;

		} //  for(Tp_Cntr...)


		//  ,       .   -     ..  

		for(Tp_Cntr=(Tp_Qnt-1); Tp_Cntr>=0; Tp_Cntr--)
		{   
			CurSampleIndex=32*Tp_Cntr+RelCntr;

			// Nm. ,      . .
			Nm=((src[CurSampleIndex]&0xFFFF0000)>>16); 
			CurSampleIndex=32*Tp_Cntr+RelCntr-1;
			//        ,     k.   M.

			Mk+=(src[CurSampleIndex]&0xFFFF0000)>>16;   

			if(Nm!=0)
			{
				//Mk=(src[CurSampleIndex]&0xFFFF0000)>>16;

				break;
			}
			//   ,     ,  ,  M1.    
		} //   

		//      F=Fref*Summa(Ni<i=2..k>)/(M1+BASE*(k-1)-Mk)      

		if(Empty_Tp_Cntr==Tp_Qnt) //         
			Frequency[i]=0.0;
		else
		{
			denominator=M1+ ((Base)*(Tp_Qnt-(Empty_Tp_Cntr+1))) - Mk;

			if(denominator==0)
				Frequency[i]=0;
			else 
				Frequency[i]=FreqS*N/denominator;
		}

	} //  for(i=0...)  

	*size=dest_qnt; 

	return LTR51_NO_ERR;      				 

}

const int min_range_id = -5000; 
const LPCSTR err_str[] = 
{
	"LTR51:   !",
	"LTR51:  !",
	"LTR51:  !",
	"LTR51:   !",
	"LTR51:   !",
	"LTR51:  !",
	"LTR51:   !",
	"LTR51:  !",
	"LTR51:  !",
	"LTR51:      !",
	"LTR51:       !",
	"LTR51:    !",
	"LTR51:   !",
	"LTR51:   !",
	"LTR51:    !",
	"LTR51:    BASE!",
	"LTR51:     !", 
	"LTR51:   !",
	"LTR51:   !    !",
	"LTR51:    !",
	"LTR51:    !",
	"LTR51:     !    !", 
	"LTR51:   !",
	"LTR51:  !",
	"LTR51:      Fs!",
	"LTR51:   Base!", 
	"LTR51:    !",
	"LTR51:    !",
	"LTR51:  !",
	"LTR51:       !",
	"LTR51:   -!", 
	"LTR51:    !",
	"LTR51:        !",
	"LTR51:           32!"
};

/*****************************************************************************************************

   

******************************************************************************************************/   

LTR51API_DllExport (LPCSTR) LTR51_GetErrorString(int Error_Code)
{
	//      
	// :
	// ErrorCode -  
	//   -    

	LPCSTR ret_val = NULL;
	int ret_val_index;
	double error_range;

	error_range=-min_range_id + 1+sizeof err_str / sizeof err_str[0];

	if ((-Error_Code < -min_range_id )|| (Error_Code < -error_range))
	{
		ret_val = LTR_GetErrorString(Error_Code);
	}
	else
	{
		ret_val_index=-Error_Code + min_range_id-1;
		ret_val = err_str[ret_val_index];
	}

	return ret_val;
}


/*****************************************************************************************************

      

******************************************************************************************************/   

INT LTR51_EvaluateFrequencies(PTLTR51 hnd)
{

	//hnd->Fref=F_REF/(hnd->Fs+1);
	//hnd->Fbase=hnd->Fref/hnd->Base;

	return 0;  
}

/**********************************************************************************************

      

************************************************************************************************/    

LTR51API_DllExport (INT) LTR51_WriteEEPROM(PTLTR51 hnd, INT Address, BYTE val) 
{

	DWORD command[2], command_ack;
	INT slot_num     =hnd->Channel.cc -1; 
	INT err=0;
	INT parity=0;

	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR;


	if((Address<0)||(Address)>=512)             
		return LTR51_ERR_WRONG_EEPROM_ADDR;           

	command[0]=Address; //   
	command[0]<<=16;
	command[0]|=WRITE_EEPROM|(slot_num<<8);  //    DSP

	parity=eval_parity(command[0]);
	command[0]|=(parity)<<5;


	command[1]=val; //   
	command[1]<<=16;
	command[1]|=WRITE_EEPROM|(slot_num<<8);

	parity=eval_parity(command[1]);
	command[1]|=(parity)<<5;

	if((err=(LTR_Send(&(hnd->Channel), command, 2, TIMEOUT_CMD_SEND)))!=2) 
		if(err<0)
			return err;
		else 		 
			return LTR51_ERR_CANT_SEND_COMMAND;      


	if((err=(LTR_Recv(&(hnd->Channel), &command_ack, NULL, 1, TIMEOUT_CMD_RECIEVE)))!=1)
		if(err<0)
			return err;
		else 
			return LTR51_ERR_CANT_WRITE_EEPROM;     


	//     

	if(check_parity(command_ack))
		return LTR51_ERR_PARITY_FROM_MODULE;


	if((command_ack&0x0000001F)==PARITY_ERROR) 
		return LTR51_ERR_PARITY_TO_MODULE; 


	if((command_ack&0x0000FF1F)!=(command[0]&0x0000FF1F)) //  . ,  ""  
		return LTR51_ERR_CANT_WRITE_EEPROM;  

	return LTR51_NO_ERR;  
}


/**********************************************************************************************

      

************************************************************************************************/       


LTR51API_DllExport (INT) LTR51_ReadEEPROM(PTLTR51 hnd, INT Address, BYTE *val) 
{

	DWORD command, command_ack;
	INT slot_num     =hnd->Channel.cc -1; 
	INT err=0;
	INT parity=0;


	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR;


	if((Address<0)||(Address)>=512)             
		return LTR51_ERR_CANT_READ_EEPROM;           

	command=Address; //   
	command<<=16;
	command|=READ_EEPROM|(slot_num<<8);  //    DSP

	parity=eval_parity(command);
	command|=(parity)<<5;


	if((err=(LTR_Send(&(hnd->Channel), &command, 1, TIMEOUT_CMD_SEND)))!=1) 
		if(err<0)
			return err;
		else 		 
			return LTR51_ERR_CANT_SEND_COMMAND;      


	if((err=(LTR_Recv(&(hnd->Channel), &command_ack, NULL, 1, TIMEOUT_CMD_RECIEVE)))!=1)
		if(err<0)
			return err;
		else 
			return LTR51_ERR_CANT_READ_EEPROM;     

	//     

	if(check_parity(command_ack))
		return LTR51_ERR_PARITY_FROM_MODULE;


	if((command_ack&0x0000001F)==PARITY_ERROR) 
		return LTR51_ERR_PARITY_TO_MODULE; 


	if((command_ack&0x0000FF1F)!=(command&0x0000FF1F)) //  . ,  ""  
		return LTR51_ERR_CANT_READ_EEPROM;  

	*val=(command_ack>>16)&0xFF;

	return LTR51_NO_ERR;  
}


/*****************************************************************************************************

     

******************************************************************************************************/ 

LTR51API_DllExport (DWORD) LTR51_CalcTimeOut(PTLTR51 hnd, int n)
{


	double Tw;
	double tmp1;
	double tmp2;
	double Fs;
	WORD Base;

	if(!hnd->SetUserPars)
	{
		Fs=FS;

		if(hnd->AcqTime<1000) 
			Base=BASE_1;
		else
			Base=BASE_2;
	}
	else
	{
		Fs=hnd->Fs;
		Base=hnd->Base;
	} 

	tmp1=n*(Fs+1);
	tmp2=Base/20E+6;
	Tw=(DWORD) 1000*tmp1*tmp2+1000; //   

	return Tw;   
}


/*****************************************************************************************************

    

******************************************************************************************************/ 

int LTR51_ReadConfRecord(PTLTR51 hnd, CHAR *version, CHAR *date, CHAR *name, CHAR *serial, CHAR *FPGA_version)
{

#define CR_LENGTH (42)
#define CRC_LENGTH (1)

	BYTE conf_rec[44]; 
	BYTE ver[2];
	BYTE FPGA_ver;

	DWORD command, command_ack[50];
	INT slot_num     =hnd->Channel.cc -1; 
	INT err=0;
	INT parity=0;
	WORD CR_CRC;
	int i;


	if(hnd==NULL)
		return LTR51_ERR_WRONG_MODULE_DESCR; 


	command=0; //   
	command|=READ_ID_RECORD|(slot_num<<8);  //    DSP

	parity=eval_parity(command);
	command|=(parity)<<5;


	if((err=(LTR_Send(&(hnd->Channel), &command, 1, TIMEOUT_CMD_SEND)))!=1) 
		if(err<0)
			return err;
		else 		 
			return LTR51_ERR_CANT_SEND_COMMAND;      


	if((err=(LTR_Recv(&(hnd->Channel), command_ack, NULL, CR_LENGTH+CRC_LENGTH+1, TIMEOUT_CMD_RECIEVE)))!=(CR_LENGTH+CRC_LENGTH+1))
		if(err<0)
			return err;
		else 
			return LTR51_ERR_CANT_READ_ID_REC;  


	for(i=0;i<CR_LENGTH;i++)
	{

		if((command_ack[i]&0x0000001F)==PARITY_ERROR) 
			return LTR51_ERR_PARITY_TO_MODULE;   


		//       -      
		//   6    -      

		if(check_parity(command_ack[i]))
			return LTR51_ERR_PARITY_FROM_MODULE;  //     

		if((command_ack[i]&0x0000FF1F)!=(command&0x0000FF1F))
			return LTR51_ERR_CANT_READ_ID_REC;  
	}    


	for(i=0;i<CR_LENGTH;i++)
		conf_rec[i]=(command_ack[i]>>16)&0xFF;

	CR_CRC=0;


	CR_CRC=CRC_Calc(conf_rec, CR_LENGTH);  

	if(CR_CRC!=(command_ack[CR_LENGTH]>>16))
		return LTR51_ERR_WRONG_ID_REC; 


	if(conf_rec[0]!=MODULE_ID)
		return LTR51_ERR_WRONG_ID_REC; 


	for(i=0;i<2;i++)
		ver[i]=*(conf_rec+1+i); //  

	sprintf(version, "%d.%d",ver[0], ver[1]);    

	for(i=0;i<14;i++)
		date[i]=*(conf_rec+3+i); //   

	for(i=0;i<8;i++)
		name[i]=*(conf_rec+17+i); //   

	for(i=0;i<17;i++)
		serial[i]=*(conf_rec+25+i); //   

	FPGA_ver=command_ack[CR_LENGTH+1]>>16; 

	sprintf(FPGA_version, "%d",FPGA_ver); //     



	return LTR51_NO_ERR;  
} 


/***********************************************************************************************

    .

************************************************************************************************/    

INT eval_parity(DWORD cmd)
{
	unsigned long index;
	INT           parity;


	parity = 0;
	for (index = 0x01UL; index <= 0x10UL; index <<= 1)
		parity ^= (cmd & index) != 0;  

	//        	
	for (index = 0x80000000UL; index >= 0x010000UL; index >>= 1)
		parity ^= (cmd & index) != 0;


	return parity;
}

/***********************************************************************************************

       .

************************************************************************************************/         

INT check_parity(DWORD cmd)
{
	if(((cmd&0x20)>>5)==eval_parity(cmd))
		return(0);
	else
		return(1);
}


/*******************************************************************************

 16- CRC

/*******************************************************************************/


WORD CRC_Calc(BYTE *b, WORD N)
/*  .
:
b --  .
N --  .
:
  .
*/
{
	WORD crc= 0;
	int i, j;

	for(i=0; i<N; i++) 
	{
		crc ^= b[i] << 8;
		for(j=0;j<8;j++) {
			if(crc&0x8000) crc = (crc<<1) ^ 0x1021;
			else crc <<= 1;
		}
	}
	return crc;
}

void CRC_Update(WORD *crc, BYTE *b, int c)
/*     ,  
   .
:
crc --      .
b --  .
c --  .
:
crc --      .
*/
{
	unsigned w;
	int i,j;

	w = *crc;
	for(i=0;i<c;i++) {
		w ^= b[i] << 8;
		for(j=0;j<8;j++)  {
			if(w&0x8000) w = (w<<1) ^ 0x1021;
			else w <<= 1;
		}
	}
	*crc = w;
}


DWORD CalkTickInterval(DWORD PrevTick, DWORD ActTick)
{
	DWORD Diff;

	if(ActTick>=PrevTick)
		Diff=ActTick-PrevTick;
	else
		Diff=(0xFFFFFFFF-PrevTick)+ActTick;

	return Diff;

}


#ifdef _MANAGED
#pragma managed(pop)
#endif
