#include <vcl.h>
#include <stdio.h>
#pragma hdrstop
#include "common.h"
#include "LtrCrate.h"
#include "LtrCrateThreads.h"
#pragma package(smart_init)

TCrate::TCrate(T_MessageQueue *mqControl, T_MessageQueue *mqDisplay)
{
    CurState = CRATE_ST_UNINIT;
    MessageQueueControl = mqControl;
    MessageQueueDisplay = mqDisplay;
    ThreadTerminationInUse = CreateMutex(NULL,FALSE,NULL);
    ip_addr = INADDR_NONE;
    memset(path, 0, sizeof(path));
    memset(pid, 0, sizeof(pid));
    memset(csn, 0, sizeof(csn));
    for (int i = 0; i <= MODULE_MAX; i++)
        {
        module[i].ModulePresence=false;
        }

    memset(module,0,sizeof(module));
    sflags=0;
    baud_rate=0;
    //
    second_mark_counter=0;
    start_mark_counter=0;
    //
    GetLocalTime(&start_time);
    total_count_recv=0;
    total_count_send=0;
    ei = 0;
    //
    mconfig=0;
    //
    CrateType = CrateType_UnknownCrate;
    memset(AllowLowSpeedModule, 1, sizeof(AllowLowSpeedModule));

    ReadRawData = false;
    Suspended = false;
}
//---------------------------------------------------------------------------
TCrate::~TCrate(void)
    {
    /*  ,    */
    if (WaitForSingleObject(client_list_access, INFINITE) == WAIT_OBJECT_0)
        {
        for (int k = 0; k < CLIENT_MAX; k++)
            {
            if (client_list[k]!=NULL && client_list[k]->crate==this)
                client_list[k]->IsAlive=false;
            }
        for (int k=0; k < CLIENT_MAX; k++)
            {
            if (client_list[k]!=NULL && client_list[k]->crate==this)
                {
                logfile.AddMessage(Format("Connection stopped for client  >> CSN=%.16s  CC=%0.4X", ARRAYOFCONST(((char*)csn, (int)client_list[k]->crate_channel))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                delete client_list[k];
                client_list[k]=NULL;
                }
            }
        ReleaseMutex(client_list_access);
        }
    /*  mutex' */
    CloseHandle(ThreadTerminationInUse);
    }

//---------------------------------------------------------------------------

//-----------------------------------------------------------------------------
bool TCrate::TModule::CreateModule(void)
    {
    DeleteModule();
    mid=0xFFFF;
    if ((recv_buf=new DWORD[MODULE_RECV_BUF_SIZE])==NULL)
        return false;
    recv_buf_is=CreateEvent(NULL, FALSE, FALSE, NULL);
    if ((send_buf=new DWORD[MODULE_SEND_BUF_SIZE])==NULL)
        return false;
    send_buf_In_Use=CreateMutex(NULL, FALSE, NULL);
    ModulePresence=true;
    return true;
    }
//----------------------------------------------------------------------------
void TCrate::TModule::DeleteModule(void)
    {
    if (ModulePresence)
        {
        ModulePresence=false;
        }
    if (recv_buf!=NULL)
        {
        delete recv_buf;
        recv_buf=NULL;
        }
    if (send_buf!=NULL)
        {
        if (send_buf_cntr!=0)
            {
            //         
            send_buf_cntr=0;
            send_buf_cntr_sended=0;
            }
        delete send_buf;
        send_buf=NULL;
        }
    if (send_buf_In_Use!=NULL)
        {
        ReleaseMutex(send_buf_In_Use);
        CloseHandle(send_buf_In_Use);
        send_buf_In_Use=NULL;
        }
    if (recv_buf_is!=NULL)
        {
        CloseHandle(recv_buf_is);
        recv_buf_is=NULL;
        }
    mid=0;                               // id 

    recv_buf_ptr=0;            //   
    send_buf_cntr=0;           //   
    send_buf_cntr_sended=0;    //   

    //   34                    //
    count_recv=0;            //      
    count_send=0;            //      
    count_FIFO_recv=0;       //     FIFO (    count_FIFO_Client)
    fifo_size=0;                        //
    }
//------------------------------------------------------------------------------

/*------------------------------------------------------------------------------------------------*/
bool TCrate::ProcessCrateError(DWORD data)
    {
    BYTE regs[16];

    if (!GetArray(CM_APP, SEL_AVR_DM|0x8000, regs, sizeof(regs)))
        {
        CancelIO();
        return false;
        }

    AnsiString msg;
    switch (data&0xFF)
    {
    case 0 :
        {
            msg=Format("Crate send fault signal >> CSN=%.16s  REGS[0..15]=", ARRAYOFCONST(((char*)csn)));
            for(int j=0; j<sizeof(regs); j++) msg+=Format(" %0.2X", ARRAYOFCONST((regs[j])));
            logfile.AddMessage(msg, log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);

            sflags|=CRATE_FLAG_HW_ERROR;
            CancelIO();
            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
        }
        break;
    case 1 :
        { //   
            WORD mconfig_new=*(WORD*)(regs+8);
            for(int i=0; i<MODULE_MAX; i++)
            {
                WORD mask=(1<<i);
                if(!(mconfig_new&mask))
                { //  
                    WORD old_mid=module[i].mid;
                    if(old_mid&&(baud_rate&mask))
                    { //      -    
                        WORD baud_rate_new=baud_rate&(~mask);
                        if (PutArray(CM_APP, SEL_AVR_DM|0x8006, (BYTE*)&baud_rate_new, 2))
                        {
                            baud_rate=baud_rate_new;
                            logfile.AddMessage(Format("RST.XXXX change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate_new))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                        } else logfile.AddMessage(Format("RST.XXXX can't change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate_new))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                    }
                    module[i].mid=0;
                    module[i].DeleteModule();
                }
                else
                {
                    bool BadmoduleName =false;
                    if((module[i].mid&0xFF)!=((module[i].mid>>8)&0xFF))
                    {
                        BadmoduleName=true;
                    }

                    if((!(mconfig&mask) && !module[i].mid)||BadmoduleName)
                    {
                        module[i].CreateModule();
                        InterlockedExchangeAdd(&mconfig, mask);
                    }
                }
            }

            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
            logfile.AddMessage(Format("RST.XXXX module configuration updated. mconfig=%0.4X", ARRAYOFCONST(((int)mconfig_new))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
        }
        break;
    case 2:
        logfile.AddMessage("Crate LTR21 Buffer Overflow", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
        break;
    case 3:
        logfile.AddMessage("Crate LTR21 DMA Buffer Miss", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
        break;
    case 4:
        logfile.AddMessage("Crate LTR021 - Unknown Sync Source", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
        break;
    default:
        logfile.AddMessage("Unknown Crate Error", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
        break;
    }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateUSB::TCrateUSB(T_MessageQueue *mqControl, T_MessageQueue *mqDisplay)
    : TCrate(mqControl, mqDisplay)
    {
    recv_buf_thread = NULL;
    send_buf_thread = NULL;
    DeviceHANDLE = NULL;
    ltr_USB = CreateInstanceUSB();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateEther::TCrateEther(T_MessageQueue *mqControl, T_MessageQueue *mqDisplay, DWORD IdleTimeout)
    : TCrate(mqControl, mqDisplay)
    {
    ping_fail_count = 0;
    data_thread = NULL;
    hAbortEvent = WSACreateEvent();
    ltr_Ether = CreateInstanceEther(hAbortEvent, IdleTimeout);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateUSB::~TCrateUSB(void)
    {
    CurState = CRATE_ST_DEAD;
    StopServiceThreads();
    for (int i=0;i<(MODULE_MAX+1);i++)
        {
        module[i].DeleteModule();
        }
    if (ltr_USB != NULL) DeleteInstanceUSB(ltr_USB);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateEther::~TCrateEther(void)
    {
    CurState = CRATE_ST_DEAD;
    StopServiceThreads();
    for (int i=0;i<(MODULE_MAX+1);i++)
        {
        module[i].DeleteModule();
        }
    if (ltr_Ether != NULL)
        DeleteInstanceEther(ltr_Ether);

    WSACloseEvent(hAbortEvent);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::SetState(t_CrateState new_state, bool NotifyThreads)
    {
    t_CrateState old_state = CurState;
    CurState = new_state;
    if (NotifyThreads && (new_state != old_state))
        {
        /*    ONLINE  -ONLINE,      */
        if ((new_state == CRATE_ST_ONLINE) || (old_state == CRATE_ST_ONLINE))
            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
        /*     DEAD    */
        if (new_state == CRATE_ST_DEAD)
            MessageQueueControl->PostMessage(LTR_SRV_bury_the_dead);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateEther::SetState(t_CrateState new_state, bool NotifyThreads)
    {
    t_CrateState old_state = CurState;
    CurState = new_state;
    switch (new_state)
        {
        case CRATE_ST_ONLINE:
            IPCrateList.SetStatus(ip_addr, IP_STATUS_ONLINE, csn);
            break;
        case CRATE_ST_CONNECTING:
            IPCrateList.SetStatus(ip_addr, IP_STATUS_CONNECTING);
            break;
        default:
            IPCrateList.SetStatus(ip_addr, IP_STATUS_OFFLINE);
            break;
        }
    if (NotifyThreads && (new_state != old_state))
        {
        /*    ONLINE  -ONLINE,      */
        if ((new_state == CRATE_ST_ONLINE) || (old_state == CRATE_ST_ONLINE))
            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
        /*     DEAD    */
        if (new_state == CRATE_ST_DEAD)
            MessageQueueControl->PostMessage(LTR_SRV_bury_the_dead);
        /*  IP       IP */
        MessageQueueDisplay->PostMessage(LTR_DISP_ethernet_list_updated);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
_CrateConnectionType TCrateUSB::GetConnType(void)
    {
    return CrateCon_USB;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
_CrateConnectionType TCrateEther::GetConnType(void)
    {
    return CrateCon_Ethernet;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::SuspendCrate(void)
    {
    if (Suspended) return;
    Suspended = true;

    StopServiceThreads();

    ltr_USB->CancelIO();
    if (ltr_USB->CloseLDevice())
        {
        logfile.AddMessage(Format("Crate suspended >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        return;
        }
    else
        {
        Suspended = false;
        logfile.AddMessage(Format("Can\'t suspend crate >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateEther::SuspendCrate(void)
    {
    if (Suspended) return;
    Suspended = true;
    StopServiceThreads();
    ltr_Ether->StopIdleTimer();
    logfile.AddMessage(Format("Crate suspended >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::ResumeCrate(void)
    {
    if (!Suspended) return;
    if (ltr_USB->OpenLDeviceByName(path))
        {
        logfile.AddMessage(Format("Crate resumed from suspend >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        Suspended = false;
        StartServiceThreads();
        return;
        }
    logfile.AddMessage(Format("Can't resume crate after suspend >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateEther::ResumeCrate(void)
    {
    if (!Suspended) return;
    ltr_Ether->RestartIdleTimer();
    if (StartServiceThreads())
        {
        logfile.AddMessage(Format("Crate resumed from suspend >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        Suspended = false;
        return;
        }
    logfile.AddMessage(Format("Can't resume crate after suspend >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::Startup(BYTE * FPGAData, char *default_ttf)
    { /*  USB   ,    */
    if (GetState() != CRATE_ST_UNINIT)
        { /*    */
        StopServiceThreads();
        sflags &= ~CRATE_FLAG_BOOT_LOADER;
        }
    SetState(CRATE_ST_CONNECTING);
    bool result = InitCrate(FPGAData, default_ttf);
    if (result)
        {
        /*     InitCrate() */
        SetState(CRATE_ST_ONLINE);
        }
    else
        {
        SetState(CRATE_ST_DEAD);
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::Startup(BYTE * FPGAData, char *default_ttf)
    { /*  TCP      */
    if (GetState() != CRATE_ST_UNINIT)
        { /*    */
        StopServiceThreads();
        sflags &= ~CRATE_FLAG_BOOT_LOADER;
        }
    SetState(CRATE_ST_CONNECTING);
    if (!StartServiceThreads())
        {
        logfile.AddMessage(Format("Can't start data thread >> Location=%s", ARRAYOFCONST((path))),
            log_flags, LOG_FLAG_DEBUG_LEVEL1 | LOG_FLAG_CRATE_INIT);
        SetState(CRATE_ST_DEAD);
        return false;
        }

    return true; /*   --     */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::InitCrate(BYTE * FPGAData, char * default_ttf)
    {
    baud_rate = 0;
    if (ltr_USB->OpenLDeviceByName(path))
        {
        //  bootloader  aplication
        char appmode = '?';
        if (ltr_USB->SwitchBootloader(true)) appmode = 'B';  // bootloader active
        if (ltr_USB->SwitchBootloader(false)) appmode = 'A'; // application active

        logfile.AddMessage("OpenLDeviceByName() -> ok", log_flags, LOG_FLAG_DEBUG_LEVEL7|LOG_FLAG_CRATE_INIT);
        TDESCRIPTION_LTR010 descr;
        char CrateName[16];
        if ((appmode != '?') && GetModuleDescription(CM_AUTO, &descr) &&
            ltr_USB->GetModuleName(CrateName, sizeof(CrateName)))
            {
            memcpy(csn, descr.Module.SerialNumber, sizeof(csn));

            logfile.AddMessage("GetModuleDescription() -> ok", log_flags, LOG_FLAG_DEBUG_LEVEL7|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage("Crate Info:", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ PIDString=%s", ARRAYOFCONST(((char*)pid))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ DeviceName   =%.16s", ARRAYOFCONST(((char*)descr.Module.DeviceName))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ SerialNumber =%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ DeviceComment=%s", ARRAYOFCONST(((char*)descr.Module.Comment))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);

            if (pid[10]=='H')
                sflags|=CRATE_FLAG_USB_HI_SPEED;

            int v1=0, v2=0, v3=0, v4=0;
            if (sscanf(descr.Cpu.Comment, "Bootloader v%d.%d.%d.%d", &v1, &v2, &v3, &v4)==4)
                {
                logfile.AddMessage(Format("+ BootLoaderVersion=%d.%d.%d.%d", ARRAYOFCONST((v1, v2, v3, v4))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                }
            if (descr.Cpu.FirmwareVersion!=0xFFFFFFFF)
                {
                logfile.AddMessage(Format("+ FirmwareVersion=%d.%d.%d.%d", ARRAYOFCONST((
                    (int)(descr.Cpu.FirmwareVersion>>24)&0xFF,
                    (int)(descr.Cpu.FirmwareVersion>>16)&0xFF,
                    (int)(descr.Cpu.FirmwareVersion>> 8)&0xFF,
                    (int)(descr.Cpu.FirmwareVersion>> 0)&0xFF))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                }

            //     -
            AnsiString section=Format("LTR010_%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber)));
            AnsiString crate_ttf;

            if (default_ttf)
                {
                crate_ttf=g_INI.ReadString(section, "fpga_ttf", default_ttf, true);
                }

            log_flags = g_INI.ReadIntegerHex(section, "log_flags", log_flags, true);

            if (appmode == 'A')
                {
                // application
                if (!strcmp(CrateName,"LTR010"))
                    {
                    CrateType=CrateType_LTR010;
                    }
                else if (!strcmp(CrateName,"LTR021"))
                    {
                    CrateType=CrateType_LTR021;
                    }
                else if (!strcmp(CrateName,"LTR030"))
                    {
                    CrateType=CrateType_LTR030;
                    }
                else if (!strcmp(CrateName,"LTR031"))
                    {
                    CrateType=CrateType_LTR031;
                    }
                else
                    {
                    CrateType=CrateType_UnknownCrate;
                    }

                switch (CrateType)
                    {
                    case CrateType_LTR010:
                    case CrateType_LTR021:
                    case CrateType_LTR030:
                    case CrateType_LTR031:
                        {
                        bool FPGALoadResult;
                        if (FPGAData!=NULL)
                            {
                            FPGALoadResult=ltr_USB->Application->LoadFPGA(FPGAData,7,0);
                            crate_ttf="  FPGA";
                            }
                        else
                            {
                            FPGALoadResult=ltr_USB->Application->LoadFPGAFromFile(crate_ttf.c_str(), 7, 0);
                            }

                        if (!FPGALoadResult)
                            {
                            logfile.AddMessage(Format("Can't LoadFPGA >> CSN=%.16s TTF_FILE=%s", ARRAYOFCONST(((char*)descr.Module.SerialNumber, crate_ttf))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                            return false;
                            }

                        ltr_USB->Application->ConvertFPGADescription(&descr);
                        logfile.AddMessage("LoadFPGA() -> ok.", log_flags, LOG_FLAG_DEBUG_LEVEL7|LOG_FLAG_CRATE_INIT);
                        logfile.AddMessage(Format("+ TTF_FILE=%s", ARRAYOFCONST((crate_ttf))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                        logfile.AddMessage(Format("+ FpgaName   =%s", ARRAYOFCONST(((char*)descr.Fpga.Name))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                        logfile.AddMessage(Format("+ FpgaVersion=%d.%d.%d.%d", ARRAYOFCONST((
                            (int)(descr.Fpga.FirmwareVersion>>24)&0xFF,
                            (int)(descr.Fpga.FirmwareVersion>>16)&0xFF,
                            (int)(descr.Fpga.FirmwareVersion>> 8)&0xFF,
                            (int)(descr.Fpga.FirmwareVersion>> 0)&0xFF))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                        logfile.AddMessage(Format("+ FpgaComment=%s", ARRAYOFCONST(((char*)descr.Fpga.Comment))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                        }
                        break;
                    } /* Switch CrateType */

                if (ltr_USB->Application->PutArray(SEL_AVR_DM|0x8006, (BYTE*)&baud_rate, 2) &&
                    ltr_USB->Application->GetModuleDescription(&descr))
                    {
                    logfile.AddMessage(Format("+ BaudRate=0x%0.4X  DMAR=%d  DMAW=%d", ARRAYOFCONST(((int)baud_rate, 7, 0))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                    }
                else
                    {
                    logfile.AddMessage(Format("Can't set baudrate && GetModuleDescription after Load FPGA >> CSN=%.16s TTF_FILE=%s", ARRAYOFCONST(((char*)descr.Module.SerialNumber, crate_ttf))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                    return false;
                    }
                } /* appmode == 'A' */
            else
                {
                // bootloader
                CrateType=CrateType_BOOTLOADER;
                sflags|=CRATE_FLAG_BOOT_LOADER;
                }
            }
        else
            {
            logfile.AddMessage(Format("Can't execute GetModuleDescription() >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            return false;
            }
        } /* if OpenDevice */
    else
        {
        logfile.AddMessage(Format("Can't open crate >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        return false;
        }

    /*   - */
    if (!StartServiceThreads())
        {
        logfile.AddMessage(Format("Can't start crate Threads >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        return false;
        }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::InitCrate(void)
    {
    baud_rate = 0;
    logfile.AddMessage(Format("Connecting IP crate %s ...", ARRAYOFCONST((path))),
        log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    if (ltr_Ether->OpenDeviceByIP(ip_addr))
        {
        logfile.AddMessage("OpenLDeviceByIP() -> ok", log_flags, LOG_FLAG_DEBUG_LEVEL7|LOG_FLAG_CRATE_INIT);
        TDESCRIPTION_LTR010 descr;
        char CrateName[16];
        if (ltr_Ether->GetModuleName(CrateName, sizeof(CrateName))&&
            ltr_Ether->GetModuleDescription(&descr))
            {
            memcpy(csn, descr.Module.SerialNumber, sizeof(csn));

            logfile.AddMessage("GetModuleDescription() -> ok", log_flags, LOG_FLAG_DEBUG_LEVEL7|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage("Crate Info:", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ Location=%s", ARRAYOFCONST(((char*)path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ DeviceName   =%.16s", ARRAYOFCONST(((char*)descr.Module.DeviceName))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ SerialNumber =%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ DeviceComment=%s", ARRAYOFCONST(((char*)descr.Module.Comment))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);

            int v1=0, v2=0, v3=0, v4=0;
            if (sscanf(descr.Cpu.Comment, "Bootloader v%d.%d.%d.%d", &v1, &v2, &v3, &v4)==4)
                {
                logfile.AddMessage(Format("+ BootLoaderVersion=%d.%d.%d.%d", ARRAYOFCONST((v1, v2, v3, v4))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                }

            if (descr.Cpu.FirmwareVersion!=0xFFFFFFFF)
                {
                logfile.AddMessage(Format("+ FirmwareVersion=%d.%d.%d.%d", ARRAYOFCONST((
                    (int)(descr.Cpu.FirmwareVersion>>24)&0xFF,
                    (int)(descr.Cpu.FirmwareVersion>>16)&0xFF,
                    (int)(descr.Cpu.FirmwareVersion>> 8)&0xFF,
                    (int)(descr.Cpu.FirmwareVersion>> 0)&0xFF))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                }

            logfile.AddMessage(Format("+ FpgaName   =%s", ARRAYOFCONST(((char*)descr.Fpga.Name))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ FpgaVersion=%d.%d.%d.%d", ARRAYOFCONST((
                (int)(descr.Fpga.FirmwareVersion>>24)&0xFF,
                (int)(descr.Fpga.FirmwareVersion>>16)&0xFF,
                (int)(descr.Fpga.FirmwareVersion>> 8)&0xFF,
                (int)(descr.Fpga.FirmwareVersion>> 0)&0xFF))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            logfile.AddMessage(Format("+ FpgaComment=%s", ARRAYOFCONST(((char*)descr.Fpga.Comment))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);

            if (!strcmp(CrateName,"LTR010"))
                {
                CrateType=CrateType_LTR010;
                }
            else if (!strcmp(CrateName,"LTR021"))
                {
                CrateType=CrateType_LTR021;
                }
            else if (!strcmp(CrateName,"LTR030"))
                {
                CrateType=CrateType_LTR030;
                }
            else if (!strcmp(CrateName,"LTR031"))
                {
                CrateType=CrateType_LTR031;
                }
            else
                {
                CrateType=CrateType_UnknownCrate;
                }

            /*    ini- */
            AnsiString section=Format("LTR010_%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber)));
            log_flags = g_INI.ReadIntegerHex(section, "log_flags", log_flags, true);

            /*  FPGA   --  ,  Ethernet,  .   fpga_info
                   . */

            /* Set baud rate */
            if (PutArray(CM_APP, SEL_AVR_DM | 0x8006, (BYTE*)&baud_rate, 2))
                {
                logfile.AddMessage(Format("+ BaudRate=0x%0.4X  DMAR=%d  DMAW=%d", ARRAYOFCONST(((int)baud_rate, 7, 0))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                }
            else
                {
                logfile.AddMessage(Format("Can't set baudrate >> CSN=%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                return false;
                }

            }
        else
            {
            logfile.AddMessage(Format("Can't execute GetModuleName() and GetModuleDescription() >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            return false;
            }
        } /* if OpenDevice */
    else
        {
        logfile.AddMessage(Format("Can't open crate >> Location=%s", ARRAYOFCONST((path))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        return false;
        }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::StartServiceThreads(void)
    {
    StopServiceThreads();
    mconfig = 0;

    DeviceHANDLE = ltr_USB->GetUSBDeviceHandle();

    if (DeviceHANDLE == NULL)
        {
        logfile.AddMessage("Can't connect to Device Before Start Thread.", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        return false;
        }

    ThreadsRunning = true;

    recv_buf_thread = new TReadUSBCrateThread(this, false);
    if (WaitForSingleObject(((TReadUSBCrateThread*)recv_buf_thread)->init_complete, INFINITE) != WAIT_OBJECT_0)
        {
        logfile.AddMessage("Can't init ReadCrateThread.", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        StopServiceThreads();
        return false;
        }

    send_buf_thread = new TWriteUSBCrateThread(this, false);
    if (WaitForSingleObject(((TWriteUSBCrateThread*)send_buf_thread)->init_complete, INFINITE) != WAIT_OBJECT_0)
        {
        logfile.AddMessage("Can't init WriteCrateThread.", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        StopServiceThreads();
        return false;
        }

    logfile.AddMessage("Crate service started.", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::StartServiceThreads(void)
    {
    StopServiceThreads();
    mconfig = 0;

    ThreadsRunning = true;
    WSAResetEvent(hAbortEvent);

    data_thread = new TEthCrateDataThread(this, false);
    if (WaitForSingleObject(((TEthCrateDataThread*)data_thread)->init_complete, INFINITE) != WAIT_OBJECT_0)
        {
        logfile.AddMessage("Can't init DataThread.", log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        StopServiceThreads();
        return false;
        }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::StopServiceThreads()
    {
    WaitForSingleObject(ThreadTerminationInUse, INFINITE);

    ltr_USB->CancelIO();

    ThreadsRunning = false;

    if (recv_buf_thread != NULL)
        recv_buf_thread->Terminate();

    if (send_buf_thread != NULL)
        send_buf_thread->Terminate();

    if (recv_buf_thread != NULL)
        {
        recv_buf_thread->WaitFor();
        delete recv_buf_thread;
        recv_buf_thread = NULL;
        }

    if (send_buf_thread != NULL)
        {
        send_buf_thread->WaitFor();
        delete send_buf_thread;
        send_buf_thread = NULL;
        }

    DeviceHANDLE = NULL;

    /*  -       
    if (!FPGAReload)
        {
        sflags = 0;
        mconfig = 0;
        for (int i = 0; i <MODULE_MAX; i++)
            {
            module[i].mid = 0;
            }
        }
    */

    ReleaseMutex(ThreadTerminationInUse);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateEther::StopServiceThreads()
    {
    WaitForSingleObject(ThreadTerminationInUse, INFINITE);

    WSASetEvent(hAbortEvent);
    ThreadsRunning = false;

    if (data_thread != NULL)
        {
        data_thread->Terminate();
        data_thread->WaitFor();
        delete data_thread;
        data_thread = NULL;
        }

    /*  -       
    if (!FPGAReload)
        {
        sflags = 0;
        mconfig = 0;
        for (int i = 0; i <MODULE_MAX; i++)
            {
            module[i].mid = 0;
            }
        }
    */

    ReleaseMutex(ThreadTerminationInUse);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrate::ParseDataFromCrate(LPDWORD lpBuffer, DWORD NumDWords)
    {
    bool result = true;
    DWORD NewDataForModules[(MODULE_MAX+1)]; // +1 -  RawData

    if ((NumDWords == 0) || (lpBuffer == NULL))
        return false;

    //  
    total_count_recv += NumDWords;
    memset(NewDataForModules,0, sizeof(NewDataForModules));
    //      - -> {0xFFFFFFFF, 0xFFFFAAXX}
    //     
    for (DWORD i = 0; i < NumDWords; i++)
        {
        DWORD data = lpBuffer[i];

        if (ReadRawData)
            {
            module[MODULE_MAX].recv_buf[module[MODULE_MAX].recv_buf_ptr]=data;
            module[MODULE_MAX].recv_buf_ptr=(module[MODULE_MAX].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;
            NewDataForModules[MODULE_MAX]++;
            }

        int ModuleSlot=(data>>8)&0xF;

        switch (data & LTR010_CMD_DATA_MASK)
            {
            case LTR010DATA:
                ei=0;
                //      
                if (!module[ModuleSlot].ModulePresence) continue;
                module[ModuleSlot].recv_buf[module[ModuleSlot].recv_buf_ptr]=data;
                module[ModuleSlot].recv_buf_ptr=(module[ModuleSlot].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;
                NewDataForModules[ModuleSlot]++;
                continue;

            case LTR010CMD:
                //      
                if(ei)
                    {
                    ei=0;
                    if((data>>8)==0xFFFFAA)
                        { //    
                        if(!ProcessCrateError(data))
                            {
                            logfile.AddMessage(Format("Crate send signal %0.2X. Can't read crate registers >> CSN=%.16s", ARRAYOFCONST(((int)data&0xFF, (char*)csn))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                            sflags |= CRATE_FLAG_HW_ERROR;
                            result = false;
                            break;
                            }
                        continue;
                        }
                    }
                // 
                if (!module[ModuleSlot].ModulePresence) continue;

                //     41, 42, 43- ,    LTR021
                if  (((data & 0xCFFFC0C0) == LTR010CMD) &&
                    (module[ModuleSlot].mid==0x2B2B || module[ModuleSlot].mid==0x2929 ||
                     module[ModuleSlot].mid==0x2A2A || module[ModuleSlot].mid==0x1515))
                    {
                    if(data&0x10000000) start_mark_counter++;
                    if(data&0x20000000) second_mark_counter++;
                    DWORD tmark=((start_mark_counter&0xFFFF)<<16)|
                        (second_mark_counter&0xFFFF);

                    for(int k=0; k<(MODULE_MAX+1); k++) // +1 -  Raw Data
                        {
                        if (!module[k].ModulePresence) continue;

                        module[k].recv_buf[module[k].recv_buf_ptr]=SIGNAL_TSTAMP;
                        module[k].recv_buf_ptr=(module[k].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;

                        module[k].recv_buf[module[k].recv_buf_ptr]=tmark;
                        module[k].recv_buf_ptr=(module[k].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;

                        NewDataForModules[k]+=2;
                        }
                    continue;
                    }

                if ((module[ModuleSlot].mid == 0x2222) && ((data & 0x80E0) == 0x80C0))
                    {   //    34- 
                    //      ,      
                    module[ModuleSlot].count_recv+=1024;
                    module[ModuleSlot].fifo_size = ((data>>16)&0xFFFF)|((data&0x11)<<16)|((data&0x11)<<16);
                    continue;
                    }

                // LTR34 echo data
                if ((module[ModuleSlot].mid == 0x2222) && ((data & 0x80E0) == 0x80E0))
                    {
                    int FIFO = (module[ModuleSlot].count_send - module[ModuleSlot].count_recv);
                    FIFO = std::max<int>(FIFO,0);
                    module[ModuleSlot].fifo_size=((data&0x11)<<16)|(FIFO>>10);
                    }

                //    
                //  LTR11 -  -  -     ,
                //        ,     
                //    
                if (((data & 0xC0C0) == LTR010CMD_RESET) ||
                    ((module[ModuleSlot].mid == 0x0B0B) && ((data & 0xF0FF) == LTR010CMD_RESET)))
                    {
                    WORD old_mid = module[ModuleSlot].mid, mask=(1<<ModuleSlot);
                    WORD NewMid=data>>16;

                    bool BadLastModuleAndNoReset=false;
                    bool BadNewModuleName = false;
                    if ((old_mid&0xFF)!=((old_mid>>8)&0xFF)&&((data&0xF0FF)!=LTR010CMD_RESET))
                        { //    LTR11
                        BadLastModuleAndNoReset=true;
                        logfile.AddMessage(Format("RST.XXXX Bad Module. slot=%d  mid=%0.4X, Data =%0.8X ", ARRAYOFCONST(((int)ModuleSlot+1, (int)module[ModuleSlot].mid, data))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                        }
                    else if ((NewMid&0xFF)!=((NewMid>>8)&0xFF))
                        {
                        BadNewModuleName = true;
                        logfile.AddMessage(Format("RST.XXXX Bad Module. slot=%d  mid=%0.4X, Data =%0.8X ", ARRAYOFCONST(((int)ModuleSlot+1, (int)NewMid, data))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                        }

                    if (old_mid!=NewMid&&!BadLastModuleAndNoReset&&!BadNewModuleName)
                        {
                        module[ModuleSlot].mid = NewMid;

                        logfile.AddMessage(Format("RST.XXXX found new module. slot=%d  mid=%0.4X (LTR%d)", ARRAYOFCONST(((int)ModuleSlot+1, (int)module[ModuleSlot].mid, ((int)module[ModuleSlot].mid&0xFF)))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                        module[ModuleSlot].count_recv = module[ModuleSlot].count_send = module[ModuleSlot].fifo_size = 0;

                        switch (module[ModuleSlot].mid)
                            {
                            case 0x0101: // LTR01 -   
                                if (ModuleSlot&1) break;
                            case 0x2222: // LTR34
                                if(old_mid==0xFFFF)
                                    {
                                    WORD baud_rate_new = baud_rate | mask;
                                    if (PutArray(CM_APP, SEL_AVR_DM | 0x8006, (BYTE*)&baud_rate_new, 2))
                                        {
                                        /*
                                        WORD NumFastModules = 0;
                                        for (int j = 0; j < MODULE_MAX; j++)
                                            {
                                            if (baud_rate&(1<<j))
                                                NumFastModules++;
                                            }
                                        */
                                        baud_rate = baud_rate_new;
                                        logfile.AddMessage(Format("RST.XXXX change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                                        //InterlockedExchangeAdd(&mconfig, mask);
                                        }
                                    else
                                        {
                                        logfile.AddMessage(Format("RST.XXXX can't change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                                        }
                                    }
                                break;
                            case 0x2B2B:
                            case 0x2929:
                            case 0x2A2A:
                            case 0x1515:
                                start_mark_counter=second_mark_counter=0; //  
                                break;
                            default:
                                break;
                            } /* switch (mid) */
                        MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
                        }
                    } /* reset */

                module[ModuleSlot].recv_buf[module[ModuleSlot].recv_buf_ptr]=data;
                module[ModuleSlot].recv_buf_ptr=(module[ModuleSlot].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;
                NewDataForModules[ModuleSlot]++;
                continue;

            case LTR010CRATE_CMD:
                {
                //    
                int CMDCode =(data>>8)&0x3F;
                int CMDParam = data&0xFF;
                switch (CMDCode)
                    {
                    case 10:
                        {
                        //   
                        if(data&0x10000000) start_mark_counter++;
                        if(data&0x20000000) second_mark_counter++;
                        DWORD tmark=((start_mark_counter&0xFFFF)<<16)|
                            (second_mark_counter&0xFFFF);

                        for (int k=0; k<(MODULE_MAX+1); k++) // +1 -  Raw Data
                            {
                            if (!module[k].ModulePresence) continue;

                            module[k].recv_buf[module[k].recv_buf_ptr]=SIGNAL_TSTAMP;
                            module[k].recv_buf_ptr=(module[k].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;

                            module[k].recv_buf[module[k].recv_buf_ptr]=tmark;
                            module[k].recv_buf_ptr=(module[k].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;

                            NewDataForModules[k]+=2;
                            }
                        }
                    case 0x3F:
                        ei++;
                        break;
                    default:
                        logfile.AddMessage(Format("Unknown Crate Command Code, FuncNumber=%d, FuncParam=%d, Data =%0.8X ", ARRAYOFCONST((CMDCode,CMDParam,data))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                        break;
                    } /* switch CMDCode */
                }
                continue;

            default:
                logfile.AddMessage(Format("Bad Crate Data (Impossible Combination) Data =%0.8X ", ARRAYOFCONST((data))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_READ);
                continue;
            } /* switch (data & LTR010_CMD_DATA_MASK) */
        } /* for i */

    for (int i=0; i<MODULE_MAX; i++)
        {
        if (!NewDataForModules[i]) continue;
        module[i].count_recv+=NewDataForModules[i];
        module[i].count_FIFO_recv+=NewDataForModules[i];
        }
    if (ReadRawData)
        {
        module[MODULE_MAX].count_recv+=NewDataForModules[MODULE_MAX];
        module[MODULE_MAX].count_FIFO_recv+=NewDataForModules[MODULE_MAX];
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
#define LTR34_BUFFER_ALERT_SIZE (1572864)
#define NUM_MIX_WORDS_FOR_ONE_FAST_MODULE   (20)
DWORD TCrate::PrepareDataForCrate(LPDWORD lpBuffer, DWORD MaxDWords, DWORD &NumDataFast)
    {
    DWORD size = 0;
    DWORD NumDataFromFastModules = 0;
    DWORD NumMixFast = NUM_MIX_WORDS_FOR_ONE_FAST_MODULE;

    if ((MaxDWords == 0) || (lpBuffer == NULL))
        return 0;

    //      
    for (int i = 0; i < (MODULE_MAX + 1); i++)
        {
        if (!module[i].ModulePresence || !module[i].send_buf_cntr)
            continue;

        if (baud_rate & (1 << i))
            {   //  
            if (module[i].mid == 0x2222)
                {
                if (module[i].count_send - module[i].count_recv >= LTR34_BUFFER_ALERT_SIZE)
                    continue; //  LTR34  
                }
            NumDataFromFastModules += module[i].send_buf_cntr;
            }

        size += module[i].send_buf_cntr;
        }

    if ((size == 0) && !mconfig)
        return 0;

    size = 0; /*    */

    if (mconfig && ThreadsRunning) //   
        {
        WORD mconf = InterlockedExchange(&mconfig, 0);
        //   STOP+RESET+STOP
        for (int i = 0; i < MODULE_MAX; i++)
            {
            if (mconf & (1 << i))
                {
                *lpBuffer++ = LTR010CMD_STOP |(i<<8);
                *lpBuffer++ = LTR010CMD_RESET|(i<<8)|(0x0101<<16);
                *lpBuffer++ = LTR010CMD_STOP |(i<<8);
                size += 3;
                module[i].count_send = 0;
                module[i].count_recv = 0;
                module[i].send_buf_cntr = 0;
                module[i].send_buf_cntr_sended = 0;
                logfile.AddMessage(Format("WST.XXXX send STOP+RESET+STOP to module. slot=%d", ARRAYOFCONST(((int)i+1))), log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
                }
            }
        }

    while ((size < MaxDWords) && ThreadsRunning)
        {
        DWORD NumUsedFastModules = 0; //   ,     
        DWORD LastSize = size;
        //      
        //  20      
        for (int i = 0; (i < (MODULE_MAX + 1)) && (size < MaxDWords); i++)
            {
            if (!module[i].ModulePresence || !module[i].send_buf_cntr)
                continue;
            DWORD NewData = module[i].send_buf[module[i].send_buf_cntr_sended];
            if (baud_rate & (1 << i))
                {   //  
                NumUsedFastModules++;
                if (module[i].mid == 0x2222)
                    {
                    if ((NewData & 0xC000) == LTR010CMD)
                        {//  ,        
                        //   34   ,     
                        module[i].count_send--; //        FIFO
                        if((NewData & 0xC0C0) == LTR010CMD_RESET)
                            {//  34 
                            module[i].count_send = 0;
                            module[i].count_recv = 0;
                            }
                        }
                    else
                        {//   34 
                        if (module[i].count_send - module[i].count_recv >= LTR34_BUFFER_ALERT_SIZE) // 100  -     
                            { //  
                            continue;
                            }
                        }
                    }

                NumDataFromFastModules--;

                if (++NumDataFast >= NumMixFast)
                    {
                    NumDataFast = 0;
                    memset(AllowLowSpeedModule, 1, sizeof(AllowLowSpeedModule));
                    }
                }
            else
                { //  
                if (!AllowLowSpeedModule[i] && NumDataFromFastModules)
                    continue;  //  ,    20   
                AllowLowSpeedModule[i]  = 0;
                }

            module[i].count_send++;
            *lpBuffer++ = NewData;
            size++;

            if (++module[i].send_buf_cntr_sended >= module[i].send_buf_cntr)
                {
                module[i].send_buf_cntr_sended = 0;
                module[i].send_buf_cntr = 0;
                }
            } /* for i */

        if (size == LastSize)
            break;
        if (NumUsedFastModules)
            NumMixFast = NUM_MIX_WORDS_FOR_ONE_FAST_MODULE * NumUsedFastModules;
        } /* while size */
    return size;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    switch (mode)
        {
        case CM_APP:
            return ltr_USB->Application->GetArray(address, data, size);
        case CM_BOOT:
            return ltr_USB->Bootloader->GetArray(address, data, size);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? ltr_USB->Application->GetArray(address, data, size)
                : ltr_USB->Bootloader->GetArray(address, data, size);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    return (mode != CM_BOOT) ? ltr_Ether->GetArray(address, data, size) : false;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    switch (mode)
        {
        case CM_APP:
            return ltr_USB->Application->PutArray(address, data, size);
        case CM_BOOT:
            return ltr_USB->Bootloader->PutArray(address, data, size);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? ltr_USB->Application->PutArray(address, data, size)
                : ltr_USB->Bootloader->PutArray(address, data, size);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    return (mode != CM_BOOT) ? ltr_Ether->PutArray(address, data, size) : false;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    switch (mode)
        {
        case CM_APP:
            return ltr_USB->Application->GetModuleDescription(description);
        case CM_BOOT:
            return ltr_USB->Bootloader->GetModuleDescription(description);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? ltr_USB->Application->GetModuleDescription(description)
                : ltr_USB->Bootloader->GetModuleDescription(description);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    return (mode != CM_BOOT) ? ltr_Ether->GetModuleDescription(description) : false;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    switch (mode)
        {
        case CM_APP:
            return false;
        case CM_BOOT:
            return ltr_USB->Bootloader->SetModuleDescription(description);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? false
                : ltr_USB->Bootloader->SetModuleDescription(description);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    return false; //   ,    ethernet  
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::CallApplication(DWORD address)
    {
    BYTE tempdata[16];

    if (!ltr_USB->Bootloader->CallApplication(address))
        return false;
    /* ,   :  application  bootloader */
    if (ltr_USB->Bootloader->GetArray(0x0 | SEL_AVR_PM, tempdata, 16))
        {
        ltr_USB->application_active = false;
        }
    else if (ltr_USB->Application->GetArray(0x0 | SEL_AVR_PM, tempdata, 16))
        {
        ltr_USB->application_active = true;
        }
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateEther::CallApplication(DWORD address)
    {
    return false; //   ,    ethernet  
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::CancelIO(void)
    {
    ltr_USB->CancelIO();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateEther::CancelIO(void)
    {
    // Do nothing
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int TCrateUSB::PingAndGetFailCount(void)
    {
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int TCrateEther::PingAndGetFailCount(void)
    {
    if (ltr_Ether->Ping())
        {
        ping_fail_count = 0;
        }
    else
        {
        if (ping_fail_count < INT_MAX)
            ping_fail_count++;
        }
    return ping_fail_count;
    }
/*------------------------------------------------------------------------------------------------*/

