#include "common.h"
#include "LtrCrateThreads.h"

/*------------------------------------------------------------------------------------------------*/
/*    exception */
#define FATAL true
class Ex
    {
private:
    const char *errmsg;
    int flags;
    int errcode;
    bool is_fatal;
public:
    Ex(bool fatal, const char *err_msg, int log_flags, int err_code = 0)
        {
        is_fatal = fatal;
        errmsg = err_msg; flags = log_flags; errcode = err_code;
        }

    Ex(const char *err_msg, int log_flags, int err_code = 0)
        {
        is_fatal = false;
        errmsg = err_msg; flags = log_flags; errcode = err_code;
        }

    Ex()
        {
        errmsg = NULL; errcode = false; flags = 0; is_fatal = false;
        }

    void Dispatch(const char *host, int host_log_flags)
        {
        if (!errmsg) return;
        if (0 == (flags & LOG_FLAG_ANY_LEVEL))
            flags |= LOG_FLAG_DEBUG_LEVEL1;
        if (errcode)
            {
            logfile.AddMessage(Format("Service thread [%s]: %s (error = %d)",
                ARRAYOFCONST((host, errmsg, errcode))), host_log_flags, flags);
            }
        else
            {
            logfile.AddMessage(Format("Service thread [%s]: %s",
                ARRAYOFCONST((host, errmsg))), host_log_flags, flags);
            }
        }

    bool IsFatal(void)
        {
        return is_fatal;
        }
    };
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TReadUSBCrateThread::TReadUSBCrateThread(TCrateUSB *_crate, bool CreateSuspended)
    : TThread(CreateSuspended)
    {
    crate = _crate;
    Priority = tpHigher;
    FreeOnTerminate = false;
    init_complete = CreateEvent(NULL, FALSE, FALSE, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TReadUSBCrateThread::~TReadUSBCrateThread(void)
    {
    CloseHandle(init_complete);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TReadUSBCrateThread::Execute(void)
    {
    DWORD i;
    DWORD index = 0;
    bool failed = false;

    DWORD CrateReadSize = CRATE_RR_SIZE_MAX_BYTE;
    if (!(crate->sflags&CRATE_FLAG_USB_HI_SPEED)) CrateReadSize /= 16;

    memset(rreq, 0, sizeof(rreq));
    for (i = 0; i < CRATE_RR_N; i++)
        {
        rreq[i].buf = &ReadOverlappedBuffer[i*CRATE_RR_SIZE_MAX_DWORD];
        rreq[i].hOver = new OVERLAPPED[1];
        memset(rreq[i].hOver,0,sizeof(OVERLAPPED));
        if ((((LPOVERLAPPED)rreq[i].hOver)->hEvent = rreq[i].hEvent = CreateEvent(NULL, FALSE, TRUE, NULL)) == NULL)
            {
            logfile.AddMessage(Format("Read Crate Thread, Error Alloc OVER %s", ARRAYOFCONST((crate->pid))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
            break;
            }
        }

    if ((i == CRATE_RR_N) && (crate->module[MODULE_MAX].CreateModule()) /*  RawData */ )
        {
        SetEvent(init_complete); //   
        DWORD SizeBytes = 0;
        crate->ei = 0;
        crate->total_count_recv = 0;
        while (crate->ThreadsRunning && !failed)
            {
            while ((WaitForSingleObject(rreq[index].hEvent, TIMEOUT_2) == WAIT_TIMEOUT)
                && (crate->ThreadsRunning));
            if (!crate->ThreadsRunning) break;

            if (!GetOverlappedResult(crate->DeviceHANDLE, (LPOVERLAPPED)rreq[index].hOver, &SizeBytes, FALSE))
                {  // GetOverlapped Result    true - false,     -    ,  -   
                logfile.AddMessage(Format("WST.XXXX Error async read data size=%d", ARRAYOFCONST(((int)SizeBytes))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
                failed = true;
                break;
                }

            /*    USB bulk I/O ,     4 */
            if (SizeBytes > 3)
                {
                if (!crate->ParseDataFromCrate(rreq[index].buf, SizeBytes >> 2))
                    {
                    failed = true;
                    break;
                    }
                }

            //      -  
            if (!ReadFile(crate->DeviceHANDLE,(BYTE*)rreq[index].buf, CrateReadSize, &SizeBytes, (LPOVERLAPPED)rreq[index].hOver))
                {
                SizeBytes = 0;
                DWORD err = GetLastError();
                if (err != ERROR_IO_PENDING)
                    {
                    failed = true;
                    logfile.AddMessage(Format("WST.XXXX Error async begin read data Error=%d", ARRAYOFCONST(((int)err))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
                    }
                index = (index + 1) % CRATE_RR_N;
                }
            } /* while ThreadsRunning && !failed */
        } /* i == CRATE_RR_N */

    logfile.AddMessage(Format("Read Crate Thread Begin Finishing Work %s", ARRAYOFCONST((crate->pid))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);

    crate->ThreadsRunning = false;

    for (int i = 0; i < CRATE_RR_N; i++)
        {
        DWORD SizeBytes2 = 0;
        if (!GetOverlappedResult(crate->DeviceHANDLE, (LPOVERLAPPED)rreq[i].hOver, &SizeBytes2, FALSE))
            crate->ltr_USB->CancelIO();
        CloseHandle(((LPOVERLAPPED)rreq[i].hOver)->hEvent);
        delete rreq[i].hOver;
        rreq[i].hOver=NULL;
        }

    crate->module[MODULE_MAX].DeleteModule();  // -  RawData
    /*         */
    if (failed)
        crate->SetState(CRATE_ST_DEAD);
    logfile.AddMessage(Format("Read Crate Thread Finished Work %s", ARRAYOFCONST((crate->pid))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TWriteUSBCrateThread::TWriteUSBCrateThread(TCrateUSB *_crate, bool CreateSuspended)
    : TThread(CreateSuspended)
    {
    crate=_crate;
    Priority=tpHigher;// tpNormal;
    FreeOnTerminate=false;
    init_complete=CreateEvent(NULL, FALSE, FALSE, NULL);
    }
//---------------------------------------------------------------------------
__fastcall TWriteUSBCrateThread::~TWriteUSBCrateThread(void)
    {
    CloseHandle(init_complete);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TWriteUSBCrateThread::Execute(void)
    {
    DWORD i;
    DWORD index = 0;
    DWORD sz;
    bool failed = false;

    memset(crate->AllowLowSpeedModule, 1, sizeof(crate->AllowLowSpeedModule));
    memset(wreq, 0, sizeof(wreq));

    for (i = 0; i < CRATE_WR_N; i++)
        {
        wreq[i].buf = &WriteOverlappedBuffer[i*CRATE_WR_SIZE_MAX_DWORD];
        wreq[i].hOver = new OVERLAPPED[1];
        memset(wreq[i].hOver,0,sizeof(OVERLAPPED));
        if ((((LPOVERLAPPED)wreq[i].hOver)->hEvent = wreq[i].hEvent = CreateEvent(NULL, FALSE, TRUE, NULL)) == NULL)
            {
            logfile.AddMessage(Format("Read Crate Thread, Error Alloc OVER %s", ARRAYOFCONST((crate->pid))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
            break;
            }
        }

    if (i == CRATE_WR_N)
        {
        SetEvent(init_complete); //   
        crate->total_count_send = 0;
        DWORD SendSize = 0;
        DWORD ReadyDWords;
        DWORD NumDataFast = 0;
        while (crate->ThreadsRunning && !failed)
            {
            while ((WaitForSingleObject(wreq[index].hEvent, TIMEOUT_2) == WAIT_TIMEOUT)
                && (crate->ThreadsRunning));
            if (!crate->ThreadsRunning) break;

            if (!GetOverlappedResult(crate->DeviceHANDLE, (LPOVERLAPPED)wreq[index].hOver, &SendSize, FALSE))
                {   // GetOverlapped Result    true - false,     -     -    
                logfile.AddMessage(Format("WST.XXXX Error async send data size=%d", ARRAYOFCONST(((int)SendSize))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
                failed = true;
                break;
                }

            //      
            ReadyDWords = 0;
            while (crate->ThreadsRunning && (ReadyDWords == 0))
                {
                ReadyDWords = crate->PrepareDataForCrate(wreq[index].buf, CRATE_WR_SIZE_MAX_DWORD,
                    NumDataFast);
                if (ReadyDWords == 0)
                    Sleep(TIMEOUT_0 / 2);
                }
            if (!crate->ThreadsRunning || (ReadyDWords == 0))
                break;

        #ifdef DEBUG_MESSAGES
            logfile.AddMessage(Format("WST.0000               sz =%8d", ARRAYOFCONST(((int)size))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL6|LOG_FLAG_CRATE_WRITE);
        #endif
            //  
            if (!WriteFile(crate->DeviceHANDLE,(BYTE*)wreq[index].buf , ReadyDWords * sizeof(DWORD), &sz, (LPOVERLAPPED)wreq[index].hOver))
                {
                int err = GetLastError();
                if (err != ERROR_IO_PENDING)
                    {
                    failed = true;
                    logfile.AddMessage(Format("WST.XXXX Error async begin write data Error=%d", ARRAYOFCONST(((int)err))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
                    }
                index = (index + 1) % CRATE_WR_N;
                }
            //  
            crate->total_count_send += ReadyDWords;
            } /* while ThreadsRunning && !failed */
        } /* i == CRATE_WR_N */

    logfile.AddMessage(Format("Write Crate Thread Begin Finishing Work %s", ARRAYOFCONST((crate->pid))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);

    crate->ThreadsRunning = false;

    for (int i = 0; i < CRATE_WR_N; i++)
        {
        DWORD SizeBytes2 = 0;
        if (!GetOverlappedResult(crate->DeviceHANDLE, (LPOVERLAPPED)wreq[i].hOver, &SizeBytes2, FALSE))
            crate->ltr_USB->CancelIO();
        CloseHandle(((LPOVERLAPPED)wreq[i].hOver)->hEvent);
        delete wreq[i].hOver;
        wreq[i].hOver=NULL;
        }

    /*         */
    if (failed)
        crate->SetState(CRATE_ST_DEAD);
    logfile.AddMessage(Format("Write Crate Thread Finished Work %s", ARRAYOFCONST((crate->pid))), crate->log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_WRITE);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TEthCratePingThread::TEthCratePingThread(TCrateEther *_crate, bool CreateSuspended)
    : TThread(CreateSuspended)
    {
    Priority = tpIdle;
    FreeOnTerminate = false;
    crate = _crate;
    hTermEvent = CreateEvent(NULL, /* manual */ TRUE, /* init */ FALSE, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TEthCratePingThread::~TEthCratePingThread(void)
    {
    CloseHandle(hTermEvent);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TEthCratePingThread::TerminateByEvent(void)
    {
    SetEvent(hTermEvent);
    TThread::Terminate();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TEthCratePingThread::Execute(void)
    {
    /*      (      ).
     *   IdleTimer.      NOP.
     *      LTR_CRATE_PING_ATTEMPTS  ,   
     *    .
     */
    HANDLE wait_list[2];
    int failcount;

    wait_list[0] = hTermEvent;
    wait_list[1] = crate->GetIdleTimerHandle();
    if (!wait_list[1])
        return;
    while (!Terminated)
        {
        switch (WaitForMultipleObjects(2, wait_list, FALSE, INFINITE))
            {
            case WAIT_OBJECT_0: /* TERMINATE EVENT */
                return;
            case WAIT_OBJECT_0 + 1: /* IDLE TIMER */
                failcount = crate->PingAndGetFailCount();
                if (failcount == 0)
                    {
                    logfile.AddMessage(Format("Ping: %s is alive",
                        ARRAYOFCONST((crate->path))),
                        crate->log_flags, LOG_FLAG_DEBUG_LEVEL6 | LOG_FLAG_CRATE_INIT);
                    }
                else
                    {
                    crate->RestartIdleTimer();
                    logfile.AddMessage(Format("Ping: %s failed %d ping(s)",
                        ARRAYOFCONST((crate->path, failcount))),
                        crate->log_flags, LOG_FLAG_DEBUG_LEVEL3 | LOG_FLAG_CRATE_INIT);
                    }
                if (failcount >= LTR_CRATE_PING_ATTEMPTS)
                    {
                    crate->SetState(CRATE_ST_DEAD);
                    return;
                    }
                break;
            case WAIT_TIMEOUT:
                break;
            default: /* ERROR */
                return;
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TEthCrateDataThread::TEthCrateDataThread(TCrateEther *_crate, bool CreateSuspended)
    : TThread(CreateSuspended)
    {
    crate = _crate;
    Priority = tpHigher;
    FreeOnTerminate = false;
    init_complete = CreateEvent(NULL, FALSE, FALSE, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TEthCrateDataThread::~TEthCrateDataThread(void)
    {
    CloseHandle(init_complete);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TEthCrateDataThread::Execute(void)
    {
    const TIMEVAL shorter_time = {TIMEOUT_ETH_SHORTER / 1000, (TIMEOUT_ETH_SHORTER % 1000) * 1000};
    const TIMEVAL longer_time =  {TIMEOUT_ETH_LONGER / 1000, (TIMEOUT_ETH_LONGER % 1000) * 1000};
    ULONG nonblocking = 1;
    bool keep_trying = true;
    int retry_count = 0;
    TEthCratePingThread *PingThread;

    memset(crate->AllowLowSpeedModule, 1, sizeof(crate->AllowLowSpeedModule));
    crate->ei = 0;
    crate->total_count_recv = 0;
    crate->total_count_send = 0;

    SetEvent(init_complete); //   

    /*     ,    */
    if (crate->GetState() == CRATE_ST_CONNECTING)
        {
        bool result = crate->InitCrate();
        if (result)
            {
            crate->SetState(CRATE_ST_ONLINE);
            }
        else
            {
            crate->SetState(CRATE_ST_DEAD);
            }
        }

    if (crate->GetState() != CRATE_ST_ONLINE)
        return;

    /*    RawData */
    if (!crate->module[MODULE_MAX].CreateModule())
        {
        Ex("Cannot create rawdata module", LOG_FLAG_CRATE_INIT | LOG_FLAG_DEBUG_LEVEL1)
            .Dispatch(crate->path, crate->log_flags);
        crate->SetState(CRATE_ST_DEAD);
        return;
        }

    /*     */
    PingThread = new TEthCratePingThread(crate, false);

    Ex("STARTED", LOG_FLAG_CRATE_INIT | LOG_FLAG_DEBUG_LEVEL1)
        .Dispatch(crate->path, crate->log_flags);

    while (keep_trying && !Terminated)
        {
        SOCKET sock = INVALID_SOCKET;
        bool connected = false;
        fd_set rset, wset, eset;
        int res;
        DWORD BytesSent = 0;
        DWORD NumSendBytes = 0;
        DWORD NumDataFast = 0;
        int RecvShift = 0;
        DWORD BufSize;
        int WaitCount;

        keep_trying = false;
        try
            {
            /*     */
            sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
            if (sock == INVALID_SOCKET)
                throw Ex(FATAL, "Cannot create socket", LOG_FLAG_CRATE_INIT, WSAGetLastError());

            /*    */
            BufSize = CRATE_ETH_SEND_TCPBUF_SIZE;
            if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char*)&BufSize, sizeof(BufSize)) == SOCKET_ERROR)
                throw Ex(FATAL, "Cannot set socket send buffer", LOG_FLAG_CRATE_INIT, WSAGetLastError());
            BufSize = CRATE_ETH_RECV_TCPBUF_SIZE;
            if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char*)&BufSize, sizeof(BufSize)) == SOCKET_ERROR)
                throw Ex(FATAL, "Cannot set socket receive buffer", LOG_FLAG_CRATE_INIT, WSAGetLastError());

            /*    non-blocking mode */
            if (ioctlsocket(sock, FIONBIO, &nonblocking) == SOCKET_ERROR)
                throw Ex(FATAL, "Cannot set socket to non-blocking mode", LOG_FLAG_CRATE_INIT, WSAGetLastError());

            Ex("Opening data connection", LOG_FLAG_CRATE_INIT  | LOG_FLAG_DEBUG_LEVEL2)
                    .Dispatch(crate->path, crate->log_flags);

            /*   */
            WaitCount = TIMEOUT_ETH_CONNECT / TIMEOUT_ETH_LONGER;
            res = crate->ltr_Ether->ConnectDevicePort(sock, LTR_ETHER_DATA_PORT);
            if (res == SOCKET_ERROR)
                {
                int error = WSAGetLastError();
                if (error != WSAEWOULDBLOCK)
                    throw Ex(FATAL, "Connection failed", LOG_FLAG_CRATE_INIT, error);
                }

            while (!connected && (WaitCount-- > 0) && !Terminated)
                {
                FD_ZERO(&wset);
                FD_SET(sock, &wset);
                FD_ZERO(&eset);
                FD_SET(sock, &eset);
                res = select(sock + 1, NULL, &wset, &eset, &longer_time);
                if (res == SOCKET_ERROR)
                    throw Ex(FATAL, "Error waiting for connection", LOG_FLAG_CRATE_INIT, WSAGetLastError());
                if (FD_ISSET(sock, &eset)) /* connection failed */
                    {
                    DWORD error;
                    int len = sizeof(error);
                    if (getsockopt(sock, SOL_SOCKET, SO_ERROR, (char*)&error, &len) == SOCKET_ERROR)
                        error = WSAEHOSTUNREACH;
                    throw Ex("Connection failed", LOG_FLAG_CRATE_INIT, error);
                    }
                if (FD_ISSET(sock, &wset)) /* connection established */
                    connected = true;
                } /* while */

            if (!connected && !Terminated)
                {
                /*  */
                throw Ex("Connection timed out", LOG_FLAG_CRATE_INIT);
                }

            /*   */
            Ex("Data connection established", LOG_FLAG_CRATE_INIT | LOG_FLAG_DEBUG_LEVEL2)
                    .Dispatch(crate->path, crate->log_flags);

            retry_count = 0; /*    */

            while (!Terminated)
                {
                if (BytesSent >= NumSendBytes)
                    {
                    crate->total_count_send += NumSendBytes / sizeof(DWORD);
                    BytesSent = 0;
                    NumSendBytes = sizeof(DWORD) *
                        crate->PrepareDataForCrate((DWORD*)SendBuf, sizeof(SendBuf) / sizeof(DWORD),
                            NumDataFast);
                    }
                FD_ZERO(&rset);
                FD_SET(sock, &rset);
                FD_ZERO(&wset);
                if (BytesSent < NumSendBytes)
                    {
                    FD_SET(sock, &wset);
                    res = select(sock + 1, &rset, &wset, NULL, &longer_time);
                    }
                else
                    {
                    res = select(sock + 1, &rset, NULL, NULL, &shorter_time);
                    }
                if (res == SOCKET_ERROR)
                    throw Ex("Error in select()", LOG_FLAG_CRATE_READ, WSAGetLastError());

                /* ----  ---- */
                if (FD_ISSET(sock, &rset))
                    {
                    res = recv(sock, RecvBuf + RecvShift, sizeof(RecvBuf) - RecvShift, 0);
                    if (res == SOCKET_ERROR)
                        {
                        int error = WSAGetLastError();
                        if (error != WSAEWOULDBLOCK)
                            throw Ex("Error reading socket", LOG_FLAG_CRATE_READ, error);
                        }
                    else if (res == 0)
                        { /*    */
                        throw Ex("Connection closed by remote host", LOG_FLAG_CRATE_READ);
                        }
                    else
                        { /*  res  */
                        crate->RestartIdleTimer();
                        int datasize = RecvShift + res;
                        RecvShift = datasize % sizeof(DWORD); /*   */
                        if (datasize >= sizeof(DWORD))
                            {
                            if (!crate->ParseDataFromCrate((DWORD*)RecvBuf, datasize / sizeof(DWORD)))
                                throw Ex("Error in crate data", LOG_FLAG_CRATE_READ);
                            /*     DWORD,     */
                            for (int i = RecvShift - 1; i >= 0; i--)
                                {
                                RecvBuf[i] = RecvBuf[datasize - RecvShift + i];
                                }
                            }
                        }
                    } /* if can read */

                /* ----  ----*/
                if (FD_ISSET(sock, &wset) && (BytesSent < NumSendBytes))
                    {
                    res = send(sock, SendBuf + BytesSent, NumSendBytes - BytesSent, 0);
                    if (res == SOCKET_ERROR)
                        {
                        int error = WSAGetLastError();
                        if (error != WSAEWOULDBLOCK)
                            throw Ex("Error writing socket", LOG_FLAG_CRATE_WRITE, error);
                        }
                    else
                        { /*  res  */
                        BytesSent += res;
                        }
                    } /* if can write */
                } /* while !Terminated */
            } /* try */

        catch (Ex ex)
            {
            ex.Dispatch(crate->path, crate->log_flags);
            keep_trying = !ex.IsFatal();
            }

        /*   */
        if (sock != INVALID_SOCKET)
            {
            if (connected)
                shutdown(sock, SD_BOTH);
            closesocket(sock);
            }

        /*   */
        if (++retry_count >= LTR_CRATE_DATACONN_ATTEMPTS)
            keep_trying = false;
        /*     */
        if (keep_trying)
            {
            WaitCount = TIMEOUT_ETH_RETRY / TIMEOUT_ETH_LONGER;
            while (WaitCount-- && !Terminated)
                {
                Sleep(TIMEOUT_ETH_LONGER);
                }
            }
        else
            { /*         */
            crate->SetState(CRATE_ST_DEAD);
            }
        } /* while (keep_trying && !Terminated) */

    Ex("SHUTTING DOWN", LOG_FLAG_CRATE_INIT | LOG_FLAG_DEBUG_LEVEL6)
        .Dispatch(crate->path, crate->log_flags);

    crate->module[MODULE_MAX].DeleteModule();

    if (PingThread)
        {
        PingThread->TerminateByEvent();
        PingThread->WaitFor();
        delete PingThread;
        }

    Ex("STOPPED", LOG_FLAG_CRATE_INIT | LOG_FLAG_DEBUG_LEVEL1)
        .Dispatch(crate->path, crate->log_flags);
    }
/*------------------------------------------------------------------------------------------------*/

