#include "LtrIniFile.h"
#include <vcl.h>

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
__fastcall TLtrIniFile::TLtrIniFile(void)
    {
    InitializeCriticalSectionAndSpinCount(&cs, 0x80000000);
    ini = NULL;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TLtrIniFile::~TLtrIniFile(void)
    {
    Close();
    DeleteCriticalSection(&cs);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::Open(AnsiString FileNameWithoutPath)
    {
    TCSLock lock(cs);
    if (ini)
        Close();
    try
        {
        ini = new TIniFile(ExtractFilePath(Application->ExeName) + FileNameWithoutPath);
        }
    catch (EIniFileException&)
        {
        ini = NULL;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::Close(void)
    {
    TCSLock lock(cs);
    if (ini)
        {
        delete ini;
        ini = NULL;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
AnsiString __fastcall TLtrIniFile::ReadString
    (AnsiString Section, AnsiString Ident, AnsiString Default, bool CreateMissing)
    {
    TCSLock lock(cs);
    if (!ini) return Default;
    try
        {
        if (!CreateMissing || ini->ValueExists(Section, Ident))
            {
            return ini->ReadString(Section, Ident, Default);
            }
        else
            {
            ini->WriteString(Section, Ident, Default);
            ini->UpdateFile();
            return Default;
            }
        }
    catch (EIniFileException&)
        {
        return Default;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int __fastcall TLtrIniFile::ReadInteger
    (AnsiString Section, AnsiString Ident, int Default, bool CreateMissing)
    {
    TCSLock lock(cs);
    if (!ini) return Default;
    try
        {
        if (!CreateMissing || ini->ValueExists(Section, Ident))
            {
            return ini->ReadInteger(Section, Ident, Default);
            }
        else
            {
            ini->WriteInteger(Section, Ident, Default);
            ini->UpdateFile();
            return Default;
            }
        }
    catch (EIniFileException&)
        {
        return Default;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int __fastcall TLtrIniFile::ReadIntegerHex
    (AnsiString Section, AnsiString Ident, int Default, bool CreateMissing)
    {
    TCSLock lock(cs);
    if (!ini) return Default;
    try
        {
        if (!CreateMissing || ini->ValueExists(Section, Ident))
            {
            return ini->ReadInteger(Section, Ident, Default);
            }
        else
            {
            ini->WriteString(Section, Ident, Format("0x%0.8X", ARRAYOFCONST((Default))));
            ini->UpdateFile();
            return Default;
            }
        }
    catch (EIniFileException&)
        {
        return Default;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool __fastcall TLtrIniFile::ReadBool
    (AnsiString Section, AnsiString Ident, bool Default, bool CreateMissing)
    {
    TCSLock lock(cs);
    if (!ini) return Default;
    try
        {
        if (!CreateMissing || ini->ValueExists(Section, Ident))
            {
            return ini->ReadBool(Section, Ident, Default);
            }
        else
            {
            ini->WriteBool(Section, Ident, Default);
            ini->UpdateFile();
            return Default;
            }
        }
    catch (EIniFileException&)
        {
        return Default;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::WriteString(AnsiString Section, AnsiString Ident, AnsiString Value)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        ini->WriteString(Section, Ident, Value);
        ini->UpdateFile();
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::WriteInteger(AnsiString Section, AnsiString Ident, int Value)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        ini->WriteInteger(Section, Ident, Value);
        ini->UpdateFile();
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::WriteIntegerHex(AnsiString Section, AnsiString Ident, int Value)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        ini->WriteString(Section, Ident, Format("0x%0.8X", ARRAYOFCONST((Value))));
        ini->UpdateFile();
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::WriteBool(AnsiString Section, AnsiString Ident, bool Value)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        ini->WriteBool(Section, Ident, Value);
        ini->UpdateFile();
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::DeleteKey(AnsiString Section, AnsiString Ident)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        ini->DeleteKey(Section, Ident);
        ini->UpdateFile();
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::ReadSection(AnsiString Section, TStrings * Strings)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        ini->ReadSection(Section, Strings);
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLtrIniFile::ClearSection(AnsiString Section)
    {
    TCSLock lock(cs);
    if (!ini) return;
    try
        {
        if (TStrings* KeyList = new TStringList)
            {
            ini->ReadSection(Section, KeyList);
            for (int i = 0; i < KeyList->Count; i++)
                {
                ini->DeleteKey(Section, KeyList->Strings[i]);
                }
            ini->UpdateFile();
            delete KeyList;
            }
        }
    catch (EIniFileException&)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
