#include <vcl.h>
#include <stdio.h>
#pragma hdrstop
#pragma pack(1)
#include <setupapi.h>
#pragma pack()
#include "LtrCrate.h"
#include "LtrServiceCrates.h"
#pragma package(smart_init)

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
struct TCrateInfo {
	char path[1024];
	char pid[32];
};
const CLSID USB_DEVICE_CLSID={0xA5DCBF10,0x6530,0x11D2,{0x90,0x1F,0x00,0xC0,0x4F,0xB9,0x51,0xED}};
//---------------------------------------------------------------------------
__fastcall TLTRServiceCrate::TLTRServiceCrate(bool CreateSuspended) : TThread(CreateSuspended)
{
    Priority=tpNormal;
    FreeOnTerminate=false;

    MessageQueueIn = new T_MessageQueue(100,100);
    if(MessageQueueIn== NULL)
    {
        logfile.AddMessage("Can't Alloc memory", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    }

    //     
    UDP_Socket = WSASocket(AF_INET, SOCK_DGRAM, IPPROTO_IP, NULL, 0, WSA_FLAG_OVERLAPPED);
    if (UDP_Socket!=INVALID_SOCKET)
    {
        bool Broadcast = true;
        if (setsockopt(UDP_Socket, SOL_SOCKET, SO_BROADCAST, (const char*)&Broadcast, sizeof(Broadcast))==SOCKET_ERROR)
        {
            logfile.AddMessage("Can't set Broadcast Socket Parameters.", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        }
        else
        {
            sockaddr_in UDP_From;
             memset(&UDP_From,0,sizeof(UDP_From));
            UDP_From.sin_family = AF_INET;
            UDP_From.sin_port = htons(LTR_ETHER_IDENTIFY_PORT);
            UDP_From.sin_addr.S_un.S_addr = INADDR_ANY;

            if(WSAConnect(UDP_Socket, (sockaddr*)&UDP_From, sizeof(UDP_From), NULL, NULL, NULL, NULL)==SOCKET_ERROR)
            {
                logfile.AddMessage("Can't connect UDP Socket", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            }
        }
    }

    UDP_WSA_Recv_Buf.len=sizeof(UDP_RecvBuf);
    UDP_WSA_Recv_Buf.buf=UDP_RecvBuf;

    memset(&UDP_Recv_Over,0, sizeof(UDP_Recv_Over));
    UDP_Recv_Over.hEvent = WSACreateEvent();
    WSASetEvent(UDP_Recv_Over.hEvent);

    init_complete=CreateEvent(NULL, FALSE, FALSE, NULL);
}
//---------------------------------------------------------------------------
__fastcall TLTRServiceCrate::~TLTRServiceCrate(void)
{
    if (UDP_Socket!=INVALID_SOCKET)
    {
        shutdown(UDP_Socket, SD_BOTH);
        closesocket(UDP_Socket);
    }
    if(UDP_Recv_Over.hEvent!=NULL) WSACloseEvent(UDP_Recv_Over.hEvent);

    if(MessageQueueIn!=NULL) delete MessageQueueIn;

    CloseHandle(init_complete);
}
//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------
/*------------------------------------------------------------------------------------------------*/
void __fastcall TLTRServiceCrate::StopServiceForDeadCrates(void)
    {
    if (WaitForSingleObject(crate_list_access, INFINITE) != WAIT_OBJECT_0)
        return;
    //***     
    for (int i = 0; i < CRATE_MAX; i++)
        {
        if (crate_list[i]->GetState() == CRATE_ST_DEAD)
            {
            logfile.AddMessage(Format("Service stopped for crate  >> CSN=%.16s  PIDString=%s  Location=%s",
                ARRAYOFCONST(((char*)crate_list[i]->csn, (char*)crate_list[i]->pid, crate_list[i]->path))),
                g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            delete crate_list[i];
            crate_list[i] = NULL;
            }
        }
    ReleaseMutex(crate_list_access);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
#if PING_IN_SERVICE
void TLTRServiceCrate::CheckForIdleCrates(void)
    {
    /*  ,      (      ).
     *   IdleTimer.      NOP.
     * ,     LTR_CRATE_PING_ATTEMPTS  ,  
     *  .
     * :    NOP      ,
     *       .
     */
    if (WaitForSingleObject(crate_list_access, 500) == WAIT_OBJECT_0)
        {
        for (int i = 0; i < CRATE_MAX; i++)
            {
            if (crate_list[i]->GetState() == CRATE_ST_ONLINE)
                {
                HANDLE hTimer = crate_list[i]->GetIdleTimerHandle();
                if (hTimer && (WaitForSingleObject(hTimer, 0) == WAIT_OBJECT_0))
                    { /*      */
                    int failcount = crate_list[i]->PingAndGetFailCount();
                    if (failcount == 0)
                        {
                        logfile.AddMessage(Format("Ping: %s is alive",
                            ARRAYOFCONST((crate_list[i]->path))),
                            g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL6 | LOG_FLAG_CRATE_INIT);
                        }
                    else
                        {
                        crate_list[i]->RestartIdleTimer();
                        logfile.AddMessage(Format("Ping: %s failed %d ping(s)",
                            ARRAYOFCONST((crate_list[i]->path, failcount))),
                            g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL3 | LOG_FLAG_CRATE_INIT);
                        }
                    if (failcount >= LTR_CRATE_PING_ATTEMPTS)
                        {
                        /*    .    
                            StopServiceForDeadCrates() */
                        crate_list[i]->SetState(CRATE_ST_DEAD);
                        }
                    /*    ,        */
                    if (failcount > 0)
                        break;
                    }
                }
            } /* for i */
        ReleaseMutex(crate_list_access);
        }
    }
#endif
/*------------------------------------------------------------------------------------------------*/

//---------------------------------------------------------------------------
//   
//---------------------------------------------------------------------------
void TLTRServiceCrate::ScanUSBCrates(void)
{
    int i=0, j, n=0;
    //***     
    TCrateInfo *crate_info=new TCrateInfo[32];
    logfile.AddMessage("Rescan attached Crates...", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    HDEVINFO hdinfo=SetupDiGetClassDevs(&USB_DEVICE_CLSID, NULL, NULL, DIGCF_PRESENT|DIGCF_DEVICEINTERFACE);
    if(hdinfo!=INVALID_HANDLE_VALUE)
    {
        SP_DEVICE_INTERFACE_DATA idata;
        memset(&idata, 0, sizeof(idata));
        idata.cbSize=sizeof(SP_DEVICE_INTERFACE_DATA);
        while(SetupDiEnumDeviceInterfaces(hdinfo, NULL, &USB_DEVICE_CLSID, i, &idata) && n<32)
        {
            DWORD rsz0=0;
            SetupDiGetDeviceInterfaceDetail(hdinfo, &idata, NULL, 0, &rsz0, NULL);
            if(GetLastError()==ERROR_INSUFFICIENT_BUFFER)
            {
                PSP_DEVICE_INTERFACE_DETAIL_DATA idata_detail=(PSP_DEVICE_INTERFACE_DETAIL_DATA)new BYTE[rsz0];
                idata_detail->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
                PSP_DEVINFO_DATA idata_info=new SP_DEVINFO_DATA;
                idata_info->cbSize=sizeof(SP_DEVINFO_DATA);
                if(SetupDiGetDeviceInterfaceDetail(hdinfo, &idata, idata_detail, rsz0, &rsz0, idata_info))
                {
                    DWORD rsz1;
                    SetupDiGetDeviceRegistryProperty(hdinfo, idata_info, SPDRP_LOCATION_INFORMATION, NULL, NULL, 0, &rsz1);
                    if(GetLastError()==ERROR_INSUFFICIENT_BUFFER)
                    {
                        char *pid=new BYTE[rsz1];
                        if(SetupDiGetDeviceRegistryProperty(hdinfo, idata_info, SPDRP_LOCATION_INFORMATION, NULL, pid, rsz1, NULL))
                        {
#ifdef DEBUG_MESSAGES
                            memset(msg, 0, sizeof(msg));
                            sprintf(msg, "Usb device information >> [%d] PIDString=%s", i, pid);
                            DWORD slen=strlen(msg);
                            if(slen<55) memset(msg+slen, ' ', 55-slen);
                            strcat(msg, "Location=");
                            strcat(msg, idata_detail->DevicePath);
                            logfile.AddMessage(msg, default_lflags&LOG_MASK_CRATE_INIT);
#endif
                            //
                            if((strstr(idata_detail->DevicePath, "vid_0471&pid_1010")!=NULL ||
                                strstr(idata_detail->DevicePath, "vid_0471&pid_2121")!=NULL ||
                                strstr(idata_detail->DevicePath, "vid_0471&pid_3030")!=NULL ||
                                strstr(idata_detail->DevicePath, "vid_0471&pid_3131")!=NULL )&&
                                idata.Flags==SPINT_ACTIVE)
                            {
                                sprintf(crate_info[n].path, "%s", idata_detail->DevicePath);
                                sprintf(crate_info[n].pid,  "%s", pid);
                                n++;
                            }
                        } else logfile.AddMessage(Format("SetupDiGetDeviceRegistryProrerty() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                        delete[] pid;
                    } else logfile.AddMessage(Format("SetupDiGetDeviceRegistryProperty() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                } else logfile.AddMessage(Format("SetupDiGetDeviceInterfaceDetail() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                delete idata_info;
                delete[] idata_detail;
            } else logfile.AddMessage(Format("SetupDiGetDeviceInterfaceDetail() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
            i++;
        }
        //
        if(n<32 && GetLastError()!=ERROR_NO_MORE_ITEMS) logfile.AddMessage(Format("SetupDiEnumDeviceInterfaces() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        if(!SetupDiDestroyDeviceInfoList(hdinfo)) logfile.AddMessage(Format("SetupDiDestroyDeviceInfoList() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        //
        if(i) logfile.AddMessage(Format("Found usb devices >> total=%d  LTRCrates=%d", ARRAYOFCONST((i, n))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        else logfile.AddMessage("Can't find any usb device", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    } else logfile.AddMessage(Format("SetupDiGetClassDevs() -> error(%d)", ARRAYOFCONST(((int)GetLastError()))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);

    if(WaitForSingleObject(crate_list_access, INFINITE)==WAIT_OBJECT_0)
    { //   
        for(i=0; i<CRATE_MAX; i++)
        {
            if ((crate_list[i]->GetState() == CRATE_ST_ONLINE)
                && (crate_list[i]->GetConnType() == CrateCon_USB))
            {
                bool CratePresenseInRegister=false;
                for(j=0; j<n; j++)
                {
                    if (!strcmp(crate_info[j].path, crate_list[i]->path))
                    {
                        CratePresenseInRegister=true;
                        break;
                    }
                }

                if (!CratePresenseInRegister)
                    crate_list[i]->SetState(CRATE_ST_DEAD);
            }
        }
        ReleaseMutex(crate_list_access);
    }

    // , .. SetState()  LTR_SRV_bury_the_dead
    //StopServiceForDeadCrates();

    if (WaitForSingleObject(crate_list_access, INFINITE)==WAIT_OBJECT_0)
        {
        //***     
        for (j = 0; j < n; j++)
            {
            bool AlreadyExists = false;
            int FreeSlot = -1;
            for (i = 0; i < CRATE_MAX; i++)
                {
                if (!crate_list[i])
                    {
                    if (FreeSlot < 0)
                        FreeSlot = i;
                    }
                else if ((crate_list[i]->GetState() != CRATE_ST_DEAD)
                        && !strcmp(crate_info[j].path, crate_list[i]->path))
                    {
                    AlreadyExists = true;
                    break;
                    }
                }

            if (AlreadyExists)
                continue;

            /*       ,    */
            if (FreeSlot >= 0)
                {
                TCrate *crate = new TCrateUSB(MessageQueueIn, MessageQueueDisplay);
                if (crate)
                    {
                    strncpy(crate->path, crate_info[j].path, sizeof(crate->path) - 1);
                    crate->path[sizeof(crate->path) - 1] = '\0';
                    strncpy(crate->pid, crate_info[j].pid, sizeof(crate->pid) - 1);
                    crate->pid[sizeof(crate->pid) - 1] = '\0';
                    crate->log_flags = g_Config.log_flags;
                    crate_list[FreeSlot] = crate;
                    /*    */
                    if (!crate->Startup(NULL, g_Config.default_ttf.c_str()))
                        {
                        logfile.AddMessage(Format("Can't init crate %s",
                            ARRAYOFCONST((crate_info[j].path))),
                            crate->log_flags, LOG_FLAG_DEBUG_LEVEL1 | LOG_FLAG_CRATE_INIT);
                        }
                    }
                }
            else /* no free slot */
                {
                logfile.AddMessage(Format("No free slot for crate %s (maximum %d reached)",
                    ARRAYOFCONST((crate_info[j].path, (int)CRATE_MAX))),
                    g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1 | LOG_FLAG_CRATE_INIT);
                break;
                }
            } /* for j */
        ReleaseMutex(crate_list_access);
        }
    delete[] crate_info;
    }
//-----------------------------------------------------------------------------
void TLTRServiceCrate::ProceedNewEthernetCrate(void)
{
    if (UDP_Socket==INVALID_SOCKET) return;

    DWORD RecvBytes=0, Flag=0;
    WSAGetOverlappedResult(UDP_Socket, &UDP_Recv_Over, &RecvBytes, FALSE, &Flag);

    if(RecvBytes>=16)
        {
        //##    source addr     
        AnsiString NewCrateIP = Format("%.16s",ARRAYOFCONST(((char *)UDP_RecvBuf)));
        DWORD ip = StrToIP(NewCrateIP);

        if (IPCrateList.AddUnique(IPLIST_DYNAMIC, ip, false))
            {
            logfile.AddMessage(Format("Found new ethernet crate, IP: %.16s",
                ARRAYOFCONST((NewCrateIP.c_str()))), g_Config.log_flags,
                LOG_FLAG_DEBUG_LEVEL1 | LOG_FLAG_CRATE_INIT);
            MessageQueueDisplay->PostMessage(LTR_DISP_ethernet_list_updated);
            if (g_Config.AutoConnectDynamic)
                MessageQueueIn->PostMessage(LTR_SRV_connect_crate, ip);
            }
        }

    //WSAResetEvent(UDP_Recv_Over.hEvent);
    memset(UDP_WSA_Recv_Buf.buf,0, UDP_WSA_Recv_Buf.len);
    WSARecv(UDP_Socket, &UDP_WSA_Recv_Buf,1, &RecvBytes, &Flag, &UDP_Recv_Over, NULL);
}
//-----------------------------------------------------------------------------
void TLTRServiceCrate::StartScanEthernetCrates()
{
    const int NumMaxRequest=1;
    if (UDP_Socket==INVALID_SOCKET) return;

    logfile.AddMessage("Start autoserach ethernet Crates...", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);

    //     
    sockaddr_in dest;
    memset(&dest,0,sizeof(dest));
    dest.sin_family = AF_INET;
    dest.sin_port = htons(LTR_ETHER_IDENTIFY_PORT);
    dest.sin_addr.S_un.S_addr = INADDR_BROADCAST;

    BYTE SendBufBuf[16];
    WSABUF SendBuf;
    SendBuf.len = sizeof(SendBufBuf);
    SendBuf.buf = SendBufBuf;
    memset(SendBuf.buf,0,SendBuf.len);

    WSAOVERLAPPED SendOver;
    memset(&SendOver,0, sizeof(SendOver));
    SendOver.hEvent = WSACreateEvent();

    DWORD Timeout=1000;
    *(int *)&SendBuf.buf[0] =0xFFFF1111;
    memcpy(&SendBuf.buf[4], "LTREther", sizeof("LTREther"));

    for (int j = 0; j < NumMaxRequest; j++)
    {
        DWORD BytesSended=0;
        DWORD Flags=0;
        int SendRes=WSASendTo(UDP_Socket, &SendBuf, 1 , &BytesSended, Flags, (sockaddr*)&dest, sizeof(dest), &SendOver, NULL);
        if (SendRes==SOCKET_ERROR)
        {
            if (WSAGetLastError()==WSA_IO_PENDING)
            {
                if(WSAWaitForMultipleEvents(1, &SendOver.hEvent, TRUE, Timeout, FALSE)==WSA_WAIT_EVENT_0)
                {
                    WSAGetOverlappedResult(UDP_Socket, &SendOver, &BytesSended, FALSE, &Flags);
                }else break;
            }else break;
        }
    }

    if(SendOver.hEvent!=NULL) WSACloseEvent(SendOver.hEvent);
}

/*------------------------------------------------------------------------------------------------*/
void TLTRServiceCrate::ConnectCrateByIP(DWORD IP)
    {
    bool AlreadyExists = false;
    int FreeSlot = -1;

    TAddrListNode addr;
    if (!addr.SetAddr(IP)) return;

    /* ,       */
    if (WaitForSingleObject(crate_list_access, INFINITE) != WAIT_OBJECT_0)
        return;
    for (int i = 0; i < CRATE_MAX; i++)
        {
        if (!crate_list[i])
            {
            if (FreeSlot < 0)
                FreeSlot = i;
            }
        else if ((crate_list[i]->GetState() != CRATE_ST_DEAD)
                && (crate_list[i]->ip_addr == addr.dwIP))
            {
            AlreadyExists = true;
            break;
            }
        }
    /*       ,    */
    if (!AlreadyExists)
        {
        AnsiString ip_str = IPToStr(addr.dwIP);
        if (FreeSlot >= 0)
            {
            TCrate *crate = new TCrateEther(MessageQueueIn, MessageQueueDisplay, TIMEOUT_ETH_IDLE);
            if (crate)
                {
                crate->ip_addr = addr.dwIP;
                strncpy(crate->path, ip_str.c_str(), sizeof(crate->path) - 1);
                crate->path[sizeof(crate->path) - 1] = '\0';
                crate->log_flags = g_Config.log_flags;
                crate_list[FreeSlot] = crate;
                /*     () */
                if (!crate->Startup(NULL, g_Config.default_ttf.c_str()))
                    {
                    logfile.AddMessage("Can't init crate " + ip_str,
                        crate->log_flags, LOG_FLAG_DEBUG_LEVEL1 | LOG_FLAG_CRATE_INIT);
                    }
                }
            }
        else /* no free slot */
            {
            logfile.AddMessage(Format("No free slot for crate %s (maximum %d reached)",
                ARRAYOFCONST((ip_str, (int)CRATE_MAX))),
                g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1 | LOG_FLAG_CRATE_INIT);
            }
        } /* !AlreadyExists */
    ReleaseMutex(crate_list_access);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TLTRServiceCrate::DisconnectCrateByIP(DWORD IP)
    {
    /*      */
    if (WaitForSingleObject(crate_list_access, INFINITE) != WAIT_OBJECT_0)
        return;
    for (int i = 0; i < CRATE_MAX; i++)
        {
        if (crate_list[i] && (crate_list[i]->GetState() != CRATE_ST_DEAD)
            && (crate_list[i]->ip_addr == IP))
            {
            crate_list[i]->SetState(CRATE_ST_DEAD);
            break;
            }
        }
    ReleaseMutex(crate_list_access);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TLTRServiceCrate::ConnectEthernetCratesByType(int IPListType)
    {
    /*   ,   AUTOCONNECT */
    TAddrList *IPList;
    TAddrListNode *addr;

    IPList = IPCrateList.ExportList(IPListType);
    if (!IPList)
        return;
    addr = IPList->GetFirstNode();
    while (addr)
        {
        if (addr->bAutoConnect)
            {
            switch (addr->iStatus)
                {
                case IP_STATUS_OFFLINE:
                    /*  --  */
                    ConnectCrateByIP(addr->dwIP);
                    break;
                case IP_STATUS_CONNECTING:
                    /*  --     */
                    DisconnectCrateByIP(addr->dwIP);
                    StopServiceForDeadCrates();
                    ConnectCrateByIP(addr->dwIP);
                    break;
                case IP_STATUS_ONLINE:
                    /*   --   */
                    break;
                }
            }
        addr = addr->next;
        } /* while (addr) */
    delete IPList;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TLTRServiceCrate::DisconnectEthernetCratesByType(int IPListType)
    {
    TAddrList *IPList;
    TAddrListNode *addr;

    IPList = IPCrateList.ExportList(IPListType);
    if (!IPList)
        return;
    addr = IPList->GetFirstNode();
    while (addr)
        {
        switch (addr->iStatus)
            {
            case IP_STATUS_CONNECTING:
            case IP_STATUS_ONLINE:
                /*   */
                DisconnectCrateByIP(addr->dwIP);
                break;
            }
        addr = addr->next;
        } /* while (addr) */
    delete IPList;
    }
/*------------------------------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
void __fastcall TLTRServiceCrate::SuspendCrates()
{
    logfile.AddMessage("LTRServiceCrate going to suspend.", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    if(WaitForSingleObject(crate_list_access, INFINITE)==WAIT_OBJECT_0)
    {
        for(int i=0; i<CRATE_MAX; i++)
        {
            if(crate_list[i]!=NULL) crate_list[i]->SuspendCrate();
        }
        ReleaseMutex(crate_list_access);
    }
}
//-----------------------------------------------------------------------------
void __fastcall TLTRServiceCrate::ResumeCrates()
{
    logfile.AddMessage("LTRServiceCrate resuming from suspend.", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    if(WaitForSingleObject(crate_list_access, INFINITE)==WAIT_OBJECT_0)
    {
        for(int i=0; i<CRATE_MAX; i++)
        {
            if(crate_list[i]!=NULL) crate_list[i]->ResumeCrate();
        }
        ReleaseMutex(crate_list_access);
    }
}

//-----------------------------------------------------------------------------
void __fastcall TLTRServiceCrate::Execute()
    {
    logfile.AddMessage("LTRServiceCrate thread started.", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);

    //  
    MessageQueueIn->PostMessage(LTR_SRV_device_changed);
    MessageQueueIn->PostMessage(LTR_SRV_server_priority_changed);
    MessageQueueIn->PostMessage(LTR_SRV_autoconnect_ip);
    MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
    MessageQueueDisplay->PostMessage(LTR_DISP_ethernet_list_updated);

    //   
    DWORD RecvBytes=0, Flag=0;
    T_Message CurrMessage;
    SetEvent(init_complete);
    while (!Terminated)
        {
        if(MessageQueueIn->PopMessage(&CurrMessage, TIMEOUT_2))
            {
#ifdef DEBUG_MESSAGES
            logfile.AddMessage(Format("Message=%d", ARRAYOFCONST((CurrMessage))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
#endif
            switch (CurrMessage.dwMessage)
                {
                case LTR_SRV_device_changed:
                    ScanUSBCrates();
                    break;
                case LTR_SRV_rescan_ethernet_crates:
                    StartScanEthernetCrates();
                    break;
                case LTR_SRV_autoconnect_ip:
                    ConnectEthernetCratesByType(IPLIST_STATIC);
                    //ConnectEthernetCratesByType(IPLIST_DYNAMIC);
                    break;
                case LTR_SRV_disconnect_all_ip:
                    DisconnectEthernetCratesByType(IPLIST_STATIC);
                    DisconnectEthernetCratesByType(IPLIST_DYNAMIC);
                    break;
                case LTR_SRV_server_priority_changed:
                    if(!SetPriorityClass(GetCurrentProcess(), g_Config.ServerPriority))
                        logfile.AddMessage("Error set server priority", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                    break;
                case LTR_SRV_suspend:
                    this->SuspendCrates();
                    break;
                case LTR_SRV_resume_suspend:
                    this->ResumeCrates();
                    break;
                case LTR_SRV_bury_the_dead:
                    StopServiceForDeadCrates();
                    break;
                case LTR_SRV_connect_crate:
                    ConnectCrateByIP(CurrMessage.dwParam);
                    break;
                case LTR_SRV_disconnect_crate:
                    DisconnectCrateByIP(CurrMessage.dwParam);
                    break;
                default:
                    logfile.AddMessage(Format("Unknown Service Crate Incoming Message=%d", ARRAYOFCONST((CurrMessage.dwMessage))), g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
                    break;
                }
            }
#if PING_IN_SERVICE
        else /*     --    () */
            {
            /*      (  ?) */
            CheckForIdleCrates();
            }
#endif

        if(WSAGetOverlappedResult(UDP_Socket, &UDP_Recv_Over, &RecvBytes, FALSE, &Flag))
            {
            ProceedNewEthernetCrate();
            }
        }

    /* Terminated */

    /*     */
    SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);

    //     ,   
    if (WaitForSingleObject(crate_list_access, INFINITE) == WAIT_OBJECT_0)
        {
        for (int i = 0; i < CRATE_MAX; i++)
            {
            if (crate_list[i])
                {
                delete crate_list[i];
                crate_list[i] = NULL;
                }
            }
        ReleaseMutex(crate_list_access);
        }
    else
        {
        logfile.AddMessage("Can't access crate list.", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
        }
    logfile.AddMessage("LTRServiceCrate thread stopped.", g_Config.log_flags, LOG_FLAG_DEBUG_LEVEL1|LOG_FLAG_CRATE_INIT);
    SetEvent(init_complete);
    }
//---------------------------------------------------------------------------

