#include <windows.h>
#include <winbase.h>
#include "MessageQueue.h"


T_MessageQueue::T_MessageQueue(DWORD NumMessages, DWORD MessageAddCount)
{
    MaxMessages = NumMessages;
    MessagePostPointer =
    MessagePopPointer =
    MessagePostCounter =
    MessagePopCounter =0;
    Initialized = false;

    if(MessageAddCount!=0) MessageNumberAdd = MessageAddCount;
    else MessageNumberAdd = 100;

    MessagePostEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if(MessagePostEvent!=NULL)
    {
        MessagePostObject = CreateMutex(NULL,FALSE,NULL);
        if(MessagePostObject!=NULL)
        {
            MessagePopObject = CreateMutex(NULL,FALSE,NULL);
            if(MessagePopObject!=NULL)
            {
                Messages = new T_Message[NumMessages];
                if(Messages!=NULL)
                {
                    Initialized = true;
                }
            }
        }
    }
}

T_MessageQueue::~T_MessageQueue(void)
{
    if (MessagePopObject!=NULL) {
        WaitForSingleObject(MessagePopObject, INFINITE);
        CloseHandle(MessagePopObject);
    }

    if (MessagePostObject!=NULL) {
        WaitForSingleObject(MessagePostObject, INFINITE);
        CloseHandle(MessagePostObject);
    }

    if (MessagePostEvent!=NULL) CloseHandle(MessagePostEvent);
    if(Messages!=NULL) delete Messages;
}

DWORD __fastcall T_MessageQueue::GetMessageCounter()
{             
   return (DWORD)(MessagePostCounter - MessagePopCounter);
}

void __fastcall T_MessageQueue::PostMessage(const T_Message &Mes)
{
    if(!Initialized) return;
    if(WaitForSingleObject(MessagePostObject, INFINITE)==WAIT_OBJECT_0)
    {
        MessagePostCounter++;
        if ((MessagePostCounter - MessagePopCounter)>=MaxMessages)
        {
            if(WaitForSingleObject(MessagePopObject, INFINITE)==WAIT_OBJECT_0)
            {
                T_Message * NewMessages = new T_Message[MaxMessages+MessageNumberAdd];
                DWORD MesNumber =(DWORD) (MessagePostCounter - MessagePopCounter);
                for(DWORD i=0;i<MesNumber;i++)
                {
                    NewMessages[i] =  Messages[MessagePopPointer];
                    if(++MessagePopPointer>=MaxMessages) MessagePopPointer=0;
                }
                MaxMessages = MaxMessages+MessageNumberAdd;
                MessagePostPointer=MesNumber;
                MessagePopPointer=0;

                delete Messages;
                Messages = NewMessages;

                ReleaseMutex(MessagePopObject);
            }
        }

        memcpy(&Messages[MessagePostPointer], &Mes, sizeof(T_Message));
        //Messages[MessagePostPointer] =  Mes;
        if(++MessagePostPointer>=MaxMessages) MessagePostPointer=0;

        SetEvent(MessagePostEvent);
        ReleaseMutex(MessagePostObject);
    }
}

void __fastcall T_MessageQueue::PostMessage(DWORD dwNewMessage, DWORD dwNewParam)
    {
    T_Message msg;
    msg.dwMessage = dwNewMessage;
    msg.dwParam = dwNewParam;
    PostMessage(msg);
    }

BOOL __fastcall T_MessageQueue::PopMessage(T_Message * Mess)
{
    if(!Initialized || Mess==NULL) return false;

    bool Result = false;

    if(WaitForSingleObject(MessagePopObject, INFINITE)==WAIT_OBJECT_0)
    {
        if(GetMessageCounter())
        {
            memcpy(Mess, &Messages[MessagePopPointer], sizeof(T_Message));
            Result = true;
            if(++MessagePopPointer>=MaxMessages) MessagePopPointer=0;
            MessagePopCounter++;
        }

        ReleaseMutex(MessagePopObject);
    }
    return Result;
}

BOOL __fastcall T_MessageQueue::PopMessage(T_Message * Mess, DWORD Timeout_ms)
{
    if(!Initialized || Mess==NULL) return false;

    bool Result = false;

    if(WaitForMessage(Timeout_ms) == WAIT_OBJECT_0)
    {
        if(WaitForSingleObject(MessagePopObject, INFINITE)==WAIT_OBJECT_0)
        {
            if(GetMessageCounter())
            {
                memcpy(Mess, &Messages[MessagePopPointer], sizeof(T_Message));
                Result = true;
                if(++MessagePopPointer>=MaxMessages) MessagePopPointer=0;
                MessagePopCounter++;
            }

            ReleaseMutex(MessagePopObject);
        }
    }
    return Result;
}


DWORD __fastcall T_MessageQueue::WaitForMessage(DWORD Timeout_ms)
{
    if(!Initialized) return WAIT_ABANDONED;
    if(GetMessageCounter()) return WAIT_OBJECT_0;
    return WaitForSingleObject(MessagePostEvent, Timeout_ms);
}

DWORD __fastcall T_MessageQueue::WaitForNewMessage(DWORD Timeout_ms)
{
    if(!Initialized) return WAIT_ABANDONED;
    return WaitForSingleObject(MessagePostEvent, Timeout_ms);
}

/*
DWORD __fastcall T_MessageQueue::WaitForMessage(T_Message * Mess, DWORD Timeout_ms)
{
    if(!Initialized) return WAIT_ABANDONED;
    BOOL MessageFind = false;
    while(!MessageFind)
    {
        DWORD WaitResult = WaitForMessage(Timeout_ms);
        if(WaitResult == WAIT_ABANDONED)break;
        if(WaitResult == WAIT_OBJECT_0)
        {
            if(WaitForNewMessage(INFINITE)==WAIT_OBJECT_0)
            {
                if(GetMessageCounter())
                {
                    DWORD MesNumber = (DWORD) (MessagePostCounter - MessagePopCounter);
                    DWORD MesPointer = MessagePopPointer;
                    for (DWORD i=0; i <MesNumber;i++) {
                        if(Messages[MesPointer].dwMessage == Mess->dwMessage)
                        {
                            Mess->dwParam = Messages[MesPointer].dwParam;
                            MessageFind = true;
                            break;
                        }
                        if(++MesPointer>=MaxMessages) MesPointer=0;
                    }
                }

                ReleaseMutex(MessagePopObject);
            }
            else Sleep(1);
        }
    }

    return WAIT_ABANDONED;
}
*/

