#include "common.h"
#include "LtrClient.h"
#include "unit1.h"
#include <shellapi.h>
//---------------------------------------------------------------------------
/*   */
HANDLE crate_list_access=NULL;
HANDLE client_list_access=NULL;
TCrate  *crate_list[CRATE_MAX];
TClient *client_list[CLIENT_MAX];
TUniqueAddrMultiList IPCrateList;
TLog logfile;
TLtrIniFile g_INI;
struct t_GlobalConfig g_Config;

//---------------------------------------------------------------------------

/*------------------------------------------------------------------------------------------------*/
DWORD StrToIP(AnsiString ip_str)
    {
    int len, n;
    unsigned int q1, q2, q3, q4;

    len = ip_str.Length();
    if ((len < 7) || (len > 15))
        return 0;
    if ((sscanf(ip_str.c_str(), "%3u.%3u.%3u.%3u%n", &q1, &q2, &q3, &q4, &n) >= 4)
        && (n == len)
        && ((q1 | q2 | q3 | q4) <= 255) /*   <= 255 */
        && ((q1 | q2 | q3 | q4) > 0)    /*     0 */
        && ((q1 & q2 & q3 & q4) < 255)  /*     255 */
        )
        {
        return ((DWORD)q4 << 24) | ((DWORD)q3 << 16) | (q2 << 8) | q1;
        }
    else
        {
        return 0;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
AnsiString IPToStr(DWORD ip)
    {
    return Format("%u.%u.%u.%u",
        ARRAYOFCONST((ip & 0xFF, (ip >> 8) & 0xFF, (ip >> 16) & 0xFF, (ip >> 24) & 0xFF)));
    }
/*------------------------------------------------------------------------------------------------*/

//---------------------------------------------------------------------------
TClient::TClient(SOCKET _socket, TCrate *_crate, WORD _crate_channel, WORD _client_flags)
{
    socket=_socket;
    crate=_crate;
    crate_channel=_crate_channel;
    buf_ptr_FIFO_recv=0;
    count_FIFO_recv=0;
    control_thread=recv_buf_thread=send_buf_thread=NULL;
    IsAlive=true;
    flags=_client_flags;

    if(flags&CC_RAW_DATA_FLAG)
    {
        count_FIFO_recv=crate->module[MODULE_MAX].count_FIFO_recv;
        buf_ptr_FIFO_recv = crate->module[MODULE_MAX].recv_buf_ptr;
        crate->ReadRawData=true;
    }

	if (_crate_channel==CC_CONTROL)
	{
		control_thread=new TControlClientThread(this, false);
	}
	else
	{
		recv_buf_thread=new TReadClientThread(this, false);
		send_buf_thread=new TWriteClientThread(this, false);
	}
}
//---------------------------------------------------------------------------
TClient::~TClient(void)
{
	IsAlive=false;

	if (socket!=SOCKET_ERROR)
	{
		shutdown(socket, SD_BOTH);
		closesocket(socket);
		socket=SOCKET_ERROR;
	}

	if(control_thread!=NULL)
	{
		control_thread->Terminate();
		control_thread->WaitFor();
		delete control_thread;
	}
	if(recv_buf_thread!=NULL)
	{
		recv_buf_thread->Terminate();
		recv_buf_thread->WaitFor();
		delete recv_buf_thread;
	}
	if(send_buf_thread!=NULL)
	{
		send_buf_thread->Terminate();
		send_buf_thread->WaitFor();
		delete send_buf_thread;
	}
}
//---------------------------------------------------------------------------
bool TClient::CheckSocket(void)
{
	fd_set fdset={1, {socket}};
	timeval tval={0, 0};
	int res=select(0, &fdset, NULL, NULL, &tval);
	return res==1 ? true : false;
}
//---------------------------------------------------------------------------
TLog::TLog(void)
{
	access=CreateMutex(NULL,FALSE,NULL);
	slist=new TStringList;
	file=NULL;
}
//---------------------------------------------------------------------------
TLog::TLog(AnsiString fname)
{
	access=CreateMutex(NULL,FALSE,NULL);
	slist=new TStringList;
	//
	file=NULL;
	Open(fname);
}
//---------------------------------------------------------------------------
TLog::~TLog(void)
{
	Close();
	delete slist;
	CloseHandle(access);
}
//---------------------------------------------------------------------------
bool TLog::Open(AnsiString fname)
{
	Close();
	file=fopen(fname.c_str(), "wt+");
	return file==NULL ? false : true;
}
//---------------------------------------------------------------------------
bool TLog::Close(void)
{
	if(WaitForSingleObject(access, INFINITE)==WAIT_OBJECT_0)
    {
	    if(file!=NULL) { fclose(file); file=NULL; }
    	ReleaseMutex(access);
    }
	return true;
}
//---------------------------------------------------------------------------
bool TLog::AddMessage(AnsiString msg, int log_flag, int log_mask)
{
	log_mask&=~LOG_FLAG_SCREEN_FILE;
	if((log_flag&log_mask)==log_mask)
	{
		if(WaitForSingleObject(access, 5000)==WAIT_OBJECT_0)
		{
			SYSTEMTIME st;
			GetLocalTime(&st);
			msg=Format("[%0.2d:%0.2d:%0.2d] ", ARRAYOFCONST((st.wHour, st.wMinute, st.wSecond)))+msg;
			if(log_flag&LOG_FLAG_SCREEN) slist->Add(msg);
			if(log_flag&LOG_FLAG_FILE && file!=NULL)
			{
				fprintf(file, "%s\n", msg.c_str());
			}
			ReleaseMutex(access);
		} else return false;
	}
	return true;
}
//---------------------------------------------------------------------------
AnsiString TLog::GetMessage(void)
{
	AnsiString astr;
	if(WaitForSingleObject(access, 5000)==WAIT_OBJECT_0)
	{
		if(slist->Count)
		{
			astr=slist->Strings[0];
			slist->Delete(0);
		}
		ReleaseMutex(access);
	}
	return astr;
}
//---------------------------------------------------------------------------
int TLog::GetCount(void)
{
	int count=0;
	if(WaitForSingleObject(access, 5000)==WAIT_OBJECT_0)
	{
		count=slist->Count;
		ReleaseMutex(access);
	}
	return count;
}

//---------------------------------------------------------------------------

