//---------------------------------------------------------------------------

#include <windows.h>
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------

#include "channel.h"
#include "ltr\\include\\ltr010api.h"
#include <stdio.h>
LPCSTR ErrorStrings[]=
{
 "   GET_ARRAY.",
 "   PUT_ARRAY.",
 "   GET_MODULE_NAME.",
 "   GET_MODULE_DESCRIPTOR.",
 "   INIT_FPGA.",
 "   RESET_FPGA.",
 "   INIT_DMAC.",
 "   LOAD_FPGA.",
 "  .",
 "   GET_INFO_FPGA."
};
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_Init(TLTR010 *module)
{
 return LTR_Init(&module->ltr);
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_Open(TLTR010 *module, DWORD saddr, WORD sport, CHAR *csn)
{
 INT res;
 if(module!=NULL && csn!=NULL)
 {
    res=LTR010_Close(module);
    if(res==LTR_OK)
    {
        module->ltr.saddr=saddr;
        module->ltr.sport=sport;
        memcpy(module->ltr.csn, csn, SERIAL_NUMBER_SIZE);
        module->ltr.cc=CC_CONTROL;
        res=LTR_Open(&module->ltr);
    }
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_Close(TLTR010 *module)
{
 return LTR_Close(&module->ltr);
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_GetArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK) {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[4]={CONTROL_COMMAND_START, CONTROL_COMMAND_GET_ARRAY};
    sbuf[2]=address;
    sbuf[3]=size;
    INT ssz=channel->Send((BYTE*)sbuf, sizeof(sbuf), 1000);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), 1000);
    if(ssz!=sizeof(sbuf)) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTR010_ERROR_GET_ARRAY;
    else if(channel->Recv(buf, size, 1000)!=(int)size) res=LTR_ERROR_RECV;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_PutArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK)
 {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[4]={CONTROL_COMMAND_START, CONTROL_COMMAND_PUT_ARRAY};
    sbuf[2]=address;
    sbuf[3]=size;
    INT ssz1=channel->Send((BYTE*)sbuf, sizeof(sbuf), 1000);
    INT ssz2=channel->Send(buf, size, 1000);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), 1000);
    if(ssz1!=sizeof(sbuf) || ssz2!=(int)size) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTR010_ERROR_PUT_ARRAY;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_GetDescription(TLTR010 *module, TDESCRIPTION_LTR010 *description)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK)
 {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[2]={CONTROL_COMMAND_START, CONTROL_COMMAND_GET_DESCRIPTION};
    INT ssz=channel->Send((BYTE*) sbuf, sizeof(sbuf), 1000);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), 1000);
    if(ssz!=sizeof(sbuf)) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTR010_ERROR_GET_MODULE_GESCR;
    else if(channel->Recv((BYTE*)description, sizeof(TDESCRIPTION_LTR010), 1000)!=sizeof(TDESCRIPTION_LTR010)) res=LTR_ERROR_RECV;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTR010_LoadFPGA(TLTR010 *module, CHAR *fname, BYTE rdma, BYTE wdma)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK)
 {
    FILE *f=fopen(fname, "rt");
    if(f!=NULL)
    {
        DWORD sz=0;
        BYTE data[EP1K50_SIZE+2];
        memset(data, 0, sizeof(data));
        data[sz++]=rdma;
        data[sz++]=wdma;
        while(!feof(f) && sz<sizeof(data))
        {
            int d;
            fscanf(f, "%3d,", &d);
            data[sz++]=d;
        }
        fclose(f);
        //
        TChannel *channel=(TChannel*)module->ltr.internal;
        DWORD ack;
        DWORD sbuf[2]={CONTROL_COMMAND_START, CONTROL_COMMAND_LOAD_FPGA};
        INT ssz1=channel->Send((BYTE*)sbuf, sizeof(sbuf), 3000);
        INT ssz2=channel->Send((BYTE*)data, sizeof(data), 3000);
        INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), 5000);
        if(ssz1!=sizeof(sbuf) || ssz2!=sizeof(data)) res=LTR_ERROR_SEND;
        else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
        else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTR_ERROR_LOAD_FPGA;
    } else res=LTR010_ERROR_OPEN_FILE;
 }
 return res;
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
LPCSTR WINAPI __export LTR010_GetErrorString(INT error)
{
 if(error>=-109 && error<=-100) return ErrorStrings[-error-100];
 else return LTR_GetErrorString(error);
}
//-----------------------------------------------------------------------------
#pragma argsused
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fwdreason, LPVOID lpvReserved)
{
 return 1;
}

