#include <stdio.h>
#include <algorithm>
#include "\LTR\include\ltrapi_ll.h"

#define MAX_CONTROL_TRANSFER_SIZE						4096


TLUSB_BASE::TLUSB_BASE()
{	
	ControlPipeInUse = false;
	WriteBulkInUse=false;
	ReadBulkInUse=false;

	hDevice = INVALID_HANDLE_VALUE;
	LastErrorNumber=0;
	memset(&ControlOverlapped, 0, sizeof(OVERLAPPED));
	memset(&ReadBulkOverlapped, 0, sizeof(OVERLAPPED));
	memset(&WriteBulkOverlapped, 0, sizeof(OVERLAPPED));
	ControlOverlapped.hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
	ReadBulkOverlapped.hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
	WriteBulkOverlapped.hEvent=CreateEvent(NULL, FALSE, FALSE, NULL);
}

TLUSB_BASE::~TLUSB_BASE()
{
    CancelIo();

	CloseLDevice();

	if (ControlOverlapped.hEvent!=NULL)
	{
		::CloseHandle(ControlOverlapped.hEvent);
		ControlOverlapped.hEvent=NULL;
	}

	if (ReadBulkOverlapped.hEvent!=NULL)
	{
		::CloseHandle(ReadBulkOverlapped.hEvent);
		ReadBulkOverlapped.hEvent=NULL;
	}

	if (WriteBulkOverlapped.hEvent!=NULL)
	{
		CloseHandle(WriteBulkOverlapped.hEvent);
		WriteBulkOverlapped.hEvent=NULL;
	}
}

HANDLE __fastcall TLUSB_BASE::GetModuleHandle()
{
	return hDevice;
}

bool __fastcall TLUSB_BASE::OpenLDevice(BYTE VirtualSlot)
{
	LastErrorNumber=LUSB_OK;

	CloseLDevice();

	if (LastErrorNumber==LUSB_OK)
	{
		char DeviceName[18];
		memset (&DeviceName,0,sizeof(DeviceName));
		//    USB

		sprintf(DeviceName, "\\\\.\\LDevUsb%d", VirtualSlot);

		OpenLDeviceByName(DeviceName);
	}

	//
	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::OpenLDeviceByName(LPTSTR DeviceName)
{
	LastErrorNumber=LUSB_OK;

	if(DeviceName!=NULL && DeviceName!="")
	{
		CloseLDevice();
		//    
		hDevice=CreateFile(DeviceName, GENERIC_READ|GENERIC_WRITE, 0, 
			NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
		if(hDevice==INVALID_HANDLE_VALUE) LastErrorNumber=LUSB_ERROR_OPEN_DEVICE;

	} else LastErrorNumber=LUSB_ERROR_INVALID_PARAMETERS;

	//
	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::CloseLDevice()
{
	LastErrorNumber=LUSB_OK;

	if(hDevice!=INVALID_HANDLE_VALUE) 
	{
		if(!CloseHandle(hDevice)) LastErrorNumber=LUSB_ERROR_CLOSE_DEVICE;
		else hDevice=INVALID_HANDLE_VALUE;
	}	

	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::CancelIo()
{
	LastErrorNumber=LUSB_OK;
	if(hDevice!=INVALID_HANDLE_VALUE) 
	{		
		if (!::CancelIo(hDevice)) LastErrorNumber=LUSB_ERROR_CANCEL_IO;		
	}else LastErrorNumber=LUSB_ERROR_DEVICE_CLOSED;
	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::ControlLDevice(DWORD ControlCode, _ControlRequest * CurrentRequest, BYTE DataBuffer[], DWORD DataBufferLength, DWORD Timeout)
{
	while(ControlPipeInUse) Sleep(10);
	ControlPipeInUse=true;

	LastErrorNumber=LUSB_OK;
	DWORD BytesReturned = 0;
	ControlCode = 0x220012; // 
	if(hDevice!=INVALID_HANDLE_VALUE) 
	{
		if (!DeviceIoControl(hDevice, ControlCode, CurrentRequest, sizeof(_ControlRequest),
			DataBuffer, DataBufferLength, &BytesReturned, &ControlOverlapped))
		{
			if (GetLastError() == ERROR_IO_PENDING)
			{
				if (WaitForSingleObject(ControlOverlapped.hEvent,Timeout)!=WAIT_OBJECT_0)
				{
					::CancelIo(hDevice);
					LastErrorNumber=LUSB_ERROR_TIMEOUT_EXPIRED;
				}                    

				if (!GetOverlappedResult(hDevice, &ControlOverlapped, &BytesReturned, false))
				{
					LastErrorNumber=LUSB_ERROR_ASYNC_ERROR;
				}                                        
			}
			else
			{
				LastErrorNumber=LUSB_ERROR_ASYNC_DATA_TRNSFER;
			}
		}

		if (BytesReturned != DataBufferLength)
		{
			LastErrorNumber=LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH;
		}
	}else LastErrorNumber=LUSB_ERROR_DEVICE_CLOSED;

	ControlPipeInUse=false;

	return LastErrorNumber!=LUSB_OK ? false : true;
}  

bool __fastcall TLUSB_BASE::CommandControl(_ControlRequest *CurrentRequest)
{
	DWORD ControlCode = 0x220012; // 
	BYTE DataBuffer[1];
	return ControlLDevice(ControlCode,CurrentRequest,DataBuffer,0,1000);
}

bool __fastcall TLUSB_BASE::WriteBulk(BYTE DataBuffer[], DWORD DataBufferLength, DWORD * BytesWritten, DWORD Timeout)
{
	while (WriteBulkInUse) Sleep(10);
	WriteBulkInUse=true;

	LastErrorNumber=LUSB_OK;
	*BytesWritten = 0;
	if(hDevice!=INVALID_HANDLE_VALUE) 
	{

		if (!WriteFile(hDevice, DataBuffer, DataBufferLength, BytesWritten, &WriteBulkOverlapped))
		{
			if (GetLastError() == ERROR_IO_PENDING)
			{
				if (WaitForSingleObject(WriteBulkOverlapped.hEvent,Timeout)!=WAIT_OBJECT_0)
				{
					::CancelIo(hDevice);
					LastErrorNumber=LUSB_ERROR_TIMEOUT_EXPIRED;
				}                    

				if (!GetOverlappedResult(hDevice, &WriteBulkOverlapped, BytesWritten, false))
				{
					LastErrorNumber=LUSB_ERROR_ASYNC_ERROR;
				}
			}else LastErrorNumber=LUSB_ERROR_ASYNC_DATA_TRNSFER;                          
		}            

		if (*BytesWritten != DataBufferLength) LastErrorNumber=LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH;
	}else LastErrorNumber=LUSB_ERROR_DEVICE_CLOSED;

	WriteBulkInUse=false;

	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::ReadBulk(BYTE DataBuffer[], DWORD DataBufferLength, DWORD * BytesRead, DWORD Timeout)
{
	while (ReadBulkInUse)Sleep(10);
	ReadBulkInUse=true;

	LastErrorNumber=LUSB_OK;
	*BytesRead=0;		

	if(hDevice!=INVALID_HANDLE_VALUE) 
	{
		if (!ReadFile(hDevice, DataBuffer, DataBufferLength, BytesRead, &ReadBulkOverlapped))
		{			
			if (GetLastError() == ERROR_IO_PENDING)
			{
				if (WaitForSingleObject(ReadBulkOverlapped.hEvent,Timeout)!=WAIT_OBJECT_0)
				{
					::CancelIo(hDevice);
					LastErrorNumber=LUSB_ERROR_TIMEOUT_EXPIRED;
				}                    

				if (!GetOverlappedResult(hDevice, &ReadBulkOverlapped, BytesRead, false))
				{
					LastErrorNumber=LUSB_ERROR_ASYNC_ERROR;
				}
			} else LastErrorNumber=LUSB_ERROR_ASYNC_DATA_TRNSFER;                            
		}            

		if (*BytesRead != DataBufferLength) LastErrorNumber=LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH;
		
	}else LastErrorNumber=LUSB_ERROR_DEVICE_CLOSED;

	ReadBulkInUse=false;

	return LastErrorNumber!=LUSB_OK ? false : true;
}


bool __fastcall TLUSB_BASE::GetLastErrorString(LPTSTR lpBuffer, DWORD nSize)
{
	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::GetModuleName(char mname[], DWORD Length)
{
	if(mname!=NULL) 
	{
		if(hDevice!=INVALID_HANDLE_VALUE) 
		{
	
			_ControlRequest Request;
			Request.RequestDirectionIN=true;
			Request.bRequest=11;
			Request.wValue=0;
			Request.wIndex=0;

			memset(mname,0,Length);			
			ControlLDevice(0,&Request,(BYTE *) mname, Length, 200);		
		}else LastErrorNumber=LUSB_ERROR_DEVICE_CLOSED;
	} else LastErrorNumber=LUSB_ERROR_INVALID_PARAMETERS;
	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::ReadControl(_ControlRequest *CurrentRequest, BYTE DataBuffer[], DWORD DataBufferLength, DWORD Timeout)
{
	LastErrorNumber=LUSB_OK;

	if (DataBuffer==NULL||DataBufferLength==NULL) LastErrorNumber=LUSB_ERROR_INVALID_PARAMETERS;

	CurrentRequest->RequestDirectionIN=true;

	DWORD SizeSended=0;

	while (LastErrorNumber==LUSB_OK&&SizeSended<DataBufferLength)
	{
		DWORD Size=std::min<int>(DataBufferLength,MAX_CONTROL_TRANSFER_SIZE);

		if (ControlLDevice(0,CurrentRequest,DataBuffer+SizeSended, Size, Timeout)) SizeSended+=Size;		
	}

	return LastErrorNumber!=LUSB_OK ? false : true;
}

bool __fastcall TLUSB_BASE::WriteControl(_ControlRequest *CurrentRequest, BYTE DataBuffer[], DWORD DataBufferLength, DWORD Timeout)
{
	LastErrorNumber=LUSB_OK;

	if (DataBuffer==NULL||DataBufferLength==NULL) LastErrorNumber=LUSB_ERROR_INVALID_PARAMETERS;

	CurrentRequest->RequestDirectionIN=false;

	DWORD SizeSended=0;

	while (LastErrorNumber==LUSB_OK&&SizeSended<DataBufferLength)
	{
		DWORD Size=std::min<int>(DataBufferLength,MAX_CONTROL_TRANSFER_SIZE);

		if (ControlLDevice(0,CurrentRequest,DataBuffer+SizeSended, Size, Timeout)) SizeSended+=Size;		
	}

	return LastErrorNumber!=LUSB_OK ? false : true;
}



