#include <stdio.h>
#include <algorithm>
#include "\LTR\include\ltrapi_ll.h"
#include "LIB\\DEV_STD\\USB_defs.h"
#include "LIB\\DEV_STD\\LUSB.h"

/*================================================================================================*/
#define LTR_ETHER_CTRL_SIG          0x314C5443 /* "CTL1" */

/*    Ethernet */
#define LTR_ETHER_CMD_INIT          0x80000000
#define LTR_ETHER_CMD_NOP           0x00000000
#define LTR_ETHER_CMD_GET_NAME      0x00000001
#define LTR_ETHER_CMD_GET_ARRAY     0x00000002
#define LTR_ETHER_CMD_PUT_ARRAY     0x00000003
//#define LTR_ETHER_CMD_       0x00

/*   (,      winsock) */
#define LTR_EE_OK               0x00
#define LTR_EE_INVALID_PARAM    0x01
#define LTR_EE_CONNECT          0x02
#define LTR_EE_ABORT            0x03
#define LTR_EE_TIMEOUT          0x04
#define LTR_EE_CONN_LOST        0x05
#define LTR_EE_READ             0x06
#define LTR_EE_WRITE            0x07
#define LTR_EE_BAD_SIG          0x08
#define LTR_EE_BAD_REPLYLEN     0x09
#define LTR_EE_CMD_FAILED       0x0A
#define LTR_EE_INTERNAL         0xFF

#define MAX_BUF_CHUNKS          2

#define MK_ERR(type, code)          (((int)(type) << 24) | ((int)(code) & 0xFFFFFF))
/*================================================================================================*/

/*================================================================================================*/
#pragma pack(1)
typedef struct
    {
    BYTE Name[16];
    BYTE SerialNumber[16];
    BYTE CpuType[16];
    BYTE Revision;
    LONG QuartzFrequency;
    BYTE Unused[128-53];
    }
    t_RawDevDescr;

typedef struct              /*     TCP */
    {
    DWORD Signature;        /* == LTR_ETHER_CTRL_SIG */
    DWORD CmdCode;          /*   */
    DWORD CmdParam;         /*   */
    DWORD VarDataLen;       /*     */
    DWORD ExpectReplyLen;   /*     */
    }
    t_EtherControlRequest;

typedef struct              /*      TCP */
    {
    DWORD Signature;        /* == LTR_ETHER_CTRL_SIG */
    DWORD ResultCode;       /*   */
    DWORD VarDataLen;       /*     */
    }
    t_EtherControlReply;
#pragma pack()

/*   "   "    WSASend/WSARecv
 *      ,   (ptr, len),
 *        .
 */
class TMultiBuf
    {
private:
    DWORD head;             /*       */
    DWORD count;            /*    */
    DWORD offset;           /*        */
    WSABUF chunk[MAX_BUF_CHUNKS]; /*   ( count ) */
public:
    TMultiBuf()             { head = count = offset = 0; }
    void Clear()            { head = count = offset = 0; }
        /* -   */
    DWORD ChunksLeft(void)  { return (count - head); }
        /* -   ,    (0 =  ) */
    bool AddChunk(LPVOID chunk_ptr, DWORD chunk_len);
        /* -         */
    LPWSABUF Head(void)     { return &chunk[head]; }
        /* -        ( 
               WSARecv()  WSASend()   ) */
    DWORD ShiftBy(DWORD bytes);
        /* -   " "   .
             ,     ,  ( head);
              ,  ,       .
                      (offset). */
    };
/*================================================================================================*/


/*      ""  */
/*------------------------------------------------------------------------------------------------*/
bool TMultiBuf::AddChunk(LPVOID chunk_ptr, DWORD chunk_len)
    {
    if ((chunk_len == 0) || !chunk_ptr) return false;
    if (count >= MAX_BUF_CHUNKS) return false;
    chunk[count].len = chunk_len;
    chunk[count].buf = (char*)chunk_ptr;
    count++;
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
DWORD TMultiBuf::ShiftBy(DWORD bytes)
    {
    while ((head < count) && (bytes >= chunk[head].len))
        { /*  ,   */
        bytes -= chunk[head].len;
        offset += chunk[head].len;
        chunk[head++].len = 0;
        }
    if ((head < count) && (bytes > 0))
        { /*      */
        chunk[head].len -= bytes;
        chunk[head].buf += bytes;
        offset += bytes;
        }
    return offset;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TLTREther::TLTREther(WSAEVENT AbortEvent, DWORD IdleTimeout)
    {
    DefaultTimeout = LTR_ETHER_DEFAULT_IO_TIMEOUT;
    LastErrorCode = 0;
    ip_addr = INADDR_NONE;
    if (IdleTimeout > 0)
        {
        hIdleTimer = CreateWaitableTimer(NULL, TRUE, NULL);
        IdleInterval.QuadPart = -10000LL * IdleTimeout; /* negative = relative, 100ns resolution */
        }
    else
        {
        hIdleTimer = NULL;
        }
    hAbortEvent = AbortEvent;
    InitializeCriticalSectionAndSpinCount(&csCtl, 0x80000000);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TLTREther::~TLTREther()
    {
    DeleteCriticalSection(&csCtl);
    if (hIdleTimer)
        CloseHandle(hIdleTimer);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void WINAPI TLTREther::StopIdleTimer(void)
    {
    if (hIdleTimer)
        CancelWaitableTimer(hIdleTimer);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void WINAPI TLTREther::RestartIdleTimer(void)
    {
    if (hIdleTimer)
        SetWaitableTimer(hIdleTimer, &IdleInterval, 0, NULL, NULL, FALSE);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
HANDLE WINAPI TLTREther::GetIdleTimerHandle(void)
    {
    return hIdleTimer;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int WINAPI TLTREther::ConnectDevicePort(SOCKET sock, int port)
    {
    sockaddr_in dest;

    memset(&dest, 0, sizeof(dest));
    dest.sin_family = AF_INET;
    dest.sin_port = htons(port);
    dest.sin_addr.s_addr = ip_addr;

    return connect(sock, (sockaddr*)&dest, sizeof(dest));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::OpenDeviceByIP(DWORD ip)
    {
    bool ok;
    SOCKET sock;
    WORD fpgainfo_raw[255];
    DWORD num_recv;

    if ((ip == INADDR_ANY) || (ip == INADDR_NONE))
        return false;
    ip_addr = ip;
    memset(fpga_info, 0, sizeof(fpga_info));

    ok = ExecCommand(LTR_ETHER_CMD_INIT, 0,
                    NULL, 0, (char*)fpgainfo_raw, sizeof(fpgainfo_raw),
                    NULL, &num_recv, LTR_ETHER_DEFAULT_CONN_TIMEOUT);
    if (ok && (num_recv >= FPGA_INFO_SIZE * sizeof(WORD)))
        {
        for (int i = 0; i < FPGA_INFO_SIZE; i++)
            {
            if ((fpgainfo_raw[i] >> 8) != 0xE0)
                {
                ok = false;
                break;
                }
            fpga_info[i] = fpgainfo_raw[i] & 0xFF;
            }
        }
    return ok;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::Ping(DWORD Timeout)
    {
    return ExecCommand(LTR_ETHER_CMD_NOP, 0,
                NULL, 0, NULL, 0,
                NULL, NULL, Timeout, Timeout);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::GetModuleDescription(TDESCRIPTION_LTR010 *description)
    {
    BYTE info1[16], info2[16];
    t_RawDevDescr ddescr;

    if (!description)
        return false;
    memset(description, 0, sizeof(TDESCRIPTION_LTR010));

    if (!(GetArray(LTR010_MODULE_DESCRIPTOR_ADDRESS, (BYTE*)&ddescr, sizeof(ddescr))
        && GetArray(LTR010_BOOT_LOADER_VERSION_ADDRESS, info1, 16)
        && GetArray(LTR010_FIRMWARE_VERSION_ADDRESS, info2, 16)))
        return false;

    // module description
    TDESCRIPTION_MODULE &mdescr = description->Module;
    strncpy(mdescr.CompanyName,  "L-CARD Inc.",       sizeof(mdescr.CompanyName));
    strncpy(mdescr.DeviceName,   ddescr.Name,         std::min(sizeof(mdescr.DeviceName), sizeof(ddescr.Name)));
    strncpy(mdescr.SerialNumber, ddescr.SerialNumber, std::min(sizeof(mdescr.SerialNumber), sizeof(ddescr.SerialNumber)));
    strncpy(mdescr.Comment,      ddescr.Unused,       std::min(sizeof(mdescr.Comment), sizeof(ddescr.Unused)));
    //mdescr.Revision=ddescr.DeviceRevision;
    // cpu description
    TDESCRIPTION_CPU &cdescr = description->Cpu;
    cdescr.Active = 1;
    strncpy(cdescr.Name, ddescr.CpuType, std::min(sizeof(cdescr.Name), sizeof(ddescr.CpuType)));
    cdescr.ClockRate = ddescr.QuartzFrequency;
    cdescr.FirmwareVersion = *(DWORD*)info2;
    sprintf(cdescr.Comment, "Bootloader v%d.%d.%d.%d", (int)info1[3], (int)info1[2], (int)info1[1], (int)info1[0]);
    // interface description
    TDESCRIPTION_INTERFACE &idescr = description->Interface;
    idescr.Active = 1;
    strcpy(idescr.Name, "Ethernet");
    strcpy(idescr.Comment, "TCP/IP Protocol");

    // fpga description
    if (fpga_info[0])
        {
        TDESCRIPTION_FPGA &fdescr=description->Fpga;
        fdescr.Active=1;
        fdescr.ClockRate=60e6;
        // Name
        char *ptag, *p1, *p2;
        ptag=strstr((char*)fpga_info, "<Name>");
        if(ptag && (p1=strchr(ptag, '{'))!=0 && (p2=strchr(ptag, '}'))!=0)
            {
            strncpy(fdescr.Name, p1+1, std::min<int>(sizeof(fdescr.Name), p2-p1-1));
            }
        // Version
        ptag=strstr((char*)fpga_info, "<Version>");
        if(ptag && (p1=strchr(ptag, '{'))!=0 && (p2=strchr(ptag, '}'))!=0)
            {

            int i, j, k;
            if(sscanf(p1, "{%d.%d.%d", &i, &j, &k)==3)
                {
                fdescr.FirmwareVersion=((i&255)<<24)|((j&255)<<16)|((k&255)<<8);
                }

            }
        // Comment
        ptag=strstr((char*)fpga_info, "<Comments>");
        if(ptag && (p1=strchr(ptag, '{'))!=0 && (p2=strchr(ptag, '}'))!=0)
            {
            strncpy(fdescr.Comment, p1+1, std::min<int>(sizeof(fdescr.Comment), p2-p1-1));
            }
    }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::GetModuleName(char* ModuleName, int ModuleNameLength)
    {
    return ExecCommand(LTR_ETHER_CMD_GET_NAME, 0,
                NULL, 0,
                ModuleName, ModuleNameLength,
                NULL, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::GetArray(DWORD address, BYTE *data, DWORD size)
    {
    return ExecCommand(LTR_ETHER_CMD_GET_ARRAY, address,
                NULL, 0,
                data, size,
                NULL, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::PutArray(DWORD address, BYTE *data, DWORD size)
    {
    return ExecCommand(LTR_ETHER_CMD_PUT_ARRAY, address,
                data, size,
                NULL, 0,
                NULL, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTREther::ExecCommand(DWORD CommandCode, DWORD CommandParam,
                        const BYTE *pSendBuf, DWORD SendSize,
                        BYTE *pRecvBuf, DWORD RecvSize,
                        DWORD *pResultCode,
                        DWORD *pReceivedBytes,
                        DWORD ConnectTimeout, DWORD OperationTimeout)
    {
    t_EtherControlRequest Request;
    t_EtherControlReply Reply;
    bool ok = false;
    bool hdr_checked = false;
    int res;
    SOCKET sock = INVALID_SOCKET;
    WSAEVENT sock_evt = WSA_INVALID_EVENT;
    DWORD timeout;
    WSAEVENT event_list[3];
    WSANETWORKEVENTS events;
    TMultiBuf sendbuf;
    TMultiBuf recvbuf;

    if (ConnectTimeout == 0)
        ConnectTimeout = DefaultTimeout;
    if (OperationTimeout == 0)
        OperationTimeout = DefaultTimeout;

    if (pResultCode)
        *pResultCode = LTR_ETHER_RESULT_SYSFAIL;
    if (pReceivedBytes)
        *pReceivedBytes = 0;

    if ((!pSendBuf && (SendSize > 0)) || (!pRecvBuf && (RecvSize > 0)))
        {
        LastErrorCode = MK_ERR(LTR_EE_INVALID_PARAM, 0);
        return false;
        }

    /*    Abort,   ,      */
    if (hAbortEvent != WSA_INVALID_EVENT)
        {
        event_list[0] = hAbortEvent;
        if (WSAWaitForMultipleEvents(1, event_list, FALSE, 0 /* timeout */, FALSE) == WAIT_OBJECT_0)
            {
            LastErrorCode = MK_ERR(LTR_EE_ABORT, 0);
            return false;
            }
        }

    /*    */
    Request.Signature = LTR_ETHER_CTRL_SIG;
    Request.VarDataLen = SendSize;
    Request.CmdCode = CommandCode;
    Request.CmdParam = CommandParam;
    Request.ExpectReplyLen = RecvSize;

    /*       */
    sendbuf.AddChunk(&Request, sizeof(Request));
    sendbuf.AddChunk((void*)pSendBuf, SendSize);

    recvbuf.AddChunk(&Reply, sizeof(Reply));
    recvbuf.AddChunk((void*)pRecvBuf, RecvSize);

    timeout = ConnectTimeout;

    EnterCriticalSection(&csCtl);
    try
        {
        sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        if (sock == INVALID_SOCKET)
            throw MK_ERR(LTR_EE_INTERNAL, WSAGetLastError());

        sock_evt = WSACreateEvent();
        if (sock_evt == WSA_INVALID_EVENT)
            throw MK_ERR(LTR_EE_INTERNAL, WSAGetLastError());

        /*  -       */

        if (WSAEventSelect(sock, sock_evt,
            FD_READ | FD_WRITE | FD_CONNECT | FD_CLOSE) == SOCKET_ERROR)
            throw MK_ERR(LTR_EE_INTERNAL, WSAGetLastError());

        res = ConnectDevicePort(sock, LTR_ETHER_CTRL_PORT);
        if (res == SOCKET_ERROR)
            {
            int err = WSAGetLastError();
            if (err != WSAEWOULDBLOCK)
                throw MK_ERR(LTR_EE_CONNECT, err);
            }

        /*    */
        while (!ok)
            {
            event_list[0] = sock_evt;
            event_list[1] = hAbortEvent;
            switch (WSAWaitForMultipleEvents(1 + (hAbortEvent != WSA_INVALID_EVENT),
                        event_list, FALSE, timeout, FALSE))
                {
                case WSA_WAIT_EVENT_0: /* Socket event */
                    if (WSAEnumNetworkEvents(sock, sock_evt, &events) == SOCKET_ERROR)
                        throw WSAGetLastError();
                    if (events.lNetworkEvents & FD_CONNECT)
                        { /*   */
                        if (events.iErrorCode[FD_CONNECT_BIT])
                            throw MK_ERR(LTR_EE_CONNECT, events.iErrorCode[FD_CONNECT_BIT]);
                        /*      */
                        timeout = OperationTimeout;
                        RestartIdleTimer();
                        } /* FD_CONNECT */

                    if (events.lNetworkEvents & FD_WRITE)
                        { /*     */
                        if (events.iErrorCode[FD_WRITE_BIT])
                            throw MK_ERR(LTR_EE_WRITE, events.iErrorCode[FD_WRITE_BIT]);
                        if (sendbuf.ChunksLeft() > 0)
                            {
                            DWORD bytes_sent;
                            res = WSASend(sock, sendbuf.Head(), sendbuf.ChunksLeft(), &bytes_sent,
                                0, NULL, NULL);
                            if (res == SOCKET_ERROR)
                                {
                                int err = WSAGetLastError();
                                if (err != WSAEWOULDBLOCK)
                                    throw MK_ERR(LTR_EE_WRITE, err);
                                }
                            else
                                {
                                sendbuf.ShiftBy(bytes_sent); /*   */
                                if (sendbuf.ChunksLeft() == 0)
                                    { /*    */
                                    shutdown(sock, SD_SEND);
                                    }
                                }
                            }
                        } /* FD_WRITE */

                    if (events.lNetworkEvents & (FD_READ | FD_CLOSE))
                        {
                        if ((events.lNetworkEvents & FD_READ) && events.iErrorCode[FD_READ_BIT])
                            throw MK_ERR(LTR_EE_READ, events.iErrorCode[FD_READ_BIT]);
                        bool again = true;
                        while (again && (recvbuf.ChunksLeft() > 0))
                            {
                            again = false;
                            DWORD bytes_recv, flags = 0;
                            res = WSARecv(sock, recvbuf.Head(), recvbuf.ChunksLeft(), &bytes_recv,
                                &flags, NULL, NULL);
                            if (res == SOCKET_ERROR)
                                { /*    FD_CLOSE,      */
                                if (events.lNetworkEvents & FD_READ)
                                    {
                                    int err = WSAGetLastError();
                                    if (err != WSAEWOULDBLOCK)
                                        throw MK_ERR(LTR_EE_READ, err);
                                    }
                                }
                            else if (bytes_recv > 0)
                                {
                                DWORD offset = recvbuf.ShiftBy(bytes_recv); /*   */
                                /*    ,    */
                                if (!hdr_checked && (offset >= sizeof(Reply)))
                                    { /*    */
                                    if (Reply.Signature != LTR_ETHER_CTRL_SIG)
                                        throw MK_ERR(LTR_EE_BAD_SIG, 0);
                                    if (Reply.VarDataLen > RecvSize)
                                        throw MK_ERR(LTR_EE_BAD_REPLYLEN, 0);
                                    /*       ,
                                          */
                                    if (!pReceivedBytes && (Reply.VarDataLen != RecvSize))
                                        throw MK_ERR(LTR_EE_BAD_REPLYLEN, 0);
                                    hdr_checked = true;
                                    }
                                /*    ,   */
                                if (hdr_checked && (offset >= sizeof(Reply) + Reply.VarDataLen))
                                    {
                                    ok = true;
                                    }
                                else
                                    {
                                    again = true;
                                    }
                                }
                            } /* while recvbuf.ChunksLeft() && again */
                            if ((events.lNetworkEvents & FD_CLOSE) && !ok)
                                throw MK_ERR(LTR_EE_CONN_LOST, events.iErrorCode[FD_CLOSE_BIT]);
                        } /* FD_READ | FD_CLOSE */

                    break;
                case WSA_WAIT_EVENT_0 + 1: /* Abort signalled by another thread */
                    throw MK_ERR(LTR_EE_ABORT, 0);
                case WSA_WAIT_TIMEOUT:
                    throw MK_ERR(LTR_EE_TIMEOUT, 0);
                case WSA_WAIT_FAILED:
                    throw MK_ERR(LTR_EE_INTERNAL, WSAGetLastError());
                } /* switch (WSAWaitForMultipleEvents(...)) */
            } /* while (!ok) */

        /*  ,       */
        if (pResultCode)
            { /*      */
            *pResultCode = Reply.ResultCode;
            }
        else
            { /*     ,     */
            if (Reply.ResultCode != LTR_ETHER_RESULT_OK)
                throw MK_ERR(LTR_EE_CMD_FAILED, 0);
            }
        if (pReceivedBytes)
            *pReceivedBytes = Reply.VarDataLen;
        }
    catch (int error_code)
        {
        if (error_code)
            LastErrorCode = error_code;
        ok = false;
        }

    /*     */
    if (sock != INVALID_SOCKET)
        {
        shutdown(sock, SD_BOTH);
        closesocket(sock);
        }
    if (sock_evt != WSA_INVALID_EVENT)
        WSACloseEvent(sock_evt);
    LeaveCriticalSection(&csCtl);
    return ok;
    }
/*------------------------------------------------------------------------------------------------*/

