#ifndef __USBAPI__
#define __USBAPI__
// -----------------------------------------------------------------------------
// ---------------------------- COMMON PART ------------------------------------
// -----------------------------------------------------------------------------
#include <windows.h>
#include <winsock2.h>
#include "ltrapitypes.h"
#include "ltrapidefine.h"
//  
#define 	LC_VERMAJOR 				2    	//   
#define 	LC_VERMINOR 				2		//   
#define 	LC_CURRENT_VERSION     	((LC_VERMAJOR<<16)|LC_VERMINOR)

#define LUSB_OK											0
#define LUSB_ERROR_INVALID_PARAMETERS					-1001
#define LUSB_ERROR_OPEN_DEVICE							-1002
#define LUSB_ERROR_CLOSE_DEVICE							-1003
#define LUSB_ERROR_DEVICE_CLOSED						-1004
#define LUSB_ERROR_CANCEL_IO							-1005
#define LUSB_ERROR_TIMEOUT_EXPIRED						-1006
#define LUSB_ERROR_ASYNC_ERROR							-1007
#define LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH			-1008
#define LUSB_ERROR_ASYNC_DATA_TRNSFER					-1009

#define LTR_ETHER_DATA_PORT         (11110)
#define LTR_ETHER_IDENTIFY_PORT     (11112)
#define LTR_ETHER_CNTRL_PORT        (11113)


extern "C" DWORD WINAPI GetDllVersion(void);

typedef enum 
{
    USB_Device=1,
    Ether_Device=2	
}_CommandDeviceType;

#pragma pack(2)
typedef struct 
{
    DWORD OveralLength;
    DWORD CommandCode;
    DWORD ReturnBytes;
    _CommandDeviceType  DeviceRequested;
}_ControlRequestEther;
#pragma pack()


//-----------------------------------------------------------------------------
//------------------------------- LTR010 --------------------------------------
//-----------------------------------------------------------------------------
#pragma pack(4)
typedef struct
{
    TDESCRIPTION_MODULE     Module;
    TDESCRIPTION_CPU        Cpu;
    TDESCRIPTION_FPGA       Fpga;
    TDESCRIPTION_INTERFACE  Interface;
}TDESCRIPTION_LTR010;
#pragma pack()
//-----------------------------------------------------------------------------
class TLTR010_BOOT_LOADER
{
 public :   
    HANDLE MainDevice;
    bool WINAPI SetUSBDevice(HANDLE Device);
    // /
    TLTR010_BOOT_LOADER();
    virtual ~TLTR010_BOOT_LOADER();

    virtual bool WINAPI CallApplication(DWORD address);

    virtual bool WINAPI GetArray(DWORD address, BYTE *data, DWORD size);
    virtual bool WINAPI PutArray(DWORD address, BYTE *data, DWORD size);

    //   
	virtual bool WINAPI GetModuleDescription(TDESCRIPTION_LTR010 *description);
	virtual bool WINAPI SetModuleDescription(TDESCRIPTION_LTR010 *description);
};
//-----------------------------------------------------------------------------
class TLTR010
{
 protected :
    BYTE fpga_info[FPGA_INFO_SIZE];
    HANDLE MainDevice;

 public :
    // /
    TLTR010();
    virtual ~TLTR010();

    bool NoRealLoadFPGAGlag;

    virtual bool WINAPI SetUSBDevice(HANDLE Device);

    virtual bool WINAPI GetArray(DWORD address, BYTE *data, DWORD size);
    virtual bool WINAPI PutArray(DWORD address, BYTE *data, DWORD size);
    //   
	virtual bool WINAPI GetModuleDescription(TDESCRIPTION_LTR010 *description);
    //     
    virtual bool WINAPI ResetFPGA(void);
    virtual bool WINAPI InitFPGA(void);
    virtual bool WINAPI InitDMAC(BYTE RDMA_PARAM, BYTE WDMA_PARAM);
	virtual bool WINAPI LoadFPGA(BYTE *data, BYTE RDMA_PARAM, BYTE WDMA_PARAM);
	virtual bool WINAPI LoadFPGAFromFile(PCHAR fname, BYTE RDMA_PARAM, BYTE WDMA_PARAM);
};

class TLTRUSB
{
    private:
        HANDLE hDevice;
        HANDLE MainDeviceClass;
    public:
        bool application_active;
        TLTRUSB();
        ~TLTRUSB();
        TLTR010 * Application;
        TLTR010_BOOT_LOADER * Bootloader;

        virtual HANDLE WINAPI GetUSBDeviceHandle(void);
        virtual void WINAPI CancelIO(void);
        virtual bool WINAPI GetModuleName(char mname[], DWORD Length);
        virtual bool WINAPI OpenLDeviceByName(LPTSTR DeviceName);
        virtual bool WINAPI CloseLDevice(void);
        virtual bool WINAPI SwitchBootloader(bool Bootloader);
};

extern "C" TLTRUSB * WINAPI CreateInstanceUSB(void);
extern "C" void WINAPI DeleteInstanceUSB(TLTRUSB * NewClass);

//-----------------------------------------------------------------------------
//------------------------------- LTR030 --------------------------------------
//-----------------------------------------------------------------------------
class TLTREther
{
    private:
        SOCKET ControlDevice;
        WSAOVERLAPPED wsaovr, wsaovw, wsaovcntrl;
        volatile bool ReadSocketInUse;
        volatile bool WriteSocketInUse;
        char DevIP[16];
    public:
        SOCKET DataDevice;
        bool application_active;
        int LastError;
        TLTREther();
        ~TLTREther();

        virtual bool WINAPI GetModuleName(char * ModuleName, int ModuleNameLength);
        virtual bool WINAPI GetModuleDescription(TDESCRIPTION_LTR010 *description);
    	virtual bool WINAPI SetModuleDescription(TDESCRIPTION_LTR010 *description);
        virtual bool WINAPI SetBaudRate(WORD BaudRate);

        virtual bool WINAPI LoadFPGA(BYTE *data, bool WriteFlash);
	    virtual bool WINAPI LoadFPGAFromFile(PCHAR fname, bool WriteFlash);

        virtual bool WINAPI RecvData(SOCKET socket, BYTE *data, DWORD size, WSAOVERLAPPED &wsaover, DWORD timeout);
        virtual bool WINAPI SendData(SOCKET socket, BYTE *data, DWORD size, WSAOVERLAPPED &wsaover, DWORD timeout);
        virtual bool WINAPI Command(_ControlRequestEther * ControlRequest, BYTE *dataIn, BYTE * DataOut, DWORD Timeout);

        virtual void WINAPI CancelIO(void);

        virtual void WINAPI CloseSocket(SOCKET & socket);
        virtual bool WINAPI OpenSocket(SOCKET & socket, int port, char * IP);

        virtual bool WINAPI OpenDeviceByIP(char * IP);
        virtual void WINAPI CloseDevice(void);
        virtual bool WINAPI SwitchBootloader(bool Bootloader);
};

extern "C" int PingHost(char * HostName, int PacketSize ,int ttl, DWORD Timeout);
extern "C" TLTREther * WINAPI CreateInstanceEther(void);
extern "C" void WINAPI DeleteInstanceEther(TLTREther * NewClass);
#endif


