//---------------------------------------------------------------------------

#include <windows.h>
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------

#include "channel.h"
#include "ltr\\include\\ltrbootapi.h"
#define SEND_RECIEVE_TIMEOUT 2000
LPCSTR ErrorStrings[]={
 "  .",
 "   GET_ARRAY.",
 "   PUT_ARRAY.",
 "   CALL_APPLICATION.",
 "   GET_DESCRIPTION.",
 "   PUT_DESCRIPTION."
};
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_Init(TLTR010 *module)
{
 return LTR010_Init(module);
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_Open(TLTR010 *module, DWORD saddr, WORD sport, CHAR *csn)
{
 return LTR010_Open(module, saddr, sport, csn);
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_Close(TLTR010 *module)
{
 return LTR010_Close(module);
}
//-----------------------------------------------------------------------------
//                  
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_GetArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK) {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[4]={CONTROL_COMMAND_START, CONTROL_COMMAND_BOOT_LOADER_GET_ARRAY};
    sbuf[2]=address;
    sbuf[3]=size;
    INT ssz=channel->Send((BYTE*)sbuf, sizeof(sbuf), SEND_RECIEVE_TIMEOUT);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), SEND_RECIEVE_TIMEOUT);
    if(ssz!=sizeof(sbuf)) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTRBOOT_ERROR_GET_ARRAY;
    else if(channel->Recv(buf, size, SEND_RECIEVE_TIMEOUT)!=(int)size) res=LTR_ERROR_RECV;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_PutArray(TLTR010 *module, BYTE *buf, DWORD size, DWORD address)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK)
 {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[4]={CONTROL_COMMAND_START, CONTROL_COMMAND_BOOT_LOADER_PUT_ARRAY};
    sbuf[2]=address;
    sbuf[3]=size;
    INT ssz1=channel->Send((BYTE*)sbuf, sizeof(sbuf), SEND_RECIEVE_TIMEOUT);
    INT ssz2=channel->Send(buf, size, SEND_RECIEVE_TIMEOUT);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), SEND_RECIEVE_TIMEOUT);
    if(ssz1!=sizeof(sbuf) || ssz2!=(int)size) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTRBOOT_ERROR_PUT_ARRAY;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_GetDescription(TLTR010 *module, TDESCRIPTION_LTR010 *description)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK)
 {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[2]={CONTROL_COMMAND_START, CONTROL_COMMAND_BOOT_LOADER_GET_DESCRIPTION};
    INT ssz=channel->Send((BYTE*) sbuf, sizeof(sbuf), SEND_RECIEVE_TIMEOUT);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), SEND_RECIEVE_TIMEOUT);
    if(ssz!=sizeof(sbuf)) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTRBOOT_ERROR_GET_DESCRIPTION;
    else if(channel->Recv((BYTE*)description, sizeof(TDESCRIPTION_LTR010), SEND_RECIEVE_TIMEOUT)!=sizeof(TDESCRIPTION_LTR010)) res=LTR_ERROR_RECV;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_SetDescription(TLTR010 *module, TDESCRIPTION_LTR010 *description)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK)
 {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[2]={CONTROL_COMMAND_START, CONTROL_COMMAND_BOOT_LOADER_SET_DESCRIPTION};
    INT ssz1=channel->Send((BYTE*)sbuf, sizeof(sbuf), SEND_RECIEVE_TIMEOUT);
    INT ssz2=channel->Send((BYTE*)description, sizeof(TDESCRIPTION_LTR010), SEND_RECIEVE_TIMEOUT);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), SEND_RECIEVE_TIMEOUT);
    if(ssz1!=sizeof(sbuf) || ssz2!=sizeof(TDESCRIPTION_LTR010)) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTRBOOT_ERROR_PUT_DESCRIPTION;
 }
 return res;
}
//-----------------------------------------------------------------------------
INT WINAPI __export LTRBOOT_CallApplication(TLTR010 *module, DWORD address)
{
 INT res=LTR_IsOpened(&module->ltr);
 if(res==LTR_OK) {
    TChannel *channel=(TChannel*)module->ltr.internal;
    DWORD ack;
    DWORD sbuf[3]={CONTROL_COMMAND_START, CONTROL_COMMAND_CALL_APPLICATION};
    sbuf[2]=address;
    INT ssz=channel->Send((BYTE*)sbuf, sizeof(sbuf), SEND_RECIEVE_TIMEOUT);
    INT rsz=channel->Recv((BYTE*)&ack, sizeof(ack), SEND_RECIEVE_TIMEOUT);
    if(ssz!=sizeof(sbuf)) res=LTR_ERROR_SEND;
    else if(rsz!=sizeof(ack)) res=LTR_ERROR_RECV;
    else if(ack!=CONTROL_ACKNOWLEDGE_GOOD) res=LTRBOOT_ERROR_CALL_APPL;
 }
 return res;
}
//-----------------------------------------------------------------------------
LPCSTR WINAPI LTRBOOT_GetErrorString(INT error)
{
 if(error>=-302 && error<=-300) return ErrorStrings[-error-300];
 else return LTR_GetErrorString(error);
}
//-----------------------------------------------------------------------------

#pragma argsused
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fwdreason, LPVOID lpvReserved)
{
    return 1;
}
//---------------------------------------------------------------------------
