#include <stdio.h>
#include <algorithm>
#include "\LTR\include\ltrapi_ll.h"
#include "LIB\\DEV_STD\\USB_defs.h"
#include "LIB\\DEV_STD\\LUSB.h"

/*================================================================================================*/
#define LTR_IP_CTRL_SIG          0x314C5443 /* "CTL1" */

/*    Ethernet */
#define LTR_IP_CMD_INIT          0x80000000
#define LTR_IP_CMD_NOP           0x00000000
#define LTR_IP_CMD_GET_NAME      0x00000001
#define LTR_IP_CMD_GET_ARRAY     0x00000002
#define LTR_IP_CMD_PUT_ARRAY     0x00000003

#define MAX_BUF_CHUNKS          2

#define WAIT_FOR_SHUTDOWN       1
/*================================================================================================*/

/*================================================================================================*/
#pragma pack(1)
typedef struct
    {
    BYTE Name[16];
    BYTE SerialNumber[16];
    BYTE CpuType[16];
    BYTE Revision;
    LONG QuartzFrequency;
    BYTE Unused[128-53];
    }
    t_RawDevDescr;

typedef struct              /*     TCP */
    {
    DWORD Signature;        /* == LTR_IP_CTRL_SIG */
    DWORD CmdCode;          /*   */
    DWORD CmdParam;         /*   */
    DWORD VarDataLen;       /*     */
    DWORD ExpectReplyLen;   /*     */
    }
    t_EtherControlRequest;

typedef struct              /*      TCP */
    {
    DWORD Signature;        /* == LTR_IP_CTRL_SIG */
    DWORD ResultCode;       /*   */
    DWORD VarDataLen;       /*     */
    }
    t_EtherControlReply;
#pragma pack()

/*   "   "    WSASend/WSARecv
 *      ,   (ptr, len),
 *        .
 */
class TMultiBuf
    {
private:
    DWORD head;             /*       */
    DWORD count;            /*    */
    DWORD offset;           /*        */
    WSABUF chunk[MAX_BUF_CHUNKS]; /*   ( count ) */
public:
    TMultiBuf()             { head = count = offset = 0; }
    void Clear()            { head = count = offset = 0; }
        /* -   */
    DWORD ChunksLeft(void)  { return (count - head); }
        /* -   ,    (0 =  ) */
    bool AddChunk(LPVOID chunk_ptr, DWORD chunk_len);
        /* -         */
    LPWSABUF Head(void)     { return &chunk[head]; }
        /* -        ( 
               WSARecv()  WSASend()   ) */
    DWORD ShiftBy(DWORD bytes);
        /* -   " "   .
             ,     ,  ( head);
              ,  ,       .
                      (offset). */
    };
/*================================================================================================*/


/*      ""  */
/*------------------------------------------------------------------------------------------------*/
bool TMultiBuf::AddChunk(LPVOID chunk_ptr, DWORD chunk_len)
    {
    if ((chunk_len == 0) || !chunk_ptr) return false;
    if (count >= MAX_BUF_CHUNKS) return false;
    chunk[count].len = chunk_len;
    chunk[count].buf = (char*)chunk_ptr;
    count++;
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
DWORD TMultiBuf::ShiftBy(DWORD bytes)
    {
    while ((head < count) && (bytes >= chunk[head].len))
        { /*  ,   */
        bytes -= chunk[head].len;
        offset += chunk[head].len;
        chunk[head++].len = 0;
        }
    if ((head < count) && (bytes > 0))
        { /*      */
        chunk[head].len -= bytes;
        chunk[head].buf += bytes;
        offset += bytes;
        }
    return offset;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TLTRIP::TLTRIP(WSAEVENT AbortEvent, DWORD IdleTimeout)
    {
    TimeoutConn = LTR_IP_DEFAULT_CONN_TIMEOUT;
    TimeoutCmd = LTR_IP_DEFAULT_IO_TIMEOUT;
    LastErrorCode = 0;
    ip_addr = INADDR_NONE;
    ctlsock = INVALID_SOCKET;
    sock_evt = WSA_INVALID_EVENT;
    Connected = false;
    if (IdleTimeout > 0)
        {
        hIdleTimer = CreateWaitableTimer(NULL, TRUE, NULL);
        IdleInterval.QuadPart = -10000LL * IdleTimeout; /* negative = relative, 100ns resolution */
        }
    else
        {
        hIdleTimer = NULL;
        }
    hAbortEvent = AbortEvent;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TLTRIP::~TLTRIP()
    {
    CloseDevice();
    if (hIdleTimer)
        CloseHandle(hIdleTimer);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void WINAPI TLTRIP::StopIdleTimer(void)
    {
    if (hIdleTimer)
        CancelWaitableTimer(hIdleTimer);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void WINAPI TLTRIP::RestartIdleTimer(void)
    {
    if (hIdleTimer)
        SetWaitableTimer(hIdleTimer, &IdleInterval, 0, NULL, NULL, FALSE);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
HANDLE WINAPI TLTRIP::GetIdleTimerHandle(void)
    {
    return hIdleTimer;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int WINAPI TLTRIP::GetLastErrorCode(void)
    {
    return LastErrorCode;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int WINAPI TLTRIP::ConnectDataSocket(SOCKET sock)
    {
    sockaddr_in dest;

    memset(&dest, 0, sizeof(dest));
    dest.sin_family = AF_INET;
    dest.sin_port = htons(LTR_IP_DATA_PORT);
    dest.sin_addr.s_addr = ip_addr;

    return connect(sock, (sockaddr*)&dest, sizeof(dest));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void WINAPI TLTRIP::SetTimeouts(DWORD ConnectTimeout, DWORD CommandTimeout)
    {
    if (ConnectTimeout)
        TimeoutConn = ConnectTimeout;
    if (CommandTimeout)
        TimeoutCmd = CommandTimeout;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::CloseDevice(void)
    {
    if (ctlsock != INVALID_SOCKET)
        {
        TCSLock lock(cs);
        if (Connected)
            {
            Connected = false;
#if WAIT_FOR_SHUTDOWN
            bool wait = (sock_evt != WSA_INVALID_EVENT);
            bool graceful = false;
            if (wait)
                wait = (WSAEventSelect(ctlsock, sock_evt, FD_CLOSE) != SOCKET_ERROR);
            shutdown(ctlsock, SD_BOTH);
            if (wait)
                {
                WSAEVENT event_list[2];
                WSANETWORKEVENTS events;
                event_list[0] = sock_evt;
                event_list[1] = hAbortEvent;
                switch (WSAWaitForMultipleEvents(1 + (hAbortEvent != WSA_INVALID_EVENT),
                        event_list, FALSE, LTR_IP_DEFAULT_CLOSE_TIMEOUT, FALSE))
                    {
                    case WSA_WAIT_EVENT_0: /* Socket event */
                        if (WSAEnumNetworkEvents(ctlsock, sock_evt, &events) == SOCKET_ERROR)
                            break;
                        if (events.lNetworkEvents & FD_CLOSE)
                            {
                            /*  shutdown */
                            graceful = true;
                            }
                        break;
                    case WSA_WAIT_TIMEOUT:
                    case WSA_WAIT_FAILED:
                        graceful = false;
                        break;
                    } /* switch */
                } /* if (wait) */
#else
            shutdown(ctlsock, SD_BOTH);
#endif
            } /* if (Connected) */
        closesocket(ctlsock);
        ctlsock = INVALID_SOCKET;
        }
    else
        {
        Connected = false;
        }
    if (sock_evt != WSA_INVALID_EVENT)
        {
        WSACloseEvent(sock_evt);
        sock_evt = WSA_INVALID_EVENT;
        }
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::OpenDeviceByIP(DWORD ip)
    {
    bool ok;
    DWORD num_recv;
    WORD fpgainfo_raw[255];

    /*     ,   */
    if (Connected)
        {
        if (!CloseDevice()) return false;
        }

    if ((ip == INADDR_ANY) || (ip == INADDR_NONE))
        return false;
    ip_addr = ip;

    memset(fpga_info, 0, sizeof(fpga_info));
    ok = ExecCommand(LTR_IP_CMD_INIT, 0,
                    NULL, 0, (char*)fpgainfo_raw, sizeof(fpgainfo_raw),
                    NULL, &num_recv);
    if (ok && (num_recv >= FPGA_INFO_SIZE * sizeof(WORD)))
        {
        for (int i = 0; i < FPGA_INFO_SIZE; i++)
            {
            if ((fpgainfo_raw[i] >> 8) != 0xE0)
                {
                ok = false;
                break;
                }
            fpga_info[i] = fpgainfo_raw[i] & 0xFF;
            }
        }
    return ok;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool __fastcall TLTRIP::ExecEnd(DWORD StatusCode, DWORD SysError)
    {
    RestartIdleTimer();
    LastErrorCode = (StatusCode & 0xFF000000) | (SysError & 0x00FFFFFF);
    if ((StatusCode & 0xFF000000) == LTR_IPERR_OK)
        {
        return true;
        }
    else
        {
        CloseDevice();
        return false;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool __fastcall TLTRIP::ExecCommand(DWORD CommandCode, DWORD CommandParam,
                        const BYTE *pSendBuf, DWORD SendSize,
                        BYTE *pRecvBuf, DWORD RecvSize,
                        DWORD *pResultCode,
                        DWORD *pReceivedBytes,
                        DWORD ForceTimeout)
    {
    t_EtherControlRequest Request;
    t_EtherControlReply Reply;
    bool ok = false;
    bool hdr_checked = false;
    bool force_write = false;
    int res;
    sockaddr_in dest;
    WSAEVENT event_list[2];
    WSANETWORKEVENTS events;
    TMultiBuf sendbuf;
    TMultiBuf recvbuf;
    DWORD timeout;

    if (pResultCode)
        *pResultCode = LTR_IP_RESULT_SYSFAIL;
    if (pReceivedBytes)
        *pReceivedBytes = 0;

    if ((!pSendBuf && (SendSize > 0)) || (!pRecvBuf && (RecvSize > 0)))
        {
        LastErrorCode = LTR_IPERR_INVALID_PARAM;
        return false;
        }

    /*    Abort,   ,      */
    if (hAbortEvent != WSA_INVALID_EVENT)
        {
        event_list[0] = hAbortEvent;
        if (WSAWaitForMultipleEvents(1, event_list, FALSE, 0 /* timeout */, FALSE) == WAIT_OBJECT_0)
            {
            LastErrorCode = LTR_IPERR_ABORT;
            return false;
            }
        }

    /*    */
    Request.Signature = LTR_IP_CTRL_SIG;
    Request.VarDataLen = SendSize;
    Request.CmdCode = CommandCode;
    Request.CmdParam = CommandParam;
    Request.ExpectReplyLen = RecvSize;

    /*       */
    sendbuf.AddChunk(&Request, sizeof(Request));
    sendbuf.AddChunk((void*)pSendBuf, SendSize);

    recvbuf.AddChunk(&Reply, sizeof(Reply));
    recvbuf.AddChunk((void*)pRecvBuf, RecvSize);

    TCSLock lock(cs);
    /*    ,    */
    if (!Connected)
        {
        memset(&dest, 0, sizeof(dest));
        dest.sin_family = AF_INET;
        dest.sin_port = htons(LTR_IP_CTRL_PORT);
        dest.sin_addr.s_addr = ip_addr;

        ctlsock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        if (ctlsock == INVALID_SOCKET)
            return ExecEnd(LTR_IPERR_INTERNAL, WSAGetLastError());

        sock_evt = WSACreateEvent();
        if (sock_evt == WSA_INVALID_EVENT)
            return ExecEnd(LTR_IPERR_INTERNAL, WSAGetLastError());

        if (WSAEventSelect(ctlsock, sock_evt, FD_CONNECT) == SOCKET_ERROR)
            return ExecEnd(LTR_IPERR_INTERNAL, WSAGetLastError());

        res = connect(ctlsock, (sockaddr*)&dest, sizeof(dest));
        if (res == SOCKET_ERROR)
            {
            int err = WSAGetLastError();
            if (err != WSAEWOULDBLOCK)
                return ExecEnd(LTR_IPERR_CONNECT, err);
            }

        timeout = (ForceTimeout) ? ForceTimeout : TimeoutConn;
        }
    else
        {
        timeout = (ForceTimeout) ? ForceTimeout : TimeoutCmd;
        force_write = true;
        }

    /*    */
    while (!ok)
        {
        bool try_write = force_write;
        /*       FD_WRITE,  
             ,     .
           :        (?)
         */
        if (force_write)
            {
            force_write = false;
            }
        else
            {
            event_list[0] = sock_evt;
            event_list[1] = hAbortEvent;
            switch (WSAWaitForMultipleEvents(1 + (hAbortEvent != WSA_INVALID_EVENT),
                        event_list, FALSE, timeout, FALSE))
                {
                case WSA_WAIT_EVENT_0: /* Socket event */
                    if (WSAEnumNetworkEvents(ctlsock, sock_evt, &events) == SOCKET_ERROR)
                        return ExecEnd(LTR_IPERR_INTERNAL, WSAGetLastError());

                    if (events.lNetworkEvents & FD_CONNECT)
                        { /*   */
                        if (events.iErrorCode[FD_CONNECT_BIT])
                            return ExecEnd(LTR_IPERR_CONNECT, events.iErrorCode[FD_CONNECT_BIT]);
                        /*      */
                        if (WSAEventSelect(ctlsock, sock_evt, FD_READ | FD_WRITE | FD_CLOSE) == SOCKET_ERROR)
                            return ExecEnd(LTR_IPERR_INTERNAL, WSAGetLastError());
                        Connected = true;
                        timeout = (ForceTimeout) ? ForceTimeout : TimeoutCmd;
                        } /* FD_CONNECT */

                    if (events.lNetworkEvents & FD_WRITE)
                        { /*     */
                        if (events.iErrorCode[FD_WRITE_BIT])
                            return ExecEnd(LTR_IPERR_WRITE, events.iErrorCode[FD_WRITE_BIT]);
                        try_write = true;
                        } /* FD_WRITE */

                    if (events.lNetworkEvents & FD_READ)
                        {
                        if (events.iErrorCode[FD_READ_BIT])
                            return ExecEnd(LTR_IPERR_READ, events.iErrorCode[FD_READ_BIT]);
                        if (recvbuf.ChunksLeft() > 0)
                            {
                            DWORD bytes_recv, flags = 0;
                            res = WSARecv(ctlsock, recvbuf.Head(), recvbuf.ChunksLeft(), &bytes_recv,
                                &flags, NULL, NULL);
                            if (res == SOCKET_ERROR)
                                {
                                int err = WSAGetLastError();
                                if (err != WSAEWOULDBLOCK)
                                    return ExecEnd(LTR_IPERR_READ, err);
                                }
                            else if (bytes_recv > 0)
                                {
                                DWORD offset = recvbuf.ShiftBy(bytes_recv); /*   */
                                /*    ,    */
                                if (!hdr_checked && (offset >= sizeof(Reply)))
                                    { /*    */
                                    if (Reply.Signature != LTR_IP_CTRL_SIG)
                                        return ExecEnd(LTR_IPERR_BAD_SIG);
                                    if (Reply.VarDataLen > RecvSize)
                                        return ExecEnd(LTR_IPERR_BAD_REPLYLEN);
                                    /*       ,
                                          */
                                    if (!pReceivedBytes && (Reply.VarDataLen != RecvSize))
                                        return ExecEnd(LTR_IPERR_BAD_REPLYLEN);
                                    hdr_checked = true;
                                    }
                                /*    ,   */
                                if (hdr_checked && (offset >= sizeof(Reply) + Reply.VarDataLen))
                                    {
                                    ok = true;
                                    }
                                }
                            } /* if recvbuf.ChunksLeft() */
                        } /* FD_READ */

                    if (events.lNetworkEvents & FD_CLOSE)
                        {
                        return ExecEnd(LTR_IPERR_CONN_LOST, events.iErrorCode[FD_CLOSE_BIT]);
                        } /* FD_CLOSE */

                    break;
                case WSA_WAIT_EVENT_0 + 1: /* Abort signalled by another thread */
                    return ExecEnd(LTR_IPERR_ABORT);
                case WSA_WAIT_TIMEOUT:
                    /*  */
                    return ExecEnd((sendbuf.ChunksLeft() > 0)
                        ? LTR_IPERR_TIMEOUT_W
                        : LTR_IPERR_TIMEOUT_R);
                case WSA_WAIT_FAILED:
                    return ExecEnd(LTR_IPERR_INTERNAL, WSAGetLastError());
                } /* switch (WSAWaitForMultipleEvents(...)) */
            } /* if force_write */

        if (try_write)
            {
            if (sendbuf.ChunksLeft() > 0)
                {
                DWORD bytes_sent;
                res = WSASend(ctlsock, sendbuf.Head(), sendbuf.ChunksLeft(), &bytes_sent,
                    0, NULL, NULL);
                if (res == SOCKET_ERROR)
                    {
                    int err = WSAGetLastError();
                    if (err != WSAEWOULDBLOCK)
                        return ExecEnd(LTR_IPERR_WRITE, err);
                    }
                else
                    {
                    sendbuf.ShiftBy(bytes_sent); /*   */
                    }
                }
            } /* try_write */
        } /* while (!ok) */

    /*  ,    */
    if (pResultCode)
        { /*      */
        *pResultCode = Reply.ResultCode;
        }
    else
        { /*     ,     */
        if (Reply.ResultCode != LTR_IP_RESULT_OK)
            return ExecEnd(LTR_IPERR_CMD_FAILED);
        }
    if (pReceivedBytes)
        *pReceivedBytes = Reply.VarDataLen;

    return ExecEnd(LTR_IPERR_OK);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::Ping(DWORD Timeout)
    {
    return ExecCommand(LTR_IP_CMD_NOP, 0,
                NULL, 0, NULL, 0,
                NULL, NULL, Timeout);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::GetModuleDescription(TDESCRIPTION_LTR010 *description)
    {
    BYTE info1[16], info2[16];
    t_RawDevDescr ddescr;

    if (!description)
        return false;
    memset(description, 0, sizeof(TDESCRIPTION_LTR010));

    if (!(GetArray(LTR010_MODULE_DESCRIPTOR_ADDRESS, (BYTE*)&ddescr, sizeof(ddescr))
        && GetArray(LTR010_BOOT_LOADER_VERSION_ADDRESS, info1, 16)
        && GetArray(LTR010_FIRMWARE_VERSION_ADDRESS, info2, 16)))
        return false;

    // module description
    TDESCRIPTION_MODULE &mdescr = description->Module;
    strncpy(mdescr.CompanyName,  "L-CARD Inc.",       sizeof(mdescr.CompanyName));
    strncpy(mdescr.DeviceName,   ddescr.Name,         std::min(sizeof(mdescr.DeviceName), sizeof(ddescr.Name)));
    strncpy(mdescr.SerialNumber, ddescr.SerialNumber, std::min(sizeof(mdescr.SerialNumber), sizeof(ddescr.SerialNumber)));
    strncpy(mdescr.Comment,      ddescr.Unused,       std::min(sizeof(mdescr.Comment), sizeof(ddescr.Unused)));
    //mdescr.Revision=ddescr.DeviceRevision;
    // cpu description
    TDESCRIPTION_CPU &cdescr = description->Cpu;
    cdescr.Active = 1;
    strncpy(cdescr.Name, ddescr.CpuType, std::min(sizeof(cdescr.Name), sizeof(ddescr.CpuType)));
    cdescr.ClockRate = ddescr.QuartzFrequency;
    cdescr.FirmwareVersion = *(DWORD*)info2;
    sprintf(cdescr.Comment, "Bootloader v%d.%d.%d.%d", (int)info1[3], (int)info1[2], (int)info1[1], (int)info1[0]);
    // interface description
    TDESCRIPTION_INTERFACE &idescr = description->Interface;
    idescr.Active = 1;
    strcpy(idescr.Name, "Ethernet");
    strcpy(idescr.Comment, "TCP/IP Protocol");

    // fpga description
    if (fpga_info[0])
        {
        TDESCRIPTION_FPGA &fdescr = description->Fpga;
        fdescr.Active = 1;
        fdescr.ClockRate = 60e6;
        // Name
        char *ptag, *p1, *p2;
        ptag=strstr((char*)fpga_info, "<Name>");
        if(ptag && (p1=strchr(ptag, '{'))!=0 && (p2=strchr(ptag, '}'))!=0)
            {
            strncpy(fdescr.Name, p1+1, std::min<int>(sizeof(fdescr.Name), p2-p1-1));
            }
        // Version
        ptag=strstr((char*)fpga_info, "<Version>");
        if(ptag && (p1=strchr(ptag, '{'))!=0 && (p2=strchr(ptag, '}'))!=0)
            {

            int i, j, k;
            if(sscanf(p1, "{%d.%d.%d", &i, &j, &k)==3)
                {
                fdescr.FirmwareVersion=((i&255)<<24)|((j&255)<<16)|((k&255)<<8);
                }

            }
        // Comment
        ptag=strstr((char*)fpga_info, "<Comments>");
        if(ptag && (p1=strchr(ptag, '{'))!=0 && (p2=strchr(ptag, '}'))!=0)
            {
            strncpy(fdescr.Comment, p1+1, std::min<int>(sizeof(fdescr.Comment), p2-p1-1));
            }
    }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::GetModuleName(char* ModuleName, int ModuleNameLength)
    {
    return ExecCommand(LTR_IP_CMD_GET_NAME, 0,
                NULL, 0,
                ModuleName, ModuleNameLength,
                NULL, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::GetArray(DWORD address, BYTE *data, DWORD size)
    {
    return ExecCommand(LTR_IP_CMD_GET_ARRAY, address,
                NULL, 0,
                data, size,
                NULL, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool WINAPI TLTRIP::PutArray(DWORD address, BYTE *data, DWORD size)
    {
    return ExecCommand(LTR_IP_CMD_PUT_ARRAY, address,
                data, size,
                NULL, 0,
                NULL, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

