// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the LTR030API_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// LTR030API_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.

#ifndef LTR030MAPI_H_
#define LTR030MAPI_H_

#include <windows.h>
#include "LIB/DEV_STD/USB_defs.h"


/*================================================================================================*/
#ifdef LTR030FAPI_EXPORTS
  #define LTR030FAPI_DllExport(type) __declspec(dllexport) type APIENTRY
#else
  #define LTR030FAPI_DllExport(type) __declspec(dllimport) type APIENTRY
#endif


/*   */
#define LTR030F_OK (0)
#define LTR030F_ERROR_INVALID_PARAMETERS (-2001)
#define LTR030F_ERROR_OPEN_FLASH (-2002)
#define LTR030F_ERROR_FLASH_COMMAND	(-2003)
#define LTR030F_ERROR_NO_COMMAND_FOR_INTERFACE (-2004)
#define LTR030F_ERROR_BULK_FIFO_WRITE_DEAD (-2005)
/*================================================================================================*/


/*================================================================================================*/
typedef enum
	{
    e_Interface_USB = 1,
    e_Interface_Ether = 2,
    e_Interface_LPT1 = 3,
    e_Interface_LPT2 = 4,
    e_Interface_LPT3 = 5,
	} e_Interface;

#pragma pack(1)
typedef struct
	{
    BYTE Name[ 16 ];
    BYTE SerialNumber[ 16 ];
    BYTE CpuType[ 16 ];
    BYTE Revision;
    long QuartzFrequency;
    BYTE Unused[ 128-53 ];    
	} t_DEVICE_DESCR;

typedef struct
	{
    BYTE CPUName[ 16 ];
    DWORD CurrCPUFreq;
    DWORD CurrSystemFreq;
    WORD SDRAM_Size_Mb;
    WORD SD_Card_Presense;
    WORD Temperature;
    BYTE Unused[ 64-30 ];    
	} t_DEVICE_STATUS;

typedef struct
	{
	DWORD FirmwareVersion;
	BYTE Unused[ 12 ];
    t_DEVICE_DESCR Descr;
    t_DEVICE_STATUS Status;   
	} t_FULL_DEVICE_DESCR_STATUS;

typedef struct 
	{
    HANDLE USBClassHandle;
	HANDLE LPTClassHandle;
    HANDLE USBFIFOHandle;
    e_Interface CurrInterface;
	BOOL Application;
	} TLTR030F, *PTLTR030F;
#pragma pack()
/*================================================================================================*/

/*================================================================================================*/
#ifdef __cplusplus
extern "C" {  // only need to export C interface if
	// used by C++ source code
#endif

LTR030FAPI_DllExport( int ) LTR030F_CallApplication(PTLTR030F device, WORD Address);
LTR030FAPI_DllExport( int ) LTR030F_CloseDevice(PTLTR030F device);
/*   ,    LPT USB Ethernet */
LTR030FAPI_DllExport( int ) LTR030F_CloseFullDevice(PTLTR030F device);
LTR030FAPI_DllExport( int ) LTR030F_DSP_Read_Command(PTLTR030F device, WORD CommandCode,
    WORD TransferBulk, BYTE Data[], DWORD DataLength, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_DSP_Write_Command(PTLTR030F device, WORD CommandCode,
    WORD TransferBulk, BYTE Data[], DWORD DataLength, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_GetDeviceDescr(PTLTR030F device,
	t_FULL_DEVICE_DESCR_STATUS *FULL_DEVICE_DESCR_STATUS);
LTR030FAPI_DllExport( int ) LTR030F_GetDeviceName(PTLTR030F device, CHAR DeviceName[],
    WORD NameLength);
LTR030FAPI_DllExport( int ) LTR030F_OpenDevice(PTLTR030F device, BYTE DeviceSlot);
LTR030FAPI_DllExport( int ) LTR030F_ProgrammDSPFlash(PTLTR030F device, DWORD Address, BYTE Data[],
	DWORD DataLength, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_ReadDSPFlash(PTLTR030F device, DWORD Address, BYTE Data[],
	DWORD DataLength, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_ResetDSP(PTLTR030F device);
LTR030FAPI_DllExport( int ) LTR030F_SearchDevices(PTLTR030F device, short DeviceSlots[],
	int DeviceSlotsLength);
LTR030FAPI_DllExport( int ) LTR030F_SimpleControlLDevice(PTLTR030F device,
    _ControlRequest *CurrentRequest, BYTE DataBuffer[], DWORD DataBufferLength, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_SimpleReadBulk(PTLTR030F device, BYTE DataBuffer[],
	DWORD DataBufferLength, DWORD *BytesRead, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_SimpleWriteBulk(PTLTR030F device, BYTE DataBuffer[],
	DWORD DataBufferLength, DWORD *BytesWritten, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_SpecialDSPFlashCommand(PTLTR030F device, WORD CommandCode,
    BYTE Data[], DWORD Size);

/*
 *     S-500     FIFO
 * FIFOByteLength -  FIFO (   -    FIFO)
 * NumberQueue -    
 * SizeBytesQueue -     
 * WriteSizeBytesQueue -    ,   DSP    ISP,
 *      (512 ),     
 *   DSP      
 * 512 ,         512 ,
 *  WriteSizeBytesQueue     512 
 * ReadSizeBytesQueue -  ,      (512 )
 * LTR030_StartBulkFIFO -    ,  DSP  DMA  (  
 *     Init_DMa -   DMa )
 */
LTR030FAPI_DllExport( int ) LTR030F_CreateBulkFIFO(PTLTR030F device, DWORD FIFOReadByteLength,
	DWORD FifoWriteByteLength, DWORD ReadNumberQueue, DWORD WriteNumberQueue,
	DWORD ReadSizeBytesQueue, DWORD WriteSizeBytesQueue);
LTR030FAPI_DllExport( int ) LTR030F_DisposeBulkFIFO(PTLTR030F device);
LTR030FAPI_DllExport( int ) LTR030F_FlushBulkFIFO(PTLTR030F device);
LTR030FAPI_DllExport( int ) LTR030F_GetBulkFIFOReadReady(PTLTR030F device, DWORD *CounterReadReady);
LTR030FAPI_DllExport( int ) LTR030F_GetBulkFIFOWriteFree(PTLTR030F device,
    DWORD *CounterWriteWORDFree);
LTR030FAPI_DllExport( int ) LTR030F_ReadBulkFIFO(PTLTR030F device, WORD *Data, DWORD DataLengthWORD,
	DWORD *WordsReaded, DWORD Timeout);
LTR030FAPI_DllExport( int ) LTR030F_StartBulkFIFO(PTLTR030F device);
LTR030FAPI_DllExport( int ) LTR030F_StopBulkFIFO(PTLTR030F device);
LTR030FAPI_DllExport( int ) LTR030F_WriteBulkFIFO(PTLTR030F device, WORD *Data,
    DWORD DataLengthWORD, DWORD *WordsWritted, DWORD Timeout);

#ifdef __cplusplus
}
#endif
/*================================================================================================*/

#endif /*#ifndef LTR030API_H_*/