#include "Logfile.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
TLogfile::TLogfile()
    : MaxLevel(0), beginning(true), hFile(INVALID_HANDLE_VALUE)
    {
    pLogBuffer = new TStringList;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TLogfile::~TLogfile()
    {
    CloseFile();
    if (pLogBuffer)
        delete pLogBuffer;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
AnsiString __fastcall TLogfile::RelativeFileName
    (
    const AnsiString &FileName      /*   (  ':', '/', '\' -   EXE) */
    )
    {
    if (!FileName.IsEmpty() && (FileName.LastDelimiter(":/\\") == 0))
        {
        return ExtractFilePath(Application->ExeName) + FileName;
        }
    else
        {
        return FileName;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::OpenFile  /*   */
    (
    const AnsiString &FileName      /*   (  ':', '/', '\' -   EXE) */
    )
    {
    TCSLock lock(cs);
    CloseFile();

    if (FileName.IsEmpty())
        return;

    AnsiString fname = RelativeFileName(FileName);
    hFile = CreateFile(fname.c_str(), GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS,
        FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile != INVALID_HANDLE_VALUE)
        {
        LONG hipart = 0;
        SetFilePointer(hFile, 0, &hipart, FILE_END);
        beginning = true;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::CloseFile /*   */
    (void)
    {
    TCSLock lock(cs);
    Flush(NULL, 0);
    if (hFile != INVALID_HANDLE_VALUE)
        {
        CloseHandle(hFile);
        hFile = INVALID_HANDLE_VALUE;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::SetLevel  /*    */
    (
    int inLevel                     /*     */
    )
    {
    TCSLock lock(cs);
    MaxLevel = inLevel;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::Write     /*     */
    (
    const AnsiString &Message,      /*   */
    const AnsiString &TaskName,     /*  "" */
    int Level                       /*   ( = ) */
    )
    {
    TCSLock lock(cs);

    if (!pLogBuffer || (Level > MaxLevel))
        return;

    try
        {
        /*    , ,     */
        SYSTEMTIME st;
        GetLocalTime(&st);
        AnsiString DateTime = Format("[%0.2d.%0.2d.%0.2d %0.2d:%0.2d:%0.2d.%0.3d]", ARRAYOFCONST((
            st.wDay, st.wMonth, st.wYear % 100, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds)));
        pLogBuffer->Add(DateTime + " (" + Level + ") " + TaskName + ": " + Message);
        }
    catch (...)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::Flush     /*          */
    (
    TStrings *pDisplay,             /*     (TRichEdit)  NULL */
    int MaxDisplayLines             /*  scrollback   (0 =  ) */
    )
    {
    TCSLock lock(cs);

    if (!pLogBuffer) return;
    if (pLogBuffer->Count == 0) return;

    try
        {
        if (pDisplay)
            {
            pDisplay->BeginUpdate();
            if (beginning && (pDisplay->Count > 0))
                {
                pDisplay->Add("");
                }
            for (int i = 0; i < pLogBuffer->Count; i++)
                {
                if (MaxDisplayLines > 0)
                    {
                    while (pDisplay->Count >= MaxDisplayLines)
                        {
                        pDisplay->Delete(0);
                        }
                    }
                pDisplay->Add(pLogBuffer->Strings[i]);
                }
            pDisplay->EndUpdate();
            }

        if (hFile != INVALID_HANDLE_VALUE)
            {
            AnsiString msg;
            DWORD nw;
            if (beginning)
                {
                msg =
                "========================================"
                "========================================"
                "\r\n";
                WriteFile(hFile, msg.data(), msg.Length(), &nw, NULL);
                }
            for (int i = 0; i < pLogBuffer->Count; i++)
                {
                msg = pLogBuffer->Strings[i] + "\r\n";
                if (!WriteFile(hFile, msg.data(), msg.Length(), &nw, NULL))
                    break;
                }
            }

        beginning = false;
        }
    catch (...)
        {
        }
    if (hFile != INVALID_HANDLE_VALUE)
        FlushFileBuffers(hFile);
    pLogBuffer->Clear();
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
