#include <vcl.h>
#include <stdio.h>
#pragma hdrstop
#include "common.h"
#include "LtrCrate.h"
#include "LtrCrateThreads.h"
#pragma package(smart_init)

TCrate::TCrate(T_MessageQueue *mqControl, T_MessageQueue *mqDisplay)
{
    CurState = CRATE_ST_UNINIT;
    MessageQueueControl = mqControl;
    MessageQueueDisplay = mqDisplay;
    ThreadTerminationInUse = CreateMutex(NULL,FALSE,NULL);
    ip_addr = INADDR_NONE;
    memset(path, 0, sizeof(path));
    memset(pid, 0, sizeof(pid));
    memset(csn, 0, sizeof(csn));

    memset(module, 0, sizeof(module));

    /*     RAW  USER  */
    module[MODULE_INDEX_RAW].CreateModule();
    module[MODULE_INDEX_USER].CreateModule();

    sflags=0;
    baud_rate=0;
    //
    second_mark_counter=0;
    start_mark_counter=0;
    //
    GetLocalTime(&start_time);
    total_count_recv=0;
    total_count_send=0;
    special_code = false;
    //
    mconfig=0;
    //
    CrateType = CRATE_TYPE_UNKNOWN;
    memset(AllowLowSpeedModule, 1, sizeof(AllowLowSpeedModule));

    ReadRawData = false;
    Suspended = false;
}
//---------------------------------------------------------------------------
TCrate::~TCrate(void)
    {
    /*  ,    */
    if (WaitForSingleObject(client_list_access, INFINITE) == WAIT_OBJECT_0)
        {
        for (int k = 0; k < CLIENT_MAX; k++)
            {
            if (client_list[k]!=NULL && client_list[k]->crate==this)
                client_list[k]->IsAlive=false;
            }
        for (int k=0; k < CLIENT_MAX; k++)
            {
            if (client_list[k]!=NULL && client_list[k]->crate==this)
                {
                g_Log.Write(Format("Connection stopped for client " +
                    client_list[k]->peer_addr_str + ": CSN=%.16s  CC=%0.4X",
                    ARRAYOFCONST(((char*)csn, (int)client_list[k]->crate_channel))),
                    LOGTASK_CLIENT_CTL, LOGL_INFO);
                delete client_list[k];
                client_list[k]=NULL;
                }
            }
        ReleaseMutex(client_list_access);
        }
    /*  mutex' */
    CloseHandle(ThreadTerminationInUse);
    }

//---------------------------------------------------------------------------

//-----------------------------------------------------------------------------
bool TCrate::TModule::CreateModule(void)
    {
    DeleteModule();
    mid = 0xFFFF;
    recv_buf = new DWORD[MODULE_RECV_BUF_SIZE];
    send_buf = new DWORD[MODULE_SEND_BUF_SIZE];
    recv_buf_is = CreateEvent(NULL, FALSE, FALSE, NULL);
    send_buf_In_Use = CreateMutex(NULL, FALSE, NULL);
    if (recv_buf && send_buf && recv_buf_is && send_buf_In_Use)
        return ModulePresence = true;
    /*  --  ,   */
    DeleteModule();
    return false;
    }
//----------------------------------------------------------------------------
void TCrate::TModule::DeleteModule(void)
    {
    ModulePresence = false;
    if (recv_buf)
        {
        delete[] recv_buf;
        recv_buf = NULL;
        }
    if (send_buf)
        {
        if (send_buf_cntr != 0)
            {
            //         
            send_buf_cntr=0;
            send_buf_cntr_sended=0;
            }
        delete[] send_buf;
        send_buf = NULL;
        }
    if (send_buf_In_Use)
        {
        ReleaseMutex(send_buf_In_Use);
        CloseHandle(send_buf_In_Use);
        send_buf_In_Use = NULL;
        }
    if (recv_buf_is!=NULL)
        {
        CloseHandle(recv_buf_is);
        recv_buf_is = NULL;
        }
    mid = 0;                               // id 
    recv_buf_ptr=0;            //   
    send_buf_cntr=0;           //   
    send_buf_cntr_sended=0;    //   

    //   34                    //
    count_recv=0;            //      
    count_send=0;            //      
    count_FIFO_recv=0;       //     FIFO (    count_FIFO_Client)
    fifo_size=0;                        //
    }
//------------------------------------------------------------------------------

/*------------------------------------------------------------------------------------------------*/
bool TCrate::ProcessCrateError(DWORD data)
    {
    BYTE regs[16];

    if (!GetArray(CM_APP, SEL_AVR_DM|0x8000, regs, sizeof(regs)))
        {
        CancelIO();
        return false;
        }

    AnsiString msg;
    switch (data&0xFF)
    {
    case 0 :
        {
            msg = Format("Crate sent fault signal CSN=%.16s  REGS[0..15]=", ARRAYOFCONST(((char*)csn)));
            for (int j = 0; j < sizeof(regs); j++)
                {
                msg += Format(" %0.2X", ARRAYOFCONST((regs[j])));
                }
            g_Log.Write(msg, LOGTASK_CRATE_READ, LOGL_ERROR);

            sflags |= CRATE_FLAG_HW_ERROR;
            CancelIO();
            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
        }
        break;
    case 1 :
        { //   
            WORD mconfig_new=*(WORD*)(regs+8);
            for(int i=0; i<MODULE_MAX; i++)
            {
                WORD mask=(1<<i);
                if(!(mconfig_new&mask))
                { //  
                    WORD old_mid=module[i].mid;
                    if(old_mid&&(baud_rate&mask))
                    { //      -    
                        WORD baud_rate_new=baud_rate&(~mask);
                        if (PutArray(CM_APP, SEL_AVR_DM|0x8006, (BYTE*)&baud_rate_new, 2))
                            {
                            baud_rate=baud_rate_new;
                            g_Log.Write(Format("RST.XXXX change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate_new))),
                                LOGTASK_CRATE_READ, LOGL_VERBOSE);
                            }
                        else
                            {
                            g_Log.Write(Format("RST.XXXX cannot change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate_new))),
                                LOGTASK_CRATE_READ, LOGL_ERROR);
                            }
                    }
                    module[i].mid=0;
                    module[i].DeleteModule();
                }
                else
                {
                    bool BadmoduleName =false;
                    if((module[i].mid&0xFF)!=((module[i].mid>>8)&0xFF))
                    {
                        BadmoduleName=true;
                    }

                    if((!(mconfig&mask) && !module[i].mid)||BadmoduleName)
                    {
                        module[i].CreateModule();
                        InterlockedExchangeAdd(&mconfig, mask);
                    }
                }
            }

            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
            g_Log.Write(Format("RST.XXXX module configuration updated. mconfig=%0.4X",
                ARRAYOFCONST(((int)mconfig_new))), LOGTASK_CRATE_READ, LOGL_INFO);
        }
        break;
    case 2:
        g_Log.Write("Crate LTR21 Buffer Overflow", LOGTASK_CRATE_READ, LOGL_ERROR);
        break;
    case 3:
        g_Log.Write("Crate LTR21 DMA Buffer Miss", LOGTASK_CRATE_READ, LOGL_ERROR);
        break;
    case 4:
        g_Log.Write("Crate LTR021 - Unknown Sync Source", LOGTASK_CRATE_READ, LOGL_ERROR);
        break;
    default:
        g_Log.Write("Unknown Crate Error", LOGTASK_CRATE_READ, LOGL_ERROR);
        break;
    }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateUSB::TCrateUSB(T_MessageQueue *mqControl, T_MessageQueue *mqDisplay)
    : TCrate(mqControl, mqDisplay)
    {
    recv_buf_thread = NULL;
    send_buf_thread = NULL;
    DeviceHANDLE = NULL;
    ltr_USB = CreateInstanceUSB();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateIP::TCrateIP(T_MessageQueue *mqControl, T_MessageQueue *mqDisplay, DWORD IdleTimeout)
    : TCrate(mqControl, mqDisplay)
    {
    ping_fail_count = 0;
    data_thread = NULL;
    hAbortEvent = WSACreateEvent();
    ltr_ip = CreateInstanceIP(hAbortEvent, IdleTimeout);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateUSB::~TCrateUSB(void)
    {
    CurState = CRATE_ST_DEAD;
    StopServiceThreads();
    for (int i = 0; i < MODULE_ARRAYLEN; i++)
        {
        module[i].DeleteModule();
        }
    if (ltr_USB != NULL)
        DeleteInstanceUSB(ltr_USB);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateIP::~TCrateIP(void)
    {
    CurState = CRATE_ST_DEAD;
    StopServiceThreads();
    for (int i = 0; i < MODULE_ARRAYLEN; i++)
        {
        module[i].DeleteModule();
        }
    if (ltr_ip != NULL)
        DeleteInstanceIP(ltr_ip);
    WSACloseEvent(hAbortEvent);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::SetState(t_CrateState new_state, bool NotifyThreads)
    {
    t_CrateState old_state = CurState;
    CurState = new_state;
    if (NotifyThreads && (new_state != old_state))
        {
        /*    ONLINE  -ONLINE,      */
        if ((new_state == CRATE_ST_ONLINE) || (old_state == CRATE_ST_ONLINE))
            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
        /*     DEAD    */
        if (new_state == CRATE_ST_DEAD)
            MessageQueueControl->PostMessage(LTR_SRV_bury_the_dead);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateIP::SetState(t_CrateState new_state, bool NotifyThreads)
    {
    t_CrateState old_state = CurState;
    CurState = new_state;
    switch (new_state)
        {
        case CRATE_ST_ONLINE:
            IPCrateList.SetStatus(ip_addr, IP_STATUS_ONLINE, csn);
            break;
        case CRATE_ST_CONNECTING:
            IPCrateList.SetStatus(ip_addr, IP_STATUS_CONNECTING);
            break;
        default:
            IPCrateList.SetStatus(ip_addr, IP_STATUS_OFFLINE);
            break;
        }
    if (NotifyThreads && (new_state != old_state))
        {
        /*    ONLINE  -ONLINE,      */
        if ((new_state == CRATE_ST_ONLINE) || (old_state == CRATE_ST_ONLINE))
            MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
        /*     DEAD    */
        if (new_state == CRATE_ST_DEAD)
            MessageQueueControl->PostMessage(LTR_SRV_bury_the_dead);
        /*  IP       IP */
        MessageQueueDisplay->PostMessage(LTR_DISP_ip_list_updated);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
BYTE TCrateUSB::GetConnType(void)
    {
    return CRATE_IFACE_USB;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
BYTE TCrateIP::GetConnType(void)
    {
    return CRATE_IFACE_TCPIP;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::SuspendCrate(void)
    {
    if (Suspended) return;
    Suspended = true;

    StopServiceThreads();

    ltr_USB->CancelIO();
    if (ltr_USB->CloseLDevice())
        {
        g_Log.Write(Format("Crate %s suspended", ARRAYOFCONST((path))),
            LOGTASK_CRATE_CTL, LOGL_INFO);
        return;
        }
    else
        {
        Suspended = false;
        g_Log.Write(Format("Cannot suspend crate %s", ARRAYOFCONST((path))),
            LOGTASK_CRATE_CTL, LOGL_ERROR);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateIP::SuspendCrate(void)
    {
    if (Suspended) return;
    Suspended = true;
    StopServiceThreads();
    ltr_ip->StopIdleTimer();
    g_Log.Write(Format("Crate %s suspended", ARRAYOFCONST((path))),
        LOGTASK_CRATE_CTL, LOGL_INFO);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::ResumeCrate(void)
    {
    if (!Suspended) return;
    if (ltr_USB->OpenLDeviceByName(path))
        {
        g_Log.Write(Format("Crate %s resumed from suspend", ARRAYOFCONST((path))),
            LOGTASK_CRATE_CTL, LOGL_INFO);
        Suspended = false;
        StartServiceThreads();
        return;
        }
    g_Log.Write(Format("Cannot resume crate %s after suspend", ARRAYOFCONST((path))),
        LOGTASK_CRATE_CTL, LOGL_ERROR);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateIP::ResumeCrate(void)
    {
    if (!Suspended) return;
    ltr_ip->RestartIdleTimer();
    if (StartServiceThreads())
        {
        g_Log.Write(Format("Crate %s resumed from suspend", ARRAYOFCONST((path))),
            LOGTASK_CRATE_CTL, LOGL_INFO);
        Suspended = false;
        return;
        }
    g_Log.Write(Format("Cannot resume crate %s after suspend", ARRAYOFCONST((path))),
        LOGTASK_CRATE_CTL, LOGL_ERROR);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::Startup(BYTE * FPGAData, char *default_ttf)
    { /*  USB   ,    */
    if (GetState() != CRATE_ST_UNINIT)
        { /*    */
        StopServiceThreads();
        sflags &= ~CRATE_FLAG_BOOT_LOADER;
        }
    SetState(CRATE_ST_CONNECTING);
    bool result = InitCrate(FPGAData, default_ttf);
    if (result)
        {
        /*     InitCrate() */
        SetState(CRATE_ST_ONLINE);
        }
    else
        {
        SetState(CRATE_ST_DEAD);
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::Startup(BYTE * FPGAData, char *default_ttf)
    { /*  TCP      */
    if (GetState() != CRATE_ST_UNINIT)
        { /*    */
        StopServiceThreads();
        sflags &= ~CRATE_FLAG_BOOT_LOADER;
        }
    SetState(CRATE_ST_CONNECTING);
    if (!StartServiceThreads())
        {
        g_Log.Write(Format("Cannot start data thread for crate %s", ARRAYOFCONST((path))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        SetState(CRATE_ST_DEAD);
        return false;
        }

    return true; /*   --     */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::InitCrate(BYTE * FPGAData, char * default_ttf)
    {
    g_Log.Write(Format("Connecting USB crate %s ...", ARRAYOFCONST((path))),
        LOGTASK_CRATE_INIT, LOGL_INFO);

    baud_rate = 0;

    if (!ltr_USB->OpenLDeviceByName(path))
        {
        g_Log.Write(Format("Cannot open crate %s", ARRAYOFCONST((path))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }

    g_Log.Write("OpenLDeviceByName() -> ok", LOGTASK_CRATE_INIT, LOGL_DEBUG3);

    //  bootloader  aplication
    char appmode = '?';
    if (ltr_USB->SwitchBootloader(true))
        {
        appmode = 'B'; // bootloader active
        }
    else
        {
        g_Log.Write(Format("Bootloader mode select failed  on %s (%d)",
            ARRAYOFCONST((path, GetLastError()))), LOGTASK_CRATE_INIT, LOGL_DEBUG2);
        }
    if (ltr_USB->SwitchBootloader(false))
        {
        appmode = 'A'; // application active
        }
    else
        {
        g_Log.Write(Format("Application mode select failed on %s (%d)",
            ARRAYOFCONST((path, GetLastError()))), LOGTASK_CRATE_INIT, LOGL_DEBUG2);
        }
    if (appmode == '?')
        {
        g_Log.Write(Format("Cannot set application mode for crate %s", ARRAYOFCONST((path))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }

    TDESCRIPTION_LTR010 descr;
    char CrateName[16];

    if (!ltr_USB->GetModuleName(CrateName, sizeof(CrateName)))
        {
        g_Log.Write(Format("Error in GetModuleName(%s)",
            ARRAYOFCONST((path))), LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }
    g_Log.Write("GetModuleName() -> ok", LOGTASK_CRATE_INIT, LOGL_DEBUG3);

    if (!GetModuleDescription(CM_AUTO, &descr))
        {
        g_Log.Write(Format("Error in GetModuleDescription(%s)",
            ARRAYOFCONST((path))), LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }
    g_Log.Write("GetModuleDescription() -> ok", LOGTASK_CRATE_INIT, LOGL_DEBUG3);

    memcpy(csn, descr.Module.SerialNumber, sizeof(csn));
    g_Log.Write("Crate Info:", LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ PIDString = %s", ARRAYOFCONST(((char*)pid))), LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ DeviceName = %.16s", ARRAYOFCONST(((char*)descr.Module.DeviceName))), LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ SerialNumber = %.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber))), LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ DeviceComment = %s", ARRAYOFCONST(((char*)descr.Module.Comment))), LOGTASK_CRATE_INIT, LOGL_VERBOSE);

    if (pid[10] == 'H')
        sflags |= CRATE_FLAG_USB_HI_SPEED;

    int v1 = 0, v2 = 0, v3 = 0, v4 = 0;
    if (sscanf(descr.Cpu.Comment, "Bootloader v%d.%d.%d.%d", &v1, &v2, &v3, &v4)==4)
        {
        g_Log.Write(Format("+ BootLoaderVersion = %d.%d.%d.%d", ARRAYOFCONST((v1, v2, v3, v4))), LOGTASK_CRATE_INIT, LOGL_VERBOSE);
        }
    if (descr.Cpu.FirmwareVersion!=0xFFFFFFFF)
        {
        g_Log.Write(Format("+ FirmwareVersion = %d.%d.%d.%d", ARRAYOFCONST((
            (int)(descr.Cpu.FirmwareVersion>>24)&0xFF,
            (int)(descr.Cpu.FirmwareVersion>>16)&0xFF,
            (int)(descr.Cpu.FirmwareVersion>> 8)&0xFF,
            (int)(descr.Cpu.FirmwareVersion>> 0)&0xFF))),
            LOGTASK_CRATE_INIT, LOGL_VERBOSE);
        }

    //     -
    AnsiString section = Format("LTR010_%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber)));
    AnsiString crate_ttf = "";

    if (default_ttf)
        {
        crate_ttf =
            TLogfile::RelativeFileName(g_INI.ReadString(section, "fpga_ttf", default_ttf, false));
        }

    if (appmode == 'A')
        {
        // application
        if (!strcmp(CrateName, "LTR010"))
            {
            CrateType = CRATE_TYPE_LTR010;
            }
        else if (!strcmp(CrateName, "LTR021"))
            {
            CrateType = CRATE_TYPE_LTR021;
            }
        else if (!strcmp(CrateName, "LTR030"))
            {
            CrateType = CRATE_TYPE_LTR030;
            }
        else if (!strcmp(CrateName, "LTR031"))
            {
            CrateType = CRATE_TYPE_LTR031;
            }
        else
            {
            CrateType = CRATE_TYPE_UNKNOWN;
            }

        switch (CrateType)
            {
            case CRATE_TYPE_LTR010:
            case CRATE_TYPE_LTR021:
            case CRATE_TYPE_LTR030:
            case CRATE_TYPE_LTR031:
                {
                bool FPGALoadResult = true;
                if (FPGAData)
                    {
                    FPGALoadResult = ltr_USB->Application->LoadFPGA(FPGAData, 7, 0);
                    crate_ttf = "User FPGA";
                    }
                else if (!crate_ttf.IsEmpty())
                    {
                    FPGALoadResult = ltr_USB->Application->LoadFPGAFromFile(crate_ttf.c_str(), 7, 0);
                    }

                if (!FPGALoadResult)
                    {
                    g_Log.Write(Format("Cannot LoadFPGA CSN=%.16s TTF_FILE=%s", ARRAYOFCONST(((char*)descr.Module.SerialNumber, crate_ttf))),
                        LOGTASK_CRATE_INIT, LOGL_ERROR);
                    return false;
                    }

                ltr_USB->Application->ConvertFPGADescription(&descr);
                g_Log.Write("LoadFPGA() -> ok.", LOGTASK_CRATE_INIT, LOGL_DEBUG3);
                g_Log.Write(Format("+ TTF_FILE = %s", ARRAYOFCONST((crate_ttf))), LOGTASK_CRATE_INIT, LOGL_VERBOSE);
                g_Log.Write(Format("+ FpgaName = %s", ARRAYOFCONST(((char*)descr.Fpga.Name))), LOGTASK_CRATE_INIT, LOGL_VERBOSE);
                g_Log.Write(Format("+ FpgaVersion = %d.%d.%d.%d", ARRAYOFCONST((
                    (int)(descr.Fpga.FirmwareVersion>>24)&0xFF,
                    (int)(descr.Fpga.FirmwareVersion>>16)&0xFF,
                    (int)(descr.Fpga.FirmwareVersion>> 8)&0xFF,
                    (int)(descr.Fpga.FirmwareVersion>> 0)&0xFF))),
                    LOGTASK_CRATE_INIT, LOGL_VERBOSE);
                g_Log.Write(Format("+ FpgaComment = %s", ARRAYOFCONST(((char*)descr.Fpga.Comment))),
                    LOGTASK_CRATE_INIT, LOGL_VERBOSE);
                }
                break;
            } /* Switch CrateType */

        if (ltr_USB->Application->PutArray(SEL_AVR_DM|0x8006, (BYTE*)&baud_rate, 2) &&
            ltr_USB->Application->GetModuleDescription(&descr))
            {
            g_Log.Write(Format("+ BaudRate = 0x%0.4X  DMAR = %d  DMAW = %d", ARRAYOFCONST(((int)baud_rate, 7, 0))),
                LOGTASK_CRATE_INIT, LOGL_VERBOSE);
            }
        else
            {
            g_Log.Write(Format("Cannot set baudrate && GetModuleDescription after Load FPGA CSN=%.16s TTF_FILE=%s", ARRAYOFCONST(((char*)descr.Module.SerialNumber, crate_ttf))),
                LOGTASK_CRATE_INIT, LOGL_ERROR);
            return false;
            }
        } /* appmode == 'A' */
    else
        {
        // bootloader
        CrateType = CRATE_TYPE_BOOTLOADER;
        sflags |= CRATE_FLAG_BOOT_LOADER;
        }

    /*   - */
    if (!StartServiceThreads())
        {
        g_Log.Write(Format("Cannot start crate service threads for crate %s", ARRAYOFCONST((path))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }

    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::InitCrate(void)
    {
    baud_rate = 0;
    g_Log.Write(Format("Connecting IP crate %s ...", ARRAYOFCONST((path))),
        LOGTASK_CRATE_INIT, LOGL_INFO);
    if (!ltr_ip->OpenDeviceByIP(ip_addr))
        {
        g_Log.Write(Format("Cannot open crate %s: %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }

    g_Log.Write("OpenLDeviceByIP() -> ok", LOGTASK_CRATE_INIT, LOGL_DEBUG3);
    TDESCRIPTION_LTR010 descr;
    char CrateName[16];

    if (!ltr_ip->GetModuleName(CrateName, sizeof(CrateName)))
        {
        g_Log.Write(Format("Error in GetModuleName(%s): %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }
    g_Log.Write("GetModuleName() -> ok", LOGTASK_CRATE_INIT, LOGL_DEBUG3);

    if (!ltr_ip->GetModuleDescription(&descr))
        {
        g_Log.Write(Format("Error in GetModuleDescription(%s): %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }
    g_Log.Write("GetModuleDescription() -> ok", LOGTASK_CRATE_INIT, LOGL_DEBUG3);

    memcpy(csn, descr.Module.SerialNumber, sizeof(csn));
    g_Log.Write("Crate Info:", LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ Location = %s", ARRAYOFCONST(((char*)path))), LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ DeviceName = %.16s", ARRAYOFCONST(((char*)descr.Module.DeviceName))), LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ SerialNumber = %.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber))), LOGTASK_CRATE_INIT, LOGL_INFO);
    g_Log.Write(Format("+ DeviceComment = %s", ARRAYOFCONST(((char*)descr.Module.Comment))), LOGTASK_CRATE_INIT, LOGL_VERBOSE);

    int v1 = 0, v2 = 0, v3 = 0, v4 = 0;
    if (sscanf(descr.Cpu.Comment, "Bootloader v%d.%d.%d.%d", &v1, &v2, &v3, &v4) == 4)
        {
        g_Log.Write(Format("+ BootLoaderVersion = %d.%d.%d.%d", ARRAYOFCONST((v1, v2, v3, v4))),
            LOGTASK_CRATE_INIT, LOGL_VERBOSE);
        }

    if (descr.Cpu.FirmwareVersion != 0xFFFFFFFF)
        {
        g_Log.Write(Format("+ FirmwareVersion = %d.%d.%d.%d", ARRAYOFCONST((
            (int)(descr.Cpu.FirmwareVersion >> 24) & 0xFF,
            (int)(descr.Cpu.FirmwareVersion >> 16) & 0xFF,
            (int)(descr.Cpu.FirmwareVersion >> 8) & 0xFF,
            (int)(descr.Cpu.FirmwareVersion) & 0xFF))),
            LOGTASK_CRATE_INIT, LOGL_VERBOSE);
        }

    g_Log.Write(Format("+ FpgaName = %s", ARRAYOFCONST(((char*)descr.Fpga.Name))),
        LOGTASK_CRATE_INIT, LOGL_VERBOSE);
    g_Log.Write(Format("+ FpgaVersion = %d.%d.%d.%d", ARRAYOFCONST((
        (int)(descr.Fpga.FirmwareVersion >> 24) & 0xFF,
        (int)(descr.Fpga.FirmwareVersion >> 16) & 0xFF,
        (int)(descr.Fpga.FirmwareVersion >> 8) & 0xFF,
        (int)(descr.Fpga.FirmwareVersion) & 0xFF))),
        LOGTASK_CRATE_INIT, LOGL_VERBOSE);
    g_Log.Write(Format("+ FpgaComment = %s", ARRAYOFCONST(((char*)descr.Fpga.Comment))),
        LOGTASK_CRATE_INIT, LOGL_VERBOSE);

    if (!strcmp(CrateName, "LTR010"))
        {
        CrateType = CRATE_TYPE_LTR010;
        }
    else if (!strcmp(CrateName, "LTR021"))
        {
        CrateType = CRATE_TYPE_LTR021;
        }
    else if (!strcmp(CrateName, "LTR030"))
        {
        CrateType = CRATE_TYPE_LTR030;
        }
    else if (!strcmp(CrateName, "LTR031"))
        {
        CrateType = CRATE_TYPE_LTR031;
        }
    else
        {
        CrateType = CRATE_TYPE_UNKNOWN;
        }

    /*    ini- */
    //AnsiString section = Format("LTR010_%.16s", ARRAYOFCONST(((char*)descr.Module.SerialNumber)));

    /*  FPGA   --  ,  LAN,  .   fpga_info
           . */

    /* Set baud rate */
    if (ltr_ip->PutArray(SEL_AVR_DM | 0x8006, (BYTE*)&baud_rate, 2))
        {
        g_Log.Write(Format("+ BaudRate = 0x%0.4X  DMAR = %d  DMAW = %d", ARRAYOFCONST(((int)baud_rate, 7, 0))),
            LOGTASK_CRATE_INIT, LOGL_VERBOSE);
        }
    else
        {
        g_Log.Write(Format("Cannot set baudrate on %s : %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::StartServiceThreads(void)
    {
    StopServiceThreads();
    mconfig = 0;

    DeviceHANDLE = ltr_USB->GetUSBDeviceHandle();

    if (DeviceHANDLE == NULL)
        {
        g_Log.Write("Cannot start service threads on unconnected device.",
            LOGTASK_CRATE_INIT, LOGL_ERROR);
        return false;
        }

    ThreadsRunning = true;

    recv_buf_thread = new TReadUSBCrateThread(this, false);
    if (WaitForSingleObject(((TReadUSBCrateThread*)recv_buf_thread)->init_complete, INFINITE) != WAIT_OBJECT_0)
        {
        g_Log.Write("Cannot init ReadCrateThread.", LOGTASK_CRATE_INIT, LOGL_ERROR);
        StopServiceThreads();
        return false;
        }

    send_buf_thread = new TWriteUSBCrateThread(this, false);
    if (WaitForSingleObject(((TWriteUSBCrateThread*)send_buf_thread)->init_complete, INFINITE) != WAIT_OBJECT_0)
        {
        g_Log.Write("Cannot init WriteCrateThread.", LOGTASK_CRATE_INIT, LOGL_ERROR);
        StopServiceThreads();
        return false;
        }

    g_Log.Write("Crate service started.", LOGTASK_CRATE_INIT, LOGL_INFO);
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::StartServiceThreads(void)
    {
    StopServiceThreads();
    mconfig = 0;

    ThreadsRunning = true;
    WSAResetEvent(hAbortEvent);

    data_thread = new TIPCrateDataThread(this, false);
    if (WaitForSingleObject(((TIPCrateDataThread*)data_thread)->init_complete, INFINITE) != WAIT_OBJECT_0)
        {
        g_Log.Write("Cannot init DataThread.", LOGTASK_CRATE_INIT, LOGL_ERROR);
        StopServiceThreads();
        return false;
        }

    g_Log.Write("Crate service started.", LOGTASK_CRATE_INIT, LOGL_INFO);
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::StopServiceThreads()
    {
    WaitForSingleObject(ThreadTerminationInUse, INFINITE);

    ltr_USB->CancelIO();

    ThreadsRunning = false;

    if (recv_buf_thread != NULL)
        recv_buf_thread->Terminate();

    if (send_buf_thread != NULL)
        send_buf_thread->Terminate();

    EndThread(recv_buf_thread);
    EndThread(send_buf_thread);

    DeviceHANDLE = NULL;

    /*  -       
    if (!FPGAReload)
        {
        sflags = 0;
        mconfig = 0;
        for (int i = 0; i <MODULE_MAX; i++)
            {
            module[i].mid = 0;
            }
        }
    */

    ReleaseMutex(ThreadTerminationInUse);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateIP::StopServiceThreads()
    {
    WaitForSingleObject(ThreadTerminationInUse, INFINITE);

    WSASetEvent(hAbortEvent);
    ThreadsRunning = false;

    EndThread(data_thread);

    /*  -       
    if (!FPGAReload)
        {
        sflags = 0;
        mconfig = 0;
        for (int i = 0; i <MODULE_MAX; i++)
            {
            module[i].mid = 0;
            }
        }
    */

    ReleaseMutex(ThreadTerminationInUse);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TCrate::ProcessSyncMark(BYTE mark_flags, DWORD *pNewDataCounters)
    {
    if ((mark_flags & 0x30) == 0)
        return;
    if (mark_flags & 0x10) start_mark_counter++;
    if (mark_flags & 0x20) second_mark_counter++;
    DWORD tmark =
        ((start_mark_counter & 0xFFFF) << 16)
        | (second_mark_counter & 0xFFFF);

    for (int k = 0; k < MODULE_ARRAYLEN; k++)
        {
        if (module[k].ModulePresence)
            {
            module[k].recv_buf[module[k].recv_buf_ptr] = SIGNAL_TSTAMP;
            module[k].recv_buf_ptr = (module[k].recv_buf_ptr + 1) % MODULE_RECV_BUF_SIZE;

            module[k].recv_buf[module[k].recv_buf_ptr] = tmark;
            module[k].recv_buf_ptr = (module[k].recv_buf_ptr + 1) % MODULE_RECV_BUF_SIZE;

            pNewDataCounters[k] += 2;
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrate::ParseDataFromCrate(LPDWORD lpBuffer, DWORD NumDWords)
    {
    bool result = true;
    DWORD NewDataForModules[MODULE_ARRAYLEN];

    if ((NumDWords == 0) || (lpBuffer == NULL))
        return false;

    //  
    total_count_recv += NumDWords;
    memset(NewDataForModules, 0, sizeof(NewDataForModules));
    //      - -> {0xFFFFFFFF, 0xFFFFAAXX}
    //     
    for (DWORD i = 0; i < NumDWords; i++)
        {
        DWORD data = lpBuffer[i];

        if (ReadRawData && module[MODULE_INDEX_RAW].ModulePresence)
            {
            module[MODULE_INDEX_RAW].recv_buf[module[MODULE_INDEX_RAW].recv_buf_ptr]=data;
            module[MODULE_INDEX_RAW].recv_buf_ptr=(module[MODULE_INDEX_RAW].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;
            NewDataForModules[MODULE_INDEX_RAW]++;
            }

        int ModuleSlot = (data >> 8) & 0xF;

        /*    */
        if (special_code && ((data & 0xFFFFFF00) == 0xFFFFAA00))
            {
            if (!ProcessCrateError(data))
                {
                g_Log.Write(Format("Crate sent signal %0.2X. Cannot read crate registers. CSN=%.16s",
                    ARRAYOFCONST(((int)data&0xFF, (char*)csn))),
                    LOGTASK_CRATE_READ, LOGL_ERROR);
                sflags |= CRATE_FLAG_HW_ERROR;
                result = false;
                }
            continue; //   switch
            }
        special_code = false;

        switch (data & LTR010_CMD_DATA_MASK)
            {
            /*    (   LTR030) */
            case LTR010USERDATA:
                ModuleSlot = MODULE_INDEX_USER;
                /* no break! */
            /*   */
            case LTR010DATA:
                //     
                if (module[ModuleSlot].ModulePresence)
                    {
                    module[ModuleSlot].recv_buf[module[ModuleSlot].recv_buf_ptr]=data;
                    module[ModuleSlot].recv_buf_ptr=(module[ModuleSlot].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;
                    NewDataForModules[ModuleSlot]++;
                    }
                break;

            case LTR010CMD:
                //   
                if (!module[ModuleSlot].ModulePresence) break;

                //     41, 42, 43- ,    LTR021
                if  (((data & 0xCFFFC0C0) == LTR010CMD) &&
                    (module[ModuleSlot].mid==0x2B2B || module[ModuleSlot].mid==0x2929 ||
                     module[ModuleSlot].mid==0x2A2A || module[ModuleSlot].mid==0x1515))
                    {
                    ProcessSyncMark(data >> 24, NewDataForModules);
                    break;
                    }

                if ((module[ModuleSlot].mid == 0x2222) && ((data & 0x80E0) == 0x80C0))
                    {   //    34- 
                    //      ,      
                    module[ModuleSlot].count_recv+=1024;
                    module[ModuleSlot].fifo_size = ((data>>16)&0xFFFF)|((data&0x11)<<16)|((data&0x11)<<16);
                    break;
                    }

                // LTR34 echo data
                if ((module[ModuleSlot].mid == 0x2222) && ((data & 0x80E0) == 0x80E0))
                    {
                    int FIFO = (module[ModuleSlot].count_send - module[ModuleSlot].count_recv);
                    FIFO = std::max<int>(FIFO,0);
                    module[ModuleSlot].fifo_size=((data&0x11)<<16)|(FIFO>>10);
                    }

                //    
                //  LTR11 -  -  -     ,
                //        ,     
                //    
                if (((data & 0xC0C0) == LTR010CMD_RESET) ||
                    ((module[ModuleSlot].mid == 0x0B0B) && ((data & 0xF0FF) == LTR010CMD_RESET)))
                    {
                    WORD old_mid = module[ModuleSlot].mid, mask=(1<<ModuleSlot);
                    WORD NewMid=data>>16;

                    bool BadLastModuleAndNoReset=false;
                    bool BadNewModuleName = false;
                    if ((old_mid&0xFF)!=((old_mid>>8)&0xFF)&&((data&0xF0FF)!=LTR010CMD_RESET))
                        { //    LTR11
                        BadLastModuleAndNoReset=true;
                        g_Log.Write(Format("RST.XXXX Bad Module. slot=%d  mid=%0.4X, Data =%0.8X ", ARRAYOFCONST(((int)ModuleSlot+1, (int)module[ModuleSlot].mid, data))),
                             LOGTASK_CRATE_READ, LOGL_ERROR);
                        }
                    else if ((NewMid&0xFF)!=((NewMid>>8)&0xFF))
                        {
                        BadNewModuleName = true;
                        g_Log.Write(Format("RST.XXXX Bad Module. slot=%d  mid=%0.4X, Data =%0.8X ", ARRAYOFCONST(((int)ModuleSlot+1, (int)NewMid, data))),
                            LOGTASK_CRATE_READ, LOGL_ERROR);
                        }

                    if (old_mid!=NewMid&&!BadLastModuleAndNoReset&&!BadNewModuleName)
                        {
                        module[ModuleSlot].mid = NewMid;

                        g_Log.Write(Format("RST.XXXX found new module. slot=%d  mid=%0.4X (LTR%d)", ARRAYOFCONST(((int)ModuleSlot+1, (int)module[ModuleSlot].mid, ((int)module[ModuleSlot].mid&0xFF)))),
                            LOGTASK_CRATE_READ, LOGL_INFO);
                        module[ModuleSlot].count_recv = module[ModuleSlot].count_send = module[ModuleSlot].fifo_size = 0;

                        switch (module[ModuleSlot].mid)
                            {
                            case 0x0101: // LTR01 -   
                                if (ModuleSlot&1) break;
                            case 0x2222: // LTR34
                                if(old_mid==0xFFFF)
                                    {
                                    WORD baud_rate_new = baud_rate | mask;
                                    if (PutArray(CM_APP, SEL_AVR_DM | 0x8006, (BYTE*)&baud_rate_new, 2))
                                        {
                                        /*
                                        WORD NumFastModules = 0;
                                        for (int j = 0; j < MODULE_MAX; j++)
                                            {
                                            if (baud_rate&(1<<j))
                                                NumFastModules++;
                                            }
                                        */
                                        baud_rate = baud_rate_new;
                                        g_Log.Write(Format("RST.XXXX change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate))),
                                            LOGTASK_CRATE_READ, LOGL_VERBOSE);
                                        //InterlockedExchangeAdd(&mconfig, mask);
                                        }
                                    else
                                        {
                                        g_Log.Write(Format("RST.XXXX cannot change baud rate to 0x%0.4X", ARRAYOFCONST(((int)baud_rate))),
                                            LOGTASK_CRATE_READ, LOGL_ERROR);
                                        }
                                    }
                                break;
                            case 0x2B2B:
                            case 0x2929:
                            case 0x2A2A:
                            case 0x1515:
                                start_mark_counter=second_mark_counter=0; //  
                                break;
                            default:
                                break;
                            } /* switch (mid) */
                        MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
                        }
                    } /* reset */

                module[ModuleSlot].recv_buf[module[ModuleSlot].recv_buf_ptr]=data;
                module[ModuleSlot].recv_buf_ptr=(module[ModuleSlot].recv_buf_ptr+1)%MODULE_RECV_BUF_SIZE;
                NewDataForModules[ModuleSlot]++;
                break;

            case LTR010CRATE_CMD:
                {
                //    
                /*    -    :
                 *  -       ( ..   )
                 *    :   FFFFFFFF,  FFFFAAxx
                 *      - 0xAA & 0xC0 == 0x80      
                 *    (  );    special_code
                 *  -    
                 *    : x000FF10,  28 = start,  29 = 1sec
                 *
                 *       16  .
                 */
                WORD CmdCode = data & 0xFFFF;
                switch (CmdCode)
                    {
                    case 0xFF10: //   
                        ProcessSyncMark(data >> 24, NewDataForModules);
                        break;
                    case 0xFFFF: //  
                        special_code = true;
                        break;
                    default:
                        g_Log.Write(Format(
                            "Unknown crate command code (data word %0.8X)",
                            ARRAYOFCONST((data))),
                            LOGTASK_CRATE_READ, LOGL_ERROR);
                        break;
                    } /* switch CmdCode */
                }
                break;

            default:
                g_Log.Write(Format("Bad Crate Data (Invalid bit combination) Data =%0.8X ", ARRAYOFCONST((data))),
                    LOGTASK_CRATE_READ, LOGL_ERROR);
                break;
            } /* switch (data & LTR010_CMD_DATA_MASK) */
        } /* for i */

    for (int i = 0; i < MODULE_MAX; i++)
        {
        if (NewDataForModules[i])
            {
            module[i].count_recv += NewDataForModules[i];
            module[i].count_FIFO_recv += NewDataForModules[i];
            }
        }
    if (NewDataForModules[MODULE_INDEX_USER])
        {
        module[MODULE_INDEX_USER].count_recv += NewDataForModules[MODULE_INDEX_USER];
        module[MODULE_INDEX_USER].count_FIFO_recv += NewDataForModules[MODULE_INDEX_USER];
        }
    if (ReadRawData)
        {
        module[MODULE_INDEX_RAW].count_recv += NewDataForModules[MODULE_INDEX_RAW];
        module[MODULE_INDEX_RAW].count_FIFO_recv += NewDataForModules[MODULE_INDEX_RAW];
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
#define LTR34_BUFFER_ALERT_SIZE (1572864)
#define NUM_MIX_WORDS_FOR_ONE_FAST_MODULE   (20)
DWORD TCrate::PrepareDataForCrate(LPDWORD lpBuffer, DWORD MaxDWords, DWORD &NumDataFast)
    {
    DWORD size = 0;
    DWORD NumDataFromFastModules = 0;
    DWORD NumMixFast = NUM_MIX_WORDS_FOR_ONE_FAST_MODULE;

    if ((MaxDWords == 0) || (lpBuffer == NULL))
        return 0;

    //      
    for (int i = 0; i < MODULE_MAX; i++)
        {
        if (!module[i].ModulePresence || !module[i].send_buf_cntr)
            continue;

        if (baud_rate & (1 << i))
            {   //  
            if (module[i].mid == 0x2222)
                {
                if (module[i].count_send - module[i].count_recv >= LTR34_BUFFER_ALERT_SIZE)
                    continue; //  LTR34  
                }
            NumDataFromFastModules += module[i].send_buf_cntr;
            }

        size += module[i].send_buf_cntr;
        }

    if ((size == 0) && !mconfig)
        return 0;

    size = 0; /*    */

    if (mconfig && ThreadsRunning) //   
        {
        WORD mconf = InterlockedExchange(&mconfig, 0);
        //   STOP+RESET+STOP
        for (int i = 0; i < MODULE_MAX; i++)
            {
            if (mconf & (1 << i))
                {
                *lpBuffer++ = LTR010CMD_STOP |(i<<8);
                *lpBuffer++ = LTR010CMD_RESET|(i<<8)|(0x0101<<16);
                *lpBuffer++ = LTR010CMD_STOP |(i<<8);
                size += 3;
                module[i].count_send = 0;
                module[i].count_recv = 0;
                module[i].send_buf_cntr = 0;
                module[i].send_buf_cntr_sended = 0;
                g_Log.Write(Format("WST.XXXX send STOP+RESET+STOP to module. slot=%d", ARRAYOFCONST(((int)i+1))),
                    LOGTASK_CRATE_WRITE, LOGL_INFO);
                }
            }
        }

    while ((size < MaxDWords) && ThreadsRunning)
        {
        DWORD NumUsedFastModules = 0; //   ,     
        DWORD LastSize = size;
        //      
        //  20      
        for (int i = 0; (i < MODULE_MAX) && (size < MaxDWords); i++)
            {
            if (!module[i].ModulePresence || !module[i].send_buf_cntr)
                continue;
            DWORD NewData = module[i].send_buf[module[i].send_buf_cntr_sended];
            if (baud_rate & (1 << i))
                {   //  
                NumUsedFastModules++;
                if (module[i].mid == 0x2222)
                    {
                    if ((NewData & 0xC000) == LTR010CMD)
                        {//  ,        
                        //   34   ,     
                        module[i].count_send--; //        FIFO
                        if((NewData & 0xC0C0) == LTR010CMD_RESET)
                            {//  34 
                            module[i].count_send = 0;
                            module[i].count_recv = 0;
                            }
                        }
                    else
                        {//   34 
                        if (module[i].count_send - module[i].count_recv >= LTR34_BUFFER_ALERT_SIZE) // 100  -     
                            { //  
                            continue;
                            }
                        }
                    }

                NumDataFromFastModules--;

                if (++NumDataFast >= NumMixFast)
                    {
                    NumDataFast = 0;
                    memset(AllowLowSpeedModule, 1, sizeof(AllowLowSpeedModule));
                    }
                }
            else
                { //  
                if (!AllowLowSpeedModule[i] && NumDataFromFastModules)
                    continue;  //  ,    20   
                AllowLowSpeedModule[i]  = 0;
                }

            module[i].count_send++;
            *lpBuffer++ = NewData;
            size++;

            if (++module[i].send_buf_cntr_sended >= module[i].send_buf_cntr)
                {
                module[i].send_buf_cntr_sended = 0;
                module[i].send_buf_cntr = 0;
                }
            } /* for i */

        if (size == LastSize)
            break;
        if (NumUsedFastModules)
            NumMixFast = NUM_MIX_WORDS_FOR_ONE_FAST_MODULE * NumUsedFastModules;
        } /* while size */
    return size;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    switch (mode)
        {
        case CM_APP:
            return ltr_USB->Application->GetArray(address, data, size);
        case CM_BOOT:
            return ltr_USB->Bootloader->GetArray(address, data, size);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? ltr_USB->Application->GetArray(address, data, size)
                : ltr_USB->Bootloader->GetArray(address, data, size);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    if (mode == CM_BOOT)
        return false;
    bool rv = ltr_ip->GetArray(address, data, size);
    if (!rv)
        g_Log.Write(Format("GetArray() failed on %s: %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_CTL, LOGL_DEBUG2);
    return rv;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    switch (mode)
        {
        case CM_APP:
            return ltr_USB->Application->PutArray(address, data, size);
        case CM_BOOT:
            return ltr_USB->Bootloader->PutArray(address, data, size);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? ltr_USB->Application->PutArray(address, data, size)
                : ltr_USB->Bootloader->PutArray(address, data, size);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size)
    {
    if (mode == CM_BOOT)
        return false;
    bool rv = ltr_ip->PutArray(address, data, size);
    if (!rv)
        g_Log.Write(Format("PutArray() failed on %s: %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_CTL, LOGL_DEBUG2);
    return rv;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    switch (mode)
        {
        case CM_APP:
            return ltr_USB->Application->GetModuleDescription(description);
        case CM_BOOT:
            return ltr_USB->Bootloader->GetModuleDescription(description);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? ltr_USB->Application->GetModuleDescription(description)
                : ltr_USB->Bootloader->GetModuleDescription(description);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    if (mode == CM_BOOT)
        return false;
    bool rv = ltr_ip->GetModuleDescription(description);
    if (!rv)
        g_Log.Write(Format("GetModuleDescription() failed on %s: %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_CTL, LOGL_DEBUG2);
    return rv;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    switch (mode)
        {
        case CM_APP:
            return false;
        case CM_BOOT:
            return ltr_USB->Bootloader->SetModuleDescription(description);
        case CM_AUTO:
        default:
            return (ltr_USB->application_active)
                ? false
                : ltr_USB->Bootloader->SetModuleDescription(description);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description)
    {
    return false; //   ,    IP  
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateUSB::CallApplication(DWORD address)
    {
    BYTE tempdata[16];

    if (!ltr_USB->Bootloader->CallApplication(address))
        return false;
    /* ,   :  application  bootloader */
    if (ltr_USB->Bootloader->GetArray(0x0 | SEL_AVR_PM, tempdata, 16))
        {
        ltr_USB->application_active = false;
        }
    else if (ltr_USB->Application->GetArray(0x0 | SEL_AVR_PM, tempdata, 16))
        {
        ltr_USB->application_active = true;
        }
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateIP::CallApplication(DWORD address)
    {
    return false; //   ,    IP  
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateUSB::CancelIO(void)
    {
    ltr_USB->CancelIO();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateIP::CancelIO(void)
    {
    // Do nothing
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int TCrateUSB::PingAndGetFailCount(void)
    {
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int TCrateIP::PingAndGetFailCount(void)
    {
    if (ltr_ip->Ping())
        {
        ping_fail_count = 0;
        }
    else
        {
        if (ping_fail_count < INT_MAX)
            ping_fail_count++;
        g_Log.Write(Format("Ping() failed on %s: %s",
            ARRAYOFCONST((path, GetErrorMessage()))),
            LOGTASK_CRATE_CTL, LOGL_DEBUG2);
        }
    return ping_fail_count;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
AnsiString TCrateIP::GetErrorMessage(void)
    {
    DWORD err = ltr_ip->GetLastErrorCode();
    int subcode = err & 0xFFFFFF;
    AnsiString msg;
    if (subcode & 0x800000)
        subcode |= 0xFF000000;
    switch (err & 0xFF000000)
        {
        case LTR_IPERR_OK:              msg = "No error";                   break;
        case LTR_IPERR_INVALID_PARAM:   msg = "Invalid parameter in call";  break;
        case LTR_IPERR_CONNECT:         msg = "Connect failed";             break;
        case LTR_IPERR_ABORT:           msg = "Aborted";                    break;
        case LTR_IPERR_TIMEOUT_W:       msg = "Write timeout";              break;
        case LTR_IPERR_TIMEOUT_R:       msg = "Read timeout";               break;
        case LTR_IPERR_CONN_LOST:       msg = "Connection lost";            break;
        case LTR_IPERR_READ:            msg = "Read error";                 break;
        case LTR_IPERR_WRITE:           msg = "Write error";                break;
        case LTR_IPERR_BAD_SIG:         msg = "Bad signature in reply";     break;
        case LTR_IPERR_BAD_REPLYLEN:    msg = "Invalid reply length";       break;
        case LTR_IPERR_CMD_FAILED:      msg = "Command failed";             break;
        case LTR_IPERR_NOT_CONNECTED:   msg = "Not connected";              break;
        case LTR_IPERR_INTERNAL:        msg = "Internal error";             break;
        default:                        msg = "Unknwon error";              break;
        }
    if (subcode != 0)
        msg = msg + " (" + subcode + ")";
    return msg;
    }
/*------------------------------------------------------------------------------------------------*/

