#include <vcl.h>
#include <stdio.h>
#pragma hdrstop
#pragma pack(1)
#include <setupapi.h>
#pragma pack()
#include "LtrCrate.h"
#include "LtrServiceCrates.h"
#pragma package(smart_init)

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
#define USBCRATEINFO_MAX        32
#define UDP_SIG_PREFIX          "LTR"
#define UDP_SEND_INTERVAL       2000
#define UDP_MAXERRMSG           10
#define UDP_RCVBUFSIZE          131072

//---------------------------------------------------------------------------
TLTRAutoDiscovery::TLTRAutoDiscovery()
    : working(false), udp_socket(INVALID_SOCKET)
    {
    //       
    send_timer.interval.QuadPart = -10000LL * UDP_SEND_INTERVAL;

    // 
    request.buf.buf = signature;
    request.buf.len = sizeof(signature);
    ZeroMemory(&request.overlap, sizeof(request.overlap));
    reply.buf.buf = recv_buf;
    reply.buf.len = sizeof(recv_buf);
    ZeroMemory(&reply.overlap, sizeof(reply.overlap));

    //   (broadcast)
    ZeroMemory(&bcast_addr, sizeof(bcast_addr));
    bcast_addr.sin_family = AF_INET;
    bcast_addr.sin_port = htons(LTR_IP_IDENTIFY_PORT);
    bcast_addr.sin_addr.s_addr = INADDR_BROADCAST;

    //   
    ZeroMemory(&peer_addr, sizeof(peer_addr));
    peer_addr.sa_family = AF_INET;

    Randomize();
    }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TLTRAutoDiscovery::~TLTRAutoDiscovery()
    {
    Stop();
    }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void TLTRAutoDiscovery::Start()
    {
    if (working) return; //  

    request.failcount = reply.failcount = 0;

    //    
    strncpy(signature, UDP_SIG_PREFIX, sizeof(signature) - 1);
    for (int i = strlen(UDP_SIG_PREFIX) + 1; i < sizeof(signature); i++)
        {
        signature[i] = Random(256);
        }

    //    
    send_timer.handle = CreateWaitableTimer(NULL, TRUE, NULL);
    request.overlap.hEvent = CreateEvent(NULL, /*manual*/ TRUE, /*initial*/ FALSE, NULL);
    reply.overlap.hEvent = CreateEvent(NULL, /*manual*/ FALSE, /*initial*/ FALSE, NULL);

    //  
    udp_socket = WSASocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP, NULL, 0, WSA_FLAG_OVERLAPPED);
    working = (udp_socket != INVALID_SOCKET) &&
              send_timer.handle && reply.overlap.hEvent && request.overlap.hEvent;

    if (working)
        {
        DWORD broadcast = 1;
        if (setsockopt(udp_socket, SOL_SOCKET, SO_BROADCAST,
            (const char*)&broadcast, sizeof(broadcast)) == SOCKET_ERROR)
            {
            g_Log.Write("Cannot enable broadcast on UDP socket", LOGTASK_GLOBAL, LOGL_ERROR);
            working = false;
            }
        }

    if (working)
        {
        DWORD reuse = 1;
        if (setsockopt(udp_socket, SOL_SOCKET, SO_REUSEADDR,
            (const char*)&reuse, sizeof(reuse)) == SOCKET_ERROR)
            {
            g_Log.Write("Cannot enable SO_REUSEADDR on UDP socket", LOGTASK_GLOBAL, LOGL_ERROR);
            working = false;
            }
        }

    if (working)
        {
        DWORD bufsize = UDP_RCVBUFSIZE;
        if (setsockopt(udp_socket, SOL_SOCKET, SO_RCVBUF,
            (const char*)&bufsize, sizeof(bufsize)) == SOCKET_ERROR)
            {
            g_Log.Write("Cannot set UDP socket receive buffer", LOGTASK_GLOBAL, LOGL_ERROR);
            working = false;
            }
        }

    if (working)
        {
        sockaddr_in bind_addr;
        ZeroMemory(&bind_addr, sizeof(bind_addr));
        bind_addr.sin_family = AF_INET;
        bind_addr.sin_port = 0;
        bind_addr.sin_addr.s_addr = INADDR_ANY;
        if (bind(udp_socket, (sockaddr*)&bind_addr, sizeof(bind_addr)) == SOCKET_ERROR)
            {
            g_Log.Write("Cannot bind UDP socket", LOGTASK_GLOBAL, LOGL_ERROR);
            working = false;
            }
        }

    if (working)
        {
        g_Log.Write("UDP auto-discovery mode enabled", LOGTASK_GLOBAL, LOGL_INFO);
        do_Send();
        do_Receive();
        }
    else
        {
        g_Log.Write("Unable to initialize UDP auto-discovery", LOGTASK_GLOBAL, LOGL_ERROR);
        Stop();
        }
    }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void TLTRAutoDiscovery::Stop()
    {
    if (working)
        {
        g_Log.Write("UDP auto-discovery mode disabled", LOGTASK_GLOBAL, LOGL_INFO);
        working = false;
        }
    if (udp_socket != INVALID_SOCKET)
        {
        closesocket(udp_socket);
        udp_socket = INVALID_SOCKET;
        }
    if (reply.overlap.hEvent)
        CloseHandle(reply.overlap.hEvent);
    if (request.overlap.hEvent)
        CloseHandle(request.overlap.hEvent);
    if (send_timer.handle)
        CloseHandle(send_timer.handle);
    reply.overlap.hEvent = request.overlap.hEvent = send_timer.handle = NULL;
    }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void TLTRAutoDiscovery::do_Send()
    {
    if (!working) return;
    ResetEvent(request.overlap.hEvent);
    if (WSASendTo(udp_socket, &request.buf, 1, NULL, 0,
        (sockaddr*)&bcast_addr, sizeof(bcast_addr), &request.overlap, NULL) == SOCKET_ERROR)
        {
        int err = WSAGetLastError();
        if (err != WSA_IO_PENDING)
            {
            if (request.failcount < UDP_MAXERRMSG)
                {
                g_Log.Write(Format("Cannot send auto-discovery request (error %d)",
                    ARRAYOFCONST((err))), LOGTASK_GLOBAL, LOGL_DEBUG3);
                if (++request.failcount == UDP_MAXERRMSG)
                    g_Log.Write("Further auto-discovery send error messages will be suppressed",
                        LOGTASK_GLOBAL, LOGL_DEBUG3);
                }
            SetEvent(request.overlap.hEvent); //    
            }
        }
    //   (    )
    SetWaitableTimer(send_timer.handle, &send_timer.interval, 0, NULL, NULL, FALSE);
    }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void TLTRAutoDiscovery::do_Receive()
    {
    if (!working) return;
    peer_addr_size = sizeof(peer_addr);
    reply.flags = 0;
    if (WSARecvFrom(udp_socket, &reply.buf, 1, NULL, &reply.flags,
        &peer_addr, &peer_addr_size, &reply.overlap, NULL) == SOCKET_ERROR)
        {
        int err = WSAGetLastError();
        if (err != WSA_IO_PENDING)
            {
            g_Log.Write(Format("UDP receive failed (error %d), auto-discovery will not work",
                ARRAYOFCONST((err))), LOGTASK_GLOBAL, LOGL_ERROR);
            //   auto-discovery 
            Stop();
            }
        }
    //   ,   reply.overlap.hEvent
    }
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool TLTRAutoDiscovery::Worker(DWORD *pPeerIP)
    {
    HANDLE wait_list[2];
    DWORD peer_ip = 0;

    if (!working)
        {
        if (pPeerIP) *pPeerIP = 0;
        return false;
        }

    //        ,   
    wait_list[0] = request.overlap.hEvent;
    wait_list[1] = send_timer.handle;
    if (WaitForMultipleObjects(2, wait_list, /*all*/ TRUE, 0) == WAIT_OBJECT_0)
        {
        do_Send();
        }

    // ,      - 
    if (WaitForSingleObject(reply.overlap.hEvent, 0) == WAIT_OBJECT_0)
        {
        DWORD recv_len;
        if (WSAGetOverlappedResult(udp_socket, &reply.overlap, &recv_len, FALSE, &reply.flags))
            { //  
            DWORD ip = (peer_addr.sa_family == AF_INET) ?
                ((sockaddr_in*)&peer_addr)->sin_addr.s_addr : 0;

            if ((recv_len == request.buf.len)
                && (!memcmp(request.buf.buf, reply.buf.buf, request.buf.len)))
                {
                // g_Log.Write("Auto-discovery reply received from host " +
                //     IPToStr(peer_addr.sin_addr.s_addr), LOGTASK_GLOBAL, LOGL_DEBUG3);
                peer_ip = ip;
                }
            else
                {
                g_Log.Write("Invalid auto-discovery reply from host" +
                    IPToStr(ip), LOGTASK_GLOBAL, LOGL_WARNING);
                }
            }
        else
            { //    
            int err = WSAGetLastError();
            if (reply.failcount < UDP_MAXERRMSG)
                {
                g_Log.Write(Format("Auto-discovery receive error (%d)",
                    ARRAYOFCONST((err))), LOGTASK_GLOBAL, LOGL_DEBUG3);
                if (++reply.failcount == UDP_MAXERRMSG)
                    g_Log.Write("Further auto-discovery receive error messages will be suppressed",
                        LOGTASK_GLOBAL, LOGL_DEBUG3);
                }
            }
        do_Receive();
        }

    if (pPeerIP) *pPeerIP = peer_ip;
    return (peer_ip != 0);
    }
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
__fastcall TLTRServiceCrate::TLTRServiceCrate(bool CreateSuspended) : TThread(CreateSuspended)
    {
    Priority=tpNormal;
    FreeOnTerminate=false;

    MessageQueueIn = new T_MessageQueue(100,100);
    if (MessageQueueIn == NULL)
        {
        g_Log.Write("Cannot allocate memory for MessageQueue", LOGTASK_GLOBAL, LOGL_CRITICAL);
        }

    init_complete=CreateEvent(NULL, FALSE, FALSE, NULL);
    }
//---------------------------------------------------------------------------
__fastcall TLTRServiceCrate::~TLTRServiceCrate(void)
    {
    if(MessageQueueIn!=NULL) delete MessageQueueIn;
    CloseHandle(init_complete);
    }
//---------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------
/*------------------------------------------------------------------------------------------------*/
void __fastcall TLTRServiceCrate::StopServiceForDeadCrates(void)
    {
    if (WaitForSingleObject(crate_list_access, INFINITE) != WAIT_OBJECT_0)
        return;
    //***     
    for (int i = 0; i < CRATE_MAX; i++)
        {
        if (crate_list[i]->GetState() == CRATE_ST_DEAD)
            {
            g_Log.Write(Format("Service stopped for crate %s CSN=%.16s PIDString=%s",
                ARRAYOFCONST((crate_list[i]->path, (char*)crate_list[i]->csn, (char*)crate_list[i]->pid))),
                LOGTASK_CRATE_CTL, LOGL_INFO);
            delete crate_list[i];
            crate_list[i] = NULL;
            }
        }
    ReleaseMutex(crate_list_access);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*     USB,      */
void __fastcall TLTRServiceCrate::ScanUSBCrates(void)
    {
    const CLSID USB_DEVICE_CLSID =
        {
        0xA5DCBF10, 0x6530, 0x11D2, { 0x90, 0x1F, 0x00, 0xC0, 0x4F, 0xB9, 0x51, 0xED }
        };
    const struct { WORD vid; WORD pid; } CrateID[] =
        {
        { 0x0471, 0x1010 }, { 0x0471, 0x2121 }, { 0x0471, 0x3030 }, { 0x0471, 0x3131 }
        };

    struct t_CrateInfo
        {
        PSP_DEVICE_INTERFACE_DETAIL_DATA pDevDetail;
        char *pIDString;
        bool connected;
        }
        AttachedCrates[USBCRATEINFO_MAX];
    HDEVINFO hDevInfo;
    DWORD MemberIdx = 0;
    int NumAttachedCrates = 0;
    int err;

    g_Log.Write("Scanning USB crates", LOGTASK_CRATE_CTL, LOGL_VERBOSE);

    /*     USB  */
    hDevInfo =
        SetupDiGetClassDevs(&USB_DEVICE_CLSID, NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
    if (hDevInfo == INVALID_HANDLE_VALUE)
        {
        err = GetLastError();
        g_Log.Write(Format("Cannot enumerate USB (SetupDiGetClassDevs() error %d)",
            ARRAYOFCONST((err))), LOGTASK_CRATE_CTL, LOGL_VERBOSE);
        return;
        }

    /*    */
    while (true)
        {
        DWORD DataSize;
        SP_DEVICE_INTERFACE_DATA IfaceData;
        PSP_DEVICE_INTERFACE_DETAIL_DATA pDetail;
        char *pID;
        SP_DEVINFO_DATA DevinfoData;
        bool is_crate;

        ZeroMemory(&IfaceData, sizeof(IfaceData));
        IfaceData.cbSize = sizeof(IfaceData);
        if (SetupDiEnumDeviceInterfaces(hDevInfo, NULL, &USB_DEVICE_CLSID, MemberIdx++, &IfaceData))
            {
            /*  idata     */
            if (!(IfaceData.Flags & SPINT_ACTIVE))
                continue;

            wchar_t guid_wstr[64];
            StringFromGUID2(IfaceData.InterfaceClassGuid, guid_wstr, 63);
            AnsiString guid_str = WideCharToString(guid_wstr);

            DataSize = 0;
            err = 0;
            if (SetupDiGetDeviceInterfaceDetail(hDevInfo, &IfaceData, NULL, 0, &DataSize, NULL) ||
                ((err = GetLastError()) != ERROR_INSUFFICIENT_BUFFER))
                {
                g_Log.Write(Format("USB device %s skipped (SetupDiGetDeviceInterfaceDetail() error %d)",
                    ARRAYOFCONST((guid_str, err))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                continue;
                }
            pDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(DataSize);
            if (!pDetail)
                {
                g_Log.Write(Format("USB device %s skipped (cannot allocate %d bytes for detail)",
                    ARRAYOFCONST((guid_str, DataSize))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                continue;
                }
            pDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
            DevinfoData.cbSize = sizeof(DevinfoData);
            if (!SetupDiGetDeviceInterfaceDetail(hDevInfo, &IfaceData, pDetail, DataSize,
                &DataSize, &DevinfoData))
                {
                err = GetLastError();
                g_Log.Write(Format("USB device %s skipped (SetupDiGetDeviceInterfaceDetail() error %d)",
                    ARRAYOFCONST((guid_str, err))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                free(pDetail);
                continue;
                }
            /*   *pDetail  DevinfoData */
            /*  LOCATION_INFORMATION */
            DataSize = 0;
            SetupDiGetDeviceRegistryProperty(hDevInfo, &DevinfoData, SPDRP_LOCATION_INFORMATION,
                NULL, NULL, 0, &DataSize);
            if (DataSize == 0)
                {
                g_Log.Write(Format("USB device %s skipped (SetupDiGetDeviceRegistryProperty() failed)",
                    ARRAYOFCONST((guid_str))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                free(pDetail);
                continue;
                }
            pID = (char*)malloc(DataSize);
            if (!pID)
                {
                g_Log.Write(Format("USB device %s skipped (cannot allocate %d bytes for location)",
                    ARRAYOFCONST((guid_str, DataSize))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                free(pDetail);
                continue;
                }
            if (!SetupDiGetDeviceRegistryProperty(hDevInfo, &DevinfoData, SPDRP_LOCATION_INFORMATION,
                NULL, pID, DataSize, &DataSize))
                {
                err = GetLastError();
                g_Log.Write(Format("USB device %s skipped (SetupDiGetDeviceRegistryProperty() error %d)",
                    ARRAYOFCONST((guid_str, err))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                free(pID);
                free(pDetail);
                continue;
                }
            /* ,   ,   VID/PID */
            is_crate = false;
            for (int i = 0; !is_crate && (i < sizeof(CrateID)/sizeof(CrateID[0])); i++)
                {
                char substr[32];
                sprintf(substr, "vid_%04X&pid_%04X", CrateID[i].vid, CrateID[i].pid);
                if (strstr(pDetail->DevicePath, substr))
                    is_crate = true;
                }
            if (is_crate)
                {
                g_Log.Write(Format("USB device, CRATE: %s, PIDString = %s",
                    ARRAYOFCONST((pDetail->DevicePath, pID))), LOGTASK_CRATE_CTL, LOGL_DEBUG2);
                if (NumAttachedCrates < USBCRATEINFO_MAX)
                    {
                    AttachedCrates[NumAttachedCrates].pDevDetail = pDetail;
                    AttachedCrates[NumAttachedCrates].pIDString = pID;
                    AttachedCrates[NumAttachedCrates].connected = false;
                    NumAttachedCrates++;
                    /* !!!    ,     !!! */
                    continue;
                    }
                else
                    {
                    g_Log.Write(Format("USB crate skipped (maximum %d reached)",
                    ARRAYOFCONST(((int)USBCRATEINFO_MAX))), LOGTASK_CRATE_CTL, LOGL_DEBUG1);
                    }
                }
            else
                {
                g_Log.Write(Format("USB device, alien: %s, PIDString = %s",
                    ARRAYOFCONST((pDetail->DevicePath, pID))), LOGTASK_CRATE_CTL, LOGL_DEBUG3);
                }
            free(pID);
            free(pDetail);
            }
        else /* SetupDiEnumDeviceInterfaces returned false */
            {
            err = GetLastError();
            if (err == ERROR_NO_MORE_ITEMS)
                {
                break; /*    */
                }
            else
                {
                g_Log.Write(Format("Cannot enumerate USB (SetupDiEnumDeviceInterfaces() error %d)",
                    ARRAYOFCONST((err))), LOGTASK_CRATE_CTL, LOGL_VERBOSE);
                break; /*    ,    */
                }
            }
        } /* while (true) */
    SetupDiDestroyDeviceInfoList(hDevInfo);

    /*   AttachedCrates[]       */
    /*       ,  pIPString,  pDevDetail */

    if (NumAttachedCrates > 0)
        {
        g_Log.Write(Format("Found %d USB crates", ARRAYOFCONST((NumAttachedCrates))),
            LOGTASK_CRATE_CTL, LOGL_VERBOSE);
        }
    else
        {
        g_Log.Write("No USB crates found", LOGTASK_CRATE_CTL, LOGL_VERBOSE);
        }

    /*       ,      */
    if (WaitForSingleObject(crate_list_access, INFINITE) == WAIT_OBJECT_0)
        {
        /*   */
        for (int ci = 0; ci < CRATE_MAX; ci++)
            {
            if (crate_list[ci]
                && (crate_list[ci]->GetState() != CRATE_ST_DEAD)
                && (crate_list[ci]->GetConnType() == CRATE_IFACE_USB))
                {
                bool still_connected = false;
                for (int ai = 0; ai < NumAttachedCrates; ai++)
                    {
                    if (!strcmp(AttachedCrates[ai].pDevDetail->DevicePath, crate_list[ci]->path))
                        {
                        AttachedCrates[ai].connected = true; /* ,     */
                        still_connected = true;
                        break;
                        }
                    }
                if (!still_connected)
                    crate_list[ci]->SetState(CRATE_ST_DEAD);
                }
            }
        /*   */
        for (int ai = 0; ai < NumAttachedCrates; ai++)
            {
            if (!AttachedCrates[ai].connected)
                {
                int FreeSlot = -1;
                for (int ci = 0; (FreeSlot < 0) && (ci < CRATE_MAX); ci++)
                    {
                    if (!crate_list[ci])
                        FreeSlot = ci;
                    }
                /*       ,    */
                if (FreeSlot >= 0)
                    {
                    TCrate *crate = new TCrateUSB(MessageQueueIn, MessageQueueDisplay);
                    if (crate)
                        {
                        strncpy(crate->path, AttachedCrates[ai].pDevDetail->DevicePath,
                            sizeof(crate->path) - 1);
                        crate->path[sizeof(crate->path) - 1] = '\0';
                        strncpy(crate->pid, AttachedCrates[ai].pIDString, sizeof(crate->pid) - 1);
                        crate->pid[sizeof(crate->pid) - 1] = '\0';
                        crate_list[FreeSlot] = crate;
                        /*    */
                        if (!crate->Startup(NULL, g_Config.default_ttf.c_str()))
                            {
                            g_Log.Write(Format("Cannot initialize crate %s",
                                ARRAYOFCONST((AttachedCrates[ai].pDevDetail->DevicePath))),
                                LOGTASK_CRATE_INIT, LOGL_ERROR);
                            }
                        }
                    else
                        {
                        g_Log.Write(Format("Cannot create instance for crate %s",
                            ARRAYOFCONST((AttachedCrates[ai].pDevDetail->DevicePath))),
                            LOGTASK_CRATE_INIT, LOGL_ERROR);
                        }
                    }
                else /* no free slot */
                    {
                    g_Log.Write(Format("No free slot for crate %s (maximum %d reached)",
                        ARRAYOFCONST((AttachedCrates[ai].pDevDetail->DevicePath, (int)CRATE_MAX))),
                        LOGTASK_CRATE_INIT, LOGL_ERROR);
                    break;
                    }
                } /* if !connected */
            } /* for ai */

        ReleaseMutex(crate_list_access);
        }

    /*   */
    while (--NumAttachedCrates >= 0)
        {
        free(AttachedCrates[NumAttachedCrates].pIDString);
        free(AttachedCrates[NumAttachedCrates].pDevDetail);
        }
    }

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLTRServiceCrate::ConnectCrateByIP(DWORD IP)
    {
    bool AlreadyExists = false;
    int FreeSlot = -1;

    TAddrListNode addr;
    if (!addr.SetAddr(IP)) return;

    /* ,       */
    if (WaitForSingleObject(crate_list_access, INFINITE) != WAIT_OBJECT_0)
        return;
    for (int i = 0; i < CRATE_MAX; i++)
        {
        if (!crate_list[i])
            {
            if (FreeSlot < 0)
                FreeSlot = i;
            }
        else if ((crate_list[i]->GetState() != CRATE_ST_DEAD)
                && (crate_list[i]->ip_addr == addr.dwIP))
            {
            AlreadyExists = true;
            break;
            }
        }
    /*       ,    */
    if (!AlreadyExists)
        {
        AnsiString ip_str = IPToStr(addr.dwIP);
        if (FreeSlot >= 0)
            {
            TCrate *crate = new TCrateIP(MessageQueueIn, MessageQueueDisplay, TIMEOUT_IP_IDLE);
            if (crate)
                {
                crate->ip_addr = addr.dwIP;
                strncpy(crate->path, ip_str.c_str(), sizeof(crate->path) - 1);
                crate->path[sizeof(crate->path) - 1] = '\0';
                crate_list[FreeSlot] = crate;
                /*     () */
                if (!crate->Startup(NULL, g_Config.default_ttf.c_str()))
                    {
                    g_Log.Write("Cannot initialize crate " + ip_str,
                        LOGTASK_CRATE_INIT, LOGL_ERROR);
                    }
                }
            }
        else /* no free slot */
            {
            g_Log.Write(Format("No free slot for crate %s (maximum %d reached)",
                ARRAYOFCONST((ip_str, (int)CRATE_MAX))),
                LOGTASK_CRATE_INIT, LOGL_ERROR);
            }
        } /* !AlreadyExists */
    ReleaseMutex(crate_list_access);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLTRServiceCrate::DisconnectCrateByIP(DWORD IP)
    {
    /*      */
    if (WaitForSingleObject(crate_list_access, INFINITE) != WAIT_OBJECT_0)
        return;
    for (int i = 0; i < CRATE_MAX; i++)
        {
        if (crate_list[i] && (crate_list[i]->GetState() != CRATE_ST_DEAD)
            && (crate_list[i]->ip_addr == IP))
            {
            crate_list[i]->SetState(CRATE_ST_DEAD);
            break;
            }
        }
    ReleaseMutex(crate_list_access);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLTRServiceCrate::ConnectAllIPCratesByType(int IPListType)
    {
    /*   ,   AUTOCONNECT */
    TAddrList *IPList;
    TAddrListNode *addr;

    IPList = IPCrateList.ExportList(IPListType);
    if (!IPList)
        return;
    addr = IPList->GetFirstNode();
    while (addr)
        {
        if (addr->bAutoConnect)
            {
            switch (addr->iStatus)
                {
                case IP_STATUS_OFFLINE:
                    /*  --  */
                    ConnectCrateByIP(addr->dwIP);
                    break;
                case IP_STATUS_CONNECTING:
                    /*  --     */
                    DisconnectCrateByIP(addr->dwIP);
                    StopServiceForDeadCrates();
                    ConnectCrateByIP(addr->dwIP);
                    break;
                case IP_STATUS_ONLINE:
                    /*   --   */
                    break;
                }
            }
        addr = addr->next;
        } /* while (addr) */
    delete IPList;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLTRServiceCrate::DisconnectAllIPCratesByType(int IPListType)
    {
    TAddrList *IPList;
    TAddrListNode *addr;

    IPList = IPCrateList.ExportList(IPListType);
    if (!IPList)
        return;
    addr = IPList->GetFirstNode();
    while (addr)
        {
        switch (addr->iStatus)
            {
            case IP_STATUS_CONNECTING:
            case IP_STATUS_ONLINE:
                /*   */
                DisconnectCrateByIP(addr->dwIP);
                break;
            }
        addr = addr->next;
        } /* while (addr) */
    delete IPList;
    }
/*------------------------------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
void __fastcall TLTRServiceCrate::SuspendCrates()
{
    g_Log.Write("LTRServiceCrate going to suspend.", LOGTASK_CRATE_CTL, LOGL_VERBOSE);
    if(WaitForSingleObject(crate_list_access, INFINITE)==WAIT_OBJECT_0)
    {
        for(int i=0; i<CRATE_MAX; i++)
        {
            if(crate_list[i]!=NULL) crate_list[i]->SuspendCrate();
        }
        ReleaseMutex(crate_list_access);
    }
}
//-----------------------------------------------------------------------------
void __fastcall TLTRServiceCrate::ResumeCrates()
{
    g_Log.Write("LTRServiceCrate resuming from suspend.", LOGTASK_CRATE_CTL, LOGL_VERBOSE);
    if(WaitForSingleObject(crate_list_access, INFINITE)==WAIT_OBJECT_0)
    {
        for(int i=0; i<CRATE_MAX; i++)
        {
            if(crate_list[i]!=NULL) crate_list[i]->ResumeCrate();
        }
        ReleaseMutex(crate_list_access);
    }
}

//-----------------------------------------------------------------------------
void __fastcall TLTRServiceCrate::Execute()
    {
    DWORD found_ip;
    T_Message CurrMessage;

    g_Log.Write("LTRServiceCrate thread started.", LOGTASK_CRATE_CTL, LOGL_VERBOSE);

    //  
    MessageQueueIn->PostMessage(LTR_SRV_device_changed);
    MessageQueueIn->PostMessage(LTR_SRV_server_priority_changed);
    MessageQueueIn->PostMessage(LTR_SRV_autoconnect_ip);
    MessageQueueDisplay->PostMessage(LTR_DISP_crate_list_updated);
    MessageQueueDisplay->PostMessage(LTR_DISP_ip_list_updated);

    //  auto-discovery
    if (g_Config.EnableAutoDiscovery)
        AutoDiscovery.Start();

    //   
    SetEvent(init_complete);

    while (!Terminated)
        {
        if(MessageQueueIn->PopMessage(&CurrMessage, TIMEOUT_2))
            {
#ifdef DEBUG_MESSAGES
            g_Log.Write(Format("Message=%d", ARRAYOFCONST((CurrMessage))), LOGTASK_CRATE_CTL, LOGL_DEBUG3);
#endif
            switch (CurrMessage.dwMessage)
                {
                case LTR_SRV_device_changed:
                    ScanUSBCrates();
                    break;
                case LTR_SRV_autodiscovery_changed:
                    if (g_Config.EnableAutoDiscovery)
                        {
                        AutoDiscovery.Start();
                        }
                    else
                        {
                        AutoDiscovery.Stop();
                        }
                    break;
                case LTR_SRV_autoconnect_ip:
                    ConnectAllIPCratesByType(IPLIST_STATIC);
                    ConnectAllIPCratesByType(IPLIST_DYNAMIC);
                    break;
                case LTR_SRV_disconnect_all_ip:
                    DisconnectAllIPCratesByType(IPLIST_STATIC);
                    DisconnectAllIPCratesByType(IPLIST_DYNAMIC);
                    break;
                case LTR_SRV_server_priority_changed:
                    if(!SetPriorityClass(GetCurrentProcess(), g_Config.ServerPriority))
                        g_Log.Write("Error setting server priority", LOGTASK_CRATE_CTL, LOGL_ERROR);
                    break;
                case LTR_SRV_suspend:
                    this->SuspendCrates();
                    break;
                case LTR_SRV_resume_suspend:
                    this->ResumeCrates();
                    break;
                case LTR_SRV_bury_the_dead:
                    StopServiceForDeadCrates();
                    break;
                case LTR_SRV_connect_crate:
                    ConnectCrateByIP(CurrMessage.dwParam);
                    break;
                case LTR_SRV_disconnect_crate:
                    DisconnectCrateByIP(CurrMessage.dwParam);
                    break;
                default:
                    g_Log.Write(Format("Unknown Service Crate Incoming Message=%d", ARRAYOFCONST((CurrMessage.dwMessage))),
                        LOGTASK_CRATE_CTL, LOGL_ERROR);
                    break;
                }
            }

        //       (UDP auto-discover)
        if (AutoDiscovery.Worker(&found_ip))
            {
            if (IPCrateList.AddUnique(IPLIST_DYNAMIC, found_ip, g_Config.AutoConnectDynamic))
                {
                g_Log.Write("Found new IP crate: " + IPToStr(found_ip),
                    LOGTASK_CRATE_INIT, LOGL_INFO);
                MessageQueueDisplay->PostMessage(LTR_DISP_ip_list_updated);
                if (g_Config.AutoConnectDynamic)
                    MessageQueueIn->PostMessage(LTR_SRV_connect_crate, found_ip);
                }
            }

        } /* while (!Terminated) */

    /* Terminated */

    AutoDiscovery.Stop();

    /*     */
    SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);

    //     ,   
    if (WaitForSingleObject(crate_list_access, INFINITE) == WAIT_OBJECT_0)
        {
        for (int i = 0; i < CRATE_MAX; i++)
            {
            if (crate_list[i])
                {
                delete crate_list[i];
                crate_list[i] = NULL;
                }
            }
        ReleaseMutex(crate_list_access);
        }
    else
        {
        g_Log.Write("Cannot access crate list.", LOGTASK_CRATE_CTL, LOGL_ERROR);
        }
    g_Log.Write("LTRServiceCrate thread stopped.", LOGTASK_CRATE_CTL, LOGL_VERBOSE);
    SetEvent(init_complete);
    }
//---------------------------------------------------------------------------

