#ifndef AddrListH
#define AddrListH

#include <windows.h>
#include "critical.h"

/*================================================================================================*/
#define ADDRLIST_MULTI_NUM      2   /*      */
#define IPLIST_STATIC           0
#define IPLIST_DYNAMIC          1

enum
    {
    IP_STATUS_INVALID,
    IP_STATUS_OFFLINE,
    IP_STATUS_CONNECTING,
    IP_STATUS_ONLINE
    };
/*================================================================================================*/

/*================================================================================================*/
/*   IP- */
class TAddrListNode
    {
public:
    DWORD dwIP;                     /*     */
    char szSerial[16];              /*     ( status = online) */
    int iStatus;                    /*  (/   ) */
    bool bAutoConnect;              /*     */
    TAddrListNode *next;            /*      NULL */

    /*   ( INADDR_NONE, next = NULL) */
    TAddrListNode();
    /*   ( ,  next   NULL) */
    TAddrListNode(const TAddrListNode &Source);
    /*   ( ,   next) */
    TAddrListNode& operator=(const TAddrListNode &Source);

    /*         */
    bool SetAddr(DWORD dwSetIP);
    /*    */
    void SetStatus(int iNewStatus, LPCSTR lpszSerial = NULL);
    };

/*   IP- (  ) */
class TAddrList
    {
private:
    TAddrListNode *first;           /*     */
    TAddrListNode *last;            /*     */
public:
    /*    (first == last == NULL) */
    TAddrList();
    /*    */
    TAddrList(TAddrList const &Source);
    /*   */
    TAddrList& operator=(const TAddrList &Source);
    /*  (    ) */
    ~TAddrList();

    /*      (  ) */
    TAddrListNode *GetFirstNode(void);
    /*    IP-  .    NULL */
    TAddrListNode *FindFirst(DWORD dwFindIP);
    /*      ( true,  ) */
    bool Add(DWORD dwAddIP, bool bAutoConnect);
    /*  ( )  IP-   */
    bool Delete(DWORD dwDeleteIP);
    /*   */
    void Clear(void);
friend
    void MkCopy(const TAddrList &Source, TAddrListNode *&NewFirst, TAddrListNode *&NewLast);
    };

/*  IP-       */
class TUniqueAddrMultiList
    {
private:
    TCritical cs;
    TAddrList List[ADDRLIST_MULTI_NUM];
public:
    TUniqueAddrMultiList();
    ~TUniqueAddrMultiList();

    /*     .   ,  ,  -1 */
    int Find(DWORD dwFindIP, TAddrListNode *pNodeCopy = NULL);
    /*     .  ,  true */
    bool AddUnique(int ListIndex, DWORD dwAddIP, bool bAutoConnect);
    /*     .  ,  true */
    bool Delete(int ListIndex, DWORD dwDeleteIP);
    /*     IP- */
    bool SetStatus(DWORD dwFindIP, int iNewStatus, LPCSTR lpszSerial = NULL);
    /*  AutoConnect  IP- */
    bool SetAutoConnect(DWORD dwFindIP, bool bNewValue);
    /*    */
    void ClearList(int ListIndex);
    /*     */
    TAddrList *ExportList(int ListIndex);
    /*    */
    void Clear(void);
    };
/*================================================================================================*/

#endif
