!include "Sections.nsh"

!define PROGNAME_D      "LTR Server"
!define PROGNAME_I      "ltrserver"
!define PROGVER_D       "1.5.1.1"
!define PROGVER_I       "1511"
!define REGKEY_VENDOR   "Software\L-Card"
!define REGKEY_PROGRAM  "${REGKEY_VENDOR}\${PROGNAME_I}"
!define REGKEY_UNINST   "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PROGNAME_D}"
!define SC_PROGRAMS     "$SMPROGRAMS\${PROGNAME_D}"
!define OLD_DLL_IN_SYS  "$SYSDIR\ltrapi_ll.dll"

LoadLanguageFile "${NSISDIR}\Contrib\Language files\English.nlf"
LoadLanguageFile "${NSISDIR}\Contrib\Language files\Russian.nlf"

# ------------------------------------------------------------------------------
# Localized strings
LangString SecMain      ${LANG_ENGLISH} "Main program (required)"
LangString SecManual    ${LANG_ENGLISH} "Manual"
LangString LnkPrograms  ${LANG_ENGLISH} "Start Menu shortcuts"
LangString LnkDesktop   ${LANG_ENGLISH} "Desktop icons"
LangString LnkQuick     ${LANG_ENGLISH} "QuickLaunch shortcut"
LangString FnameStart   ${LANG_ENGLISH} "Start ${PROGNAME_D}.lnk"
LangString FnameManual  ${LANG_ENGLISH} "${PROGNAME_D} manual.lnk"
LangString FnameUninst  ${LANG_ENGLISH} "Uninstall ${PROGNAME_D}.lnk"
LangString TxtOldDLL    ${LANG_ENGLISH} "The file ${OLD_DLL_IN_SYS} exists.\
    $\nThis is an internal-use DLL which ${PROGNAME_D} now stores \
    inside the program directory, and it is recommended to delete this file \
    from $SYSDIR unless you are sure that you need it there.\
    $\nDo you want the installer to delete ${OLD_DLL_IN_SYS} now?"
LangString SecMain      ${LANG_RUSSIAN} "  ()"
LangString Secmanual    ${LANG_RUSSIAN} ""
LangString LnkPrograms  ${LANG_RUSSIAN} "   $\"$\""
LangString LnkDesktop   ${LANG_RUSSIAN} "   "
LangString LnkQuick     ${LANG_RUSSIAN} "  "
LangString FnameStart   ${LANG_RUSSIAN} " ${PROGNAME_D}.lnk"
LangString FnameManual  ${LANG_RUSSIAN} " ${PROGNAME_D}.lnk"
LangString FnameUninst  ${LANG_RUSSIAN} " ${PROGNAME_D}.lnk"
LangString TxtOldDLL    ${LANG_RUSSIAN} "    ${OLD_DLL_IN_SYS}.\
    $\n  DLL,  ${PROGNAME_D}   \
      ,      \
     $SYSDIR,   ,    .\
    $\n ,      ${OLD_DLL_IN_SYS}?"


# The name of the installer
Name "${PROGNAME_D}"

# The file to write
OutFile "${PROGNAME_I}${PROGVER_I}_setup.exe"

# The default installation directory
InstallDir $PROGRAMFILES\L-Card\${PROGNAME_I}

# Registry key to check for directory (so if you install again, it will 
# overwrite the old one automatically)
InstallDirRegKey HKLM ${REGKEY_PROGRAM} "InstallPath"

Page components
Page directory
Page instfiles
UninstPage uninstConfirm
UninstPage instfiles

# ------------------------------------------------------------------------------
section "!$(SecMain)"
SectionIn RO
setOutPath $INSTDIR

# Check and suggest removal of old DLL
IfFileExists ${OLD_DLL_IN_SYS} old_dll no_old_dll
old_dll:
MessageBox MB_YESNO "$(TxtOldDLL)" /SD IDNO IDNO no_old_dll
delete "${OLD_DLL_IN_SYS}"
no_old_dll:

file ltrserver.exe
file ltrserver.ini
file ltrapi_ll.dll
file ltr010v3_0_3.ttf
file ltr010v3_0_5.ttf
file ltr010v3_5_1.ttf
file ltr010v3_5_3.ttf
file ltr010v3_5_5.ttf

#IfFileExists file_to_check_for jump_if_present

# Save install path
WriteRegStr HKLM ${REGKEY_PROGRAM} "InstallPath" "$INSTDIR"

# Create uninstaller
writeUninstaller $INSTDIR\uninstall.exe

# Create "add/remove programs" entry
WriteRegStr HKLM "${REGKEY_UNINST}" "DisplayName" "${PROGNAME_D} ${PROGVER_D}"
WriteRegStr HKLM "${REGKEY_UNINST}" "UninstallString" "$\"$INSTDIR\uninstall.exe$\""
sectionEnd
 
# ------------------------------------------------------------------------------
section "$(SecManual)" mansec_idx
setOutPath $INSTDIR\manual

file ..\..\doc\ltrserver.pdf

sectionEnd

# ------------------------------------------------------------------------------
section "$(LnkPrograms)"

CreateDirectory "${SC_PROGRAMS}"
!insertmacro SectionFlagIsSet ${mansec_idx} ${SF_SELECTED} man noman
man:
createShortCut "${SC_PROGRAMS}\$(FnameManual)"  "$INSTDIR\manual\ltrserver.pdf"
noman:
createShortCut "${SC_PROGRAMS}\$(FnameStart)"   "$INSTDIR\ltrserver.exe"
createShortCut "${SC_PROGRAMS}\$(FnameUninst)"  "$INSTDIR\uninstall.exe"
sectionEnd

# ------------------------------------------------------------------------------
section /o "$(LnkDesktop)"
createShortCut "$DESKTOP\$(FnameStart)"         "$INSTDIR\ltrserver.exe"
sectionEnd

# ------------------------------------------------------------------------------
section /o "$(LnkQuick)"
createShortCut "$QUICKLAUNCH\$(FnameStart)"     "$INSTDIR\ltrserver.exe"
sectionEnd

# ------------------------------------------------------------------------------
section "Uninstall"
 
# Always delete uninstaller first
delete $INSTDIR\uninstall.exe

# delete files
delete $INSTDIR\ltrserver.exe
delete $INSTDIR\ltrserver.ini
delete $INSTDIR\ltrapi_ll.dll
delete $INSTDIR\*.ttf
delete $INSTDIR\*.log
delete $INSTDIR\manual\ltrserver.pdf
rmdir $INSTDIR\manual
rmdir $INSTDIR

# delete shortcuts
delete "${SC_PROGRAMS}\*.lnk"
rmdir "${SC_PROGRAMS}"
delete "$DESKTOP\$(FnameStart)"
delete "$QUICKLAUNCH\$(FnameStart)"

# delete uninstall entry
DeleteRegKey HKLM "${REGKEY_UNINST}"

# delete registry key
DeleteRegKey HKLM "${REGKEY_PROGRAM}"
DeleteRegKey /ifempty HKLM "${REGKEY_VENDOR}"
 
sectionEnd

# ------------------------------------------------------------------------------
# Language selection dialog
Function .onInit
	Push "" ; terminator
	Push ${LANG_ENGLISH}
	Push English
	Push ${LANG_RUSSIAN}
	Push Russian
	Push A ; auto count
	LangDLL::LangDialog "Installer Language" "Please select the language of the installer"
	Pop $LANGUAGE
	StrCmp $LANGUAGE "cancel" 0 +2
		Abort
FunctionEnd
