#include "Logfile.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
__fastcall TLogfile::TLogfile(void)
    {
    TaskCount = 0;
    TaskNames = NULL;
    TaskLevels = NULL;
    pLogBuffer = new TStringList;
    hFile = INVALID_HANDLE_VALUE;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
__fastcall TLogfile::~TLogfile(void)
    {
    Flush();
    if (TaskLevels)
        delete[] TaskLevels;
    if (TaskNames)
        delete[] TaskNames;
    if (pLogBuffer)
        delete pLogBuffer;
    if (hFile != INVALID_HANDLE_VALUE)
        CloseHandle(hFile);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::Configure /*   */
    (
    const AnsiString &FileName,     /*   (  ':', '/', '\' -   EXE) */
    unsigned int inTaskCount,       /*   "" */
    const AnsiString *inTaskNames,  /*   TaskCount  "" */
    const int *inTaskLevels         /*       */
    )
    {
    TCSLock lock(cs);

    Flush();

    if (hFile != INVALID_HANDLE_VALUE)
        {
        CloseHandle(hFile);
        hFile = INVALID_HANDLE_VALUE;
        }

    if (TaskLevels)
        {
        delete[] TaskLevels;
        TaskLevels = NULL;
        }
    if (TaskNames)
        {
        delete[] TaskNames;
        TaskNames = NULL;
        }
    TaskCount = 0;
    TaskNames = new AnsiString[inTaskCount];
    if (!TaskNames)
        return;
    TaskLevels = new int[inTaskCount];
    if (!TaskLevels)
        {
        delete[] TaskNames;
        TaskNames = NULL;
        return;
        }
    for (unsigned int i = 0; i < inTaskCount; i++)
        {
        TaskNames[i] = inTaskNames[i];
        TaskLevels[i] = inTaskLevels[i];
        }
    TaskCount = inTaskCount;

    if (!FileName.IsEmpty())
        {
        AnsiString fname;
        if (FileName.LastDelimiter(":/\\") == 0)
            {
            fname = ExtractFilePath(Application->ExeName) + FileName;
            }
        else
            {
            fname = FileName;
            }
        hFile = CreateFile(fname.c_str(), GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_ALWAYS,
            FILE_ATTRIBUTE_NORMAL, NULL);
        if (hFile != INVALID_HANDLE_VALUE)
            {
            LONG hipart = 0;
            SetFilePointer(hFile, 0, &hipart, FILE_END);
            }
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::Write     /*     */
    (
    AnsiString Message,             /*   */
    unsigned int TaskID,            /*  "" */
    int Level                       /*   ( = ) */
    )
    {
    TCSLock lock(cs);

    if ((TaskID >= TaskCount) || (!pLogBuffer))
        return;
    if (Level > TaskLevels[TaskID])
        return;

    try
        {
        /*    , ,     */
        SYSTEMTIME st;
        GetLocalTime(&st);
        AnsiString msg = Format("[%0.2d.%0.2d.%0.4d %0.2d:%0.2d:%0.2d.%0.3d] [%s %d] ", ARRAYOFCONST((
            st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds,
            TaskNames[TaskID].c_str(), Level))) + Message;
        /*    */
        pLogBuffer->Add(msg);
        }
    catch (...)
        {
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::Flush     /*       */
    (void)
    {
    Flush(NULL, 0);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __fastcall TLogfile::Flush     /*          */
    (
    TStringList *pDisplay,          /*     (TRichEdit)  NULL */
    int MaxDisplayLines             /*  scrollback   (0 =  ) */
    )
    {
    TCSLock lock(cs);

    if (!pLogBuffer) return;
    if (pLogBuffer->Count == 0) return;

    try
        {
        if (pDisplay)
            {
            pDisplay->BeginUpdate();
            for (int i = 0; i < pLogBuffer->Count; i++)
                {
                if (MaxDisplayLines > 0)
                    {
                    while (pDisplay->Count >= MaxDisplayLines)
                        {
                        pDisplay->Delete(0);
                        }
                    }
                pDisplay->Add(pLogBuffer->Strings[i]);
                }
            pDisplay->EndUpdate();
            }

        if (hFile != INVALID_HANDLE_VALUE)
            {
            AnsiString msg;
            DWORD nw;
            for (int i = 0; i < pLogBuffer->Count; i++)
                {
                msg = pLogBuffer->Strings[i] + "\r\n";
                if (!WriteFile(hFile, msg.data(), msg.Length(), &nw, NULL))
                    break;
                }
            }
        }
    catch (...)
        {
        }
    if (hFile != INVALID_HANDLE_VALUE)
        FlushFileBuffers(hFile);
    pLogBuffer->Clear();
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
